#!/usr/bin/perl

use strict;
use AVStrings;
use Getopt::Long;

use constant DEF_LANG => 'en';

# prints out Strings from Archive or File

# get_string.pl -id $id [-lang $language -type $type -newl $newline]

my ($id,$language,$type,$newl);

GetOptions(
           "id=s" => \$id,
					 "lang:s" => \$language,
					 "type:s" => \$type,
					 "newl:s" => \$newl,
          );

my $source = AVStrings::SOURCE_FILE;
$source = AVStrings::SOURCE_DB if $type eq 'db';

$language = get_lang() unless $language;

$newl = '\n' if $newl eq '';

my $avstring = AVStrings->new($language,$source);
my $phstring = $avstring->getStrings($id);

foreach my $key ( keys %$phstring) {
  # Get String from default language we 
	if ( $phstring->{$key}->{$language} eq '' ) {
	  $language = DEF_LANG;
		$phstring = $avstring->getStrings($key,$language);
	}

  $phstring->{$key}->{$language} =~ s/\\n/$newl/g;

  print $phstring->{$key}->{$language}."\n";
}






=head1 get_lang

Returns the XKeyBoard Layout Setting.

=cut

sub get_lang  {

  my $lang;
  if (-e "/etc/lang.conf") {
    open(XKBL,"<","/etc/lang.conf");
    binmode(XKBL);
    $lang = <XKBL>;
    close(XKBL);
	}
  if ($lang eq "") {
    open(XKBL,"<","/home/archivista/.xkb-layout");
    binmode(XKBL);
    my $lang = <XKBL>;
    close(XKBL);
	}

  # cut of the _CH from 'de_CH','fr_CH' or 'it_CH'
  $lang = substr($lang,0,2);

  # Not German/French it must be english. Other Languages are not supported.
	if ($lang eq 'fr') {
    # is ok
	} elsif ($lang eq 'it') {
	  # is ok
	} elsif ($lang eq 'de') {
    # is ok
	} else {
    $lang = DEF_LANG;
	}

  $lang = DEF_LANG if $lang ne 'de' && $lang ne 'fr' && $lang ne 'it';

  return $lang;
}
