#!/bin/bash
. /etc/profile
export LC_ALL="en_US.ISO-8859-1"
. ${0%/*}/strings.in
STATUS_TIT=$ARCHIVISTA_BOX
webconfig=0
if [ "$1" = -webconfig ];then
  webconfig=1
	shift
fi
tmp=`mktemp`
tmp2=`mktemp`
backup_check ()
{
	read ddays < <(grep "days=" $1)
	read dtime < <(grep "time=" $1)
	if [ "$ddays" -a "$dtime" ]; then
	  if [ "$ddays" != "days=0" ]; then
		  echo "$2 enabled at $dtime o'clock for $ddays"
		else
		  echo "$2 not enabled"
		fi
	else
		echo "$2 not enabled"
	fi
}

cron_status ()
{
	read m h x x days junk < <(grep "$1" /etc/crontab)
	if [ "$h" -a "$m" ]; then
		[[ $m = [0-9] ]] && m=0$m
		echo "$2 enabled at $h:$m o'clock for days $days"
	else
		echo "$2 not enabled"
	fi
}

/sbin/ifconfig vmbr0 > $tmp
/sbin/ethtool vmbr0 >> $tmp
file=/etc/network/interfaces
cat $file | perl -e '@a=<>;$a=join("",@a);$a=~/(gateway\s)(.*?)(\n)/s;print "Gateway: $2\n"' >>$tmp
DNS=`perl -p -e 'print $_ s/(nameserver)(\s)(.+$)(\n)/$3 /' /etc/resolv.conf | perl -p -e 'print $_ s/(.*)(\s)(options)(.*$)/$1/' -`
(
	if grep -q UP $tmp; then
		grep "inet addr" $tmp | sed 's/\([[:alnum:]]\)  /\1\n/g'
	else
		echo not activated
	fi

	grep "Gateway" $tmp
	echo "DNS: $DNS"

	sed -n 's/.*\(HWaddr.*\)/\1/p' $tmp

	grep -e Speed -e Duplex -e "Link detect" $tmp
	echo

	if grep -q 0 /etc/https.conf; then
		echo "Web server https (SSL) disabled"
	else
		echo "Web server https (SSL) enabled"
	fi

  mysql=/home/data/archivista/mysql
  if [ -e $mysql ]; then 
	  if grep -q '^ftp' /etc/inetd.conf; then
		  echo "FTP/SMB server enabled"
	  else
		  echo "FTP/SMB server disabled"
	  fi

	  read junk junk id < <(grep '^server-id' /etc/mysql/my.cnf)
	  case $id in
	  1) 
	    if grep -q '^log-bin' /etc/mysql/my.cnf; then
		    echo "Database in master mode"
	    else
		    echo "Database in normal mode"
	     fi
	     ;;
	  2) echo "Database in slave mode" ;;
	  esac

	  if grep -q "autostart=1" /etc/cups-webconfig.conf 2>/dev/null; then
		  echo "PDF printing enabled"
		  # TODO: Maybe parse cups config for allowed IP ranges ...
	  else
		  echo "PDF printing disabled"
	  fi
	  echo
	fi

	if ps -C sshd >/dev/null; then
		echo "Remote access (SSH) active"
	else
		echo "Remote access (SSH) not active"
	fi

	if [ -e /etc/sshd.conf ]; then
		if grep -q "ssh_autostart=1" /etc/sshd.conf 2>/dev/null; then
			echo "Remote access (SSH) enabled permanently"
		fi
	fi

	if ps -C x11vnc >/dev/null; then
		echo "Graphical remote access (VNC) active"
	else
		echo "Graphical remote access (VNC) not active"
	fi
	if grep -q "autostart=1" /etc/vnc.conf 2>/dev/null; then
		echo "Graphical remote access (VNC) enabled permanently"
	fi
	echo

  if [ -d $mysql ]; then
		if [ -f "/etc/net-backup-webconfig.conf" ]; then 
	  	backup_check "/etc/net-backup-webconfig.conf" "Network backup"
		fi
		if [ -f "/etc/rsync-backup-webconfig.conf" ]; then 
	  	backup_check "/etc/rsync-backup-webconfig.conf" "Rsync network backup"
		fi
		if [ -f "/etc/usb-backup-webconfig.conf" ]; then 
	  	backup_check "/etc/usb-backup-webconfig.conf" "USB hard-disk backup"
		fi
	  echo
	fi

	echo "Hard-disk usage"
	df -h / /home/data | tr -s ' ' |
		sed '1d; s,\([^ ]*\) \([^ ]*\) \([^ ]*\) \([^ ]*\) \([^ ]*\) \([^ ]*\),\5 of \2 used on \6,'
	echo

  cat /boot/boot.msg | perl -e '@a=<>;$a=join("",@a);$a=~/(\(build\s)(.*?)(\))/;print "ArchivistaBox ($2)\n"'
	cat /proc/version | perl -e '$a=<>;$a =~ s/(.*?)(\()(.*)/$1/;print "$a\n"'
) | sed -e 's/^[[:space:]]\+//' -e 's/inet /Inet /' -e 's/HWaddr /HWaddr:/' \
        -e 's/Bcast:/Bcast: /' -e 's/Mask:/Mask: /' -e 's/addr:/addr: /' > $tmp2

if [ $webconfig -eq 0 ];then
  xterm -e "dialog --no-cancel --title \"$STATUS_TIT\" --msgbox \"`cat $tmp2`\" 0 0"
else
  cat $tmp2
fi

rm -f $tmp $tmp2
