#!/bin/bash
# Copyright (C) 2017 Archivista GmbH
# write an iso to cd/dvd
# Return codes: 0: success, 1: no config, 2: device not ok
. ${0%/*}/gnomesu.in
WODIM_TIT=$ARCHIVISTA_BOX
WODIM_WRITE_TO=`findit "WODIM_WRITE_TO" $zeile`
WODIM_NO_CFG=`findit "WODIM_NO_CFG" $zeile`
WODIM_IN_DISK=`findit "WODIM_IN_DISK" $zeile`
M_TITLE=`findit "M_ARCH_OCR_WRIT_FOLDER" $zeile`
goroot "$M_TITLE"
iso="$1"
# include shared code for speed setup
. ${0%/*}/wodim-setup.in
# Convert numeric speed variable to the format wodim recognizes.
speed2speedopt()
{
	if [ "$speed" = "0" ]; then
		speed=
	else
		speed="speed=$speed"
	fi
}
# Returns the CD/DVD capacity of the disc present in the device in bytes.
get_media_size()
{
	local capacity=`dvd+rw-mediainfo $1 2>/dev/null | sed -n 's/ *Track Size: *\([^*]*\).*/\1/p'`
	capacity=${capacity:-0} # zero if none at all
	if [ "$capacity" = "0" ]; then
		capacity=`wodim -atip 2>/dev/null | grep 'lead out' | gawk '{print $6}'`
	fi
	echo $((capacity * 2048)) # as the size is returned in CD sectors
}
# Returns the ISO size of the directories specified in bytes.
get_iso_size()
{
	local isosize=`mkisofs -rJ --graft-points -q --print-size "$@"`
	isosize=${isosize:-0} 
	echo $((isosize * 2048)) # as the size is returned in CD sectors
}
log="/home/data/archivista/av.log"
# If we are called with just a iso file we just write that
if [ "$iso" ]; then
  speed=0
  fsize=$(stat -c%s "/etc/wodim.conf")
  if [ $fsize -gt 200 ]; then
	  speed=`wodim_speed $speed`
	else 
    [ -e /etc/wodim.conf ] && . /etc/wodim.conf
	fi
	speed2speedopt
	devices=
	for dev in /dev/sr*; do
	  echo "dev:$dev" >>/tmp/log
		if [ ! -e $dev ]; then
			echo "No CD/DVD device found." >> $log
			continue
		fi
		media_size=`get_media_size $dev`
		if [ $media_size = 0 ]; then
			echo "No media found in $dev." >> $log
			continue
		fi
		devices="$devices $dev"
		devicecount=$(( $devicecount + 1 ))
		break # for now, later search for more
	done
	devices=${devices## }
	if [ -z "$devices" ]; then
	  echo "eror with $devices" >>/tmp/log
		echo "No devices with media available." >> $log
		exit 2
	fi
	dev="$devices"
	infobox "$WODIM_WRITE_TO$dev."
	wodimerr=0
	wodim dev=$dev $speed "$iso" 2>/dev/null >/dev/null || odimerr=$?
	if [ $wodimerr != 0 ]; then
		echo "Error writing the archive to $dev." >> $log
		wodimerr=4
	else
		echo "Successfully written archive to $dev." >> $log
	fi
	exit $wodimerr
else
	echo "$0 isofile"
fi

