
package Archivista::BL::Attribute;
use strict;
use Archivista::BL::Form;
use Archivista::BL::Scan;
use Archivista::BL::OCR;
use Archivista::BL::Barcode;

sub _init {
  my ($self) = @_;
  $self->{'attributes'} = {};
  $self->{'modified'} = {};
}

sub new {
  my ($cls) = @_;
  my $self = {};
  bless $self, $cls;
  $self->_init;
  return $self;
}

sub id {
  my ($self,$attributeId) = @_;
  if (defined $attributeId) {
    $self->{'attribute_id'} = $attributeId;
  } else {
    return $self->{'attribute_id'};
  }
}

sub parentId {
  my ($self,$parentId) = @_;
  # Hold the id of the parent. 
  # Parents are parameters, users, documents, pages ...
  # $archive->document->attribute => documentId is parentId
  if (defined $parentId) {
    $self->{'parent_id'} = $parentId;
  } else {
    return $self->{'parent_id'};
  }
}

sub archive {
  my ($self,$archive) = @_;
  # Hold the archive object
  if (defined $archive) {
    $self->{'archiveO'} = $archive;
  } else {
    return $self->{'archiveO'};
  }
}

sub parent {
  my ($self,$parent) = @_;
  # Hold the parent object, the object who invoque the attribute package
  # (parameter, user, document, page ... )
  # $archive->document->attribute => document object
  if (defined $parent) {
    $self->{'parentO'} = $parent;
  } else {
    return $self->{'parentO'};
  }
}

sub all {
  my ($self) = @_;
  return $self->{'attributes'};
}

sub value {
  my ($self,$value,$modified) = @_;
  my $parentId = $self->parentId;
  my $key = $self->id;
  if (defined $value) {
    # Check for standard modifications
    if ($parentId eq "FeldBreite" && $key eq "Inhalt") {
			# Parameter FeldBreite must be between 300 and 9000
	    if ($value != -1) {
	      $value = "3000" if ($value < 300 or $value > 9000);
			}
    }
    $self->{'attributes'}->{$key} = $value;
  } else {
    return $self->{'attributes'}->{$key};
  }
  if (defined $modified) {
    $self->{'modified'}->{$key} = $modified;
  } else {
    $self->{'modified'}->{$key} = 1;
  }
}

sub remove {
  my ($self) = @_;
  my $key = $self->id;
  delete $self->{'attributes'}->{$key};
  delete $self->{'modified'}->{$key};
}

sub modified {
  my ($self) = @_;
  my $key = $self->id;
  return $self->{'modified'}->{$key};
}

sub field {
  my ($self,$field) = @_;
  my $parentId = $self->parentId;
  if ($parentId =~ /^FelderObj/) {
    return Archivista::BL::Form->fieldObj($self,$field);
  } elsif ($parentId =~ /^FelderTab/) {
    return Archivista::BL::Form->fieldTab($self,$field);
  }
}

sub fields {
  my ($self,$retDS) = @_;	# Return data structur (ARRAY or HASH)
  # POST: pointer to Hash(fieldName,fieldObject)
  # fieldObject is the same object returned by Archivista::BL::Attribute::field
  my $parentId = $self->parentId;
  if ($parentId =~ /^FelderObj/) {
    return Archivista::BL::Form->fieldObjs($self);
  } elsif ($parentId =~ /^FelderTab/) {
    return Archivista::BL::Form->fieldTabs($self,$retDS);
  }
}

sub label {
  my ($self,$label) = @_;
  return Archivista::BL::Form->labelObj($self,$label);
}

sub form {
  my ($self) = @_;
  return Archivista::BL::Form->self($self);
}

sub scan {
  my ($self,$definition) = @_;
  # Create a link from 
  # $archive->parameter('ScannenDefinitionen')->attribute('Inhalt')
  # to the scan field parser module
  return Archivista::BL::Scan->definition($self,$definition);
}

sub scans {
  my ($self,$retDS) = @_;
  # Return all scan definitions
  return Archivista::BL::Scan->definitions($self,$retDS);
}

sub ocr {
  my ($self,$definition) = @_;
  # Create a link from 
  # $archive->parameter('OCRSets')->attribute('Inhalt')
  # to the ocr field parser module
  return Archivista::BL::OCR->definition($self,$definition);
}

sub ocrs {
  my ($self,$retDS) = @_;
  # Return all ocr definitions
  return Archivista::BL::OCR->definitions($self,$retDS);
}

sub barcode {
  my ($self,$definition) = @_;
  # Create a link from
  # $archive->parameter('Barcodes')->attribute('Inhalt')
  # to the barcode parser module
  return Archivista::BL::Barcode->definition($self,$definition);
}

sub barcodes {  
  my ($self,$retDS) = @_; # Return all barcode definitions
  return Archivista::BL::Barcode->definitions($self,$retDS);
}

1;

