
package Archivista::BL::Barcode;
use strict;
use Archivista::Config;

sub _init {
  my ($self) = @_;
	$self->{'attribute'}->id("Art");
	$self->{'attribute'}->value("parameter");
	$self->{'attribute'}->id("Tabelle");
	$self->{'attribute'}->value("parameter");
  $self->{'attribute'}->id("Inhalt");
}

sub _initBarcodeDefinition {
  my ($self) = @_;
	$self->name($self->_parseBarcodeDefinition(0));
	$self->position($self->_parseBarcodeDefinition(1));
	$self->firstBarcodeType($self->_parseBarcodeDefinition(2));
  $self->orientation($self->_parseBarcodeDefinition(3));
	$self->checkCharacter($self->_parseBarcodeDefinition(4));
	$self->secondBarcodeType($self->_parseBarcodeDefinition(5));
	$self->verticalStretch($self->_parseBarcodeDefinition(6));
	$self->engine($self->_parseBarcodeDefinition(7));
	$self->script($self->_parseBarcodeDefinition(8));
	$self->afterpage($self->_parseBarcodeDefinition(9));
	my $cplus=21;
	for (my $c=0;$c<=31;$c++) {
		my $c1=$c+$cplus;
		$self->processing($c,$self->_parseBarcodeDefinition($c1));
	}
}

sub _parseBarcodeDefinition {
  my ($self,$keyId,$barcodeDefinitionValue) = @_; # keyId=Pos in def (0;1;2;3..)
  my ($definition,@definition);
  foreach my $def (split /\r\n/, $self->{'definition'}) {
		my @values = split /;/, $def;
		if ($values[0] eq $self->name) {
			if (defined $barcodeDefinitionValue) {
				# Setter method
				$values[$keyId] = $barcodeDefinitionValue;
			} else {
		  	# Getter method
				return $values[$keyId];
			}
		}
		$definition = join ";", @values;
		push @definition, $definition . ";";
	}
	# For setter method, update the definition
	$self->{'definition'} = join "\r\n", @definition;
	# Update the attribute
	# The new values are so up-to-date. If we make an $archive->parameter->update
	# the new values are automatically saved to the database
	$self->{'attribute'}->value($self->{'definition'});
  return
}

sub _parseCSV {
	my ($keyId,$barcodeDefinitionValue) = @_;
	my @values = split /,/, $barcodeDefinitionValue;
	return $values[$keyId];
}

sub _createBarcodeDefinition {
  my ($self) = @_;
	my $definition;
  $definition = $self->name.";";
  $definition .= $self->{'left'}.",";
	$definition .= $self->{'top'}.",";
	$definition .= $self->{'width'}.",";
	$definition .= $self->{'height'}.",";
	$definition .= $self->{'tolerance'}.",";
	$definition .= $self->trials.",";
	$definition .= $self->manual.";";
	$definition .= $self->firstBarcodeType.";";
	$definition .= $self->orientation.";";
	$definition .= $self->checkCharacter.";";
	$definition .= $self->secondBarcodeType.";";
	$definition .= $self->verticalStretch.";";
	$definition .= $self->engine.";";
	$definition .= $self->script.";";
	$definition .= $self->afterpage.";";
	# Empty values
	for (my $i = 0; $i < 11; $i++) {
		$definition .= "-1;";
	}
  for (my $i = 0; $i <= 31; $i++) {
		$definition .= $self->processing($i).";";
	}
	return $definition;
}

sub _allBarcodeDefinitions {
  my ($self,$attribute) = @_;
	my @barcodeDefinitions;
	foreach my $definition (split /\r\n/, $attribute->value) {
		my @values = split /;/, $definition;
		push @barcodeDefinitions, $values[0];
	}
	return \@barcodeDefinitions;
}

sub _parseToTwain {
  my ($value) = @_;
  return int (($value * 56.692) + 0.499);
}

sub _parseToMM {
  my ($value) = @_;
	return int (($value / 56.692) + 0.499);
}

sub definition {
  my ($cls,$attribute,$barcodeDefinition) = @_;
	my $parent = $attribute->parent;
  my $self = {};
	bless $self, $cls;
  $self->{'attribute'} = $attribute;
	$self->{'parentO'} = $parent;
	$self->{'definition'} = $attribute->value;
	$self->{'barcode_definition'} = $barcodeDefinition;
  $self->{'barcode_definition_id'} = $barcodeDefinition;
	$self->_initBarcodeDefinition;
  $self->_init;
	return $self;
}

sub definitions {
	my ($cls,$attribute,$retDS) = @_;
  # Return an array or hash representation of all barcode definitions
	# Pointer to Array(DefName)
	# Pointer to Hash(DefName, Object of Archivista::BL::Barcode)
	#            Hash(DefName, 'delete') -> 0/1
	#            Hash(DefName, 'update') -> 1
	$retDS = "ARRAY" if (! defined $retDS);
	my $padefinitions = $cls->_allBarcodeDefinitions($attribute);
  if (uc($retDS) eq "ARRAY") {
		return $padefinitions;
	} elsif (uc($retDS) eq "HASH") {
	  my $count = 0;
		my %barcodeDefinitions;
		foreach my $definition (@$padefinitions) {
			$barcodeDefinitions{$definition} = 
				$cls->definition($attribute,$definition);
			# First definition can't be deleted
			if ($count == 0) {
		  	$barcodeDefinitions{$definition}{'delete'} = 0;
			} else {
				$barcodeDefinitions{$definition}{'delete'} = 1;
			}
			$barcodeDefinitions{$definition}{'update'} = 1;
			$count++;
		}
		return \%barcodeDefinitions;
	}	
}

sub add {
  my ($self) = @_;
  $self->{'definition'} .= "\r\n" if (length($self->{'definition'}) > 0);
	$self->{'definition'} .= $self->_createBarcodeDefinition;
	# Set the new definition as attribute value of the object (parameter)
  $self->{'attribute'}->value($self->{'definition'});
	# Execute the update method of the parameter object
	# So we save the attribute values to the parameter table
	if ($self->{'parentO'}->exists == 0) {
		$self->{'parentO'}->insert;
	}
	$self->{'parentO'}->update;
}

sub remove {
  my ($self) = @_;
  my @newDefinitions;
	my @oldDefinitions = split /\r\n/, $self->{'definition'};
  foreach my $definition (@oldDefinitions) {
	  my @values = split /;/, $definition;
		if ($values[0] ne $self->id) {
			push @newDefinitions, $definition;
		}
	}
	$self->{'definition'} = join "\r\n", @newDefinitions;
	$self->{'attribute'}->value($self->{'definition'});
	$self->{'parentO'}->update;
}

sub resetProcessingBarcode {
  my ($self,$idx) = @_;
  # Processing barcode definitions are stored 
	# in fields with index from 21 to 28
	# We have idx between 0 .. 7 
	$idx = $idx + 21;
  my @newDefinitions;
	my @oldDefinitions = split /\r\n/, $self->{'definition'};
	foreach my $definition (@oldDefinitions) {
		my @values = split /;/, $definition;
		if ($values[0] eq $self->id) {
			$values[$idx] = "-1,-1,-1,-1,-1";
			$definition = join ";", @values;
		  $definition = $definition . ";";
		}
		push @newDefinitions, $definition;
	}
	$self->{'definition'} = join "\r\n", @newDefinitions;
	$self->{'attribute'}->value($self->{'definition'});
	$self->{'parentO'}->update;
}

sub update {
  my ($self) = @_;
  my @newDefinitions;
	my @oldDefinitions = split /\r\n/, $self->{'definition'};
	foreach my $definition (@oldDefinitions) {
	  my @values = split /;/, $definition;
		if ($values[0] eq $self->id) {
			$definition = $self->_createBarcodeDefinition;
		}
		push @newDefinitions, $definition;
	}
	$self->{'definition'} = join "\r\n", @newDefinitions;
	$self->{'attribute'}->value($self->{'definition'});
	$self->{'parentO'}->update;
}

sub name {
  my ($self,$value) = @_;
  if (defined $value) {
		$self->{'barcode_definition'} = $value;
		$self->_parseBarcodeDefinition(0,$value);
	} else {
		if (length($self->{'barcode_definition'}) > 1) {
			return $self->{'barcode_definition'};
		} else {
			return "";
		}
	}
}

sub id {
  my ($self) = @_;
	return $self->{'barcode_definition_id'};
}

sub position {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'position'} = $value;
		$self->_parseBarcodeDefinition(1,$value);
	  $self->{'left'} = _parseCSV(0,$value);
		$self->{'top'} = _parseCSV(1,$value);
		$self->{'width'} = _parseCSV(2,$value);
		$self->{'height'} = _parseCSV(3,$value);
		$self->{'tolerance'} = _parseCSV(4,$value);
		$self->{'trials'} = _parseCSV(5,$value);
		$self->{'manual'} = _parseCSV(6,$value);
	} elsif (length($self->{'position'}) > 0) {
		my @position;
		push @position, $self->{'left'};
		push @position, $self->{'top'};
		push @position, $self->{'width'};
		push @position, $self->{'height'};
		push @position, $self->{'tolerance'};
		push @position, $self->{'trials'};
		push @position, $self->{'manual'};
		$self->{'position'} = join ",", @position;
		return $self->{'position'};
	} else {
		return 0;
	}
}

sub left {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'left'} = _parseToTwain($value);
	} elsif (length($self->{'left'}) > 0) {
		return _parseToMM($self->{'left'});
	} else {
		return 0;
	}
}

sub top {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'top'} = _parseToTwain($value);
	} elsif (length($self->{'top'}) > 0) {
		return _parseToMM($self->{'top'});
	} else {
		return 0;
	}
}

sub width {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'width'} = _parseToTwain($value);
	} elsif (length($self->{'width'}) > 0) {
		return _parseToMM($self->{'width'});
	} else {
		return 0;
	}
}

sub height {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'height'} = _parseToTwain($value);
	} elsif (length($self->{'height'}) > 0) {
		return _parseToMM($self->{'height'});
	} else {
		return 0;
	}
}

sub tolerance {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'tolerance'} = _parseToTwain($value);
	} elsif (length($self->{'tolerance'}) > 0) {
		return _parseToMM($self->{'tolerance'});
	} else {
		return 0;
	}
}

sub trials {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'trials'} = $value;
	} else {
		if (length($self->{'trials'}) > 0) {
			return $self->{'trials'};
		} else {
			return 1;
		}
	}
}

sub manual {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'manual'} = $value;
	} elsif (length($self->{'manual'}) > 0) {
		return $self->{'manual'};
	} else {
		return 0;
	}
}

sub firstBarcodeType {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'first_barcode_type'} = $value;
		$self->_parseBarcodeDefinition(2,$value);
	} else {
		return $self->{'first_barcode_type'};
	}
}

sub orientation {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'orientation'} = $value;
	  $self->_parseBarcodeDefinition(3,$value);	
	} else {
		return $self->{'orientation'};
	}
}

sub checkCharacter {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'check_character'} = $value;
		$self->_parseBarcodeDefinition(4,$value);
	} else {
		return $self->{'check_character'};
	}
}

sub secondBarcodeType {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'second_barcode_type'} = $value;
		$self->_parseBarcodeDefinition(5,$value);
	} else {
		return $self->{'second_barcode_type'};
	}
}

sub verticalStretch {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'vertical_stretch'} = $value;
		$self->_parseBarcodeDefinition(6,$value);
	} elsif (length($self->{'vertical_stretch'}) > 0) {
		return $self->{'vertical_stretch'};
	} else {
		return 0;
	}
}

sub afterpage {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'afterpage'} = $value;
		$self->_parseBarcodeDefinition(9,$value);
	} elsif ($self->{'afterpage'} > 0) {
		return $self->{'afterpage'};
	} else {
		return 0;
	}
}

sub script {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'script'} = $value;
		$self->_parseBarcodeDefinition(8,$value);
	} elsif (length($self->{'script'}) > 0) {
		return $self->{'script'};
	} else {
		return "";
	}
}

sub engine {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'engine'} = $value;
		$self->_parseBarcodeDefinition(7,$value);
	} elsif (length($self->{'engine'}) > 0) {
		return $self->{'engine'};
	} else {
		return 0;
	}
}

sub processing {
  my ($self,$idx,$value) = @_;
	if (defined $value)  {
		$self->{'processing_'.$idx} = $value;
		$self->processingBarcodeNumber($idx,_parseCSV(0,$value));
		$self->processingBarcodeLength($idx,_parseCSV(1,$value));
		$self->processingBarcodeAttribute($idx,_parseCSV(2,$value));
		$self->processingBarcodeStart($idx,_parseCSV(3,$value));
		$self->processingBarcodeCharacter($idx,_parseCSV(4,$value));
	} else {
	  my @processing;
		push @processing, $self->processingBarcodeNumber($idx);
		push @processing, $self->processingBarcodeLength($idx);
		push @processing, $self->processingBarcodeAttribute($idx);
		push @processing, $self->processingBarcodeStart($idx);
		push @processing, $self->processingBarcodeCharacter($idx);
		$self->{'processing_'.$idx} = join ",", @processing;
    undef @processing;
		return $self->{'processing_'.$idx};
	}
}

sub processingBarcodeNumber {
  my ($self,$idx,$value) = @_;
	if (defined $value) {
		$self->{'processing_'.$idx.'_number'} = $value; 
	} else {
	  if (length($self->{'processing_'.$idx.'_number'}) > 0) {
			return $self->{'processing_'.$idx.'_number'};
		} else {
			return -1;
		}
	}
}

sub processingBarcodeLength {
  my ($self,$idx,$value) = @_;
	if (defined $value) {
		$self->{'processing_'.$idx.'_length'} = $value;
	} else {
	  if (length($self->{'processing_'.$idx.'_length'}) > 0) {
			return $self->{'processing_'.$idx.'_length'};
		} else {
			return -1;
		}
	}
}

sub processingBarcodeAttribute {
  my ($self,$idx,$value) = @_;
	if (defined $value) {
		$self->{'processing_'.$idx.'_attribute'} = $value;
	} else {
		if (length($self->{'processing_'.$idx.'_attribute'}) > 0) {
			return $self->{'processing_'.$idx.'_attribute'};
		} else {
			return -1;
		}
	}
}

sub processingBarcodeStart {
	my ($self,$idx,$value) = @_;
	if (defined $value) {
		$self->{'processing_'.$idx.'_start'} = $value;
	} else {
	  if (length($self->{'processing_'.$idx.'_start'}) > 0) {
			return $self->{'processing_'.$idx.'_start'};
		} else {
			return -1;
		}
	}
}

sub processingBarcodeCharacter {
	my ($self,$idx,$value) = @_;
	if (defined $value) {
		$self->{'processing_'.$idx.'_character'} = $value;
	} else {
	  if (length($self->{'processing_'.$idx.'_character'}) > 0) {
			return $self->{'processing_'.$idx.'_character'};
		} else {
			return -1;
		}
	}
}

1;
