
package Archivista::BL::Document;
use strict;
use Archivista::BL::Page;
use Archivista::BL::Attribute;
use Archivista::DL::Archive;
use Archivista::DL::ArchivePages;
use Archivista::Util::Collection;

sub _init {
  my ($self,$archive,$documentId) = @_;
  my $db = $archive->db;
	$self->{'attribute'} = Archivista::BL::Attribute->new();
  $self->{'page_collection'} = Archivista::Util::Collection->new();
  $self->{'archiveT'} = Archivista::DL::Archive->new($db);
  $self->{'archivePagesT'} = Archivista::DL::ArchivePages->new($db);
  my $archiveT = $self->{'archiveT'};
  # Check if getter or setter method
  if (defined $documentId) {
    # Setter: load document from data layer
    $archiveT->select($self,$documentId);
  } else {
    # Getter: insert a new document to data layer
    $self->attribute("Seiten")->value(0);
		$documentId = $archiveT->insert();
  }
  $self->archive($archive);
  $self->id($documentId);
}

sub _pageCollection {
  my ($self,$pageId,$page) = @_;
  if (defined $page) {
    $self->collection("page")->element($pageId,$page);	
  } else {
    return $self->collection("page")->element($pageId);
  }
}

sub _exception {
  my ($self) = @_;
	if (length($@) > 0) {
	  my $exception = $@;
		my $config = Archivista::Config->new;
		my $io = Archivista::Util::IO->new;
		$io->append($config->get("LOG_FILE"),$exception);
		undef $@;
		delete $self->{'db'};
	}
}

sub new {
  my ($cls,$archive,$documentId) = @_;
  # If documentId is set, then create an empty document
  # else load the selected document from data layer
  my $self = {};
  bless $self, $cls;
  $self->_init($archive,$documentId);
  return $self;
}

sub id {
  my ($self,$documentId) = @_;
	if (defined $documentId) {
		$self->{'doc_id'} = $documentId;
	} else {
  	return $self->{'doc_id'};
	}
}

sub curPageId {
  my ($self,$pageId) = @_;
  if (defined $pageId) {
    $self->{'cur_page_id'} = $pageId;
  } else {
    return $self->{'cur_page_id'};
  }
}

sub archive {
  my ($self,$archive) = @_;
	if (defined $archive) {
		$self->{'archive'} = $archive;
	} else {
		return $self->{'archive'};
	}
}

sub attribute {
  my ($self,$attributeId) = @_;
  $self->{'attribute'}->id($attributeId) if (defined $attributeId);
  return $self->{'attribute'};
}

sub collection {
  my ($self,$collectionType) = @_;
	return $self->{$collectionType.'_collection'};
}

sub page {
  my ($self,$pageId) = @_;
  my $page;
  if (defined $pageId) {
    # Check for pageId or pageNumber
		if (length($pageId) <= 3) {
			$pageId = $self->pageIdByDocAndPageNr($self->id,$pageId);
		}
		$page = Archivista::BL::Page->new($self,$pageId);
		$self->_pageCollection($pageId,$page);
    $self->curPageId($pageId);
	} else {
    my $curPageId = $self->curPageId();
    if (defined $curPageId) {
      $page = $self->_pageCollection($curPageId);
    } else {
			$page = Archivista::BL::Page->new($self);
			$pageId = $page->id;
			$self->_pageCollection($pageId,$page);
		}
  }
  # Check for exception
	$self->_exception;
  return $page;
}

sub pageIdByDocAndPageNr {
  my ($self,$documentId,$pageNumber) = @_;
	return $documentId * 1000 + $pageNumber;
}

sub update {
  my ($self) = @_;
  my $archiveT = $self->{'archiveT'};
  my $attribute = $self->{'attribute'};
  my $documentId = $self->id;
  $archiveT->update($documentId,$attribute);
  # Check for exception
	$self->_exception;
}

sub delete {
  my ($self) = @_;
  my $archiveT = $self->{'archiveT'};
  my $archivePagesT = $self->{'archivePagesT'};
	my $documentId = $self->id;
  # Clear document from collection
  $self->archive->clearDocument($documentId);
	# Delete document from archive table
  $archiveT->delete($documentId);
	# Delete related pages from archive pages table
	$archivePagesT->deleteByDocumentId($documentId);
  # Check for exception
	$self->_exception;
}

sub clearPage {
  my ($self,$pageId) = @_;
  undef $self->{'cur_page_id'};
	if (defined $pageId) {
		$self->collection("page")->remove($pageId);
	} else {
		$self->collection("page")->clear;
	}
}

1;

