
package Archivista::BL::OCR;
use strict;
use Archivista::Config;
use Archivista::Util::FS;

sub _init {
  my ($self) = @_;
  $self->{'attribute'}->id("Art");
  $self->{'attribute'}->value("parameter");
  $self->{'attribute'}->id("Tabelle");
  $self->{'attribute'}->value("parameter");
  $self->{'attribute'}->id("Inhalt");
}

sub _initOCRDefinition {
  my ($self) = @_;
  $self->name($self->_parseOCRDefinition(0));
  $self->lang1($self->_parseOCRDefinition(1));
  $self->lang2($self->_parseOCRDefinition(2));
  $self->lang3($self->_parseOCRDefinition(3));
  $self->lang4($self->_parseOCRDefinition(4));
  $self->lang5($self->_parseOCRDefinition(5));
  $self->quality($self->_parseOCRDefinition(6));
  $self->checkOrientation($self->_parseOCRDefinition(7));
  $self->cleanBeforeRecognition($self->_parseOCRDefinition(8));
  $self->suppressScaling($self->_parseOCRDefinition(9));
  $self->withoutBWConversion($self->_parseOCRDefinition(10));
  $self->tableCellsFromLines($self->_parseOCRDefinition(11));
  $self->noOverlappedCells($self->_parseOCRDefinition(12));
  $self->oneRow($self->_parseOCRDefinition(13));
}

sub _parseOCRDefinition {
  my ($self,$keyId,$ocrDefinitionValue) = @_;
  my ($definition,@definition);
  foreach my $def (split /\r\n/, $self->{'definition'}) {
    my @values = split /;/, $def;
    if ($values[0] eq $self->id()) {
      if (defined $ocrDefinitionValue) {
        # Setter method
        $values[$keyId] = $ocrDefinitionValue;
      } else {
        # Getter method
        return $values[$keyId];
      }
    }
    $definition = join ";", @values;
    push @definition, $definition . ";";
  }
  # For setter method, update the definition
  $self->{'definition'} = join "\r\n", @definition;
  # Update the attribute
  # The new values are so up-to-date. If we make an $archive->parameter->update
  # the new values are automatically saved to the database
  $self->{'attribute'}->value($self->{'definition'});
  return
}

sub _createOCRDefinition {
  my ($self) = @_;
  my $definition;
  $definition.=$self->name.";";
  $definition.=$self->lang1.";";
  $definition.=$self->lang2.";";
  $definition.=$self->lang3.";";
  $definition.=$self->lang4.";";
  $definition.=$self->lang5.";";
  $definition.=$self->quality.";";
  $definition.=$self->checkOrientation.";";
  $definition.=$self->cleanBeforeRecognition.";";
  $definition.=$self->suppressScaling.";";
  $definition.=$self->withoutBWConversion.";";
  $definition.=$self->tableCellsFromLines.";";
  $definition.=$self->noOverlappedCells.";";
  $definition.=$self->oneRow.";";
  return $definition;
}

sub _allOCRDefinitions {
  my ($self,$attribute) = @_;
  my @ocrDefinitions;
  foreach my $definition (split /\r\n/, $attribute->value) {
    my @values = split /;/, $definition;
    push @ocrDefinitions, $values[0];
  }
  return \@ocrDefinitions;
}

sub definition {
  my ($cls,$attribute,$ocrDefinition) = @_;
  my $parent = $attribute->parent;
  my $self = {};
  bless $self, $cls;
  $self->{'ocr_definition'}=$ocrDefinition;
  $self->{'ocr_definition_id'}=$ocrDefinition;
  $self->{'attribute'} = $attribute;
  $self->{'parentO'} = $parent;
  $self->{'definition'} = $attribute->value;
  $self->_initOCRDefinition;
  $self->_init;
  return $self;
}

sub definitions {
  my ($cls,$attribute,$retDS) = @_;
  # Return an array or hash representation of all ocr definitions
  # Pointer to Array(DefName)
  # Pointer to Hash(DefName, Object of Archivista::BL::OCR)
  #            Hash(DefName, 'delete') -> 0/1
  #            Hash(DefName, 'update') -> 1
  $retDS = "ARRAY" if (! defined $retDS);
  my $padefinitions = $cls->_allOCRDefinitions($attribute);
  if (uc($retDS) eq "ARRAY") {
    return $padefinitions;
  } elsif (uc($retDS) eq "HASH") {
    my %ocrDefinitions;
    my $count=0;
    foreach my $definition (@$padefinitions) {
      $ocrDefinitions{$definition} = $cls->definition($attribute,$definition);
      if (( $count==0 ) || (( $count > 0 ) && (($count+1) < @$padefinitions))) {
				$ocrDefinitions{$definition}{'delete'} = 0;
      } else {
				$ocrDefinitions{$definition}{'delete'} = 1;
      }
      $count++;
      $ocrDefinitions{$definition}{'update'} = 1;
    }
    return \%ocrDefinitions;
  }
}

sub add {
  my ($self) = @_;
  $self->{'definition'} .= "\r\n" if (length($self->{'definition'}) > 0);
  $self->{'definition'} .= $self->_createOCRDefinition;
  # Set the new definition as attribute value of the object (parameter)
  $self->{'attribute'}->value($self->{'definition'});
  # Execute the update method of the parameter object
  # So we save the attribute values to the parameter table
  if ($self->{'parentO'}->exists == 0) {
    $self->{'parentO'}->insert;
  }
  $self->{'parentO'}->update;
}

sub remove {
  my ($self) = @_;
  my @newDefinitions;
  my @oldDefinitions = split /\r\n/, $self->{'definition'};
  foreach my $definition (@oldDefinitions) {
    my @values = split /;/, $definition;
    if ($values[0] ne $self->id()) {
      push @newDefinitions, $definition;
    }
  }
  $self->{'definition'} = join "\r\n", @newDefinitions;
  $self->{'attribute'}->value($self->{'definition'});
  $self->{'parentO'}->update;
}

sub update {
  my ($self) = @_;
  my @newDefinitions;
  my @oldDefinitions = split /\r\n/, $self->{'definition'};
  foreach my $definition (@oldDefinitions) {
    my @values = split /;/, $definition;
    if ($values[1] eq $self->id()) {
      $definition = $self->_createOCRDefinition;
    }
    push @newDefinitions, $definition;
  }
  $self->{'definition'} = join "\r\n", @newDefinitions;
  $self->{'attribute'}->value($self->{'definition'});
  $self->{'parentO'}->update;
}

sub name {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'ocr_definition'} = $value;
    $self->_parseOCRDefinition(0,$value);
  } else {
    return $self->{'ocr_definition'};
  }
}

sub id {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'ocr_definition_id'} = $value;
    $self->_parseOCRDefinition(0,$value);
  } else {
    return $self->{'ocr_definition_id'};
  }
}

sub lang1 {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'lang1'} = $value;
    $self->_parseOCRDefinition(1,$value);
  } else {
    return $self->{'lang1'};
  }
}

sub lang2 {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'lang2'} = $value;
    $self->_parseOCRDefinition(2,$value);
  } else {
    return $self->{'lang2'};
  }
}

sub lang3 {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'lang3'} = $value;
    $self->_parseOCRDefinition(3,$value);
  } else {
    return $self->{'lang3'};
  }
}

sub lang4 {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'lang4'} = $value;
    $self->_parseOCRDefinition(4,$value);
  } else {
    return $self->{'lang4'};
  }
}

sub lang5 {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'lang5'} = $value;
    $self->_parseOCRDefinition(5,$value);
  } else {
    return $self->{'lang5'};
  }
}

sub quality {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'quality'} = $value;
    $self->_parseOCRDefinition(6,$value);
  } else {
    return $self->{'quality'};
  }
}

sub checkOrientation {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'checkOrientation'} = $value;
    $self->_parseOCRDefinition(7,$value);
  } else {
    return $self->{'checkOrientation'};
  }
}

sub cleanBeforeRecognition {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'cleanBeforeRecognition'} = $value;
    $self->_parseOCRDefinition(8,$value);
  } else {
    return $self->{'cleanBeforeRecognition'};
  }
}

sub suppressScaling {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'suppressScaling'} = $value;
    $self->_parseOCRDefinition(9,$value);
  } else {
    return $self->{'suppressScaling'};
  }
}

sub withoutBWConversion {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'withoutBWConversion'} = $value;
    $self->_parseOCRDefinition(10,$value);
  } else {
  	return $self->{'withoutBWConversion'};
  }
}

sub tableCellsFromLines {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'tableCellsFromLines'} = $value;
    $self->_parseOCRDefinition(11,$value);
  } else {
  	return $self->{'tableCellsFromLines'};
  }
}

sub noOverlappedCells {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'noOverlappedCells'} = $value;
    $self->_parseOCRDefinition(12,$value);
  } else {
    return $self->{'noOverlappedCells'};
  }
}

sub oneRow {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'oneRow'} = $value;
    $self->_parseOCRDefinition(13,$value);
  } else {
  	return $self->{'oneRow'};
  }
}

1;

