
package Archivista::BL::Page;
use strict;
use Archivista::Config;
use Archivista::BL::Attribute;
use Archivista::DL::ArchivePages;

sub _init {
	my ($self,$document,$pageId) = @_;
	my $db = $document->archive->db;
	$self->{'attribute'} = Archivista::BL::Attribute->new();
	$self->{'archivePagesT'} = Archivista::DL::ArchivePages->new($db);
	my $archivePagesT = $self->{'archivePagesT'};
	# Check if getter or setter method
	if (defined $pageId) {
		# Setter: load page from data layer
		$archivePagesT->select($self,$pageId);
	} else {
		# Getter: insert a new page to data layer
		$pageId = _newPageId($document);
		$archivePagesT->insert($pageId);
	}
	$self->{'page_id'} = $pageId;
}

sub _newPageId {
	my ($document) = @_;
  my $documentId = $document->id;
	my $nrOfPages = $document->attribute("Seiten")->value + 1;
	$document->attribute("Seiten")->value($nrOfPages);
	$document->update;
	return $document->pageIdByDocAndPageNr($documentId,$nrOfPages);
}

sub _exception {
  my ($self) = @_;
	if (length($@) > 0) {
	  my $exception = $@;
		my $config = Archivista::Config->new;
		my $io = Archivista::Util::IO->new;
		$io->append($config->get("LOG_FILE"),$exception);
		undef $@;
		delete $self->{'db'};
	}
}

sub new {
  my ($cls,$document,$pageId) = @_;
	my $self = {};
  bless $self, $cls;
  $self->_init($document,$pageId);
  return $self;
}

sub id {
	my ($self) = @_;
	return $self->{'page_id'};
}

sub attribute {
	my ($self,$attributeId) = @_;
  # Set the attribute id for the setter method
	$self->{'attribute'}->id($attributeId) if (defined $attributeId);
	return $self->{'attribute'};
}

sub update {
  my ($self) = @_;
	my $archivePagesT = $self->{'archivePagesT'};
	my $attribute = $self->{'attribute'};
	my $pageId = $self->id;
	$archivePagesT->update($pageId,$attribute);
  # Check for exception
	$self->_exception;
}

sub delete {
  my ($self) = @_;
	my $archivePagesT = $self->{'archivePagesT'};
	my $pageId = $self->id;
	$archivePagesT->delete($pageId);
	# Check for exception
	$self->_exception;
}

1;

