
package Archivista::BL::Parameter;
use strict;
use Archivista::BL::Attribute;
use Archivista::DL::Parameter;

sub _init {
  my ($self,$archive,$type,$parameterId) = @_;
  my $db = $archive->db;
  $self->{'attribute'} = Archivista::BL::Attribute->new();
  $self->{'parameterT'} = Archivista::DL::Parameter->new($db,$type);
  my $parameterT = $self->{'parameterT'};
  # Check if getter or setter method
  if (defined $parameterId) {
    # Setter: parameter from data layer
    $parameterT->select($self,$parameterId);
  } else {
    # Getter: insert a new parameter to data layer
    $parameterId = $parameterT->insert();
  }
  $self->archive($archive);
  $self->{'parameter_id'} = $parameterId;
}

sub new {
  my ($cls,$archive,$type,$parameterId) = @_;
  my $self = {};
  bless $self, $cls;
  $self->_init($archive, $type, $parameterId);
  return $self;
}

sub load {
  my ($cls,$archive,$archiveName) = @_;
  my $db = $archive->db;
  Archivista::DL::Parameter->load($db,$archiveName);
}

sub id {
  my ($self) = @_;
  return $self->{'parameter_id'};
}

sub attribute {
  my ($self,$attributeId) = @_;
  if (defined $attributeId) {
    # Set the parameter id as parent id
    $self->{'attribute'}->parentId($self->id);
    # Set the attribute id for the setter method
    $self->{'attribute'}->id($attributeId);
    # Set the parameter object to the attribute
    $self->{'attribute'}->parent($self);
    # Set the archive object to the attribute
    $self->{'attribute'}->archive($self->archive);
  }
  return $self->{'attribute'};
}

sub archive {
  my ($self,$archive) = @_;
  if (defined $archive) {
    $self->{'archive'} = $archive;
  } else {
    return $self->{'archive'};
  }
}

sub insert {
  my ($self) = @_;
  my $parameterT = $self->{'parameterT'};
  my $attribute = $self->{'attribute'};
  $parameterT->insert($self,$attribute);
}

sub update {
  my ($self) = @_;
  my $parameterT = $self->{'parameterT'};
  my $attribute = $self->{'attribute'};
  $parameterT->update($self,$attribute);
}

sub maskDefinitionName {
  my ($self,$maskDefinitionName) = @_;
  my $parameterT = $self->{'parameterT'};
  return $parameterT->maskDefinitionName($self,$maskDefinitionName);
}

sub maskParentFields {
  my ($self,$retDS) = @_; # Return data structure (ARRAY or HASH)
  my $parameterT = $self->{'parameterT'};
  return $parameterT->maskParentFields($self,$retDS);
}

sub exists {
  my ($self) = @_;
  my $parameterT = $self->{'parameterT'};
  return $parameterT->exists($self);
}

sub delete {
  my ($self) = @_;
  my $parameterT = $self->{'parameterT'};
  my $parameterId = $self->id;
  $self->archive->clearParameter($parameterId);
  $parameterT->delete($self);
}

1;

