
package Archivista::BL::Scan;
use strict;
use Archivista::Config;
use Archivista::Util::FS;

sub _init {
  my ($self) = @_;
  $self->{'attribute'}->id("Art");
  $self->{'attribute'}->value("parameter");
  $self->{'attribute'}->id("Tabelle");
  $self->{'attribute'}->value("parameter");
  $self->{'attribute'}->id("Inhalt");
}

sub _initScanDefinition {
  my ($self,$aktdef) = @_;
  my @scanval = split(/;/,$aktdef);
  $self->name($scanval[0]);
  $self->scanType($scanval[1]);
  $self->dpi($scanval[2]);
  $self->_x($scanval[3]);
  $self->_y($scanval[4]);
  $self->_left($scanval[5]);
  $self->_top($scanval[6]);
  $self->rotation($scanval[7]);
  $self->postProcessing($scanval[8]);
  $self->splitPage($scanval[9]);
  $self->ocr($scanval[10]);
  $self->adf($scanval[11]);
  # If field 11 contains a value >= 2 then we have seconds for auto-pilot
  $self->waitSeconds($scanval[11]);
  $self->numberOfPages($scanval[12]);
  $self->brightness($scanval[13]);
  $self->contrast($scanval[14]);
  $self->gamma($scanval[15]);
  $self->newDocument($scanval[16]);
  $self->adjust($scanval[17]);
  $self->notactive($scanval[18]);
  $self->truncate($scanval[19]);
  $self->newDocAfterPages($scanval[20]);
  $self->sleep($scanval[21]);
  $self->key($scanval[22]);
  $self->emptyPages($scanval[23]);
  $self->optimizeOn($scanval[24]);
  $self->optimizeRadius($scanval[25]);
  $self->optimizeOutputDPI($scanval[26]);
  $self->autoFields($scanval[27]);
  $self->boxBarcodeDef($scanval[28]);
	$self->optimizeThreshold($scanval[29]);
	$self->formRecognition($scanval[30]);
	$self->jpegCompression($scanval[31]);
	$self->detectDoubleFeed($scanval[32]);
	$self->ocrWait($scanval[33]);
	$self->addLastDoc($scanval[34]);
	$self->callConvert($scanval[35]);
}

sub _createScanDefinition {
  my ($self) = @_;
  my $definition;
  $definition = $self->name.";";
  $definition .= $self->scanType.";";
  $definition .= $self->dpi.";";
  $definition .= $self->_x.";";
  $definition .= $self->_y.";";
  $definition .= $self->_left.";";
  $definition .= $self->_top.";";
  $definition .= $self->rotation.";";
  $definition .= $self->postProcessing.";";
  $definition .= $self->splitPage.";";
  $definition .= $self->ocr.";";
  if ($self->adf >= 2) {
    $definition .= $self->waitSeconds.";";
  } else {
    $definition .= $self->adf.";";
  }
  $definition .= $self->numberOfPages.";";
  $definition .= $self->brightness.";";
  $definition .= $self->contrast.";";
  $definition .= $self->gamma.";";
  $definition .= $self->newDocument.";";
  $definition .= $self->adjust.";";
  $definition .= $self->notactive.";";
  $definition .= $self->truncate.";";
  $definition .= $self->newDocAfterPages.";";
  $definition .= $self->sleep.";";
  $definition .= $self->key.";";
  $definition .= $self->emptyPages.";";
  $definition .= $self->optimizeOn.";";
  $definition .= $self->optimizeRadius.";";
  $definition .= $self->optimizeOutputDPI.";";
  $definition .= $self->autoFields.";";
  $definition .= $self->boxBarcodeDef.";";
	$definition .= $self->optimizeThreshold.";";
	$definition .= $self->formRecognition.";";
	$definition .= $self->jpegCompression.";";
	$definition .= $self->detectDoubleFeed.";";
	$definition .= $self->ocrWait.";";
	$definition .= $self->addLastDoc.";";
	$definition .= $self->callConvert.";";
  return $definition;
}

sub _allScanDefinitions {
  my ($self,$attribute) = @_;
  my @scanDefinitions;
  foreach my $definition (split /\r\n/, $attribute->value) {
    my @values = split /;/, $definition;
    push @scanDefinitions, $values[0];
  }
  return \@scanDefinitions;
}

sub _x {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'x'} = $value;
  } else {
    return $self->{'x'};
  }
}

sub _y {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'y'} = $value;
  } else {
    return $self->{'y'};
  }
}

sub _left {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'left'} = $value;
  } else {
    return $self->{'left'};
  }
}

sub _top {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'top'} = $value;
  } else {
    return $self->{'top'};
  }
}

sub _parseToTwain {
  my ($value) = @_;
  return int (($value * 56.692) + 0.499);
}

sub _parseToMM {
  my ($value) = @_;
  return int (($value / 56.692) + 0.499);
}

sub _parseContrast {
  my ($value) = @_;
  return int ($value / 10);
}

sub _parseBrightness {
  my ($value) = @_;
  return int ($value / 10);
}

sub definition {
  my ($cls,$attribute,$scanDefinition) = @_;
  my $parent = $attribute->parent;
  my $self = {};
  bless $self, $cls;
  $self->{'attribute'} = $attribute;
  $self->{'parentO'} = $parent;
  $self->{'definition'} = $attribute->value;
  $self->{'scan_definition'} = $scanDefinition;
  $self->{'scan_definition_id'} = $scanDefinition;
  my @defs=split(/\r\n/,$self->{definition}); # get the current scan def
	my $aktdef = $defs[0];
	foreach(@defs) {
	  my $tempdef = $_;
    my @vals=split(/;/,$tempdef);
		if ($vals[0] eq $scanDefinition) {
      $aktdef=$tempdef;
			last;
		}
	}
  $self->_initScanDefinition($aktdef);
  $self->_init;
  return $self;
}

sub definitions {
  my ($cls,$attribute,$retDS) = @_;
  # Return an array or hash representation of all scan definitions
  # Pointer to Array(DefName)
  # Pointer to Hash(DefName, Object of Archivista::BL::Scan)
  #            Hash(DefName, 'delete') -> 0/1
  #            Hash(DefName, 'update') -> 1
  $retDS = "ARRAY" if (! defined $retDS);
  my $padefinitions = $cls->_allScanDefinitions($attribute);
  if (uc($retDS) eq "ARRAY") {
    return $padefinitions;
  } elsif (uc($retDS) eq "HASH") {
    my $count = 0;
    my %scanDefinitions;
    foreach my $definition (@$padefinitions) {
	    $scanDefinitions{$definition} = $cls->definition($attribute,$definition);
	    # First definition can't be deleted
	    if ($count == 0) {
	      $scanDefinitions{$definition}{'delete'} = 0;
	    } else {
	      $scanDefinitions{$definition}{'delete'} = 1;
	    }
	    $scanDefinitions{$definition}{'update'} = 1;
	    $count++;
    }
    return \%scanDefinitions;
  }  
}

sub add {
  my ($self) = @_;
	my @oldDefinitions = split /\r\n/, $self->{'definition'};
	my $ok=1;
	$self->name="New scan def" if $self->name eq "";
  foreach my $definition (@oldDefinitions) {
    my @values = split /;/, $definition;
	  if ($values[0] eq $self->name) {
		  $ok=0;
			last;
	  }
	}
  return if $ok==0;	
  $self->{'definition'} .= "\r\n" if (length($self->{'definition'}) > 0);
  $self->{'definition'} .= $self->_createScanDefinition;
  # Set the new definition as attribute value of the object (parameter)
  $self->{'attribute'}->value($self->{'definition'});
  # Execute the update method of the parameter object
  # So we save the attribute values to the parameter table
  if ($self->{'parentO'}->exists == 0) {
    $self->{'parentO'}->insert;
  }
  $self->{'parentO'}->update;
}

sub remove {
  my ($self) = @_;
  my @newDefinitions;
  my @oldDefinitions = split /\r\n/, $self->{'definition'};
  foreach my $definition (@oldDefinitions) {
    my @values = split /;/, $definition;
    if ($values[0] ne $self->id) {
	    push @newDefinitions, $definition;
    }
  }
  $self->{'definition'} = join "\r\n", @newDefinitions;
  $self->{'attribute'}->value($self->{'definition'});
  $self->{'parentO'}->update;
}

sub update {
  my ($self) = @_;
  my @newDefinitions;
  my @oldDefinitions = split /\r\n/, $self->{'definition'};
	my $ok=1;
	$self->name=$self->id if $self->name eq "";
	if ($self->id ne $self->name) {
    foreach my $definition (@oldDefinitions) {
      my @values = split /;/, $definition;
			if ($values[0] eq $self->name) {
			  $ok=0;
				last;
			}
	  }
	}
	return if $ok==0;
  foreach my $definition (@oldDefinitions) {
    my @values = split /;/, $definition;
    if ($values[0] eq $self->id) {
    	$definition = $self->_createScanDefinition;
    }
    push @newDefinitions, $definition;
  }
	my $outdef = "";
	my $nokey = 10000;
	my %defs = ();
	my @pos = ();
  for (my $nr=0;$nr<=@newDefinitions-1;$nr++) {
    my @vals = split(";",$newDefinitions[$nr]);
		my $key = $vals[22];
		$key = $nokey if $key==0;
		if ($defs{$key} ne "") {
		  for (my $c=10000;$c<19000;$c++) {
			  if ($defs{$c} eq "") {
				  $key = $c;
					last;
				}
			}
		}
		$defs{$key} = $newDefinitions[$nr];
		$pos[$nr] = $key;
	}
	@pos = sort { $a <=> $b } @pos;
	my $checknew = abs($pos[0]);
	if ($defs{$checknew} ne "") {
	  my $key = 0;
    for (my $c=10000;$c<19000;$c++) {
		  if ($defs{$c} eq "") {
			  $key = $c;
				last;
			}
		}
		if ($key>0) {
		  $defs{$key} = $defs{$pos[0]};
		  $pos[0] = $key;
		}
	} else {
		$defs{$checknew} = $defs{$pos[0]};
		$pos[0] = $checknew;
	}
	@pos = sort { $a <=> $b } @pos;
	if ($pos[0]==10001) { # we don't have any number, so give it 1 to x
	  for(my $nr=0;$nr<=@pos-1;$nr++) {
		  my $nr1 = $nr+1;
		  $defs{$nr1} = $defs{$pos[$nr]};
		  $pos[$nr]=$nr1;
		}
	}
	my $out = ""; 
	for (my $nr=0;$nr<=@pos-1;$nr++) {
	  my $def = $defs{$pos[$nr]};
		my @vals = split(";",$def);
		$vals[22] = $pos[$nr];
		$def = join(";",@vals);
		$out .= $def ."\r\n";
	}
  #$self->{'definition'} = join "\r\n", @newDefinitions; # old (not sorted)
  $self->{'definition'} = $out;
  $self->{'attribute'}->value($self->{'definition'});
  $self->{'parentO'}->update;
}

sub name {
  my ($self,$value) = @_;
  if (defined $value) {
	  $value =~ s/\;/ /g;
    $self->{'scan_definition'} = $value;
  } else {
    return $self->{'scan_definition'};
  }
}

sub id {
  my ($self) = @_;
  return $self->{'scan_definition_id'};
}

sub scanType {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'scan_type'} = $value;
    # We save the new value to the attribute
  } else {
    my $scanType = $self->{'scan_type'};
    if ($scanType >= 0 && $scanType <= 2) {
      return $scanType;
    } else {
	    return 1;
    }
  }
}

sub dpi {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'dpi'} = $value;
  } else {
    if (defined $self->{'dpi'}) {
      return $self->{'dpi'}; 
    } else {
	    return 300;
    }
  }
}

sub x {
  my ($self,$value) = @_;
  if (defined $value) {
    $value = _parseToTwain($value);
    $self->{'x'} = $value;
  } else {
    return _parseToMM($self->{'x'});
  }  
}

sub y {
  my ($self,$value) = @_;
  if (defined $value) {
    $value = _parseToTwain($value);
    $self->{'y'} = $value;
  } else {
    return _parseToMM($self->{'y'});
  }
}

sub left {
  my ($self,$value) = @_;
  if (defined $value) {
    $value = _parseToTwain($value);
    $self->{'left'} = $value;
  } else {
    return _parseToMM($self->{'left'});
  }
}

sub top {
  my ($self,$value) = @_;
  if (defined $value) {
    $value = _parseToTwain($value);
    $self->{'top'} = $value;
  } else {
    return _parseToMM($self->{'top'});
  }
}

sub rotation {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'rotation'} = $value;
  } else {
    if (defined $self->{'rotation'}) {
      return $self->{'rotation'};
    } else {
      return 0;
    }
  }
}

sub postProcessing {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'post_processing'} = $value;
  } else {
    if (defined $self->{'post_processing'}) {
      return $self->{'post_processing'};
    } else {
      return -1;
    }
  }
}

sub splitPage {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'split_page'} = $value;
  } else {
    if (defined $self->{'split_page'}) {
      return $self->{'split_page'};
    } else {
      return 0;
    }
  }
}

sub ocr {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'ocr'} = $value;
  } else {
    if (defined $self->{'ocr'}) {
      return $self->{'ocr'};
    } else {
      return -1;
    }
  }
}

sub adf {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'adf'} = $value;
  } else {
    if (defined $self->{'adf'}) {
      return $self->{'adf'};
    } else {
      return 0;
    }
  }
}

sub waitSeconds {
  my ($self,$value) = @_;
  if (defined $value) {
    if ($self->adf >= 2) {
      # We selected auto-pilot
      # Save the value as ADF to the definition
      # The value can't be <= 2
      $value = 2 if ($value < 2);
      $self->{'wait_seconds'} = $value;
      $self->{'adf'} = 2;
    }
  } else {
    if (defined $self->{'wait_seconds'}) {
      return $self->{'wait_seconds'};
    } else {
      return 2;
    }
  }
}

sub numberOfPages {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'number_of_pages'} = $value;
  } else {
    my $nrOfPages = $self->{'number_of_pages'};
    if ($nrOfPages < 1) {
      return 1;
    }  elsif ($nrOfPages > 640) {
      return 640;
    } else {
      return $nrOfPages;
    }
  }
}

sub brightness {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'brightness'} = $value;
  } else {
    if (defined $self->{'brightness'}) {
      return $self->{'brightness'};  
    } else {
      return 0;
    }
  }
}

sub contrast {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'contrast'} = $value;
  } else {
    if (defined $self->{'contrast'}) {
      return $self->{'contrast'};
    } else {
      return 0;
    }
  }
}

sub emptyPages {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'emptyPages'} = $value;
  } else {
    if (defined $self->{'emptyPages'}) {
      return $self->{'emptyPages'};
    } else {
      return 0;
    }
  }
}

sub gamma {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'gamma'} = $value;
  } else {
    if (defined $self->{'gamma'}) {
      return $self->{'gamma'};
    } else {
      return 1800;
    }
  }
}

sub newDocument {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'new_document'} = $value;
  } else {
    return $self->{'new_document'};
  }
}

sub adjust {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'adjust'} = $value;
  } else {
    return $self->{'adjust'};
  }
}

sub notactive {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'notactive'} = $value;
  } else {
    return $self->{'notactive'};
  }
}

sub truncate {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'truncate'} = $value;
  } else {
    return $self->{'truncate'};
  }
}

sub bwOptimisation {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'bw_optimisation'} = $value;
  } else {
    return $self->{'bw_optimisation'};
  }
}

sub sleep {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'sleep'} = $value;
  } else {
    return $self->{'sleep'};
  }
}

sub newDocAfterPages {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'newDocAfterPages'} = $value;
  } else {
    return $self->{'newDocAfterPages'};
  }
}

sub key {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'key'} = $value;
  } else {
    return $self->{'key'};
  }
}

sub optimizeOn {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'optimizeOn'} = $value;
  } else {
    if (defined $self->{'optimizeOn'}) {
      return $self->{'optimizeOn'};
    } else {
      return 0;
    }
  }
}

sub optimizeRadius {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'optimizeRadius'} = $value;
  } else {
    if (defined $self->{'optimizeRadius'}) {
      return $self->{'optimizeRadius'};
    } else {
      return 0;
    }
  }
}

sub optimizeOutputDPI {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'optimizeOutputDPI'} = $value;
  } else {
    if (defined $self->{'optimizeOutputDPI'}) {
      return $self->{'optimizeOutputDPI'};
    } else {
      return 0;
    }
  }
}

sub optimizeThreshold {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'optimizeThreshold'} = $value;
  } else {
    if (defined $self->{'optimizeThreshold'}) {
      return $self->{'optimizeThreshold'};
    } else {
      return 0;
    }
  }
}

sub formRecognition {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'formRecognition'} = $value;
  } else {
    if (defined $self->{'formRecognition'}) {
      return $self->{'formRecognition'};
    } else {
      return 0;
    }
  }
}

sub autoFields {
  my ($self,$value) = @_;
  if (defined $value) {
	  # as we can't use the , + ; we have to quote them
	  $value =~ s/,/\*44\*/g;
		$value =~ s/;/\*59\*/g;
    $self->{'autoFields'} = $value;
  } else {
    if (defined $self->{'autoFields'}) {		  
      return $self->{'autoFields'};
    } else {
      return '';
    }
  }
}

sub callConvert {
  my ($self,$value) = @_;
  if (defined $value) {
	  # as we can't use the , + ; we have to quote them
	  $value =~ s/,/\*44\*/g;
		$value =~ s/;/\*59\*/g;
    $self->{'callConvert'} = $value;
  } else {
    if (defined $self->{'callConvert'}) {		  
      return $self->{'callConvert'};
    } else {
      return '';
    }
  }
}

sub boxBarcodeDef {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'boxBarcodeDef'} = $value;
  } else {
    if (defined $self->{'boxBarcodeDef'}) {
      return $self->{'boxBarcodeDef'};
    } else {
      return 0;
    }
  }
}

sub jpegCompression {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'jpegCompression'} = $value;
  } else {
    if (defined $self->{'jpegCompression'}) {
      return $self->{'jpegCompression'};
    } else {
      return '';
    }
  }
}

sub detectDoubleFeed {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'detectDoubleFeed'} = $value;
  } else {
    if (defined $self->{'detectDoubleFeed'}) {
      return $self->{'detectDoubleFeed'};
    } else {
      return 0;
    }
  }
}

sub ocrWait {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'ocrWait'} = $value;
  } else {
    if (defined $self->{'ocrWait'}) {
      return $self->{'ocrWait'};
    } else {
      return 0;
    }
  }
}

sub addLastDoc {
  my ($self,$value) = @_;
  if (defined $value) {
    $self->{'addlastdoc'} = $value;
  } else {
    if (defined $self->{'addlastdoc'}) {
      return $self->{'addlastdoc'};
    } else {
      return 0;
    }
  }
}

1;

