
package Archivista::BL::User;
use strict;
use Archivista::BL::Attribute;
use Archivista::DL::User;

sub _init {
	my ($self,$userId,$host,$uid,$pwd,$level) = @_;
	my $userT = $self->{'userT'};
	$self->{'attribute'} = Archivista::BL::Attribute->new();
	# Check if getter or setter method
	if (defined $userId) {
		# Getter: insert a new user to data layer
		$userT->select($self,$userId);
	} else {
		# Setter: user from data layer
		$userId = $userT->insert($self,$host,$uid,$pwd,$level);
	}
	$self->{'user_id'} = $userId;
}

sub new {
  my ($cls,$archive,$userId,$host,$uid,$pwd,$level) = @_;
	my $self = {};
  bless $self, $cls;
  # Set the archive object
	$self->archive($archive);
	my $db = $archive->db;
	$self->{'userT'} = Archivista::DL::User->new($db);
	if (defined $userId or defined $host) {
		$self->_init($userId,$host,$uid,$pwd,$level);
	}
  return $self;
}

sub all {
  my ($cls,$archive) = @_;
	my $db = $archive->db;
	return Archivista::DL::User->all($db);
}

sub grant {
  my ($cls,$archive,$pausers) = @_; # 2-dim array[n][3] host,user,level
  my $db = $archive->db;
	Archivista::DL::User->grant($db,$pausers);
}

sub revoke {
  my ($cls,$archive,$pausers) = @_;
	my $db = $archive->db;
	Archivista::DL::User->revoke($db,$pausers);
}

sub id {
	my ($self) = @_;
	return $self->{'user_id'};
}

sub attribute {
	my ($self,$attributeId) = @_;
  # Set the attribute id for the setter method
	$self->{'attribute'}->id($attributeId) if (defined $attributeId);
	return $self->{'attribute'};
}

sub archive {
  my ($self,$archive) = @_;
	if (defined $archive) {
		$self->{'archive'} = $archive;
	} else {
		return $self->{'archive'};
	}
}

sub update {
  my ($self) = @_;
	my $userT = $self->{'userT'};
	my $attribute = $self->{'attribute'};
	my $userId = $self->id;
	$userT->update($userId,$attribute);
}

sub delete {
  my ($self) = @_;
	my $userT = $self->{'userT'};
	my $userId = $self->id;
	$self->archive->clearUser($userId);
	$userT->delete($self);
}

sub idByHostAndUser {
  my ($self,$host,$uid) = @_;
  my $userT = $self->{'userT'};
	return $userT->idByHostAndUser($host,$uid);
}

1;

