
package Archivista::DL::Archive;
use strict;
use vars qw ( @ISA );
use Archivista::DL::DB;
use Archivista::Util::Date;
@ISA = qw ( Archivista::DL::DB );

sub new {
	my ($cls,$db) = @_;
  my $self = {};
	bless $self, $cls;
  $self->db($db);
	return $self;
}

sub all {
  my ($cls,$db) = @_;
  my $self = {};
  my $config = Archivista::Config->new;
  my $globalDb = $config->get("AV_GLOBAL_DB");
  my $dbh = $db->dbh;	
	my (@archives);
	bless $self, $cls;
	$self->db($db);
	my $padatabases = $self->showDatabases();
	foreach my $database (@$padatabases) {
		if ($self->isArchivistaArchive($database) == 1) {
			push @archives, $database;
		}
	}
	return \@archives;
}

sub select {
  my ($self,$document,$documentId) = @_;
	my $dbh = $self->db->dbh;	
	my $query = "SELECT * FROM archiv WHERE Akte = $documentId";
	my $sth = $dbh->prepare($query);
	$sth->execute();
  if ($sth->rows) {
	  while (my $hash_ref = $sth->fetchrow_hashref()) {
		  foreach my $key (keys %$hash_ref) {
		    my $value = $$hash_ref{$key};
			  $document->attribute($key)->value($value,0);
		  }
	  }
  } else {
	  my $exception = "Document $documentId not found ($query)";
		$self->exception($exception,__FILE__,__LINE__);
	}
	$sth->finish();
}

sub insert {
  my ($self) = @_;
  my $date = Archivista::Util::Date->new;
  my ($query,$exception,$lastInsertId);
  my $dbh = $self->db->dbh;
	$query = "INSERT INTO archiv () VALUES ()";
	my $do = $dbh->do($query);
  $exception = "Failed to insert record ($query)";
	$self->exception($exception,__FILE__,__LINE__) if ($do != 1);
	$query = "SELECT LAST_INSERT_ID()";
	my $sth = $dbh->prepare($query);
	$sth->execute();
	while (my @row = $sth->fetchrow_array()) {
		$lastInsertId = $row[0];
	}
  $sth->finish();
  $query = "UPDATE archiv ";
	$query .= "SET Akte = $lastInsertId, ";
	$query .= "Datum = '".$date->actualDate."', ";
	$query .= "Seiten = 0 ";
	$query .= "WHERE Laufnummer = $lastInsertId";
	$do = $dbh->do($query);
	$exception = "Failed to update document $lastInsertId ($query)";
	$self->exception($exception,__FILE__,__LINE__) if ($do != 1);
	return $lastInsertId;
}

sub update {
  my ($self,$documentId,$attribute) = @_;
	my $dbh = $self->db->dbh;
	my $attributesToUpdate = $self->attributesToString($attribute);	
	if (length($attributesToUpdate) > 0) {
	  my $query = "UPDATE archiv SET $attributesToUpdate ";
	  $query .= "WHERE Akte = $documentId";
	  my $do = $dbh->do($query);
    my $exception = "Failed to update document $documentId ($query)";
	  $self->exception($exception,__FILE__,__LINE__) if ($do != 1);
  }
}

sub alter {
  my ($self,$db,$palter) = @_;
	# Pointer to array of fields
	# $i (array index) -> number of alter queries to execute
	# $$palter[$i]{'attribute_name'}
	# $$palter[$i]{'attribute_type'} -> mysql data type: varchar, int, etc
	# $$palter[$i]{'attribute_length'} -> ie 255: varchar(255)
	# $$palter[$i]{'attribute_length'} -> for varchar, int
	# $$palter[$i]{'alter_type'} -> ADD or DROP
	# $$palter[$i]{'after_attribute'} -> add new attribute after this
	my $dbh = $db->dbh;
	my ($query);
  for (my $idx = 0; $idx <= $#$palter; $idx++) {
		my $alterType = $$palter[$idx]{'alter_type'};
		my $attributeName = $$palter[$idx]{'attribute_name'};
		my $afterAttribute = $$palter[$idx]{'after_attribute'};
		my $oldAttributeName = $$palter[$idx]{'attribute_old_name'};
		my $attributeType = $self->attributeDbType(
		  $$palter[$idx]{'attribute_type'},
			$$palter[$idx]{'attribute_length'});
		if (uc($alterType) eq "ADD") {
			# ADD new attribute
			$query = "ALTER TABLE archiv ";
			$query .= "ADD COLUMN $attributeName $attributeType ";
			$query .= "AFTER $afterAttribute";
		} elsif (uc($alterType) eq "DROP") {
			# DROP existing attribute
			$query = "ALTER TABLE archiv DROP COLUMN $attributeName";
		} elsif (uc($alterType) eq "CHANGE") {
			# CHANGE existing attribute
			$query = "ALTER TABLE archiv ";
			$query .= "CHANGE $oldAttributeName $attributeName $attributeType ";
			$query .= "AFTER $afterAttribute";
		}
		my $do = $dbh->do($query);
		my $exception = "Error on alter table ($query)";
		if (! $do) {
		  $self->exception($exception,__FILE__,__LINE__);
		}
	}
}

sub attributeDbType {
  my ($self,$attributeType,$attributeLength) = @_;
	my $attributeDbType;
	if (lc($attributeType) eq "varchar") {
	  if ($attributeLength > 0) {
			$attributeDbType = "VARCHAR($attributeLength)";
		} else {
			$attributeDbType = "VARCHAR(255)";
		}
	} elsif (lc($attributeType) eq "int") {
	  if ($attributeLength > 0) {
			$attributeDbType = "INT($attributeLength)";
		} else {
			$attributeDbType = "INT(11)";
		}
	} elsif (lc($attributeType) eq "double") {
		$attributeDbType = "DOUBLE";
	} elsif (lc($attributeType) eq "datetime") {
		$attributeDbType = "DATETIME";
	} elsif (lc($attributeType) eq "tinyint") {
		$attributeDbType = "TINYINT(1)";
	}
	return $attributeDbType;
}

sub delete {
  my ($self,$documentId) = @_;
	my $dbh = $self->db->dbh;
	my $query = "DELETE FROM archiv WHERE Akte = $documentId";
	my $do = $dbh->do($query);
  my $exception = "Failed to delete document $documentId ($query)";
	$self->exception($exception,__FILE__,__LINE__) if ($do != 1);
}

1;

