
package Archivista::DL::DB;
use strict;
use DBI;
use Archivista::Config;
use Archivista::BL::Attribute;
use Archivista::Util::Exception;

sub _dumpTable {
  my ($config,$host,$uid,$pwd,$database,$table,$nodata) = @_;
	my $mysqldump = $config->get("MYSQL_DUMP");
	$mysqldump = "/usr/bin/mysqldump" if !-e $mysqldump; # 64 bit
	my $dirsep = $config->get("DIR_SEP");
  my $tmpdir = $config->get("TEMP_DIR");
  my $dump = $tmpdir.$dirsep.time()."_".$database."_".$table.".sql";
	my $cmd = "$mysqldump $nodata -h $host -u $uid --password=$pwd ".
		"$database $table > $dump";
	my $system = system($cmd);
	my $exception = "Failed to dump data from $database ($cmd)";
	exception(undef,$exception,__FILE__,__LINE__) if ($system != 0);
  return $dump;
}

sub _loadDump {
  my ($config,$host,$uid,$pwd,$dump,$database,$unlink) = @_;
  my $mysql = $config->get("MYSQL_BIN");
	$mysql = "/usr/bin/mysql" if !-e $mysql; # 64 bit
	$unlink = 0 if (! defined $unlink);
  if (-f $dump) {
    my $cmd = "$mysql -h $host -u $uid --password=$pwd $database < $dump";
    my $exception = "Failed to insert data to database $database ($cmd)"; 
		my $system = system($cmd);
		exception(undef,$exception,__FILE__,__LINE__) if ($system != 0);
  } else {
	  my $exception = "File $dump not found";
	  exception(undef,$exception,__FILE__,__LINE__);
	}
	unlink $dump if ($unlink);
}

sub _connect {
  my ($host,$database,$uid,$pwd) = @_;
  eval {
    my $dbh = DBI->connect("DBI:mysql:host=$host;database=$database",
	    $uid,$pwd,{PrintError=>0,RaiseError=>0});
		return $dbh;
	}
}

sub _archivistaArchive {
  my ($self,$dbh,$database,$loginDb) = @_;
  my $config = Archivista::Config->new();
	my $avVersion = $config->get("AV_VERSION");
	my $archivistaArchive = 0;
	my ($query,$sth,@tables);
	$dbh->do("USE $database");
	$sth = $dbh->prepare("SHOW TABLES");
	$sth->execute(); 
	while (my @row = $sth->fetchrow_array()) {
		push @tables, $row[0];
	}
	foreach my $table (@tables) {
		if ($table eq "parameter") {
			$query = "SELECT Inhalt FROM parameter WHERE Name = 'AVVersion'";
			$sth = $dbh->prepare($query);
			$sth->execute(); 
			while (my @row = $sth->fetchrow_array()) {
				if ($avVersion eq $row[0]) { $archivistaArchive = 1; }
			}
			$sth->finish();
		}
	}
	# Use again the login database
	$dbh->do("USE $loginDb");
	return $archivistaArchive;
}

sub new {
  my ($cls,$database,$host,$uid,$pwd) = @_;
  my $self = {};
  my $config = Archivista::Config->new();
  my $suHost = $config->get("MYSQL_HOST");
	my $suDb = $config->get("AV_GLOBAL_DB");
	my $suUid = $config->get("MYSQL_UID");
	my $suPwd = $config->get("MYSQL_PWD");
	if (! defined $database) { $database = $suDb; }
	if (! defined $host) {
  	$host = $suHost;
		$uid = $suUid;
		$pwd = $suPwd;
	}
  bless $self, $cls;
  $self->{'sudbh'} = _connect($suHost,$suDb,$suUid,$suPwd);
	$self->{'dbh'} = _connect($host,$database,$uid,$pwd);
	$self->{'host'} = $host;
	$self->{'database'} = $database;
  $self->{'uid'} = $uid;
	if (! defined $self->{'dbh'}) {
		my $exception = "Error on login ($host, $database, $uid, ...)";
		$self->exception($exception,__FILE__,__LINE__);		
	} elsif ($database ne $config->get("AV_GLOBAL_DB")) {
		my $checkIfArchivistaDb = $self->_archivistaArchive($self->dbh,
			$self->database,$self->database);
		if ($checkIfArchivistaDb == 0) {
			my $exception = "Not an archivista archive ($database)";
			$self->exception($exception,__FILE__,__LINE__);
		}
	}
	return $self;
}

sub create {
  my ($cls,$database,$cpDataFromDb,$host,$uid,$pwd) = @_;
  my ($query,$dump,$cmd,$system,$exception);
  my $config = Archivista::Config->new();
  my $avGlobalDb = $config->get("AV_GLOBAL_DB");
	if (! defined $host) {
		$host = $config->get("MYSQL_HOST");
		$uid = $config->get("MYSQL_UID");
		$pwd = $config->get("MYSQL_PWD");
	}
  my $mysql = $config->get("MYSQL_BIN");
	$mysql = "/usr/bin/mysql" if !-e $mysql; # 64 bit
	my $dirsep = $config->get("DIR_SEP");
  my $cfgdir = $config->get("CONFIG_DIR");
  my $tmpdir = $config->get("TEMP_DIR");
  my $dbh = _connect($host,$avGlobalDb,$uid,$pwd);
	$query = "CREATE DATABASE $database";
	if ($dbh->do($query)) {
		$dump = $cfgdir.$dirsep."dump".$dirsep."archive.sql";
		_loadDump($config,$host,$uid,$pwd,$dump,$database);
		# We must copy users and parameters from another database
		if (defined $cpDataFromDb) {
	  	my $archiveDump = 
				_dumpTable($config,$host,$uid,$pwd,$cpDataFromDb,"archiv","--no-data");
			my $parameterDump = 
				_dumpTable($config,$host,$uid,$pwd,$cpDataFromDb,"parameter",undef);
			my $userDump = 
				_dumpTable($config,$host,$uid,$pwd,$cpDataFromDb,"user",undef);
			_loadDump($config,$host,$uid,$pwd,$archiveDump,$database,1);
	  	_loadDump($config,$host,$uid,$pwd,$parameterDump,$database,1);
			_loadDump($config,$host,$uid,$pwd,$userDump,$database,1);
  	}
		my $sql = "delete from $database.parameter where Name='MailArchiving' and ".
			"Art='MailArchiving01' and Tabelle='archiv'";
		$dbh->do($sql);
		$query = "INSERT INTO archives SET name=".$dbh->quote($database);
		$dbh->do($query);
	} else {
		$exception = "Failed to create database $database ($query)";
		exception(undef,$exception,__FILE__,__LINE__);
	}
	$dbh->disconnect;
}

sub drop {
  my ($self,$database) = @_;
  my $config = Archivista::Config->new;
  my $dbmaster = $config->get("MYSQL_DB");
	if ($database ne $dbmaster) {
  	my $dbh = $self->dbh;
		my $config = Archivista::Config->new;
		my $avGlobalDb = $config->get("AV_GLOBAL_DB");
		my ($query);
  	# Now drop the database
		$query = "USE $avGlobalDb";
		$dbh->do($query);
		$query = "DROP DATABASE $database";
  	if ($dbh->do($query)) {
			$query = "DELETE FROM archives WHERE ";
			$query .= "name = ".$dbh->quote($database);
			$dbh->do($query);
		} else {
			my $exception = "Failed to drop database $database ($query)";
			exception(undef,$exception,__FILE__,__LINE__)
		}
  } else {
		my $exception = "Failed to drop database $dbmaster";
		exception(undef,$exception,__FILE__,__LINE__)
  }
}

sub db {
  my ($self,$db) = @_; # Object of Archivista::DL::DB
	if (defined $db) {
		$self->{'db'} = $db;
	} else {
		return $self->{'db'};
	}
}

sub dbh {
	my ($self,$dbh) = @_;
	if (defined $dbh) {
		$self->{'dbh'} = $dbh;
	} else {
		return $self->{'dbh'};
	}
}

sub sudbh {
  my ($self) = @_;
	return $self->{'sudbh'};
}

sub host {
  my ($self) = @_;
	return $self->{'host'};
}

sub database {
  my ($self) = @_;
	return $self->{'database'};
}

sub uid {
  my ($self) = @_;
	return $self->{'uid'};
}

sub exception {
  my ($self,$msg,$file,$line) = @_;
  Archivista::Util::Exception->add($msg,$file,$line);
}

sub attributesToString {
  my ($self,$attribute) = @_;
  my @attributes;
	my $dbh = $self->db->dbh;
	foreach my $attributeId (keys %{$attribute->all}) {
		$attribute->id($attributeId);
	  my $attributeValue = $attribute->value;
		# If attribute id is password (password cipher for mysql.user table)
		if (lc($attributeId) eq "password") {
			push @attributes,
				$attributeId." = Password(".$dbh->quote($attributeValue).")";
		} else {
		  push @attributes, $attributeId." = ".$dbh->quote($attributeValue);
		}
	}
	return join ", ", @attributes;
}

sub shiftObsoleteAttributes {
  my ($self,$database,$table,$attribute) = @_;
  my ($query,$sth);
  my $dbh = $self->db->dbh;
	my $attributeToUpdate = Archivista::BL::Attribute->new;
	$self->selectdb($database);
  my $phattributes = $self->db->describe($table);
  foreach my $key (keys %$phattributes) {
	  $attribute->id($key);
		if ($attribute->modified == 1) {
		  $attributeToUpdate->id($key);
		  $attributeToUpdate->value($attribute->value,0);
	  }
	}
  $self->selectdb($self->db->database);
  return $attributeToUpdate;
}

sub describe {
  my ($self,$table,$retDS) = @_;
  my (%attributes,@attributes);
  my $dbh = $self->dbh;
	$retDS = "HASH" if (! defined $retDS);
  my $query = "DESCRIBE $table";
	my $sth = $dbh->prepare($query);
	$sth->execute();
	while (my @row = $sth->fetchrow_array()) {
		if ($retDS eq "HASH") {
			$attributes{$row[0]} = $row[1]; 
		} elsif ($retDS eq "ARRAY") {
			push @attributes, $row[0];
		}
	}
  $sth->finish();
	if ($retDS eq "HASH") {
		return \%attributes;
	} elsif ($retDS eq "ARRAY") {
		return \@attributes;
	}
}

sub userDefinedAttributes {
  my ($self,$retDS) = @_;
  # PRE: /ARRAY/HASH
  # POST: pointer to array(attributeName) or
	# 			pointer to hash(attributeName,attributeType)
	$retDS = "ARRAY" if (! defined $retDS);
  my $idx = 0;
	my $paattributes = $self->describe("archiv","ARRAY");
  # Get the index of the field 'Notiz'
	for (my $i = 0; $i < $#$paattributes; $i++) {
		$idx = $i - 1 if ($$paattributes[$i] eq "Notiz");
	}
 	my @userDefinedAttributes = @$paattributes[4 .. $idx];
  if (uc($retDS) eq "ARRAY") {
		return \@userDefinedAttributes;
	} else {
		my %userDefinedAttributes;
		my $phattributes = $self->describe("archiv","HASH");
	  foreach my $udAttribute (@userDefinedAttributes) {
			$userDefinedAttributes{$udAttribute} = $$phattributes{$udAttribute};
		}
		return \%userDefinedAttributes;
	}
}

sub showDatabases {
	my ($self) = @_;
	my @databases;
	my $dbh = $self->db->dbh;
	my $query = "SHOW DATABASES";
	my $sth = $dbh->prepare($query);
	$sth->execute();
	while (my @row = $sth->fetchrow_array()) {
		push @databases, $row[0];
	}
	$sth->finish();
	return \@databases;
}

sub showTables {
  my ($self,$database) = @_;
	my @tables;
	my $dbh = $self->db->dbh;
	$self->selectdb($database) if (defined $database);
	my $query = "SHOW TABLES";
  my $sth = $dbh->prepare($query);
  $sth->execute();
	while (my @row = $sth->fetchrow_array()) {
		push @tables, $row[0];
	}
  $sth->finish();
	return \@tables;
}

sub selectdb {
  my ($self,$database) = @_;
	my $query = "USE $database";
	my $do = $self->db->dbh->do($query);
	my $exception = "Failed to change database to $database ($query)";
	exception($exception,__FILE__,__LINE__) if (! defined $do);
}

sub hostIsSlave {
	my ($self) = @_;
	my $isSlaveHost = 0;
	my $dbh = $self->sudbh;
	if (defined $dbh) {
	  my $sth = $dbh->prepare("SHOW SLAVE STATUS");
	  $sth->execute();
	  if ($sth->rows) {
		  my @row = $sth->fetchrow_array();
		  $isSlaveHost = 1 if ($row[9] eq 'Yes');
	  }
	  $sth->finish();
    $dbh=$self->{dbh};
		if (defined $dbh) { 
	    my @row = $dbh->selectrow_array("SHOW VARIABLES LIKE 'server%'");
	    $isSlaveHost=1 if $row[1]>1;
		} else { # no connection, so we say it is a slave
      $isSlaveHost=1;
		}
	}
	return $isSlaveHost;
}

sub isArchivistaArchive {
	my ($self,$database) = @_;
	my $dbh = $self->db->dbh;
	my $loginDb = $self->db->database;
	return $self->_archivistaArchive($dbh,$database,$loginDb);
}

1;

