
package Archivista::DL::MyUser;
use strict;
use constant GRANT => "GRANT";
use constant REVOKE => "REVOKE";
use constant SYSOP => 255;
use Archivista::Config;
use Encode;

sub _privileges {
  my ($self,$type,$host,$uid,$pwd,$level) = @_;
	Encode::from_to($pwd,"iso-8859-1","utf8");
  $type = uc($type);
  my $dbh = $self->db->dbh;
	my $dbh1 = $dbh;
  my $dbh2 = $self->db->sudbh;	
  my $database = $self->db->database;
  my $grantTo = $dbh->quote($uid)."@".$dbh->quote($host);
  my $isSYSOP = $self->_isSysop($dbh,$database,$grantTo);
	my $vers = `cat /proc/version`;
	my @parts = split(/\s/,$vers);
	if ($parts[0] eq "Linux" && $parts[1] eq "version" && $parts[2]>=5) {
		my $sql = "select user from mysql.user where user=".
			$dbh2->quote($uid)." and host=".$dbh2->quote($host);
		my @row = $dbh2->selectrow_array($sql);
		if ($row[0] eq "" && $type eq "GRANT") {
			$dbh2->do($sql);
			$sql = "create user ".
				$dbh2->quote($uid)."@".$dbh2->quote($host)." ".
				"identified by ".$dbh2->quote($pwd);
			$dbh2->do($sql);
		  $dbh2->do("flush privileges");
		}
	}
  my @tables = ("user","parameter","workflow","archiv","abkuerzungen",
		"archivseiten","archivbilder","feldlisten","adressen","adressenplz",
		"literatur","literaturrubrik","notizen",
  );
  $level = 0 if (! defined $level);
  # Table grants for archivista database
	if($isSYSOP) {;
	  $level=255; 
	  # we have the first user (SYSOP), so set back Level 255
	  my $sql = "update $database.user set Level=255 "
		        . "where User='$uid' and Host='$host'";
		$dbh->do($sql);
  }	
  # check first if we already have grant rights (so wee neet to revoke them)
	my $revokefirst = $self->checkGrant($grantTo,$dbh2,$database);
	# only change user rights if we don't have the FIRST user (SYSOP)
  foreach my $table (@tables) {
    # NOTE : Set it to "select" as default
    my $priv="select";
    my $opt="";
    my $sql = "";
    if ($level == SYSOP) {
	    # Level 255
      $priv="All";
      $opt=" with grant option";
    } elsif ($table ne "parameter" && $table ne "workflow" && 
      $table ne  "user" && ($level > 0 && $level !=250)) {
		  # not level 0 and not internal tables
      $priv = "Select,Insert,Update,Delete";
    }
    if ($type eq GRANT) {
      $sql = "TO $grantTo $opt";
    } elsif ($type eq REVOKE) {
      # We don't need $opt becaus it's only for grant
      $sql = "FROM $grantTo";
    }
    if ($sql ne "" && (($type eq REVOKE && $revokefirst==1) ||
			$type eq GRANT)) {
      $sql="$type $priv ON $database.$table $sql";
			my $sql1 = "select version()";
			my @row = $dbh->selectrow_array($sql1);
			my $vers = $row[0];
			my $vers1 = int $vers;
			if ($vers1>=5) {
			  if ($table eq "user") {
			    # mysql5 does not open user when sending a grant, so
				  # we need to do this manually (only once when granting user table)
				  $sql1 = "create user '$uid'\@$host";
				  $dbh2->do($sql1);
				  if ($level==255) {
				    $sql1 = "update mysql.user set Create_user_priv='Y' ".
			        "where user=".$dbh->quote($uid)." and host=".$dbh->quote($host);
				    $dbh2->do($sql1);
				  }
				  $dbh2->do("flush privileges");
				}
				$dbh1 = $dbh2;
			}
      $dbh1->do($sql);
    }
  }
  my $sql="";
  $sql = "set password for ".$dbh2->quote($uid)."\@".$dbh2->quote($host).
				 "=OLD_PASSWORD(".$dbh2->quote($pwd).")";
  $dbh2->do($sql);
  $sql="";
  if ($level!=SYSOP) {
    if ($type eq GRANT) {
	    # give the right to update flag field PWArt after login
      $sql = "$type Update(PWArt) ON $database.user TO $grantTo";
    } elsif ($type eq REVOKE ) {
      # remove the grant option from users
      $sql = "$type Update(PWArt) ON $database.user FROM $grantTo";
		}
    $dbh1->do($sql);
  }
 	$self->AccessLog($type,$host,$uid,$dbh2);
  my $cmd = "perl /home/cvs/archivista/jobs/adjustRights.pl";
	my $cmd2 = lc($type);
	system("$cmd $database $cmd2 $uid $host");
}

sub checkGrant {
  my ($self,$grant,$dbh,$database) = @_;
	my $sql = "show grants for $grant";
	my $prow = $dbh->selectall_arrayref($sql);
	my $found = 0;
	foreach (@$prow) {
	  my $row = $$_[0];
		if (index($row,$database)>0) {
			$found = 1;
			last;
		}
  }
	return $found;
}

sub AccessLog {
  my ($self,$type,$host,$uid,$dbh) = @_;
	return if $host eq "localhost";
  my $config = Archivista::Config->new;
  my $host1 = $config->get("MYSQL_HOST");
  my $db1 = $config->get("MYSQL_DB");
  my $user1 = $config->get("MYSQL_UID");
  my $pw1 = $config->get("MYSQL_PWD");
	if ($dbh) {
    if ($type eq GRANT) {
      my $sql = "grant select,insert,update on archivista.access to ".
	      $dbh->quote($uid)."@".$dbh->quote($host);
	    $dbh->do($sql);
	  } elsif ($type eq REVOKE) { 
	    my $sql = "revoke all on archivista.access from ".
		    $dbh->quote($uid)."@".$dbh->quote($host);
	    $dbh->do($sql);
	  }
	}
}

sub _isSysop{
  my ($self,$dbh,$db,$grantTo) = @_;
  my ($user,$host) = split('@',$grantTo);
  if($user =~ /SYSOP/){
    my $sql="select Laufnummer from $db.user where User=$user and Host=$host";
    my $row = $dbh->selectrow_arrayref($sql);
    if($row->[0] == 1){
      return 1;
    } else {
      return 0;
    }
  } else {
    return 0;
  }
}

sub myInsert {
  my ($self,$host,$uid,$pwd,$level) = @_;
  $self->_privileges(REVOKE,$host,$uid,undef,255);
  $self->_privileges(GRANT,$host,$uid,$pwd,$level);
}

sub myUpdate {
  my ($self,$attribute) = @_; # Object of Archivista::BL::Attribute
  # NOTE: the syntax $attribute->id()->value can't be used; 
  # it's not the same as $user->attribute()->value !!
  $attribute->id("Host");
  my $host = $attribute->value;
  $attribute->id("User");
  my $uid = $attribute->value;
  $attribute->id("Password");
  my $pwd = $attribute->value;
  $attribute->id("Level");
  my $level = $attribute->value;
  $self->_privileges(REVOKE,$host,$uid,undef,255);
  $self->_privileges(GRANT,$host,$uid,$pwd,$level);
}

sub myGrant {
  my ($self,$pausers) = @_; # 2-dim array[n][2] host,user,level
  for (my $i = 0; $i <= $#$pausers; $i++) {
    my $host = $$pausers[$i][0];
    my $uid = $$pausers[$i][1];
    my $level = $$pausers[$i][2];
    $self->_privileges(REVOKE,$host,$uid,undef,255);
    $self->_privileges(GRANT,$host,$uid,undef,$level);
  }
}

sub myRevoke {
  my ($self,$pausers) = @_;
  for (my $i = 0; $i <= $#$pausers; $i++) {
    my $host = $$pausers[$i][0];
    my $uid = $$pausers[$i][1];
    my $level = $$pausers[$i][2];
    $self->_privileges(REVOKE,$host,$uid,undef,$level);
  }
}

sub myDelete {
  my ($self,$user) = @_;
  my $query;
  my $dbh = $self->db->dbh;
  my $host = $user->attribute("Host")->value;
  my $uid = $user->attribute("User")->value;
  my $level = $user->attribute("Level")->value;
  $self->_privileges(REVOKE,$host,$uid,undef,255);
}

1;

