
package Archivista::DL::Parameter;
use strict;
use vars qw ( @ISA );
use Archivista::Config;
use Archivista::DL::DB;
use Archivista::Util::IO;
@ISA = qw ( Archivista::DL::DB );

sub _refreshParam {
  my ($dbh,$name,$table,$value) = @_;
	my $update = 0;
	my ($query);
  $query = "SELECT Name FROM parameter WHERE ";
  $query .= "Name = '$name' AND Tabelle = '$table' AND Inhalt = '$value'";
  my $sth = $dbh->prepare($query);
  $sth->execute;
  while (my @row = $sth->fetchrow_array()) {
    $update = 1;
  }
  $sth->finish;
  if ($update == 1) {
    $query = "UPDATE ";
  } else {
    $query = "INSERT INTO ";
  }
  $query .= "parameter SET ";
  $query .= "Name = '$name', Tabelle = '$table', Inhalt = '$value'";
  $dbh->do($query);
}

sub new {
  my ($cls,$db,$type) = @_;
  my $self = {};
  bless $self, $cls;
  $self->db($db);
  $self->type($type);
  return $self;
}

sub load {
  my ($cls,$db,$archiveName) = @_;
  my $self = {};
  bless $self, $cls;
  my ($do,$exception,$query);
  my $dbh = $db->dbh;
  my $io = Archivista::Util::IO->new;
  my $config = Archivista::Config->new;
  my $cfgdir = $config->get("CONFIG_DIR");
  my $dirsep = $config->get("DIR_SEP");
  my $baseImagePath = $config->get("BASE_IMAGE_PATH");
  my $avversion = $config->get("AV_VERSION");
  my $av5par = $cfgdir.$dirsep."AV5.PAR";
  # Parse the AV5.PAR file (from av5.zzz)
  my $pfileData = $io->read($av5par);
  foreach (split /\r\n/, $$pfileData) {
    # Values are tab separated 
    my ($type,$table,$name,@content) = split /\t/, $_;
    my $content = join "\r\n", @content;
    $query = "INSERT INTO parameter SET ";
    $query .= "Art = ".$dbh->quote($type).",";
    $query .= "Tabelle = ".$dbh->quote($table).",";
    $query .= "Name = ".$dbh->quote($name).",";
    $query .= "Inhalt = ".$dbh->quote($content);
    $do = $dbh->do($query);
    $exception = "Failed to insert parameter ($query)";
    $self->exception($exception,__FILE__,__LINE__) if ($do != 1);
  }
  # Now update the AVVersion value
  $query = "UPDATE parameter SET ";
  $query .= "Inhalt = ".$dbh->quote($avversion)." ";
  $query .= "WHERE Name = ".$dbh->quote("AVVersion");
  $do = $dbh->do($query);
  $exception = "Failed to update parameter AVVersion ($query)";
  $self->exception($exception,__FILE__,__LINE__) if ($do != 1);
  # Insert the input, output screen path
  my $input = $baseImagePath.$dirsep.$archiveName.$dirsep."input";
  _refreshParam($dbh,'PfadGrafikBatchInput','archiv',$input);
  my $output = $baseImagePath.$dirsep.$archiveName.$dirsep."output";
  _refreshParam($dbh,'PfadGrafikBatchOutput','archiv',$output);
  my $screen = $baseImagePath.$dirsep.$archiveName.$dirsep."screen";
  _refreshParam($dbh,'PfadGrafikBatchScreen','archiv',$screen);
}

sub select {
  my ($self,$parameter,$parameterId) = @_; # Object Archivista::BL::Parameter
  my $type = $self->type;
  my $dbh = $self->db->dbh;
  my $query = "SELECT * FROM parameter WHERE Name=".$dbh->quote($parameterId);
  my $sth = $dbh->prepare($query);
  $sth->execute();
  if ($sth->rows) {
    while (my $hash_ref = $sth->fetchrow_hashref()) {
			foreach my $key (keys %$hash_ref) {
	  		my $value = $$hash_ref{$key};
	  		$parameter->attribute($key)->value($value,0);
			}
    }
  } else {
    my $exception = "Parameter $parameterId not found ($query)";
    #		$self->exception($exception,__FILE__,__LINE__);
  }
  $sth->finish();
}

sub insert {
  my ($self,$parameter,$attribute) = @_;
  my $parameterId = $parameter->id;
  my $dbh = $self->db->dbh;
  my $qArt=$dbh->quote($self->type);
  my $qName=$dbh->quote($parameterId);
  my $qVal=$dbh->quote($parameter->attribute("Inhalt")->value);
  my $query = "INSERT INTO parameter (Art,Tabelle,Name,Inhalt) "; 
  $query.="values ( $qArt, 'parameter', $qName, $qVal)";
  my $do = $dbh->do($query);
  my $exception = "Failed to insert parameter $parameterId ($query)";
  $self->exception($exception,__FILE__,__LINE__) if (! $do);
}

sub update {
	my ($self,$parameter,$attribute) = @_;
  if (!$self->exists($parameter)) {
   $self->insert($parameter,$attribute);
  } else {
    my $parameterId = $parameter->id;
    my $archive = $parameter->archive;
    my $type=$self->type;
    my $dbh = $self->db->dbh;
    my $attributesToUpdate = $self->attributesToString($attribute);
		if (length($attributesToUpdate) > 0) {
			my $query = "UPDATE parameter SET $attributesToUpdate ";
			$query .= "WHERE Name = ".$dbh->quote($parameterId);
			my $do = $dbh->do($query);
			my $exception = "Failed to update parameter $parameterId ($query)";
			$self->exception($exception,__FILE__,__LINE__) if (! $do);
    }
  }
}

sub exists {
  my ($self,$parameter) = @_;
  my $dbh = $self->db->dbh;
  my $parameterId = $parameter->id;
  my $type=$self->type;
  my $exists = 0;
  my $query="SELECT Name FROM parameter WHERE Name=".$dbh->quote($parameterId);
  my $sth = $dbh->prepare($query);
  $sth->execute;
  $exists = 1 if ($sth->rows);
  $sth->finish;
  return $exists;
}

sub maskDefinitions {
  my $self = shift;
  my $db = shift;
  my %definitions;
  my $dbh = $db->dbh;
  $self->maskDefinitionsCheck($db);
  my $query = "SELECT Name,Art FROM parameter WHERE ";
  $query .= "Name LIKE 'FelderObj__'";
  my $sth = $dbh->prepare($query);
  $sth->execute;
  while (my @row = $sth->fetchrow_array()) {
		my $key = $row[0];
		my $value = $row[1];
		$key =~ s/FelderObj//;
    $definitions{$key} = $value;
  }
  $sth->finish;
  return \%definitions;
}

sub maskDefinitionsCheck {
  my ($self,$db) = @_;
  my $dbh = $db->dbh;
	my ($sql,@row,$val);
	# check first, if the first definition is there 
	$sql = "SELECT Name from parameter WHERE Name='FelderObj00'".
		" AND Tabelle='archiv'";
  @row = $dbh->selectrow_array($sql);
	if ($row[0] eq "") {
	  # first definition is not there
		$sql = "insert into parameter set Art='Input mask1'," .
		       "Name='FelderObj00',Tabelle='archiv',Inhalt=''";
		$dbh->do($sql);
	}
	$sql = "SELECT Name from parameter WHERE Name='FelderTab00' ".
		"AND Tabelle='archiv'";
  @row = $dbh->selectrow_array($sql);
	if ($row[0] eq "") {
		$sql = "insert into parameter set Art='FelderTab00'," .
		       "Name='FelderTab00',Tabelle='archiv',Inhalt=''";
		$dbh->do($sql);
	}
}

sub maskDefinitionName {
  my ($self,$parameter,$maskDefinitionName) = @_;
  my $dbh = $self->db->dbh;
  my $parameterId = $parameter->id;
  my ($query,$definition,$sth);
  if (defined $maskDefinitionName) {
    $query = "UPDATE parameter SET Art=".$dbh->quote($maskDefinitionName)." ".
			"WHERE Name = ".$dbh->quote($parameterId);
    $dbh->do($query);
  } else {
		$query = "SELECT Art FROM parameter WHERE Name = ".
	  $dbh->quote($parameterId);
  	$sth = $dbh->prepare($query);
	  $sth->execute;
  	while (my @row = $sth->fetchrow_array()) {
    	$maskDefinitionName = $row[0];
  	}
  	$sth->finish;
		return $maskDefinitionName;
  }
}

sub maskParentFields {
  # Returns all fields by hash (index, field name) which have a type 4 or 5
  # The hash key is like the array index when 'FelderObjXY' is splitted by
  # '\r\n'
  my ($self,$parameter,$retDS) = @_; # Return data structure (ARRAY or HASH)
  my $dbh = $self->db->dbh;
  my $parameterId = $parameter->id;
  $retDS = "HASH" if (! defined $retDS);
  my ($parentFields,@parentFields,%parentFields);
  my $query = "SELECT Inhalt FROM parameter ";
  $query .= "WHERE Tabelle = 'archiv' AND Name = ".$dbh->quote($parameterId);
  my $sth = $dbh->prepare($query);
  $sth->execute;
  while (my @row = $sth->fetchrow_array()) {
    $parentFields = $row[0];
  }
  $sth->finish;
  my @definitions = split /\r\n/, $parentFields;
  for (my $idx = 0; $idx <= $#definitions; $idx++) {
    my @values = split /;/, $definitions[$idx];
    if ($values[4] == 0 && ($values[5] == 4 || $values[5] == 5)) {
      if (uc($retDS) eq "HASH") {
      	$parentFields{$idx} = $values[18];
      } elsif (uc($retDS) eq "ARRAY") {
        push @parentFields, $idx;
      }
    }
  }

  if (uc($retDS) eq "HASH") {
    return \%parentFields;
  } elsif (uc($retDS) eq "ARRAY") {
    return \@parentFields;
  }
}

sub userSqlDefinitions {
	# POST: Pointer to hash(ID, Name)
  # Return all AVStart sql definitions 
  my ($self,$db) = @_;
  my $dbh = $db->dbh;
  my %sqlDefinitions;
  my $query = "SELECT Laufnummer, Name FROM parameter ";
  $query .= "WHERE Art = 'SQL' AND Tabelle = 'archiv' AND ";
  $query .= "(Inhalt NOT LIKE '%[%' or Inhalt LIKE '%[user}%')";
  my $sth = $dbh->prepare($query);
  $sth->execute;
  while (my @row = $sth->fetchrow_array()) {
    $sqlDefinitions{$row[0]} = $row[1];
  }
  $sth->finish;
  return \%sqlDefinitions;
}

sub delete {
	my ($self,$parameter) = @_;
	my $dbh = $self->db->dbh;
	my $parameterId = $parameter->id;
	my $query = "DELETE FROM parameter WHERE Name = ".$dbh->quote($parameterId);
	my $do = $dbh->do($query);
  my $exception = "Failed to delete parameter $parameterId ($query)";
  $self->exception($exception,__FILE__,__LINE__) if (! $do);
}

sub type {
	my ($self,$type) = @_; # Object of Archivista::DL::DB
  if (defined $type) {
    $self->{'type'} = $type;
  } else {
    return $self->{'type'};
  }
}

1;

