
package Archivista::DL::Session;
use strict;
use vars qw ( @ISA );
use Archivista::Config;
use Archivista::DL::DB;
@ISA = qw ( Archivista::DL::DB );

sub _init {
  my ($self) = @_;
	my $config = Archivista::Config->new;
	my $database = $config->get("AV_GLOBAL_DB");
	my $host = $config->get("MYSQL_HOST");
	my $uid = $config->get("MYSQL_UID");
	my $pwd = $config->get("MYSQL_PWD");
	my $db = Archivista::DL::DB->new($database,$host,$uid,$pwd);
	$self->{'session_db'} = $database;
	$self->db($db);
}

sub _read {
  my ($self) = @_;
	my $dbh = $self->db->dbh;
	my $sid = $self->session->id;
	my ($query,$sth);
	$query = "SELECT host,db,user,password,language FROM session WHERE ";
	$query .= "sid = ".$dbh->quote($sid);
  $sth = $dbh->prepare($query);
  $sth->execute;
	while (my @row = $sth->fetchrow_array()) {
		$self->session->host($row[0]);
		$self->session->db($row[1]);
		$self->session->user($row[2]);
		$self->session->password($row[3]);
		$self->session->language($row[4]);
	}
	$sth->finish;
}

sub _open {
  my ($self) = @_;
	my $dbh = $self->db->dbh;
	my $sid = $self->session->id;
	my ($exception,$query,$do,$sth);
  my $lang = $self->session->language;
	$query = "INSERT INTO session SET ";
	$query .= "sid = ".$dbh->quote($sid).",";
	$query .= "host = ".$dbh->quote($self->session->host).",";
	$query .= "db = ".$dbh->quote($self->session->db).",";
	$query .= "user = ".$dbh->quote($self->session->user).",";
	$query .= "password = ".$dbh->quote($self->session->password);
	if (defined $lang) {
		$query .= ",language = ".$dbh->quote($lang);
	}
	$do = $dbh->do($query);
	$exception = "Failed to open session ($query)";
	$self->exception($exception,__FILE__,__LINE__) if ($do != 1);
}

sub _deleteSessionData {
  my ($self) = @_;
	my $dbh = $self->db->dbh;
  my $sid = $self->session->id;	
  my $query = "DELETE FROM session_data WHERE ";
	$query .= "sid = ".$dbh->quote($sid);
	my $do = $dbh->do($query);
}

sub read {
	my ($self,$session) = @_; # Object of Archivista::Util::Session
	$self->{'session'} = $session;
  $self->_read;
}

sub open {
  my ($cls,$session) = @_; # Object of Archivista::Util::Session
	my $self = {};
	bless $self, $cls;
  $self->{'session'} = $session;
  $self->_init;
	$self->_open;
	return $self;
}

sub check {
  my ($cls,$sid) = @_;
  my $self = {};
	bless $self, $cls;
	$self->_init;
  my $dbh = $self->db->dbh;
	my $query = "SELECT sid FROM session WHERE ";
	$query .= "sid = ".$dbh->quote($sid)." LIMIT 1";
	my $sth = $dbh->prepare($query);
	$sth->execute;
  if ($sth->rows) {
		$self->{'session_id'} = $sid;
	} else {
		undef $self;
	}
	$sth->finish;
	return $self;
}

sub close {
  my ($self) = @_;
  $self->db->dbh->disconnect;
}

sub delete {
  my ($self) = @_;
	my $dbh = $self->db->dbh;
	my $sid = $self->session->id;
	my $query = "DELETE FROM session WHERE ";
	$query .= "sid = ".$dbh->quote($sid);
	my $do = $dbh->do($query);
  $self->_deleteSessionData;
	my $exception = "Failed to delete session ($query)";
	$self->exception($exception,__FILE__,__LINE__) if ($do <= 1);
}

sub save {
  my ($self,$session) = @_;
	my $sid = $self->session->id;
  my $dbh = $self->db->dbh;
	# First we delete the session data
	$self->_deleteSessionData;
	# Now we save the data from the hash
	foreach my $param (keys %{$session->params}) {
		my $query = "INSERT INTO session_data SET ";
	  $query .= "sid = ".$dbh->quote($sid).",";
		$query .= "param = ".$dbh->quote($param).",";
		$query .= "value = ".$dbh->quote($session->param($param));
		my $do = $dbh->do($query);
	  my $exception = "Failed to add session param ($query)";
		$self->exception($exception,__FILE__,__LINE__) if ($do != 1);
	}
}

sub session {
  my ($self) = @_;
 
  return $self->{'session'};
}

sub sessiondb {
  my ($self) = @_;
	return $self->{'session_db'};
}

1;

