
package Archivista::Util::FS;
use strict;
use Archivista::Config;
use Archivista::Util::IO;

sub new {
  my ($cls) = @_;
	my $self = {};
  bless $self, $cls;
  return $self;
}

sub countFilesOnDir {
  my ($self,$dir,$fileMask) = @_;
  # Please note: FileMask is a string that must match the filename on the
	# directory (for example: scan-20-0003.tif matches the mask 'scan-20-')
	# If the FileMask is given, only files of the directory that matches the mask
	# will be counted
  my $nrOfFiles = 0;
	my $io = Archivista::Util::IO->new;
	my $pafiles = $io->read($dir,"DIR");
	foreach my $file (@$pafiles) {
		if (defined $fileMask) {
			$nrOfFiles++ if ($file =~ /$fileMask/);	
		} else {
			$nrOfFiles++;
		}
	}
	return $nrOfFiles;
}

sub createImageFoldersForArchive {
  my ($self,$archive) = @_;
	my $config = Archivista::Config->new;
	my $dirSep = $config->get("DIR_SEP");
	my $baseImagePath = $config->get("BASE_IMAGE_PATH");
	my $baseImagePathForArchive = $baseImagePath.$dirSep.$archive.$dirSep;
	mkdir $baseImagePathForArchive;
	mkdir $baseImagePathForArchive."input";
	mkdir $baseImagePathForArchive."output";
	mkdir $baseImagePathForArchive."screen";
}

sub unlinkImageFoldersForArchive {
	my ($self,$archive) = @_;
	my $config = Archivista::Config->new;
	my $rm = $config->get("RM_RF");
	my $dirSep = $config->get("DIR_SEP");
	my $baseImagePath = $config->get("BASE_IMAGE_PATH");
  my $baseImagePathForArchive = $baseImagePath.$dirSep.$archive;
	system("$rm $baseImagePathForArchive");
}

1;

