
package Archivista::Util::Session;
use strict;
use Digest::MD5 qw(md5_hex);
use Archivista::Config;
use Archivista::DL::Session;

sub open {
  my ($cls,$database,$host,$uid,$pwd,$lang) = @_;
	my $self = {};
  bless $self, $cls;
  if (! defined $host) {
		my $config = Archivista::Config->new;
		$host = $config->get("MYSQL_HOST");
		$uid = $config->get("MYSQL_UID");
		$pwd = $config->get("MYSQL_PWD");
	}
  $self->host($host);
	$self->db($database);
	$self->user($uid);
	$self->password($pwd);
	$self->language($lang);
	$self->{'params'} = {};
	$self->{'session_id'} = md5_hex rand().time().$host.$database.$uid;
  $self->{'sessionT'} = Archivista::DL::Session->open($self);
	return $self;
}

sub check {
  my ($cls,$sid) = @_;
	my $self = {};
	bless $self, $cls;
  $self->{'session_id'} = $sid;
	$self->{'params'} = {};
	$self->{'sessionT'} = Archivista::DL::Session->check($sid);
	if (! defined $self->{'sessionT'}) {
		undef $self;
	}
	return $self;
}

sub param {
  my ($self,$param,$value) = @_;
	if (defined $value) {
		$self->{'params'}->{$param} = $value;
	} else {
		return $self->{'params'}->{$param};
	}
}

sub params {
  my ($self) = @_;
  # POST: pointer to Hash(param,value)
	return $self->{'params'};
}

sub read {
  my ($self) = @_;
	# Read an existing session
	$self->{'sessionT'}->read($self);
}

sub close {
  my ($self) = @_;
	$self->{'sessionT'}->close;
}

sub save {
  my ($self) = @_;
	$self->{'sessionT'}->save($self);
}

sub delete {
  my ($self) = @_;
	$self->{'sessionT'}->delete;
  undef $self->{'sessionT'};
}

sub id {
  my ($self) = @_;
	return $self->{'session_id'};
}

sub host {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'host'} = $value;
	} else {
		return $self->{'host'};
	}
}

sub db {
  my ($self,$value) = @_;
	if (defined $value) {
		$self->{'db'} = $value;
	} else {
		return $self->{'db'};
	}
}

sub user {
	my ($self,$value) = @_;
	if (defined $value) {
		$self->{'user'} = $value;
	} else {
		return $self->{'user'};	
	}
}

sub password {
	my ($self,$value) = @_;
	if (defined $value) {
		$self->{'password'} = $value;
	} else {
		return $self->{'password'};
	}
}

sub language {
	my ($self,$value) = @_;
	if (defined $value) {
		$self->{'language'} = $value;
	} else {
		return $self->{'language'};
	}
}

1;

