package AVStrings;
use strict;
use warnings;
use lib qw(/home/cvs/archivista/jobs/);
use Wrapper;
use DBI;
use constant DB_DEFAULT => 'archivista';
use constant TABLE_LANGUAGES => 'languages';
use constant LANG_EN => 'en';
use constant LANG_DE => 'de';
use constant LANG_FR => 'fr';
use constant LANG_IT => 'it';
use constant SOURCE_DB => 'database';
use constant SOURCE_FILE => 'file';
use constant STRINGS_FOLDER => '/usr/lib/perl5/site_perl/';
use constant STRINGS_FILE => STRINGS_FOLDER.'languages.txt';
use constant KEY_FIELD => 'id';
sub lang { wrap(@_) } # Language 'de' or 'en'
sub source { wrap(@_) } # Where to get Strings from
sub dbh { wrap(@_) } # dbh object
sub strings { wrap(@_) } # Pointer with all Strings of the file we parsed.

# Creates a New AVStrings Object.
sub new {
  my $class = shift;
  my $self = {};
  my $lang = shift;
  my $source = shift;
  bless $self,$class;
  $self->setSource($source);
  $self->setLanguages($lang);
  return $self;
}

sub setLanguages {
  my $self = shift;
  my $lang = shift;
  $self->lang($self->LANG_EN);
  $self->lang($lang) if $lang eq $self->LANG_FR;
  $self->lang($lang) if $lang eq $self->LANG_DE;
  $self->lang($lang) if $lang eq $self->LANG_IT;
}

# Sets the source. Possible Values are $self->SOURCE_DB or $self->SOURCE_FILE.
sub setSource {
  my $self = shift;
  my $source = shift;
  if($source eq $self->SOURCE_DB) {
    if (eval("require AVConfig; 1") ) {
		  # We are root Default settings can be loaded
      my $conf = AVConfig->new();
      my $host = $conf->def_host;
      my $db = $conf->def_db;
      my $user = $conf->def_user;
      my $pw = $conf->def_pw;
      my $dsn = "DBI:mysql:host=$host;database=$db;";
      my $dbh = DBI->connect($dsn,$user,$pw, {PrintError => 0});
      if ($dbh) {
        $self->dbh($dbh); # Set Database Handler
        $self->source($source);
      } else {
        # We have no Connection set Source to File
        $self->setSource($self->SOURCE_FILE);
      }
    } else {
		  # We can not load AVConfig so we are not root
			# Set Source to file
      $self->setSource($self->SOURCE_FILE);
    }
  } else {
    $self->source($self->SOURCE_FILE); # set default
    $self->_parseFile();
  }
}

# Look at getStrings. Diffrence it returns only one string.
sub string {
  my ($self,$id) = @_;
	my $phash = $self->getStrings($id);
	return ($phash->{$id}->{$self->lang()});
}

# Returns pointer to hash with strings with application or one single string
sub getStrings {
  my ($self,$id,$lang) = @_;
  my ($phstrings);
# if someone gives 'en' to us  we only want this String to be in english. So we
# have to backup first the default value. Then set the new one temporary to the
# one that was given to us, test if it is possible. And in the end Reset the
# Object Default language
##
  my $old_lang = $self->lang(); # Backup object default language
  # Set language and check if it is possible else take Class default
  # Get new Value
  $lang = $self->lang($lang);
  # Reset object default
  $self->lang($old_lang);
  # Check if we want the strings from the database
  if ($self->source eq $self->SOURCE_DB) {
    $phstrings = $self->_getStringsFromDB($id,$lang);
  } else {
    $phstrings = $self->_getStringsFromFile($id,$lang);
  }
  return $phstrings;
}

# Composes SQL String and returns a hash Reference with the Result.
sub _getStringsFromDB {
  my ($self,$id,$lang) = @_;
  my $sql = $self->_composeSQL($id,$lang);
  my $phstrings = $self->dbh->selectall_hashref($sql,$self->KEY_FIELD);
  return $phstrings;
}

# Check if the wanted string is in our parsed file.
sub _getStringsFromFile {
  my ($self,$id,$lang) = @_;
  my $phstrings = {};
  if ($id =~ /\d+\.\d+/) {
    foreach my $key (keys %{$self->strings}) {
      # ID and Application are numeric so take == instead of eq
      if ($self->strings->{$key}->{Application} == $id) {
        # Add String to Output if it has the right Application number
        $phstrings->{$key} = $self->strings->{$key};
      }
    }
  } else {
    $phstrings->{$id} = $self->strings->{$id};
  }
  return $phstrings;
}

# Composes SQL String and returns it.
sub _composeSQL {
  my ($self,$id,$lang) = @_;
  my $sql;
  $sql  = "select ".$self->KEY_FIELD.",$lang from ".$self->TABLE_LANGUAGES." ";
  # Do we want all Strings from an Application or just one id
  my $check = "id";
  $check = "Application" if ($id =~ /\d+\.\d+/);
  $sql .= " where $check = ".$self->dbh->quote($id);
  return $sql;
}

# Parses the default language File to an intern variable.
sub _parseFile {
  my $self = shift;
  my $file = $self->STRINGS_FILE;
  my %hash;
  my @lang_table = ('id','comment','de','en','fr','it');
  my $strings = $self->_getFile($file);
  my @lines = split("\r\n",$strings);
  foreach my $line (@lines) {
    my %temp;
    my @values = split("\t",$line);
    my $c = 0;
    foreach my $attribute (@lang_table) {
      $temp{$attribute} = $values[$c];
      $c++;
    }
    $hash{$values[0]} = \%temp;
  }
  $self->strings(\%hash);
}

# Reads a file and returns it.
sub _getFile {
  my ($self,$file,$txt) = @_;
  open(FIN,"<",$file);
  binmode(FIN);
  while(<FIN>) { $txt .= $_; }
  close(FIN);
  return $txt;
}

1;

