#!/usr/bin/perl

use strict;
use Digest::MD5 qw(md5_hex);
use Fcntl qw(SEEK_SET SEEK_CUR SEEK_END); # better than using 0, 1, 2
use lib qw(/home/cvs/archivista/jobs);
use AVJobs;
use Cwd;
use File::Temp qw(tempfile tempdir);
use HTML::Entities;
use constant HTMLDOC0 => '/usr/bin/htmldoc';
use constant HTMLDOC => HTMLDOC0.' --size a4 --format pdf14 '.
  '--continuous --no-embedfonts --header ... ';
use constant TXT2HTML => '/usr/bin/txt2html';
use constant WKHTMLTOPDF=>'/usr/local/bin/wkhtmltopdf';
use constant WKHTMLOPT=>'--load-error-handling ignore';
use constant ADDFILE => "/usr/bin/perl /home/cvs/archivista/jobs/addfile.pl";
use constant MAXCHECK => 1000;
use constant KDENLIVE => "kdenlive";
use constant SHOTCUT => "mlt";
use Archivista::Config; # needed for the passwords
my $config = Archivista::Config->new;
my $pw=$config->get("MYSQL_PWD");
undef $config;
my $file = shift; # kdenlive file
$file = $ENV{AV_KDENLIVE} if $file eq "";
my $mode = KDENLIVE;
$mode = SHOTCUT if $file =~ /(\.mlt)$/;
logit("adding video project $mode with $file");
$file = Cwd::abs_path($file); # we want always the full path
my $db = shift; # database to upload
$db = $ENV{AV_SCAN_DB} if $db eq "";
$db="archivista" if $db eq "";
die "usage $0 file [database] => mssing file" if !-e $file;
my @producers = (); # producers fragments
my @ids = (); # ids of producers fragments
my @in = (); # all founded files so far (producers)
my @out = (); # new file names for output (producers)
my @hash = (); # hashes to find out the correct file
my $pr1 = "<producer";
my $pr2 = "<\/producer>";
my $cont = ""; # contains the kdenlive xml file
if (-e "$file") {
	my $dbh = MySQLOpen();
  if (HostIsSlave($dbh)==0 && $dbh->do("use $db")) {
		my @parts = split(/\//,$file);
		$file = pop @parts;
		my $pin = join('/',@parts);
		my @parts1 = split(/\./,$file);
		my $ext = pop @parts1;
		my $bas = join('.',@parts1);
		readFile2("$pin/$file",\$cont);
		while ($cont =~ /($pr1.*?id=\")(.*?)(\".*?>)(.*?)($pr2)/sg) {
			push @ids,$2;
			push @producers,$4;
		}
		my $c = 0;
		my $error = 0;
		foreach my $prod (@producers) {
			if ($ids[$c] eq "black" && $mode eq SHOTCUT) {
				$c++;
				next;
			}
			my $new=doFilelinks($dbh,$db,$pw,$pin,$file,$prod,
				\@in,\@out,\@hash,$bas,$mode);
			if ($new ne "") {
				$cont=~s/($pr1.*?id=\")($ids[$c])(\".*?>)(.*?)($pr2)/$1$2$3$new$5/sg;
				$c++;
			} else {
				logit("ERROR at position $c with id ".$ids[$c]." for database $db");
				$error=1;
				last;
			}
		}
		if ($error==0) {
			postProcessing($dbh,$db,\$cont,$pin,$file,\@in,\@out,\@hash,$bas,$mode);
		}
	}
}

sub doFilelinks {
	my ($dbh,$db,$pw,$pin,$file,$producer,
		$pins,$pouts,$phash,$bas,$mode,$ppre) = @_;
	my $nat = "<property name=\"resource\">"; # file without proxy
	my $url = "<property name=\"kdenlive\:originalurl\">"; # file with proxy
	my $prop = "<\/property>";
	my $filenat = "";
	my $fileurl = "";
  $producer =~ /($nat)(.*?)($prop)/s;
	$filenat = $2 if $1 eq $nat && $2 ne "" && $3 eq $prop;
	if ($mode eq SHOTCUT && $filenat ne "") {
		my $first = substr($filenat,0,1);
		$filenat="#" if $first eq "#";
	}
  $producer =~ /($url)(.*?)($prop)/s;
	$fileurl = $2 if $1 eq $url && $2 ne "" && $3 eq $prop;
	my $fin = "";
	my $pre = ""; # scaling factor in Shotcut
	my $fout = "";
	my $check=1;
	if ($filenat ne "black" && $filenat ne "#") {
		$fin = $filenat if $filenat ne "";
		$fin = $fileurl if $fileurl ne "" && $mode eq KDENLIVE;
	}
	if ($fin ne "") {
		my @parts0 = split(':',$fin);
		$pre = shift @parts0;
		$fin = join(':',@parts0);
		if ($fin eq "") { # no preface on file name, so set file=pre and pre=''
			$fin = $pre;
			$pre = "";
		}
	  $fin = decode_entities($fin);
		$fin = "$pin/$fin" if -e "$pin/$fin";
		$fin = Cwd::abs_path($fin); # we want always the full path
		my @parts = split(/\//,$fin);
		$fin = pop @parts;
		$pin = join('/',@parts);
		my $hash = getHash($fin,$pin,$mode);
		my $hash2 = $hash;
		if ($mode eq SHOTCUT) {
			$hash2 = getHash($fin,$pin,KDENLIVE);
		}
		if ($hash ne "") {
			my $c=0;
			foreach my $fin0 (@$pins) {
				if ($$phash[$c] eq $hash) {
					$fout = $$pouts[$c];
					logit("$fin with $hash already founded, goes to $fout");
				}
				$c++;
			}
			if ($fout eq "") {
				$fout=$fin;
				logit("$fin not yet founded, check database $db");
				my $sql = "select Schlussel,Seite from archivseiten ".
					"where Schlussel=".$dbh->quote($hash2);
				my @row = $dbh->selectrow_array($sql);
				if ($row[0] eq $hash2) {
					my $lnr = int($row[1]/1000);
					updateTitelKfile($dbh,$db,$lnr,$bas);
					logit("$fin already in database $db");
				} else {
					logit("$fin NOT YET in database $db, so add it");
					addJobFile($dbh,$db,$pw,"$pin/$fin",$bas,1);
				}
				@parts = split(/\./,$fout);
				my $ext = pop @parts;
				my $base = join('.',@parts);
				while ($check>0 && $check<MAXCHECK) {
					my $found=checkValue($pouts,$fout);
					$fout = "$base-$check.$ext" if $found==1;
					$check++;
					$check=0 if $found==0;
				}
				if ($check<MAXCHECK) {
					push @$pins,$fin;
					push @$pouts,$fout;
					push @$phash,$hash;
					logit("adding $fin as $fout for $db");
				} else {	
					logit("error with finding output for $fin");
				}
			}
		} else {
			$check=MAXCHECK;
		}
		if ($check<MAXCHECK) {
			my $fout1 = encode_entities($fout);
			$fout1 = "$pre:$fout1" if $pre ne "";
		 	$producer =~ s/($nat)(.*?)($prop)/$1$fout1$3/;
			if ($mode eq KDENLIVE) {
	  		$producer =~ s/($url)(.*?)($prop)/$1$fout1$3/;
				my $org = "<property name=\"kdenlive\:original\.resource\">";
			  $producer =~ s/($org)(.*?)($prop)/$1$fout1$3/;
				my $hnew="<property name=\"kdenlive\:original\.kdenlive\:file_hash\">";
		  	$producer =~ s/($hnew)(.*?)($prop)/$1$hash$3/;
			}
		} else {
			$producer="";
		}
	}
	return $producer;
}

sub postProcessing {
	my ($dbh,$db,$pcont,$pin,$file,$pins,$pouts,$phash,$bas,$mode) = @_;
	my $key = "<mlt ";
	my $root = "\" root=\"";
	my $last = "\">";
	$$pcont =~ s/($key)(.*?)($root)(.*)($last)/$1$2$3.$5/;
	my $proxy = "<property name=\"kdenlive\:proxy\">"; # file with proxy
	my $fhash = "<property name=\"kdenlive\:file_hash\">"; # file with proxy
	my $warp = "<property name=\"warp_resource\">"; # file with proxy
	my $prop = "<\/property>";
	$$pcont =~ s/(\n)(.*?)($proxy)(.*?)($prop)(\n)/\n/g;
	$$pcont =~ s/(\n)(.*?)($warp)(.*?)($prop)(\n)/\n/g;
	$$pcont =~ s/(\n)(.*?)($fhash)(.*?)($prop)(\n)/\n/g;
	my $fback = getFile($file,$mode,"backup");
	my $ftxt = getFile($file,$mode,"txt");
	my $fold = getFile($file,$mode,"old");
	my $fhtml = getFile($file,$mode,"html");
	my $fpdf = getFile($file,$mode,"pdf");
	my $fzip = getFile($file,$mode,"zip");
	my $pout = tempdir("/tmp/kde-XXXX");
	my $cmd = "cp -f \"$pin/$file\" \"$pout/$fback\"";
	my $res=doit($cmd);
	if ($res==0) {
		my $res2 = writeFile("$pout/$file",\$cont,1);
		$res=-1 if $res2!=1;
	}
	if ($res==0) {
		my $out = "";
		my $old = "";
		my $c=0;
		foreach my $fin (@$pins) {
			my $hash = $$phash[$c];
			my $fout = $$pouts[$c];
			$out .= "$hash $fout\n\n";
			$old .= "$hash $fin\n\n";
			$c++;
		}
		my $res2 = writeFile("$pout/$fold",\$old,1);
		$res2 = writeFile("$pout/$ftxt",\$out,1);
		$res=-1 if $res2!=1;

	}
	$cmd = TXT2HTML." \"$pout/$ftxt\" >\"$pout/$fhtml\"";
	$res=doit($cmd) if $res==0;
	$cmd = WKHTMLTOPDF." ".WKHTMLOPT." \"$pout/$fhtml\" \"$pout/$fpdf\"";
	$res=doit($cmd) if $res==0;
	if ($res==0 && -e "$pout/$file" && -e "$pout/$fback" && -e "$pout/$fpdf") {
		my @parts = split(/\./,$fzip);
		my $ext = pop @parts;
		my $base = join('.',@parts);
		my $check = 1;
		while (-e "/tmp/$fzip" && $check<MAXCHECK) {
			$fzip  = "$base-$check.$ext";
			$check++;
		}
		if (!-e "/tmp/$fzip") {
			$cmd = "zip -q -j '/tmp/$fzip' "."'$pout/$file' ".
				"'$pout/$fold' '$pout/$fback' '$pout/$ftxt' '$pout/$fpdf'";
			$res=doit("$cmd 2>/dev/null 0>/dev/null");
			addJobFile($dbh,$db,$pw,"/tmp/$fzip",$bas,0) if $res==0;
		}
	}
	doit("rm -f $pout/*");
	doit("rmdir $pout");
	if ($res==0) {
		logit("$mode project $file added to $db");
	} else {
		logit("error creating $mode $file for $db");
	}
}

sub getHash {
	my ($fin,$pin,$mode) = @_;
	my $file = "$pin/$fin";
	my $hash = "";
	my $rest = "";
	if (-e "$file") {
		if ($mode eq SHOTCUT) {
			my $mb = 1000000; #my $mb = 1048576 => not 1 MB, only 1000000 bytes
			my $mb2 = $mb*2;
			my $buf = "";
			my $buf1 = "";
			my $buf2 = "";
			eval { open (FH, '< :raw', "$file") or die $!; };
			my $totsize = -s "$file";
			if ($totsize > $mb2) {
			  my $lastmb = $totsize - $mb;
			  my $first = sysread( FH, $buf1, $mb);
			  seek FH, $lastmb, SEEK_SET;
				my $last = sysread( FH, $buf2, $mb);
			  $buf = $buf1.$buf2;
			} else {
			  my $first = sysread( FH, $buf, $totsize);
			}
			close(FH);
			$hash = md5_hex "$buf";
		} else {
			my $cmd = "md5sum \"$pin/$fin\"";
			my $check = `$cmd`;
			($hash,$rest) = split(/\s/,$check);
		}
		return $hash;
	}
	return "";
}

sub checkValue {
	my ($pouts,$check) = @_;
	my $found = 0;
	my @found = grep(/^$check/,@$pouts);
	$found=1 if $found[0] ne "";
	return $found;
}
	
sub doit {
	my ($cmd) = @_;
	my $res=system($cmd);
	logit("$res=>$cmd");
	return $res;
}

sub getFile {
	my ($file,$in,$out) = @_;
	$file =~ s/(.*?)(\.$in)$/$1.$out/;
	return $file;
}

sub addJobFile {
	my ($dbh,$db,$pw,$fname,$bas,$dontkill) = @_;
  my $sql = "insert into archivista.jobs set " .
		"job='OFFICE',status=110,host='localhost',db=".$dbh->quote($db).",".
		"user='root',pwd=".$dbh->quote($pw);
	$dbh->do($sql);
	# now get back the last row number
	$sql="select last_insert_id()";
	my @row=$dbh->selectrow_array($sql);
	my $id=$row[0];
	if ($id>0) {
		$sql = "insert into archivista.jobs_data set jid ='$id',".
			"param='FILENAME',value=".$dbh->quote($fname);
		$dbh->do($sql);
		if ($dontkill==1) {
			$sql="insert into archivista.jobs_data ".
				"set jid='$id',param='DONTKILL',value='1'";
			$dbh->do($sql);
		}
		$sql="insert into archivista.jobs_data set jid='$id',param='KFILE',".
			"value=".$dbh->quote($bas);
		$dbh->do($sql);
		$sql="update archivista.jobs set status=100 where id=$id";
		$dbh->do($sql);
	}
}

