#!/usr/bin/perl

=head1 Program to start/stop master/slave configuration

masterslave.pl ipslave start|stop

=cut

use strict;
use Archivista::Config;
my $config = Archivista::Config->new;
my $pw = $config->get("MYSQL_PWD");
undef $config;
use lib qw(/home/cvs/archivista/jobs);
use AVJobs;

my $slaveip = shift; # if AVMultimedia based, only start/stop
my $mode = shift;
my $slaveuser = shift;
$slaveuser = "slave" if $slaveuser eq "";
my $slavepw = shift;
$slavepw = "slave" if $slavepw eq "";
my $ip = `ifconfig vmbr0`;
$ip =~ s/^(.*?)([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})(.*)$/$2/s;
if ($mode eq "" && ($slaveip eq "start" || $slaveip eq "stop")) {
	my $cmdline = `cat /proc/cmdline`;
	my $master = "";
	$cmdline =~ /(.*?)(master)(.*?)(\s)(.*)/;
	if ($2 eq "master" && $4 eq " ") {
		my $master0 = $3;
		my @parts = split(/\./,$master0);
		shift @parts;
		$master = join('.',@parts);
	}
	my $slave = "";
	$cmdline =~ /(.*?)(slave)(.*?)(\s)(.*)/;
	if ($2 eq "slave" && $4 eq " ") {
		my $slave0 = $3;
		my @parts = split(/\./,$slave0);
		shift @parts;
		$slave = join('.',@parts);
	}
	if ($master ne "" && $master eq $ip && $slave ne "") {
		$mode = $slaveip;
		$slaveip = $slave;
	}
}
if ($slaveip ne "" && ($mode eq "start" || $mode eq "stop")) {
  my $dbh = MySQLOpen();
  if ($dbh) {
    if (HostIsSlave($dbh)==0) {
	    my $ssh = "sshpass -p $pw";
	    my $opt = "-o UserKnownHostsFile=/dev/null ".
	              "-o StrictHostKeyChecking=no";
			my $ssh1 = "$ssh ssh $opt \"root\"\@\"$slaveip\" ";
		  my $res = 0;
		  if ($mode eq "start") {
			  $res=master($dbh,$ip,$pw,$slaveip,$slaveuser,$slavepw,$ssh,$ssh1,$opt);
			} elsif ($mode eq "stop") {
			  $res=masterstop($pw,$ssh1);
			}
			logit("result:$res");
	  }
  }
} else {
  print "$0 slaveip start|stop [slaveuser slavepw]\n";
	die;
}

sub master {
  my ($dbh,$ip,$rootpw,$slaveip,$slaveuser,$slavepw,$ssh,$ssh1,$opt) = @_;
  my $sql = "grant replication slave on *.* to '$slaveuser'\@'$slaveip' ".
            "identified by '$slavepw'";
  $dbh->do($sql);
  $sql = "flush privileges";
  $dbh->do($sql);
	mastertablelock($dbh);
  my ($file,$pos) = $dbh->selectrow_array("show master status");
	$pos=0 if $file eq "";
	my $cmd = "$ssh1 /etc/init.d/mysql stop";
	my $res = 0;
	for (my $c1=1;$c1<=5;$c1++) {
		$res = doit($cmd,$rootpw);
		last if $res==0;
		sleep $c1;
	}
	if ($res==0) {
	  $cmd = "$ssh rsync -e 'ssh $opt' -arv --delete --exclude '*-bin.*' ".
		       "--exclude '*.info' --delete-excluded -P ". # --inplace
	         "/home/data/archivista/mysql $slaveip:/home/data/archivista/";
	  $res = doit($cmd,$rootpw);
		if ($res==0) {
			if (isMariaDB()==0) {
			  $cmd = "$ssh1 \"sed -i -e \\\"s/.*server-id.*/server-id = 2/\\\" ".
					"-e \\\"s/.*master-host.*\$/master-host = $ip/\\\" ".
					"-e \\\"s/.*master-user.*\$/master-user = $slaveuser/\\\" ".
					"-e \\\"s/.*master-password.*\$/master-password = $slavepw/\\\" ".
					" /etc/mysql/my.cnf\"";
			} else {
		  	$cmd = "$ssh1 \"sed -i -e \\\"s/.*server-id.*/server-id = 2/\\\" ".
					" /etc/mysql/my.cnf\"";
			}
			$res = doit($cmd,$rootpw);
			if ($res==0) {
			  sleep 2;
			  $cmd = "$ssh1 /usr/bin/mysqld_safe --skip-slave-start &";
				$res = doit($cmd,$rootpw);
			  if ($res==0) {
				  sleep 2;
			    $cmd = "$ssh1 mysql -uroot -p$rootpw -hlocalhost <<-EOT\n".
                 "CHANGE MASTER TO MASTER_HOST='$ip',".
                 "MASTER_USER='$slaveuser',MASTER_PASSWORD='$slavepw',".
								 "MASTER_LOG_FILE='$file',MASTER_LOG_POS=$pos;".
								 "START SLAVE;\nEOT";
					$res = doit($cmd,$rootpw);
					if ($res==0) {
            sleep 2;
			      $cmd = "$ssh1 /etc/init.d/mysql stop";
						$res = doit($cmd,$rootpw);
						if ($res==0) {
							logit("starting database server on slave");
							print "starting database server on slave\n";
			        $cmd = "$ssh1 \"/etc/init.d/mysql start 2>/dev/null >/dev/null\"";
						  $res = doit($cmd,$rootpw);
						}
					}
				}
			}
		}
	}
	$sql = "unlock tables";
	$dbh->do($sql);
	return $res;
}

sub isMariaDB {
	my $dbnew=0;
	$dbnew=1 if `/usr/bin/dpkg --list | grep 'maria'` ne "";
	return $dbnew;
}

sub mastertablelock {
  my ($dbh) = @_;
	my $all = 0;
	my $sql = "";
	my @databases = ();
	if ($all==0) {
    my $prow = $dbh->selectall_arrayref("show databases");
	  foreach my $prow1 (@$prow) {
	    push @databases,$$prow1[0];
	  }
	  my $sql1 = "";
		foreach my $database (@databases) {
		  next if $database eq "information_schema";
		  next if $database eq "performance_schema";
		  $sql = "use $database";
			$dbh->do($sql);
      $prow = $dbh->selectall_arrayref("show tables");
	    foreach my $prow1 (@$prow) {
	      my $table = $$prow1[0];
			  my $pos = index($table,"log");
				if ($pos>=0 && $database eq "mysql") {
				  # nothing
				} else {
			    $pos = index($table,"sessionweb");
				  if ($pos==0 && ($database eq "archivista" || 
													$database eq "avsession")) {
				    # nothing
					} else {
			      $sql1 .= ", " if $sql1 ne "";
			      $sql1 .= "$database.$table read";
					}
				}
			}
		}
		if ($sql1 ne "") {
		  $sql = "lock table $sql1" if $sql1 ne "";
	  }
	} else {
    $sql = "FLUSH TABLES WITH READ LOCK";
	}
	my $res = $dbh->do($sql);
	return $res;
}

sub masterstop {
  my ($rootpw,$ssh1) = @_;
	my $cmd1 = "SLAVE STOP";
	$cmd1 = "STOP SLAVE" if isMariaDB()==1;
  my $cmd = "$ssh1 mysql -uroot -p$rootpw -hlocalhost <<-EOT\n".
            "$cmd1;RESET SLAVE\nEOT";
	my $res = doit($cmd,$rootpw);
	if ($res==0) {
		if (isMariaDB()==1) {
	  	$cmd = "$ssh1 \"sed -i -e \\\"s/.*server-id.*/server-id = 1/\\\" ".
		  	" /etc/mysql/my.cnf\"";
		} else {
	  	$cmd = "$ssh1 \"sed -i -e \\\"s/.*server-id.*/server-id = 1/\\\" ".
		  	"-e \\\"s/.*\\\(master-.*\\\)/# \\1/\\\" /etc/mysql/my.cnf\"";
		}
		$res = doit($cmd,$rootpw);
	}
	if ($res==0) {
    sleep 2;
	  $cmd = "$ssh1 /etc/init.d/mysql stop";
	  $res = doit($cmd,$rootpw);
		if ($res==0) {
			logit("starting database server on slave");
			print "starting database server on slave\n";
		  $cmd = "$ssh1 \"/etc/init.d/mysql start 2>/dev/null >/dev/null\"";
		  $res = doit($cmd,$rootpw);
		}
	}
	return $res;
}

sub doit {
  my ($cmd,$rootpw) = @_;
	my $res=system($cmd);
	$cmd=~s/$rootpw/*****/g;
	logit("$res=>$cmd");
	return $res;
}

