#!/usr/bin/perl

use strict;
use File::Copy;
my %val;
$val{cups} = "/var/spool/cups-pdf/ANONYMOUS";
my $pf="/home/data/archivista/ftp";
$val{log} = "/home/data/archivista/av.log";
$val{webconfig} = "/etc/webconfig";
$val{dirs}="$pf/office $pf/pdf $pf/tiff $pf/tosca $pf/lbs $pf/axapta ".
	"$pf/xerox $pf/scanbox $pf/go $pf/lists $val{webconfig} $val{cups}";
$val{dirsep} = "/";
initFolders(); # add all needed folders
my $wt=`find $val{dirs} -maxdepth 2 -type d -printf '%p '`;
print "Watching: $wt \n";
open(INOTIFY, "inotifywait -r -m -e create -e close_write -e moved_to $wt |");
my $quit_loop=1;
while (<INOTIFY>) {
  # parse the inotify event output, that is in the style:
  # /tmp/, CLOSE_WRITE CLOSE a
  # we remove the middle part and insert a / separator
  my $name = $_;
  chomp $name; 
	#logit($name);
	#$name =~ s/ [^ ]*,[^ ]* //;
	$name =~ s/( MOVED\_TO )//;
	$name =~ s/( CLOSE_WRITE,CLOSE )//;
  # during testing removed directories generate some IGNORED line
  next if /IGNORED/;
  if (/CREATE/) {
    print "CREATE\n";
    if (/CREATE ISDIR/) {
      # not interested in subdirs
      print "ISDIR\n";
      #$watch="$watch $name";
      last;
    } else {
      next;
    }
  } else {
    # what kind of file?
    my $job = "";
    if ($name =~ /\/ftp\/temp\//) {
      next;
    } elsif ($name =~ /\/ftp\/office\//) {
      $job="OFFICE";
    } elsif ($name =~ /\/ftp\/lbs\//) {
      $job="LBS";
    } elsif ($name =~ /\/cups-pdf\//) {
      $job="CUPS";
    } elsif ($name =~ /\/ftp\/tiff\//) {
      $job="TIFF";
    } elsif ($name =~ /\/ftp\/pdf\//) {
      $job="PDF";
    } elsif ($name =~ /\/xerox\//) {
      if ($name =~ /\.XST/) {
        $job="XEROX";
      } else {
        next;
  		}
    } elsif ($name =~ /\/tosca\//) {
      if ($name =~ /\.txt/) {
        $job="TOSCA";
      } else {
        next;
	  	}
    } elsif ($name =~ /\/axapta\//) {
      if ($name =~ /\.TXT/) {
        $job="AXAPTA";
      } else {
        next;
			}
    } elsif ($name =~ /\/scanbox\//) {
      if ($name =~ /(\-1\-0\.pnm)$/ ||
		      $name =~ /(\-1\-0\.jpg)$/ ||
		      $name =~ /(\-1\-0\.pnm\.gz)$/) {
          $job="SCANBOX";
			}
      if ($name =~ /(\.pnm)$/ ||
		    $name =~ /(\.jpg)$/ ||
		    $name =~ /(\.pnm\.gz)$/) {
				my @parts = split(/\./,$name);
				my $name1 = $parts[0].'.txt';
		    system("touch $name1");
				$name = $name1;
			}
	  } elsif ($name =~ /\/webconfig\//) {
		  $job="WEBCONFIG";
    } elsif ($name =~ /\/ftp\/lists\//) {
		  $job="FIELDLISTS";
    } elsif ($name =~ /\/ftp\/go\//) {
      if ($name =~ /\.txt/) {
        $job="FTP";
      } elsif ($name =~ /\.ftp/) {
        $job="FTPSLOW";
      } else {
        next;
      }
    }
		if ($job ne "WEBCONFIG") {
      saveJob($job,$name,\%val); # save to job for later processing
		} else {
		  system("/usr/bin/perl /home/cvs/archivista/jobs/webconfig.pl $name &");
		}
  }
  print "EOL\n";
}
# TODO: find out how to do in perl, or not use open (" |")
system ("killall inotifywait");
close INOTIFY;
if ($val{dbh} ne "") {
  $val{dbh}->disconnect();
}






=head1 saveJob

Save the file name and all informations to the job table

=cut

sub saveJob {
  my $job = shift;
  my $name = shift;
	my $pval = shift;
  return if $job eq "";
	logit($name);
  if ($$pval{host} eq "") {
	  eval {
      require DBI;
      require Archivista::Config; # needed for the passwords
      my $config = Archivista::Config->new;
      $$pval{host} = $config->get("MYSQL_HOST");
      $$pval{db} = $config->get("MYSQL_DB");
      $$pval{user} = $config->get("MYSQL_UID");
      $$pval{pw} = $config->get("MYSQL_PWD");
      undef $config;
      if (MySQLOpen($pval)) {
        updateJobsTable($$pval{dbh}); # update the job table
			}
		}
	}
  $$pval{dbh}->disconnect();
	if (MySQLOpen($pval)) {
    my $sql = "insert into jobs set " .
            "job = '$job', status = 110"."," .
            "host = ".$val{dbh}->quote($val{host})."," .
            "db = ".$val{dbh}->quote($val{db})."," .
      	    "user = ".$val{dbh}->quote($val{user})."," .
    	      "pwd = ".$val{dbh}->quote($val{pw}).";";
    $val{dbh}->do($sql);
    # now get back the last row number
    $sql="select last_insert_id()";
    my @row=$val{dbh}->selectrow_array($sql);
    my $id=$row[0];
    if ($id>0) {
      $sql = "insert into jobs_data set " .
           "jid = '$id'," .
           "param = 'FILENAME'," .
           "value = ".$val{dbh}->quote($name).";";
      $val{dbh}->do($sql);
			$sql = "update jobs set status=100 where id=$id";
			$val{dbh}->do($sql);
    }
	}
}




=head2 updateJobsTable($dbh);

Update the jobs table for all types of imports

=cut

sub updateJobsTable {
  my $dbh = shift;
  # update jobs table!
  my $sql = "alter table jobs change job job " .
    "enum('OCRRECPAGE','SANE','SCANBOX','WEB','WEBCONF','OFFICE','CUPS',".
		"'TEMPPDF','FTP','MAIL','JOBS','PDF','TIFF','LBS','TOSCA',".
		"'AXAPTA','XEROX','FTPSLOW','EXPORT','LASTDOC','FIELDLISTS') " .
    "NOT NULL default 'SANE'";
  $dbh->do($sql);
}



=head2 initFolders

Init all needed folders

=cut

sub initFolders {
  # for now, the directories must exist, thus create them if necessary
	my $webconfig = "/etc/webconfig";
  system("mkdir $webconfig") if !-e $webconfig;
  system ("mkdir -p /var/spool/cups-pdf/ANONYMOUS");
  system ("chmod 777 /var/spool/cups-pdf/ANONYMOUS");
  system ("chown nobody:nogroup /var/spool/cups-pdf/ANONYMOUS");
	my @folders = `find /home/data/archivista/ftp/office -type d -name '*'`;
	foreach my $folder (@folders) {
		chomp $folder;
		my $out = $folder;
		$out =~ s/\&/and/g;
		$out =~ s/\s/_/g;
		$out =~ s/,/_/g;
		$out =~ s/'/_/g;
		if (!-e "$out" && "$folder" ne "$out") {
			my $cmd = "mv \"$folder\" \"$out\"";
			my $res=system($cmd);
			logit("$res=>$cmd");
		}
	}
}






=head2 $dbh=MySQLOpen(%$val)

Open a MySQL connection and gives back a db handler

=cut

sub MySQLOpen {
  my $pval = shift;
  my ($ds);
  $ds = "DBI:mysql:host=$$pval{host};database=$$pval{db}";
  $$pval{dbh}=DBI->connect($ds,$$pval{user},$$pval{pw});
  return $$pval{dbh};
}





=head1 logit($message)

Writes a message to a log file

=cut

sub logit {
  my $stamp   = TimeStamp();
  my $message = shift;
  # $log file name comes from outside
  my @parts = split($val{dirsep},$0);
  my $prg = pop @parts;
  open( FOUT, ">>$val{log}" );
  binmode(FOUT);
  my $logtext = $prg . " " . $stamp . " " . $message . "\n";
  print FOUT $logtext;
  close(FOUT);
}






=head2 $stamp=TimeStamp 

Actual date/time stamp (20040323130556)

=cut

sub TimeStamp {
  my @t = localtime( time() );
  my ( $stamp, $y, $m, $d, $h, $mi, $s );
  $y     = $t[5] + 1900;
  $m     = $t[4] + 1;
  $m     = sprintf( "%02d", $m );
  $d     = sprintf( "%02d", $t[3] );
  $h     = sprintf( "%02d", $t[2] );
  $mi    = sprintf( "%02d", $t[1] );
  $s     = sprintf( "%02d", $t[0] );
  $stamp = $y . $m . $d . $h . $mi . $s;
  return $stamp;
}




