#!/usr/bin/perl

use lib qw(/home/cvs/archivista/jobs);
use AVJobs;
my $db = "avomdb";
my $path = "/home/archivista/data/omdb";
my $err = 0;
my $initdb = shift;
mkdir $path if !-d $path;
my @files = ();
my $dbh=MySQLOpen();
if (checkdb($dbh,$db,$initdb)==0) {
	logit("database $db seems to be ready");
	$err=getFiles($path,\@files);
	foreach my $file (@files) {
		$err=tableit($dbh,$path,$file) if $err==0;
	}
	system("chown -Rf archivista.archivista $path") if $err==0;
}
logit("omdb job for database $db ended");

sub getFiles {
	my ($path,$pfiles) = @_;
	my $hfile = "$path/all.html";
	my $site = "https://www.omdb.org";
	my $cont = "";
	my $cmd = "";
	my $err = 0;
	if (!-e $hfile) {
		$cmd = "curl $site/de/ch/content/Help:DataDownload --output $hfile";
		$err=doit($cmd);
	}
	if ($err==0 && -e $hfile) {
		readFile2($hfile,\$cont,0);
		while ($cont =~ /(https:\/\/)(.*?)(\.csv\.bz2)/g) {
			my $file = "$1$2$3$4";
			my @parts = split(/\//,$file);
			my $fin = pop @parts;
			@parts = split(/\./,$fin);
			pop @parts;
			my $fout = join('.',@parts);
			if (!-e "$path/$fout") {
				$cmd = "curl $file --output \"$path/$fin\"";
				$err=doit($cmd) if $err==0;
				$cmd = "cd $path;bunzip2 $path/$fin";
				$err=doit($cmd) if $err==0;
				if ($err==0) {
					push @$pfiles,$fout;
					unlinkFile("$path/$fin");
				}
			} else {
				push @$pfiles,$fout;
			}
			last if $err>0;
		}
	}
	return $err;
}

sub doit {
	my ($cmd) = @_;
	my $res=system($cmd);
	logit("$res=>$cmd");
	return $res;
}
	
sub tableit {
	my ($dbh,$path,$file) = @_;
	logit("process $file");
	my $cont = "";
	my $err = 0;
	my ($table,$ext) = split(/\./,$file);
	my $sql = "show tables like '$table'";
	my @row = $dbh->selectrow_array($sql);
	if ($row[0] eq "") {
		readFile2("$path/$file",\$cont);
		$cont =~ s/(\"\\\n)/"/mg;
		$cont =~ s/(\\\n)//mg;
		$cont = fromUTF8($cont);
		my @lines = split(/\n/,$cont);
		my $title = shift @lines;
		$title =~ s/^(\")//g;
		$title =~ s/(\")$//g;
		my @fields = split(/\"\,\"/,$title);
		my $sql = "create table if not exists $table (";
		my $first=1;
		foreach my $field (@fields) {
			$field = "key0" if $field eq "key";
			my $note=0;
			if ($first==1) {
				$first=0;
			} else {
				$sql.=", ";
			}
			if ($field eq "date") {
			 	$sql.="$field datetime default NULL";
			} elsif ($field eq "id" || $field =~ /(\_id)/) {
				$sql.="$field int(11) not null default '0'";
			} elsif ($field eq "abstract") {
				$sql.="$field text";
				$note=1;
			} else {
				my $lang=250;
				if (($table eq "all_people_aliases" || $table eq "all_movie_aliases" ||
					$table eq "all_movie_aliases_iso") and $field eq "name") {
					$lang=1000;
				} else {
					$lang=1000 if $field eq "role" && $table eq "all_casts";
					$lang=1000 if $field eq "source" && $table eq "image_licenses";
				}
				$sql.="$field varchar($lang) not null default ''";
			}
			$sql.=",key $field"."I ($field)" if $note==0;
		}
		$sql.=")";
		$err=mysqldo($dbh,$sql);
		if ($err==0) {
			$sql = "select count(*) from $table";
			my @row = $dbh->selectrow_array($sql);
			if ($row[0]==0) {
				my $c0=0;
				foreach my $line (@lines) {
					$line =~ s/\\N/""/g;
					$line =~ s/^(\")//g;
					$line =~ s/(\")$//g;
					my @values = split(/\"\,\"/,$line);
					$sql = "insert into $table set ";
					my $c=0;
					foreach my $field (@fields) {
						$field = "key0" if $field eq "key";
						my $val = $values[$c];
						if ($val ne "") {
							$sql .= "," if$c>0;
							$val .= " 00:00:00" if $field eq "date";
							$val =~ s/(\\\")/"/g;
							$sql .= "$field=".$dbh->quote($val);
						}
						$c++;
					}
					$err=mysqldo($dbh,$sql);
					if ($err!=0) {
						logit("$line at $c0");
						last;
					}
					last if $err!=0;
				}
				$c0++;
			}
		} else {
			logit("table $table already done");
		}
	}
	return $err;
}

sub checkdb {
	my ($dbh,$db,$initdb) = @_;
	my $err=0;
	if ($dbh) {
		if (HostIsSlave($dbh)==0) {
			logit("db server ready");
			my $sql = "show databases like '$db'";
			my @row = $dbh->selectrow_array($sql);
			my $dbcheck = $row[0];
			if ($dbcheck ne "" && $initdb eq "initdb") {
				logit("remove db $db");
				my $sql1 = "drop database $db";
				$err=mysqldo($dbh,$sql1);
				$dbcheck="" if $err==0;
			}
			if ($dbcheck eq "") {
				logit("create db $db");
				my $sql1 = "create database $db";
				$err=mysqldo($dbh,$sql1) if $err==0;
			}
			if ($err==0) {
				@row = $dbh->selectrow_array($sql);
				if ($row[0] eq $db) {
					logit("db $db ready for processing");
					$sql = "use $db";
					$err=mysqldo($dbh,$sql);
				}
			}
		}
	}
	return $err;
}

sub mysqldo {
	my $err=0;
	my ($dbh,$sql) = @_;
	$dbh->do($sql);
	if ($dbh->errstr) {
		$err=1;
		my $msg = $dbh->errstr;
		logit("$msg=>$sql");
	}
	return $err;
}
