#!/usr/bin/perl

=head1 xerox2axis.pl

Moves a given pdf file to the ftp folder and creates an axis file

=cut

use strict;
use lib qw(/home/cvs/archivista/jobs);
use AVJobs;
use File::Copy;

my $file1 = shift; # get the file name (XST)
my $filenoext = $file1;
$filenoext =~ s/(.*)(\.XST$)/$1/;
my $file = $filenoext.'.pdf';
my $file2 = $filenoext.'.PDF';
logit("$file--$file1--$file2");
my $dir = "/home/data/archivista/ftp/go";
my $ds = '/'; # Unix=/, Windows=\
if (-e $file2) { $file = $file2; }
if (-e $file) {
  my ($fname,$db,$scandef) = getFileDB($file);
  my $fout = "$dir$ds$fname";
  foreach (my $c=0;$c<100;$c++) {
	  last if (!-e $fout);
  	$fname=TimeStamp().'.pdf';
    $fout = "$dir$ds$fname";
  }
  if (!-e $fout) {
    move($file,$fout);
  	$file=$fname;
    my $pinfo = getInfos("$dir$ds",$file,$db,$scandef);
		my $pxrxinfo = getXRXFields($file1);
		my @fields;
		foreach my $key (keys %{$pxrxinfo}) {
		  my $field_name = $pxrxinfo->{$key}->{"MetaDataFieldName"};
		  my $field_value = $pxrxinfo->{$key}->{"MetaDataValue"};
			if ($field_name eq "UserID" || $field_name eq "Profiles") {
			  # get user and scan def (if available) directly 
			  $pinfo->{$field_name}=$field_value;
			} else {
			  $field_name = "Titel" if $field_name eq "Title";
			  $field_value=escape($field_value);
			  $pinfo->{"Fields"} .= ":" if $pinfo->{'Fields'} ne "";
			  $pinfo->{"Fields"} .= "$field_name=$field_value";
			}
		}
    # create axis-file from the infos and delete the csv-file
    createAXIS($pinfo,$fout);
    logit("$file");
	}
  unlink($file1) if -e $file1;
}



sub getField {
  my $file = shift;
	my $field = shift;
	my $text = "";
	my $infos = getXRXFields($file);
	foreach my $key (keys %{$infos}) {
	    if ($infos->{$key}->{"MetaDataFieldName"} eq $field) {
			  return $infos->{$key}->{"MetaDataValue"};
			}
	}
}



=head2 \%infos = getXRXFields($file)

Returns the Infos in the xrx_dscrpt_metadata section from the Xerox File.
$infos->{1}->{MetaDataFieldName}
$infos->{2}->{MetaDataValue}

=cut

sub getXRXFields {
  my $file = shift;
	my $text = "";
  my %infos;
  readFile($file,\$text);
  if ($text =~ /\[description xrx_dscrpt_metadata\](.+)end/s) {
    my $fields = $1;
    while ( $fields =~ /(\s\d\{.*?\}){1}/s) {
      $fields =~ s/(\s(\d)\{(.*?)\}){1}//s;
      my $id = $2;
      my $info = $3;
      my %tmp;
      foreach my $line (split("\n",$info)) {
			  if ($line =~ /^\s+\*?\s\w+\s(\w+)\s=\s(.+);$/) {
				  my $name = $1;
					my $value = $2;
          # Remove Quotes
          $value =~ s/^"//; 
					$value =~ s/"$//;
          $tmp{$name} = $value;
        }
      }
      $infos{$id} = \%tmp;
    }
  } else {
    %infos = {};
  }
  return \%infos;
}

