
package PL::BarcodeSettings;
use strict;

sub new {
  my ($cls,$archiveO) = @_; # Object of Archivista::BL::Archive (APCL)
	my $self = {};
  bless $self, $cls;
	my @fields = ("name","first_barcode_type","second_barcode_type",
								"orientation","check_character","vertical_stretch",
								"single_pages","afterpage","engine","script");
  $self->{'archiveO'} = $archiveO;
	$self->{'field_list'} = \@fields;
  return $self;
}

sub fields {
  my ($self,$cgiO) = @_;
	my $archiveO = $self->archive;
  my $langO = $self->archive->lang;
  my $pabarcodePosition = $archiveO->barcodePosition("ARRAY");
	my $phbarcodePosition = $archiveO->barcodePosition("HASH");
  my $pabarcodeTypes = $archiveO->barcodeTypes("ARRAY");
	my $phbarcodeTypes = $archiveO->barcodeTypes("HASH");
	my $pabarcodeOrientation = $archiveO->barcodeOrientation("ARRAY");
	my $phbarcodeOrientation = $archiveO->barcodeOrientation("HASH");
  my $pabarcodeVerticalStretch = $archiveO->barcodeVerticalStretch("ARRAY");
	my $phbarcodeVerticalStretch = $archiveO->barcodeVerticalStretch("HASH");
	# Add automatic to barcodeTypes
	unshift @$pabarcodeTypes, 0;
	$$phbarcodeTypes{0} = $langO->string("AUTOMATIC");
	# Add automatic to barcodeOrientation
	unshift @$pabarcodeOrientation, 0;
	$$phbarcodeOrientation{0} = $langO->string("AUTOMATIC");
  my (%fields);	
  $fields{'list'} = $self->{'field_list'};
  $fields{'name'}{'label'} = $langO->string("NAME");
	$fields{'name'}{'name'} = "bc_name";
	$fields{'name'}{'type'} = "textfield";
	$fields{'name'}{'update'} = 1;
	$fields{'manual'}{'label'} = $langO->string("BARCODE_POSITION");
	$fields{'manual'}{'name'} = "bc_manual";
	$fields{'manual'}{'type'} = "select";
	$fields{'manual'}{'array_values'} = $pabarcodePosition;
	$fields{'manual'}{'hash_values'} = $phbarcodePosition;
  $fields{'manual'}{'update'} = 1;	
  $fields{'left'}{'label'} = $langO->string("LEFT")." (mm)";
	$fields{'left'}{'name'} = "bc_left";
	$fields{'left'}{'type'} = "textfield";
	$fields{'left'}{'update'} = 1;
	$fields{'top'}{'label'} = $langO->string("TOP")." (mm)";
	$fields{'top'}{'name'} = "bc_top";
	$fields{'top'}{'type'} = "textfield";
	$fields{'top'}{'update'} = 1;
	$fields{'width'}{'label'} = $langO->string("WIDTH")." (mm)";
	$fields{'width'}{'name'} = "bc_width";
	$fields{'width'}{'type'} = "textfield";
	$fields{'width'}{'update'} = 1;
	$fields{'height'}{'label'} = $langO->string("HEIGHT")." (mm)";
	$fields{'height'}{'name'} = "bc_height";
	$fields{'height'}{'type'} = "textfield";
	$fields{'height'}{'update'} = 1;
	$fields{'tolerance'}{'label'} = $langO->string("TOLERANCE_VALUE")." (mm)";
	$fields{'tolerance'}{'name'} = "bc_tolerance";
	$fields{'tolerance'}{'type'} = "textfield";
	$fields{'tolerance'}{'update'} = 1;
	$fields{'trials'}{'label'} = $langO->string("TRIALS");
	$fields{'trials'}{'name'} = "bc_trials";
	$fields{'trials'}{'type'} = "textfield";
	$fields{'trials'}{'update'} = 1;
  $fields{'first_barcode_type'}{'label'} = $langO->string("FIRST_BARCODE_TYPE");
	$fields{'first_barcode_type'}{'name'} = "first_bc_type";
	$fields{'first_barcode_type'}{'type'} = "select";
	$fields{'first_barcode_type'}{'array_values'} = $pabarcodeTypes;
	$fields{'first_barcode_type'}{'hash_values'} = $phbarcodeTypes;
	$fields{'first_barcode_type'}{'update'} = 1;
  $fields{'second_barcode_type'}{'label'} = 
		$langO->string("SECOND_BARCODE_TYPE");
	$fields{'second_barcode_type'}{'name'} = "second_bc_type";
	$fields{'second_barcode_type'}{'type'} = "select";
	$fields{'second_barcode_type'}{'array_values'} = $pabarcodeTypes;
	$fields{'second_barcode_type'}{'hash_values'} = $phbarcodeTypes;
	$fields{'second_barcode_type'}{'update'} = 1;
  $fields{'orientation'}{'label'} = $langO->string("ORIENTATION");
	$fields{'orientation'}{'name'} = "bc_orientation";
	$fields{'orientation'}{'type'} = "select";
	$fields{'orientation'}{'array_values'} = $pabarcodeOrientation;
	$fields{'orientation'}{'hash_values'} = $phbarcodeOrientation;
	$fields{'orientation'}{'update'} = 1;
  $fields{'check_character'}{'label'} = $langO->string("CHECK_CHARACTER");
	$fields{'check_character'}{'name'} = "bc_check_character";
	$fields{'check_character'}{'type'} = "textfield";
  $fields{'check_character'}{'update'} = 1;
	$fields{'vertical_stretch'}{'label'} = $langO->string("VERTICAL_STRETCH");
	$fields{'vertical_stretch'}{'name'} = "bc_vertical_stretch";
	$fields{'vertical_stretch'}{'type'} = "select";
	$fields{'vertical_stretch'}{'array_values'} = $pabarcodeVerticalStretch;
	$fields{'vertical_stretch'}{'hash_values'} = $phbarcodeVerticalStretch;
	$fields{'vertical_stretch'}{'update'} = 1;
  my @aengine = (0,1,2);
  my %hengine = (0=>$langO->string("ENGINE_DEF"),
	                1=>$langO->string("ENGINE_FIRST"),
	                2=>$langO->string("ENGINE_SECOND"));
	$fields{'engine'}{'label'} = $langO->string("ENGINE");
	$fields{'engine'}{'name'} = "bc_engine";
	$fields{'engine'}{'type'} = "select";
	$fields{'engine'}{'array_values'} = \@aengine;
	$fields{'engine'}{'hash_values'} = \%hengine;
	$fields{'engine'}{'update'} = 1;
  $fields{'script'}{'label'} = $langO->string("SCRIPT");
	$fields{'script'}{'name'} = "bc_script";
	$fields{'script'}{'type'} = "textfield";
  $fields{'script'}{'update'} = 1;
	$fields{'single_pages'}{'label'} = $langO->string("SINGLE_PAGES");
	$fields{'single_pages'}{'name'} = "bc_single_pages";
	$fields{'single_pages'}{'type'} = "checkbox";
  $fields{'single_pages'}{'update'} = 1;
	$fields{'afterpage'}{'label'} = $langO->string("AFTERPAGE");
	$fields{'afterpage'}{'name'} = "bc_afterpage";
	$fields{'afterpage'}{'type'} = "checkbox";
  $fields{'afterpage'}{'update'} = 1;
	if ($cgiO->param("adm") eq "edit") {
		my $id = $cgiO->param("id");
		if (defined $id) {
		  my $def = $archiveO->parameter("Barcodes")
				->attribute("Inhalt")->barcode($id);
			$fields{'name'}{'value'} = $def->name;
			$fields{'single_pages'}{'value'} = $def->manual;
			$fields{'afterpage'}{'value'} = $def->afterpage;
		  $fields{'left'}{'value'} = $def->left;
			$fields{'top'}{'value'} = $def->top;
			$fields{'width'}{'value'} = $def->width;
			$fields{'height'}{'value'} = $def->height;
			$fields{'tolerance'}{'value'} = $def->tolerance;
			$fields{'trials'}{'value'} = $def->trials;
			$fields{'first_barcode_type'}{'value'} = $def->firstBarcodeType;
			$fields{'second_barcode_type'}{'value'} = $def->secondBarcodeType;
			$fields{'orientation'}{'value'} = $def->orientation;
			$fields{'check_character'}{'value'} = $def->checkCharacter;
			$fields{'vertical_stretch'}{'value'} = $def->verticalStretch;
			$fields{'engine'}{'value'} = $def->engine;
		  $fields{'engine'}{'value'} = 0 if $def->engine<0 || $def->engine>2;
			$fields{'script'}{'value'} = $def->script;
			$fields{'script'}{'value'} = "" if $fields{'script'}{'value'} eq "-1";
		}
	}
  $fields{'displayBackFormButton'} = 1;
	if ($cgiO->param("adm") eq "edit") {
		my $id = $cgiO->param("id");
		if (defined $id) {
		  my $def = $archiveO->parameter("Barcodes")
				->attribute("Inhalt")->barcode($id);
			$fields{'name'}{'value'} = $def->name;
			$fields{'single_pages'}{'value'} = $def->manual;
			$fields{'afterpage'}{'value'} = $def->afterpage;
		  $fields{'left'}{'value'} = $def->left;
			$fields{'top'}{'value'} = $def->top;
			$fields{'width'}{'value'} = $def->width;
			$fields{'height'}{'value'} = $def->height;
			$fields{'tolerance'}{'value'} = $def->tolerance;
			$fields{'trials'}{'value'} = $def->trials;
			$fields{'first_barcode_type'}{'value'} = $def->firstBarcodeType;
			$fields{'second_barcode_type'}{'value'} = $def->secondBarcodeType;
			$fields{'orientation'}{'value'} = $def->orientation;
			$fields{'check_character'}{'value'} = $def->checkCharacter;
			$fields{'vertical_stretch'}{'value'} = $def->verticalStretch;
			$fields{'engine'}{'value'} = $def->engine;
		  $fields{'engine'}{'value'} = 0 if $def->engine<0 || $def->engine>2;
			$fields{'script'}{'value'} = $def->script;
			$fields{'script'}{'value'} = "" if $fields{'script'}{'value'} eq "-1";
		}
	}
  $fields{'displayBackFormButton'} = 1;
  return \%fields;
}

sub save {
  my ($self,$cgiO) = @_;
	my $archiveO = $self->archive;
  my $id = $cgiO->param("id");
  my ($barcodeDef);
	my $name = $cgiO->param("bc_name");
	my $firstBarcodeType = $cgiO->param("first_bc_type");
	my $secondBarcodeType = $cgiO->param("second_bc_type");
	my $orientation = $cgiO->param("bc_orientation");
	my $checkCharacter = $cgiO->param("bc_check_character");
	my $verticalStretch = $cgiO->param("bc_vertical_stretch");
	my $engine = $cgiO->param("bc_engine");
	$engine = 0 if $engine<0 || $engine>2;
	my $script = $cgiO->param("bc_script");
	my $singlepages = $cgiO->param("bc_single_pages");
  $singlepages=0 if $singlepages != 1;
	my $afterpage = $cgiO->param("bc_afterpage");
  $afterpage=0 if $afterpage != 1;
  if ($cgiO->param("submit") eq $self->archive->lang->string("SAVE")) {
		if (defined $id) {
			# Update the scan definition
		  $barcodeDef = $archiveO->parameter("Barcodes")
				->attribute("Inhalt")->barcode($id);
		} else {
			$barcodeDef = $archiveO->parameter("Barcodes")
				->attribute("Inhalt")->barcode;
		}
    $barcodeDef->name($name);
		$barcodeDef->firstBarcodeType($firstBarcodeType);
		$barcodeDef->secondBarcodeType($secondBarcodeType);
		$barcodeDef->orientation($orientation);
		$barcodeDef->checkCharacter($checkCharacter);
		$barcodeDef->verticalStretch($verticalStretch);
		$barcodeDef->engine($engine);
		$barcodeDef->script($script);
		$barcodeDef->manual($singlepages);
		$barcodeDef->afterpage($afterpage);
 		if (defined $id) {
			$barcodeDef->update;
		} else {
			$barcodeDef->add;
	  }
	}
}

sub elements {
  my ($self) = @_;
	my $archiveO = $self->archive;
  my $langO = $archiveO->lang;
	my $pabarcodeDefs = $archiveO->barcodeDefinitions("ARRAY");
  my $phbarcodeDefs = $archiveO->barcodeDefinitions("HASH");
	my (@list,%barcodeDefs);
	# Attributes to display on table
	my @attributes = ("Name");
	foreach my $def (@$pabarcodeDefs) {
		push @list, $def;
		my $name = $$phbarcodeDefs{$def}->name;
		$barcodeDefs{$def}{'Name'} = $name;
		$barcodeDefs{$def}{'delete'} = $$phbarcodeDefs{$def}{'delete'};
		$barcodeDefs{$def}{'update'} = $$phbarcodeDefs{$def}{'update'};
	}
  # Display 'new' Link for new entries
  $barcodeDefs{'new'} = 1;
  $barcodeDefs{'attributes'} = \@attributes;
  $barcodeDefs{'list'} = \@list;
	return \%barcodeDefs;
}

sub delete {
  my ($self,$cgiO) = @_;
	my $archiveO = $self->archive;
	my $id = $cgiO->param("id");
	if (defined $id) {
		$archiveO->parameter("Barcodes")
			->attribute("Inhalt")->barcode($id)->remove;
	}
}

sub archive {
  my ($self) = @_;
	return $self->{'archiveO'};
}

1;

