
package PL::DatabaseAdministration;
use strict;
use constant OCR_NONE => '';
use constant OCR_FR => "Commercial (ArchivistaBox AddOn)";
use constant OCR_TESS => "Tesseract (OpenSource)";
use constant OCR_GNU => "Ocrad 0.25pre2 (OpenSource)";
use constant ENGINE_NONE => '';
use constant ENGINE_MYSQL => "MySQL";
use constant ENGINE_SPHINX => "Sphinx";

sub new {
  my ($cls,$archiveO) = @_; # Object of Archivista::BL::Archive (APCL)
  my $self = {};
  bless $self, $cls;
  my @fields = ("photo_mode","old_icons","hide_exticons","columns",
		"minimalmode","minimalfields","hide_title_field","title_field_width",
    "publish_field_name","filename","versions","versionkey","anz_feldlisten",
		"showeditfield","apilink","apifield","empty_field",
		"ocr","ocr_cluster","pdf","pdf_jpegquality",
		"pdfsingle","barcodes2","cold2","fulltext","show_ocr","empty_field",
		"show_fieldsocr","hide_owner","download_link","save_values",
		"joker_comboany","empty_field","jpeg_quality","jpeg_quality2",
    "preview_scaling","sources_offline","turbo_videos_off","camera_md5",
		"office_images","empty_field","archive_sql","archive_mbyte",
		"archive_files","iso_mbyte","empty_field","access_log","access_hash",
  );
  $self->{'archiveO'}   = $archiveO;
  $self->{'field_list'} = \@fields;
  return $self;
}

sub fields {
  my ($self,$cgiO) = @_;
  my $archiveO = $self->archive;
  my $langO = $self->archive->lang;
  my $pauserDefinedAttributes = $archiveO->userDefinedAttributes;
  # Add en empty element
  unshift @$pauserDefinedAttributes, "";
	my $pThemes = {1=>$langO->string("THEME_OLD_ICONS"),
		0=>$langO->string("THEME_NEW_ICONS"),2=>$langO->string("THEME_NEW_DARK")};
	my $pUsage = {0=>$langO->string("USAGE_DOCS"),
		1=>$langO->string("USAGE_PHOTOS"),2=>$langO->string("USAGE_CINEMA")};
  my (%fields);
  $fields{'list'}                       = $self->{'field_list'};
  $fields{'hide_title_field'}{'label'}  = $langO->string("HIDE_TITLE_FIELD");
  $fields{'hide_title_field'}{'name'}   = "hide_title_field";
  $fields{'hide_title_field'}{'type'}   = "checkbox";
  $fields{'hide_title_field'}{'update'} = 1;
  $fields{'hide_exticons'}{'label'}  = $langO->string("HIDE_EXTICONS");
  $fields{'hide_exticons'}{'name'}   = "hide_exticons";
  $fields{'hide_exticons'}{'type'}   = "checkbox";
  $fields{'hide_exticons'}{'update'} = 1;
  $fields{'old_icons'}{'label'}  = $langO->string("THEME");
  $fields{'old_icons'}{'name'}   = "old_icons";
  $fields{'old_icons'}{'type'}  = "select";
  $fields{'old_icons'}{'array_values'} = $pThemes;
  $fields{'old_icons'}{'update'}       = 1;
  $fields{'columns'}{'label'}  = $langO->string("COLUMNS");
  $fields{'columns'}{'name'}   = "columns";
  $fields{'columns'}{'type'}   = "textfield";
  $fields{'columns'}{'update'} = 1;
  $fields{'minimalmode'}{'label'}  = $langO->string("MINIMALMODE");
  $fields{'minimalmode'}{'name'}   = "minimalmode";
  $fields{'minimalmode'}{'type'}   = "checkbox";
  $fields{'minimalmode'}{'update'} = 1;
  $fields{'minimalfields'}{'label'}  = $langO->string("MINIMALFIELDS");
  $fields{'minimalfields'}{'name'}   = "minimalfields";
  $fields{'minimalfields'}{'type'}   = "checkbox";
  $fields{'minimalfields'}{'update'} = 1;
  $fields{'title_field_width'}{'label'}  = $langO->string("TITLE_FIELD_WIDTH");
  $fields{'title_field_width'}{'name'}   = "title_field_width";
  $fields{'title_field_width'}{'type'}   = "textfield";
  $fields{'title_field_width'}{'update'} = 1;
  $fields{'anz_feldlisten'}{'label'}  = $langO->string("MAX_FIELDLIST");
  $fields{'anz_feldlisten'}{'name'}   = "anz_feldlisten";
  $fields{'anz_feldlisten'}{'type'}   = "textfield";
  $fields{'anz_feldlisten'}{'update'} = 1;
  $fields{'publish_field_name'}{'label'} = $langO->string("PUBLISH_FIELD_NAME");
  $fields{'publish_field_name'}{'name'}  = "publish_field_name";
  $fields{'publish_field_name'}{'type'}  = "select";
  $fields{'publish_field_name'}{'array_values'} = $pauserDefinedAttributes;
  $fields{'publish_field_name'}{'update'}       = 1;
  $fields{'filename'}{'label'} = $langO->string("FILENAME");
  $fields{'filename'}{'name'}  = "filename";
  $fields{'filename'}{'type'}  = "select";
  $fields{'filename'}{'array_values'} = $pauserDefinedAttributes;
  $fields{'filename'}{'update'}       = 1;
  $fields{'versions'}{'label'} = $langO->string("VERSIONS");
  $fields{'versions'}{'name'}  = "versions";
  $fields{'versions'}{'type'}  = "select";
  $fields{'versions'}{'array_values'} = $pauserDefinedAttributes;
  $fields{'versions'}{'update'}       = 1;
  $fields{'versionkey'}{'label'} = $langO->string("VERSIONKEY");
  $fields{'versionkey'}{'name'}  = "versionkey";
  $fields{'versionkey'}{'type'}  = "select";
  $fields{'versionkey'}{'array_values'} = $pauserDefinedAttributes;
  $fields{'versionkey'}{'update'}       = 1;
  $fields{'showeditfield'}{'label'}  = $langO->string("SHOW_EDIT_FIELD");
  $fields{'showeditfield'}{'name'}   = "showeditfield";
  $fields{'showeditfield'}{'type'}   = "checkbox";
  $fields{'showeditfield'}{'update'} = 1;
  $fields{'apilink'}{'label'}  = $langO->string("APILINK");
  $fields{'apilink'}{'name'}   = "apilink";
  $fields{'apilink'}{'type'}   = "textfield";
  $fields{'apilink'}{'update'} = 1;
  $fields{'apifield'}{'label'} = $langO->string("APIFIELD");
  $fields{'apifield'}{'name'}  = "apifield";
  $fields{'apifield'}{'type'}  = "select";
  $fields{'apifield'}{'array_values'} = $pauserDefinedAttributes;
  $fields{'apifield'}{'update'}       = 1;
  my @ocr = (OCR_NONE,OCR_FR,OCR_TESS,OCR_GNU);
  $fields{'ocr'}{'label'}  = $langO->string("OCR_RECOGNITION");
  $fields{'ocr'}{'name'}   = "ocr_recognition";
  $fields{'ocr'}{'type'}   = "select";
  $fields{'ocr'}{'array_values'} = \@ocr;
  $fields{'ocr'}{'update'} = 1;
	$fields{'ocr_cluster'}{'label'}  = $langO->string("OCR_CLUSTER");
  $fields{'ocr_cluster'}{'name'}   = "ocr_cluster";
  $fields{'ocr_cluster'}{'type'}   = "textfield";
  $fields{'ocr_cluster'}{'update'} = 1;
  $fields{'pdf'}{'label'}  = $langO->string("PDF_FILES_ON_OCR");
  $fields{'pdf'}{'name'}   = "pdf_files";
  $fields{'pdf'}{'type'}   = "checkbox";
  $fields{'pdf'}{'update'} = 1;
  $fields{'pdfsingle'}{'label'}  = $langO->string("PDFWHOLEDOC");
  $fields{'pdfsingle'}{'name'}   = "pdfsingle";
  $fields{'pdfsingle'}{'type'}   = "checkbox";
  $fields{'pdfsingle'}{'update'} = 1;
  $fields{'pdf_jpegquality'}{'label'}  = $langO->string("PDF_JPEGQUALITY");
  $fields{'pdf_jpegquality'}{'name'}   = "pdf_jpegquality";
  $fields{'pdf_jpegquality'}{'type'}   = "textfield";
  $fields{'pdf_jpegquality'}{'update'} = 1;
  $fields{'barcodes2'}{'label'}  = $langO->string("BARCODES2");
  $fields{'barcodes2'}{'name'}   = "barcodes2";
  $fields{'barcodes2'}{'type'}   = "checkbox";
  $fields{'barcodes2'}{'update'} = 1;
	$fields{'cold2'}{'label'}  = $langO->string("COLD2");
  $fields{'cold2'}{'name'}   = "cold2";
  $fields{'cold2'}{'type'}   = "checkbox";
  $fields{'cold2'}{'update'} = 1;
  my @engine = (ENGINE_NONE,ENGINE_MYSQL,ENGINE_SPHINX);
  $fields{'fulltext'}{'label'}  = $langO->string("SEARCH_FULLTEXT");
  $fields{'fulltext'}{'name'}   = "fulltext";
  $fields{'fulltext'}{'type'}   = "select";
  $fields{'fulltext'}{'array_values'} = \@engine;
  $fields{'fulltext'}{'update'} = 1;
  $fields{'show_ocr'}{'label'}  = $langO->string("SHOW_OCR");
  $fields{'show_ocr'}{'name'}   = "show_ocr";
  $fields{'show_ocr'}{'type'}   = "checkbox";
  $fields{'show_ocr'}{'update'} = 1;
  $fields{'show_fieldsocr'}{'label'}  = $langO->string("SHOW_FIELDSOCR");
  $fields{'show_fieldsocr'}{'name'}   = "show_fieldsocr";
  $fields{'show_fieldsocr'}{'type'}   = "checkbox";
  $fields{'show_fieldsocr'}{'update'} = 1;
  $fields{'hide_owner'}{'label'}  = $langO->string("HIDE_OWNER");
  $fields{'hide_owner'}{'name'}   = "hide_owner";
  $fields{'hide_owner'}{'type'}   = "checkbox";
  $fields{'hide_owner'}{'update'} = 1;
  $fields{'download_link'}{'label'}  = $langO->string("DOWNLOAD_LINK");
  $fields{'download_link'}{'name'}   = "download_link";
  $fields{'download_link'}{'type'}   = "checkbox";
  $fields{'download_link'}{'update'} = 1;
  $fields{'save_values'}{'label'}  = $langO->string("SAVE_VALUES");
  $fields{'save_values'}{'name'}   = "save_values";
  $fields{'save_values'}{'type'}   = "checkbox";
  $fields{'save_values'}{'update'} = 1;
	$fields{'joker_comboany'}{'label'}  = $langO->string("JOKER_COMBOANY");
  $fields{'joker_comboany'}{'name'}   = "joker_comboany";
  $fields{'joker_comboany'}{'type'}   = "checkbox";
  $fields{'joker_comboany'}{'update'} = 1;
  $fields{'photo_mode'}{'label'}  = $langO->string("USAGE");
  $fields{'photo_mode'}{'name'}   = "photo_mode";
	$fields{'photo_mode'}{'type'}   = "select";
  $fields{'photo_mode'}{'array_values'} = $pUsage;
  $fields{'photo_mode'}{'update'} = 1;
  $fields{'jpeg_quality'}{'label'}  = $langO->string("JPEG_QUALITY");
  $fields{'jpeg_quality'}{'name'}   = "jpeg_quality";
  $fields{'jpeg_quality'}{'type'}   = "textfield";
  $fields{'jpeg_quality'}{'update'} = 1;
  $fields{'jpeg_quality2'}{'label'}  = $langO->string("JPEG_QUALITY2");
  $fields{'jpeg_quality2'}{'name'}   = "jpeg_quality2";
  $fields{'jpeg_quality2'}{'type'}   = "checkbox";
  $fields{'jpeg_quality2'}{'update'} = 1;
  $fields{'preview_scaling'}{'label'}  = $langO->string("PREVIEW_SCALING");
  $fields{'preview_scaling'}{'name'}   = "preview_scaling";
  $fields{'preview_scaling'}{'type'}   = "textfield";
  $fields{'preview_scaling'}{'update'} = 1;
	$fields{'sources_offline'}{'label'}  = $langO->string("SOURCES_OFFLINE");
  $fields{'sources_offline'}{'name'}   = "sources_offline";
  $fields{'sources_offline'}{'type'}   = "checkbox";
  $fields{'sources_offline'}{'update'} = 1;
	$fields{'turbo_videos_off'}{'label'}  = $langO->string("TURBO_VIDEOS");
  $fields{'turbo_videos_off'}{'name'}   = "turbo_videos_off";
  $fields{'turbo_videos_off'}{'type'}   = "checkbox";
  $fields{'turbo_videos_off'}{'update'} = 1;
  $fields{'camera_md5'}{'label'}  = $langO->string("CAMERA_MD5");
  $fields{'camera_md5'}{'name'}   = "camera_md5";
  $fields{'camera_md5'}{'type'}   = "checkbox";
  $fields{'camera_md5'}{'update'} = 1;
  $fields{'office_images'}{'label'}  = $langO->string("OFFICE_IMAGES");
  $fields{'office_images'}{'name'}   = "office_images";
  $fields{'office_images'}{'type'}   = "checkbox";
  $fields{'office_images'}{'update'} = 1;
  $fields{'archive_sql'}{'label'} =
  $langO->string("CONDITION_ARCHIVING") . " (SQL)";
  $fields{'archive_sql'}{'name'}   = "archive_sql";
  $fields{'archive_sql'}{'type'}   = "textfield";
  $fields{'archive_sql'}{'update'} = 1;
  $fields{'archive_mbyte'}{'label'} =
  $langO->string("ARCHIVE_SIZE") . " (MByte)";
  $fields{'archive_mbyte'}{'name'}   = "archive_mbyte";
  $fields{'archive_mbyte'}{'type'}   = "textfield";
  $fields{'archive_mbyte'}{'update'} = 1;
  $fields{'archive_files'}{'label'}  = $langO->string("ARCHIVE_FILES");
  $fields{'archive_files'}{'name'}   = "archive_files";
  $fields{'archive_files'}{'type'}   = "textfield";
  $fields{'archive_files'}{'update'} = 1;
  $fields{'iso_mbyte'}{'label'} = $langO->string("ISO_MBYTE") . " (MByte)";
  $fields{'iso_mbyte'}{'name'}   = "iso_mbyte";
  $fields{'iso_mbyte'}{'type'}   = "textfield";
  $fields{'iso_mbyte'}{'update'} = 1;
  $fields{'access_log'}{'label'}  = $langO->string("ACCESS_LOG");
  $fields{'access_log'}{'name'}   = "access_log";
  $fields{'access_log'}{'type'}   = "checkbox";
  $fields{'access_log'}{'update'} = 1;
  $fields{'access_hash'}{'label'}  = $langO->string("ACCESS_HASH");
  $fields{'access_hash'}{'name'}   = "access_hash";
  $fields{'access_hash'}{'type'}   = "textfield";
  $fields{'access_hash'}{'update'} = 1;
  # In this form we won't the back button
  $fields{'displayBackFormButton'} = 0;
  my $hideTitleField =
    $archiveO->parameter("FeldTitel")->attribute("Inhalt")->value;
  my $hideExtIcons =
    $archiveO->parameter("HideExtIcons")->attribute("Inhalt")->value;
  my $oldIcons =
    $archiveO->parameter("OldIcons")->attribute("Inhalt")->value;
  my $columns =
    $archiveO->parameter("Columns")->attribute("Inhalt")->value;
  my $minimalmode =
    $archiveO->parameter("MinimalMode")->attribute("Inhalt")->value;
  my $minimalfields =
    $archiveO->parameter("MinimalFields")->attribute("Inhalt")->value;
  my $titleFieldWidth =
    $archiveO->parameter("FeldBreite")->attribute("Inhalt")->value;
  my $publishFieldName =
    $archiveO->parameter("PublishField")->attribute("Inhalt")->value;
  my $filename =
    $archiveO->parameter("FILENAME")->attribute("Inhalt")->value;
  my $versions =
    $archiveO->parameter("VERSIONS")->attribute("Inhalt")->value;
  my $versionkey =
    $archiveO->parameter("VERSIONKEY")->attribute("Inhalt")->value;
  my $anz_feldlisten =
    $archiveO->parameter("FeldlistenAnzahl")->attribute("Inhalt")->value;
  my $showeditfield = 
	  $archiveO->parameter("NoRichEdit")->attribute("Inhalt")->value;
  my $apilink =
    $archiveO->parameter("APILINK")->attribute("Inhalt")->value;
  my $apifield =
    $archiveO->parameter("APIFIELD")->attribute("Inhalt")->value;
  my $ocrRecognition =
    $archiveO->parameter("JobsOCRRecognition")->attribute("Inhalt")->value;
	my $ocr_cluster = 
    $archiveO->parameter("OCR_CLUSTER")->attribute("Inhalt")->value;
  my $pdfFiles = $archiveO->parameter("PDFFiles")->attribute("Inhalt")->value;
	my $pdfsingle =
	  $archiveO->parameter("PDFWHOLEDOC")->attribute("Inhalt")->value;
	my $pdf_jpegquality =
	  $archiveO->parameter("PDF_JPEGQUALITY")->attribute("Inhalt")->value;
	my $barcodes2 =
	  $archiveO->parameter("BARCODES2")->attribute("Inhalt")->value;
	my $cold2 =
	  $archiveO->parameter("COLD2")->attribute("Inhalt")->value;
	my $fulltext =
	  $archiveO->parameter("SEARCH_FULLTEXT")->attribute("Inhalt")->value;
	my $show_ocr =
	  $archiveO->parameter("SHOW_OCR")->attribute("Inhalt")->value;
	my $show_fieldsocr =
	  $archiveO->parameter("SHOW_FIELDSOCR")->attribute("Inhalt")->value;
	my $hide_owner =
	  $archiveO->parameter("HIDE_OWNER")->attribute("Inhalt")->value;
  my $photoMode = $archiveO->parameter("PhotoMode")->attribute("Inhalt")->value;
  my $archiveSQL =
    $archiveO->parameter("ArchivSQL")->attribute("Inhalt")->value;
  my $archiveMByte =
    $archiveO->parameter("ArchivMByte")->attribute("Inhalt")->value;
  my $archiveFiles =
    $archiveO->parameter("ArchivDateien")->attribute("Inhalt")->value;
  my $isoMByte =
    $archiveO->parameter("isoMByte")->attribute("Inhalt")->value;
  my $jpegQuality =
    $archiveO->box_parameter("JpegQuality")->attribute("Inhalt")->value;
  my $jpegQuality2 =
    $archiveO->box_parameter("JpegQuality2")->attribute("Inhalt")->value;
  my $prevScaling =
    $archiveO->box_parameter("PrevScaling")->attribute("Inhalt")->value;
	my $sources_offline =
    $archiveO->parameter("SourcesOffline")->attribute("Inhalt")->value;
	my $turbo_videos_off =
    $archiveO->parameter("TurboVideosOff")->attribute("Inhalt")->value;
  my $camera_md5 =
    $archiveO->parameter("CameraMD5")->attribute("Inhalt")->value;
  my $office_images =
    $archiveO->parameter("OfficeImages")->attribute("Inhalt")->value;
	my $accesslog =
	  $archiveO->parameter("ACCESS_LOG")->attribute("Inhalt")->value;
	my $download_link =
	  $archiveO->parameter("DOWNLOAD_LINK")->attribute("Inhalt")->value;
  my $save_values =
	  $archiveO->parameter("SAVE_VALUES")->attribute("Inhalt")->value;
 	my $joker_comboany =
	  $archiveO->parameter("JOKER_COMBOANY")->attribute("Inhalt")->value;
  $fields{'hide_title_field'}{'value'} = $hideTitleField;
  if ( defined $titleFieldWidth ) {
    $fields{'title_field_width'}{'value'} = $titleFieldWidth;
  } else {
    $fields{'title_field_width'}{'value'} = 3000;
  }
	$fields{'hide_exticons'}{'value'} = $hideExtIcons;
	$fields{'old_icons'}{'value'} = $oldIcons;
  if ( defined $columns ) {
    $fields{'columns'}{'value'} = $columns;
  } else {
    $fields{'columns'}{'value'} = 1;
  }
  if ( defined $minimalmode ) {
    $fields{'minimalmode'}{'value'} = $minimalmode;
  } else {
    $fields{'minimalmode'}{'value'} = 0;
  }
  if ( defined $minimalfields ) {
    $fields{'minimalfields'}{'value'} = $minimalfields;
  } else {
    $fields{'minimalfields'}{'value'} = 0;
  }
  if ( defined $publishFieldName ) {
    $fields{'publish_field_name'}{'value'} = $publishFieldName;
  }
  if ( defined $filename ) {
    $fields{'filename'}{'value'} = $filename;
  }
  if ( defined $versions ) {
    $fields{'versions'}{'value'} = $versions;
  }
  if ( defined $versionkey ) {
    $fields{'versionkey'}{'value'} = $versionkey;
  }
  if ( defined $anz_feldlisten ) {
	  $anz_feldlisten=0 if $anz_feldlisten<0;
		$anz_feldlisten=9999 if $anz_feldlisten>9999;
    $fields{'anz_feldlisten'}{'value'} = $anz_feldlisten;
  } else {
    $fields{'anz_feldlisten'}{'value'} = 0;
  }
  if ( defined $showeditfield ) {
    $fields{'showeditfield'}{'value'} = $showeditfield;
  } else {
    $fields{'showeditfield'}{'value'} = 0;
  }
  if ( defined $apilink ) {
    $fields{'apilink'}{'value'} = $apilink;
  }
  if ( defined $apifield ) {
    $fields{'apifield'}{'value'} = $apifield;
  }
  if ( defined $pdfFiles ) {
    $fields{'pdf'}{'value'} = $pdfFiles;
  } else {
    $fields{'pdf'}{'value'} = 1;
  }
  if ( defined $pdfsingle ) {
    $fields{'pdfsingle'}{'value'} = $pdfsingle;
  } else {
    $fields{'pdfsingle'}{'value'} = 0;
  }
  if ( defined $pdf_jpegquality ) {
    $fields{'pdf_jpegquality'}{'value'} = $pdf_jpegquality;
  } else {
    $fields{'pdf_jpegquality'}{'value'} = 5;
  }
  if ( defined $barcodes2 ) {
    $fields{'barcodes2'}{'value'} = $barcodes2;
  } else {
    $fields{'barcodes2'}{'value'} = 0;
  }
  if ( defined $cold2 ) {
    $fields{'cold2'}{'value'} = $cold2;
  } else {
    $fields{'cold2'}{'value'} = 0;
  }
  $fields{'fulltext'}{'value'} = ENGINE_MYSQL;
  if ( defined $fulltext ) {
	  if ($fulltext==0) {
      $fields{'fulltext'}{'value'} = ENGINE_NONE;
		} elsif ($fulltext==1) {
      $fields{'fulltext'}{'value'} = ENGINE_MYSQL;
		} elsif ($fulltext==2) {
      $fields{'fulltext'}{'value'} = ENGINE_SPHINX;
		}
  }
  if ( defined $show_ocr ) {
    $fields{'show_ocr'}{'value'} = $show_ocr;
  } else {
    $fields{'show_ocr'}{'value'} = 0;
	}
  if ( defined $show_fieldsocr ) {
    $fields{'show_fieldsocr'}{'value'} = $show_fieldsocr;
  } else {
    $fields{'show_fieldsocr'}{'value'} = 0;
	}
  if ( defined $hide_owner ) {
    $fields{'hide_owner'}{'value'} = $hide_owner;
  } else {
    $fields{'hide_owner'}{'value'} = 0;
	}
  $fields{'ocr'}{'value'} = OCR_FR;
  if ( defined $ocrRecognition ) {
	  if ($ocrRecognition==0) {
      $fields{'ocr'}{'value'} = OCR_NONE;
		} elsif ($ocrRecognition==1) {
      $fields{'ocr'}{'value'} = OCR_FR;
		} elsif ($ocrRecognition==3) {
      $fields{'ocr'}{'value'} = OCR_GNU
		} elsif ($ocrRecognition==2) {
      $fields{'ocr'}{'value'} = OCR_TESS
		}
  }
  if ( defined $ocr_cluster ) {
    $fields{'ocr_cluster'}{'value'} = $ocr_cluster;
  } else {
    $fields{'ocr_cluster'}{'value'} = "";
	}
  if ( defined $download_link ) {
    $fields{'download_link'}{'value'} = $download_link;
  } else {
    $fields{'download_link'}{'value'} = 1;
	}
  if ( defined $save_values ) {
    $fields{'save_values'}{'value'} = $save_values;
  } else {
    $fields{'save_values'}{'value'} = 1;
	}
  if ( defined $joker_comboany ) {
    $fields{'joker_comboany'}{'value'} = $joker_comboany;
  } else {
    $fields{'joker_comboany'}{'value'} = '';
	}
  if (defined $accesslog) {
	  $fields{access_log}{value} = $accesslog;
	} else {
	  $fields{access_log}{value} = 0;
	}
	$fields{access_hash}{value} = "";
  $fields{'photo_mode'}{'value'} = $photoMode;
  if ( defined $archiveSQL ) {
    $fields{'archive_sql'}{'value'} = $archiveSQL;
  }
  if ( defined $archiveMByte ) {
    $fields{'archive_mbyte'}{'value'} = $archiveMByte;
  } else {
    $fields{'archive_mbyte'}{'value'} = 600;
  }
  if ( defined $isoMByte ) {
    $fields{'iso_mbyte'}{'value'} = $isoMByte;
  } else {
    $fields{'iso_mbyte'}{'value'} = 600;
  }
  if ( defined $jpegQuality ) {
    $fields{'jpeg_quality'}{'value'} = $jpegQuality;
  } else {
    $fields{'jpeg_quality'}{'value'} = 33;
  }
  if ( defined $jpegQuality2 ) {
    $fields{'jpeg_quality2'}{'value'} = $jpegQuality2;
  } else {
    $fields{'jpeg_quality2'}{'value'} = 0;
  }
  if ( defined $prevScaling ) {
    $fields{'preview_scaling'}{'value'} = $prevScaling;
  } else {
    $fields{'preview_scaling'}{'value'} = 50;
  }
  if ( defined $sources_offline ) {
    $fields{'sources_offline'}{'value'} = $sources_offline;
  } else {
    $fields{'sources_offline'}{'value'} = 0;
  }
  if ( defined $turbo_videos_off ) {
    $fields{'turbo_videos_off'}{'value'} = $turbo_videos_off;
  } else {
    $fields{'turbo_videos_off'}{'value'} = 0;
  }
  if ( defined $camera_md5 ) {
    $fields{'camera_md5'}{'value'} = $camera_md5;
  } else {
    $fields{'camera_md5'}{'value'} = 0;
  }
  if ( defined $office_images ) {
    $fields{'office_images'}{'value'} = $office_images;
  } else {
    $fields{'office_images'}{'value'} = 0;
	}
  if ( defined $archiveFiles ) {
    $fields{'archive_files'}{'value'} = $archiveFiles;
  } else {
    $fields{'archive_files'}{'value'} = 1000;
  }
  return \%fields;
}

sub save {
  my ($self,$cgiO,$archiveO) = @_;
  my $hideTitleField     = $cgiO->param("hide_title_field");
  my $hideExtIcons       = $cgiO->param("hide_exticons");
  my $oldIcons           = $cgiO->param("old_icons");
  my $columns            = $cgiO->param("columns");
  my $minimalmode        = $cgiO->param("minimalmode");
  my $minimalfields      = $cgiO->param("minimalfields");
  my $titleFieldWidth    = $cgiO->param("title_field_width");
  my $publishFieldName   = $cgiO->param("publish_field_name");
  my $filename           = $cgiO->param("filename");
  my $versions           = $cgiO->param("versions");
  my $versionkey         = $cgiO->param("versionkey");
  my $anz_feldlisten     = $cgiO->param("anz_feldlisten");
  my $showeditfield      = $cgiO->param("showeditfield");
  my $apilink            = $cgiO->param("apilink");
  my $apifield           = $cgiO->param("apifield");
  my $photoMode          = $cgiO->param("photo_mode");
  my $pdfFiles           = $cgiO->param("pdf_files");
	my $pdfsingle          = $cgiO->param("pdfsingle");
	my $pdf_jpegquality    = $cgiO->param("pdf_jpegquality");
	my $barcodes2          = $cgiO->param("barcodes2");
	my $cold2              = $cgiO->param("cold2");
	my $fulltext           = $cgiO->param("fulltext");
	my $show_ocr           = $cgiO->param("show_ocr");
	my $show_fieldsocr     = $cgiO->param("show_fieldsocr");
	my $hide_owner         = $cgiO->param("hide_owner");
  my $ocrRecognition     = $cgiO->param("ocr_recognition");
	my $ocr_cluster				 = $cgiO->param("ocr_cluster");
  my $download_link      = $cgiO->param("download_link");
  my $save_values        = $cgiO->param("save_values");
  my $joker_comboany     = $cgiO->param("joker_comboany");
  my $archiveSQL         = $cgiO->param("archive_sql");
  my $archiveMByte       = $cgiO->param("archive_mbyte");
  my $archiveFiles       = $cgiO->param("archive_files");
  my $isoMByte           = $cgiO->param("iso_mbyte");
  my $jpegQuality        = $cgiO->param("jpeg_quality");
  my $jpegQuality2       = $cgiO->param("jpeg_quality2");
  my $prevScaling        = $cgiO->param("preview_scaling");
  my $sources_offline   = $cgiO->param("sources_offline");
  my $turbo_videos_off   = $cgiO->param("turbo_videos_off");
  my $camera_md5         = $cgiO->param("camera_md5");
  my $office_images      = $cgiO->param("office_images");
	my $accesslog          = $cgiO->param("access_log");
	my $accesshash         = $cgiO->param("access_hash");
  $hideTitleField = 0 if ( !defined $hideTitleField );
  $hideExtIcons = 0 if ( !defined $hideExtIcons );
  $oldIcons = 0 if ( !defined $oldIcons );
  $columns = 1 if ( !defined $columns );
  $minimalmode = 0 if ( !defined $minimalmode );
  $minimalfields = 0 if ( !defined $minimalfields );
  $showeditfield = 0 if ( !defined $showeditfield );
  my $ocr = 0; 
	if ($ocrRecognition eq OCR_FR) {
	  $ocr = 1;
	} elsif ($ocrRecognition eq OCR_GNU) {
	  $ocr = 3;
	} elsif ($ocrRecognition eq OCR_TESS) {
	  $ocr = 2;
	}
  $anz_feldlisten     = 0 if ( !defined $anz_feldlisten );
  $pdfFiles           = 0 if ( !defined $pdfFiles );
  $pdfsingle          = 0 if ( !defined $pdfsingle );
	$pdf_jpegquality    = 5 if ( !defined $pdf_jpegquality );
  $barcodes2          = 0 if ( !defined $barcodes2 );
  $cold2              = 0 if ( !defined $cold2 );
  $photoMode          = 0 if ( !defined $photoMode );
	$accesslog          = 0 if ( !defined $accesslog);
	$fulltext           = 0 if ( !defined $fulltext);
	$show_ocr           = 0 if ( !defined $show_ocr);
	$show_fieldsocr     = 0 if ( !defined $show_fieldsocr);
	$hide_owner         = 0 if ( !defined $hide_owner);
	$download_link      = 0 if ( !defined $download_link);
	$save_values        = 0 if ( !defined $save_values);
	$joker_comboany     = 0 if ( !defined $joker_comboany);
	$jpegQuality2       = 0 if ( !defined $jpegQuality2);
  $office_images      = 0 if ( !defined $office_images );
  $sources_offline    = 0 if ( !defined $sources_offline );
  $turbo_videos_off   = 0 if ( !defined $turbo_videos_off );
  $camera_md5         = 0 if ( !defined $camera_md5 );
  $archiveO->parameter("FeldTitel")->attribute("Inhalt")
    ->value($hideTitleField);
  $archiveO->parameter->update;
  $archiveO->parameter("HideExtIcons")->attribute("Inhalt")
    ->value($hideExtIcons);
  $archiveO->parameter->update;
  $archiveO->parameter("OldIcons")->attribute("Inhalt")
    ->value($oldIcons);
  $archiveO->parameter->update;
  $archiveO->parameter("Columns")->attribute("Inhalt")
    ->value($columns);
  $archiveO->parameter->update;
  $archiveO->parameter("MinimalMode")->attribute("Inhalt")
    ->value($minimalmode);
  $archiveO->parameter->update;
  $archiveO->parameter("MinimalFields")->attribute("Inhalt")
    ->value($minimalfields);
  $archiveO->parameter->update;
  $archiveO->parameter("FeldlistenAnzahl")->attribute("Inhalt")
    ->value($anz_feldlisten);
  $archiveO->parameter->update;
  $archiveO->parameter("FeldBreite")->attribute("Inhalt")
    ->value($titleFieldWidth);
  $archiveO->parameter->update;
  $archiveO->parameter("NoRichEdit")->attribute("Inhalt")
    ->value($showeditfield);
  $archiveO->parameter->update;
  $archiveO->parameter("JobsOCRRecognition")->attribute("Inhalt")
    ->value($ocr);
  $archiveO->parameter->update;
  $archiveO->parameter("OCR_CLUSTER")->attribute("Inhalt")
    ->value($ocr_cluster);
  $archiveO->parameter->update;
  $archiveO->parameter("PublishField")->attribute("Inhalt")
    ->value($publishFieldName);
  $archiveO->parameter->update;
	$archiveO->parameter("FILENAME")->attribute("Inhalt")
    ->value($filename);
  $archiveO->parameter->update;
	$archiveO->parameter("VERSIONS")->attribute("Inhalt")
    ->value($versions);
  $archiveO->parameter->update;
	$archiveO->parameter("VERSIONKEY")->attribute("Inhalt")
    ->value($versionkey);
  $archiveO->parameter->update;
	$archiveO->parameter("APILINK")->attribute("Inhalt")
    ->value($apilink);
  $archiveO->parameter->update;
	$archiveO->parameter("APIFIELD")->attribute("Inhalt")
    ->value($apifield);
  $archiveO->parameter->update;
  $archiveO->parameter("PhotoMode")->attribute("Inhalt")->value($photoMode);
  $archiveO->parameter->update;
  $archiveO->parameter("PDFFiles")->attribute("Inhalt")->value($pdfFiles);
  $archiveO->parameter->update;
  $archiveO->parameter("PDFWHOLEDOC")->attribute("Inhalt")->value($pdfsingle);
  $archiveO->parameter->update;
  $archiveO->parameter("PDF_JPEGQUALITY")
		->attribute("Inhalt")->value($pdf_jpegquality);
  $archiveO->parameter->update;
  $archiveO->parameter("BARCODES2")->attribute("Inhalt")->value($barcodes2);
  $archiveO->parameter->update;
  $archiveO->parameter("COLD2")->attribute("Inhalt")->value($cold2);
  $archiveO->parameter->update;
	my $t = $archiveO->parameter("SEARCH_FULLTEXT")->attribute("Inhalt")->value();
  my $text = 1; 
	if ($fulltext eq ENGINE_NONE) {
	  $text = 0;
	} elsif ($fulltext eq ENGINE_MYSQL) {
	  $text = 1;
	} elsif ($fulltext eq ENGINE_SPHINX) {
	  $text = 2;
	}
  $archiveO->parameter("SEARCH_FULLTEXT")
		->attribute("Inhalt")->value($text);
  $archiveO->parameter->update;
	if ($text==2 && $t != $text) { # only update if it was changed
	  if (!-e '/tmp/sphinx.wrk') {
		  if ($archiveO->db->host eq "localhost") {
        my $config = Archivista::Config->new;
        my $host = $config->get("MYSQL_HOST");
        my $db = $archiveO->db->database();
        my $user = $config->get("MYSQL_UID");
        my $pw = $config->get("MYSQL_PWD");
        undef $config;
		    my $dbh1 = $archiveO->db->sudbh;
        my $sql = "INSERT INTO archivista.jobs SET status=110,job='WEBCONF',".
               "host=".$dbh1->quote($host).",db=".$dbh1->quote($db).",".
               "user=".$dbh1->quote($user).",pwd=".$dbh1->quote($pw);
			  $dbh1->do($sql);
				my @row = $dbh1->selectrow_array("SELECT LAST_INSERT_ID()");
        $sql = "INSERT INTO archivista.jobs_data SET jid=$row[0],".
               "param='WEBC_MODE',value='SPHINX_INDEX'";
				$dbh1->do($sql);
        $sql = "UPDATE archivista.jobs SET status=100 where id=$row[0]";
				$dbh1->do($sql);
			} else {
		    print STDERR "sphinx indexer only at localhost allowed...\n";
			}
		} else {
		  print STDERR "sphinx indexer already in progress...\n";
		}
	}
  $archiveO->parameter("SHOW_OCR")
		->attribute("Inhalt")->value($show_ocr);
  $archiveO->parameter->update;
  $archiveO->parameter("SHOW_FIELDSOCR")
		->attribute("Inhalt")->value($show_fieldsocr);
  $archiveO->parameter->update;
  $archiveO->parameter("HIDE_OWNER")
		->attribute("Inhalt")->value($hide_owner);
  $archiveO->parameter->update;
  $archiveO->parameter("DOWNLOAD_LINK")
		->attribute("Inhalt")->value($download_link);
  $archiveO->parameter->update;
  $archiveO->parameter("SAVE_VALUES")
		->attribute("Inhalt")->value($save_values);
  $archiveO->parameter->update;
  $archiveO->parameter("JOKER_COMBOANY")
		->attribute("Inhalt")->value($joker_comboany);
  $archiveO->parameter->update;
  $archiveO->parameter("ArchivSQL")->attribute("Inhalt")->value($archiveSQL);
  $archiveO->parameter->update;
  $archiveO->parameter("ArchivMByte")->attribute("Inhalt")
    ->value($archiveMByte);
  $archiveO->parameter->update;
  $archiveO->parameter("ArchivDateien")->attribute("Inhalt")
    ->value($archiveFiles);
  $archiveO->parameter->update;
  $archiveO->parameter("isoMByte")->attribute("Inhalt")
    ->value($isoMByte);
  $archiveO->parameter->update;
  $archiveO->box_parameter("JpegQuality")->attribute("Inhalt")
    ->value($jpegQuality);
  $archiveO->parameter->update;
  $archiveO->box_parameter("JpegQuality2")->attribute("Inhalt")
    ->value($jpegQuality2);
  $archiveO->box_parameter->update;
  $archiveO->box_parameter("PrevScaling")->attribute("Inhalt")
    ->value($prevScaling);
  $archiveO->box_parameter->update;
  $archiveO->parameter("SourcesOffline")->attribute("Inhalt")
    ->value($sources_offline);
  $archiveO->parameter->update;
  $archiveO->parameter("TurboVideosOff")->attribute("Inhalt")
    ->value($turbo_videos_off);
  $archiveO->parameter->update;
  $archiveO->parameter("CameraMD5")->attribute("Inhalt")
    ->value($camera_md5);
  $archiveO->parameter->update;
	if ($camera_md5==1) { # update database with md5 strings
	  if (!-e '/tmp/md5.wrk') {
		  if ($archiveO->db->host eq "localhost") {
        my $config = Archivista::Config->new;
        my $host = $config->get("MYSQL_HOST");
        my $db = $archiveO->db->database();
        my $user = $config->get("MYSQL_UID");
        my $pw = $config->get("MYSQL_PWD");
        undef $config;
		    my $dbh1 = $archiveO->db->sudbh;
        my $sql = "INSERT INTO archivista.jobs SET status=110,job='WEBCONF',".
               "host=".$dbh1->quote($host).",db=".$dbh1->quote($db).",".
               "user=".$dbh1->quote($user).",pwd=".$dbh1->quote($pw);
			  $dbh1->do($sql);
				my @row = $dbh1->selectrow_array("SELECT LAST_INSERT_ID()");
        $sql = "INSERT INTO archivista.jobs_data SET jid=$row[0],".
               "param='WEBC_MODE',value='CAMERA_MD5'";
				$dbh1->do($sql);
        $sql = "INSERT INTO archivista.jobs_data SET jid=$row[0],".
               "param='database',value=".$dbh1->quote($db);
				$dbh1->do($sql);
        $sql = "UPDATE archivista.jobs SET status=100 where id=$row[0]";
				$dbh1->do($sql);
			} else {
		    print STDERR "must be localhost for adding md5 hashes...\n";
			}
		} else {
		  print STDERR "adding md5 hashes already in progress...\n";
		}
	}
  $archiveO->parameter("OfficeImages")->attribute("Inhalt")
    ->value($office_images);
  $archiveO->parameter->update;
	my $checkok=0;
	if ($accesslog==0) {
	  my $dbh = $archiveO->db->dbh;
		if ($archiveO->db->host eq "localhost") {
		  $dbh = $archiveO->db->sudbh;
		}
		my $sql = "select hash,id from access order by id limit 1";
		my @row = $dbh->selectrow_array($sql);
		my $hash = $row[0];
		my $id = $row[1];
		my $hash1 = substr($hash,0,8);
		if ($hash1 eq $accesshash && $id >0 && $accesshash ne "") {
      $sql = "update access set hash = '$hash"."a' where id=$id";
			$dbh->do($sql);
			$checkok=1;
	  } else {
	    $checkok=0;
		}
	} else {
	  $checkok=1;
	}
	if ($checkok==1) {
    $archiveO->parameter("ACCESS_LOG")->attribute("Inhalt")->value($accesslog);
    $archiveO->parameter->update;
	}
	if ($ocr_cluster ne "") {
	  my $dbh = $archiveO->db->dbh;
		if ($archiveO->db->host eq "localhost") {
		  $dbh = $archiveO->db->sudbh;
		}
    my $db = $archiveO->db->database();
		my @hosts = split(/\,/,$ocr_cluster);
		my $base = "";
		my $ext = "";
		foreach my $host (@hosts) {
			my @parts = split(/\./,$host);
			if ($parts[3] ne "") {
				$base = $parts[0].'.'.$parts[1].'.'.$parts[2];
				$ext = $parts[3];
			} else {
				$ext = $parts[0];
			}
			if ($ext ne "" && $base ne "") {
				$host=$base.'.'.$ext;
				my $userpw = "ocr";
				my $sql = "select Laufnummer from $db.user where user=".
					$dbh->quote($userpw)." and host=".$dbh->quote($host)." limit 1";
				my ($userid)=$dbh->selectrow_array($sql);
				if ($userid>0) {
					my $user = $archiveO->user($userid);
					$user->attribute("Password")->value($userpw);
					$user->attribute("ZugriffWeb")->value(0);
					$user->update;
				} else {
					my $userid = $archiveO->user($host,$userpw,$userpw,1)->id;
					my $user = $archiveO->user($userid);
					$user->attribute("ZugriffWeb")->value(0);
					$user->update;
				}
			} elsif ($base eq "" && $ext eq "1") {
				open(FOUT,">/etc/webconfig/mysqlconf");
				print FOUT "MYSQLCONF\n";
				print FOUT "old1=bind-address \n";
				print FOUT "old2= 127.0.0.1\n";
				print FOUT 'new1=# bind-address '."\n";
				print FOUT 'new2= 127.0.0.1'."\n";
				close(FOUT);
				my $sql = "alter table $db.archivseiten add Index ZippedI (Zipped)";
				$dbh->do($sql);
			}
		}
	}
}

sub archive {
  my ($self) = @_;
  return $self->{'archiveO'};
}

1;

