
package PL::FieldDefinition;
use strict;

sub new {
  my ($cls,$archiveO) = @_;
	my $self = {};
  bless $self, $cls;
  my @fields = ("field_old_name","field_name","field_type",
		"field_length","field_position");
  $self->{'archiveO'} = $archiveO;
	$self->{'field_list'} = \@fields;
  return $self;
}

sub fields {
  my ($self,$cgiO) = @_;
	my $archiveO = $self->archive;
  my $langO = $self->archive->lang;
	my $pafieldTypes = $self->archive->attributeDataTypesByArray;
	my $phfieldTypes = $self->archive->attributeDataTypesByHash;
	my $pauserDefinedAttributes = $self->archive->userDefinedAttributes;
	# Add pages to the user defined attribute list (position after pages -> first
	# user defined attribute
	unshift @$pauserDefinedAttributes, "Seiten";
	my %userDefinedAttributes;
  foreach my $attribute (@$pauserDefinedAttributes) {
		$userDefinedAttributes{$attribute} = $attribute;
	}
	$userDefinedAttributes{'Seiten'} = $langO->string("PAGES");
	my (%fields);
  $fields{'list'} = $self->{'field_list'};
	$fields{'field_old_name'}{'name'} = "field_old_name";
	$fields{'field_old_name'}{'type'} = "hidden";
	$fields{'field_old_name'}{'update'} = 1;
  $fields{'field_name'}{'label'} = $langO->string("FIELD_NAME");
	$fields{'field_name'}{'name'} = "field_name";
	$fields{'field_name'}{'type'} = "textfield";
	$fields{'field_name'}{'update'} = 1;
	$fields{'field_type'}{'label'} = $langO->string("TYPE");
	$fields{'field_type'}{'name'} = "field_type";
	$fields{'field_type'}{'type'} = "select";
	$fields{'field_type'}{'array_values'} = $pafieldTypes;
	$fields{'field_type'}{'hash_values'} = $phfieldTypes;
	$fields{'field_type'}{'update'} = 0;
	$fields{'field_length'}{'label'} = $langO->string("LENGTH");
	$fields{'field_length'}{'name'} = "field_length";
	$fields{'field_length'}{'type'} = "textfield";
	$fields{'field_length'}{'update'} = 0;
	$fields{'field_position'}{'label'} = $langO->string("POSITION_AFTER");
	$fields{'field_position'}{'name'} = "field_position";
	$fields{'field_position'}{'type'} = "select";
	$fields{'field_position'}{'array_values'} = $pauserDefinedAttributes;
	$fields{'field_position'}{'hash_values'} = \%userDefinedAttributes;
	$fields{'field_position'}{'update'} = 1;
  if ($cgiO->param("adm") eq "edit") {
		my $id = $cgiO->param("id");
		if (defined $id) {
			my $phattribute = $archiveO->describeAttribute($id);
			$fields{'field_old_name'}{'value'} = $$phattribute{$id}{'attribute_name'};
			$fields{'field_name'}{'value'} = $$phattribute{$id}{'attribute_name'};
			$fields{'field_type'}{'value'} = 
				$$phfieldTypes{$$phattribute{$id}{'attribute_type'}};
			$fields{'field_type'}{'hidden_value'} = 
				$$phattribute{$id}{'attribute_type'};
			$fields{'field_length'}{'value'} = 
				$$phattribute{$id}{'attribute_length'};
			$fields{'field_length'}{'hidden_value'} =	
				$$phattribute{$id}{'attribute_length'};
			$fields{'field_position'}{'value'} = 
				$$phattribute{$id}{'attribute_after'};
			# On update, delete the attribute itself from the attribute position
			# dropdown
			my @udAttributes = 
				grep !/$id/,@{$fields{'field_position'}{'array_values'}};
			$fields{'field_position'}{'array_values'} = \@udAttributes;
		}
	}
  $fields{'displayBackFormButton'} = 1;
  return \%fields;
}

sub save {
  my ($self,$cgiO) = @_;
	my $archiveO = $self->archive;
  my $id = $cgiO->param("id");
  if (length($cgiO->param("field_name")) > 0) {
  	my (@alter);
		$alter[0]{'attribute_name'} = $cgiO->param("field_name");
		$alter[0]{'attribute_type'} = $cgiO->param("field_type");
		$alter[0]{'attribute_length'} = $cgiO->param("field_length");
		$alter[0]{'after_attribute'} = $cgiO->param("field_position");
		if (defined $id) {
			$alter[0]{'attribute_old_name'} = $cgiO->param("field_old_name");
			$alter[0]{'alter_type'} = "CHANGE";
		} else {
			$alter[0]{'alter_type'} = "ADD";
		}
		$archiveO->alter(\@alter);
	}
}

sub elements {
  my ($self) = @_;
	my $archiveO = $self->archive;
  my $langO = $archiveO->lang;
	my $pauserDefinedAttributes = $archiveO->userDefinedAttributes;
	my (@list,%fields);
	# Attributes to display on table
	# Same name as in archivista.languages, $langO->string() will use this
	# attribute definitions (upper case)
	my @attributes = ("field_name");
	foreach my $attribute (@$pauserDefinedAttributes) {
		push @list, $attribute;
		$fields{$attribute}{'field_name'} = $attribute;
		$fields{$attribute}{'delete'} = 1;
		$fields{$attribute}{'update'} = 1;
	}
  $fields{'new'} = 1;
  $fields{'attributes'} = \@attributes;
  $fields{'list'} = \@list;
	return \%fields;
}

sub delete {
  my ($self,$cgiO) = @_;
	my $archiveO = $self->archive;
	my $id = $cgiO->param("id");
	if (defined $id) {
	  my (@alter);
		$alter[0]{'attribute_name'} = $id;
		$alter[0]{'alter_type'} = "DROP";
		$archiveO->alter(\@alter);
	}
	$self->deleteObjTab($id,"Obj");
	$self->deleteObjTab($id,"Tab");
}

sub deleteObjTab {
	my ($self,$field,$type) = @_;
	my $archiveO = $self->archive;
	my $dbh = $archiveO->db->dbh;
	$type = "Obj" if $type ne "Tab";
	my $fpos=0; # field name in Tab is on first position
	$fpos=18 if $type eq "Obj"; # field name is on 18th position
	my $sql="select Inhalt,Laufnummer from parameter where ".
		"Name like 'Felder$type%' and Tabelle='archiv'";
	my $prow = $dbh->selectall_arrayref($sql);
	foreach my $prow1 (@$prow) {
		my $obj = $$prow1[0];
		my $lnr = $$prow1[1];
		my @lines = split("\r\n",$obj);
		my @new = ();
		my $c=0;
		foreach my $line (@lines) {
			my @parts = split(';',$line);
			if ($parts[$fpos] ne $field) {
				push @new,$line;
			}
		}
		my $new = join("\r\n",@new);
		$sql = "update parameter set Inhalt=".$dbh->quote($new).
			" where Laufnummer=$lnr";
		$dbh->do($sql);
	}
}

sub archive {
  my ($self) = @_;
	return $self->{'archiveO'};
}

1;

