#!/usr/bin/perl

package AVJobs;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
use ExactImage;
use Digest::MD5 qw(md5_hex);
use File::Temp qw(tempfile mkdtemp);
use Archive::Zip;
require Exporter;
use Encode;
use DBI;

@ISA    = qw(Exporter);
@EXPORT = qw (getScanDefByNumber
getScanDefByName
getScanDefById
getParameterRead
getBoxParameterRead
getParameterReadWithType
getFields
getFileExtension
savePage
updateDoc
saveDoc
createThumbsFirst
checkNewDoc
rotate
checkEmptyPage
readFile
readFile2
writeFile
optimizeBW
FTPaddPages
FTPparseFile
FTPsplitFile
CUPSwaitAndMove
CUPSparseFile
CUPSgetPSFile
jobStart
jobStop
jobCheckStop
jobCheckInit
checkInitialized
archivingDatabase
archivingDatabase1
getValidDatabases
checkDatabase
chooseXValues
restartOCRbatch
MySQLOpen
MySQLOpen2
MySQLClose
HostIsSlave
HostIsSlaveSimple
logit
MainAccessLog
recordAddOrUpdate
copyValues
checkLinkFields
DocumentLock
DocumentUnlock
importformatvalue 
exportformatvalue 
emptypath
sqlrange
addLogEntryImport
getLang
findit
findit1
escape
unescape
keyvalue
TimeStamp
DateStamp
DocumentAddDatForm
DocumentAddDatSQL
getBlobFile
getBlobTable
createThumbsAndSave
getConnectionValues
OpenOfficeConvert
HTMLtoPDF
HTMLtoText
CheckFileNamePathBase
selectValue
OCRDoOpenSourceLang
getXRandr
archivingDocSetOwner
OCRDoOpenSource
getOCRVals
check64bit
MainUpdateText
MainUpdateTextUpdate
getZeros
getZeroFile
getFileDB
getInfos
getIdentify
createAXIS
getBlockTableCheck 
archivingDocPathCheck
archivingDocMoveFile
archivingDocFolder
dumpKeyInfos
getArchivingExt
archivingDocExtractImages
archivingDocSource
fromUTF8
toUTF8
correctFile
md5File
moveFile
checkFile
checkDir
notFile
notDir
unlinkFile
MainNoteHash
smallram
checkmaxi
md5add
md5add2
md5get
md5check
checklastdocadd
OCRScriptPDFCheck
exportfiles
splitEDVName
getEDVName
updateTitelKfile
isSourceExternal
speechRecognition
exportDbToDisk
unzipFirstFile
writeTempFile
);

use DBI;
use File::Copy;
use Archivista::Config;    # only needed for connection

use constant LOG_FORMREC => -1; # first we need to process form recognition
use constant LOG_DONE => 0; # everything is ok, ocr not done
use constant LOG_WORK => 1; # someone is working with the document
use constant LOG_AXISDONE => 2; # axis job is done, sane-client can get it
use constant LOG_OCR => 3; # the ocr engine is saving pdf files
use constant LOG_OCRDONE => 4; # the ocr recognition is done
use constant LOG_SCANSTART => 5; # we did start the scanadf job
use constant LOG_SCANADF => 6; # scanadf is working (on pages)
use constant LOG_SCANDONE => 7; # scanadf job is done, but not yet sane-post.pl

use constant MAX_PAGES => 640; # max. of pages per document (640)
use constant HTML_WIDTH => 1600; # desired width of html pages
use constant HTML_HEIGHT => 1200; # desired height of html pages
use constant HTML_SCROLL => 1050; # desired scrolling height
use constant HTML_SCALE => 1; # scale factor after getting screenshot
use constant HTML_SCALE2 => 2; # scale factor after cropping

my %val;
$val{dirsep} = "/"; # seperate a directory ((windows/linux)
$val{dirimg} = "/home/data/archivista/images/"; # base path to images
$val{tmpdir} = "/home/data/archivista/tmp/"; # temp dir (in case we need file)
$val{log}       = '/home/data/archivista/av.log';
$val{ftpin} = "/home/data/archivista/ftp/go/"; # where we get the axis files
$val{ftpgs} = "gs -dNOPAUSE -dNOPROMPT -dBATCH -q"; # gs with main attributes
$val{cupsin} = "/var/spool/cups-pdf/ANONYMOUS/"; # in folder for cups files
$val{cupsps}  = "/var/spool/cups/"; # folder where we can find the orig. ps file
$val{pdfinfo} = "pdfinfo";          # program to extract pdf information
$val{pdftk}   = "pdftk";            # program to extract pdf pages
$val{pdf2txt} = "pdftotext";        # program to extract text from pdf file
$val{cupsinfo} = "lpstat_cups -W completed -o";    # check what job we got
$val{cupsinfo64} = "lpstat -W completed -o";    # check what job we got
$val{coldplus} = "/home/data/archivista/cust/cold/coldplus.pl";   # job coldplus
$val{ftpplus} = "/home/data/archivista/cust/ftp/ftpplus.pl";   # job ftpplus
$val{sane} = "/usr/bin/perl /home/cvs/archivista/jobs/sane-client.pl";
$val{autoprg} = "/home/data/archivista/cust/autofields/"; # path autoprogs
$val{ftpprocess} = "/etc/ftp-process.conf"; # send ftp/pdf not to local database
$val{frpath} = '/home/archivista/.wine/drive_c/Programs/Av5e/';

my %ocr = (0=>'Afrikaans',1=>'Albanian',2=>'Basque',3=>'Breton',
	4=>'Bulgarian',5=>'Belarusian',6=>'Catalan',7=>'Chechen',8=>'Croatian',
	9=>'Czech',10=>'Danish',11=>'Digits',12=>'Dutch',13=>'DutchBelgian',
	14=>'English',15=>'Estonian',16=>'Fijian',17=>'Finnish',18=>'French',
	19=>'German',20=>'GermanNewSpelling',21=>'Greek',22=>'Hawaiian',
	23=>'Hungarian',24=>'Icelandic',25=>'Indonesian',26=>'Irish',27=>'Italian',
	28=>'Kabardian',29=>'Latin',30=>'Latvian',31=>'Lithuanian',32=>'Macedonian',
	33=>'Maori',34=>'RomanianMoldavia',35=>'Norwegian',36=>'NorwegianBokmal',
	37=>'NorwegianNynorsk',38=>'Ossetic',39=>'Polish',40=>'PortugueseStandard',
	41=>'PortugueseBrazilian',42=>'RhaetoRomanic',43=>'Romanian',44=>'Russian',
	45=>'Mixed',46=>'Samoan',47=>'SerbianCyrillic',48=>'Slovak',49=>'Slovenian',
	50=>'Spanish',51=>'Swahili',52=>'Swedish',53=>'Tagalog',54=>'Tatar',
	55=>'CrimeanTatar',56=>'Turkish',57=>'Ukrainian',58=>'Basic',59=>'C++',
	60=>'Cobol',61=>'Fortran',62=>'Java',63=>'Pascal',64=>'Chemistry',
	65=>'Esperanto');

# where we get the Archivista connection
my $config = Archivista::Config->new;
$val{host} = $config->get("MYSQL_HOST");
$val{db}   = $config->get("MYSQL_DB");
$val{user} = $config->get("MYSQL_UID");
$val{pw}   = $config->get("MYSQL_PWD");
undef $config;

$val{avversion} = 520; # actual archivista internal database number
$val{avfoldermax} = 9999; # max. number of folders in archive
$val{avuser} = 500; # archivista user
$val{avowner} = 100; # group for archivista user
# paths for ocr and archiving batch jobs
$val{jobpf} = "/home/archivista/.wine/drive_c/Programs/Av5e/";
$val{jobwr} = "AV5AUTO.WRK";
$val{jobst} = "AV5AUTO.STP";
$val{joben} = "AV5AUTO.END";



=head2 $scandef=getScanDefByNumber($dbh,$db1,$scanid)

Read a scan definition by its number and gives back the string

=cut

sub getScanDefByNumber {
  my ($dbh,$db1,$scanid) = @_;
  # read the scan definitions from the database
  my $scandef = getParameterRead( $dbh, $db1, "ScannenDefinitionen" );
  my @scannen = split( "\r\n", $scandef );
  $scanid = 0 if $scannen[$scanid] eq "";
  return $scannen[$scanid];
}



=head2 $scandef=getScanDefByName($dbh,$db1,$scanname)

Read scan definition from the database db1 and gives back $scandef
  
=cut

sub getScanDefByName {
  my ($dbh,$db1,$scanname) = @_;
  my ( $scandef, @scannen, @scanval, $scanid, $c );
  # read the scan definitions from the database
  $scandef = getParameterRead( $dbh, $db1, "ScannenDefinitionen" );
  @scannen = split( "\r\n", $scandef );
  # give back the first definition in case we don't find it
  $scanid = 0;
  $c      = 0;
  foreach (@scannen) {
    # go through all definitions
    @scanval = split( ";", $_ );
    if ( $scanval[0] eq $scanname ) {
      $scanid = $c;
    }
    $c++;
  }
  return $scannen[$scanid];
}



=head2 $scandef=getScanDefById($dbh,$db1,$key)

Read scan definition from the database db1 and gives back $scandef
  
=cut

sub getScanDefById {
  my ($dbh,$db1,$key) = @_;
  my ( $scandef, @scannen, @scanval, $scanid, $c );
  # read the scan definitions from the database
  $scandef = getParameterRead( $dbh, $db1, "ScannenDefinitionen" );
  @scannen = split( "\r\n", $scandef );
  # give back the first definition in case we don't find it
  $scanid = 0;
  $c = 0;
	if ($key>0) {
    foreach (@scannen) {
      # go through all definitions
      @scanval = split( ";", $_ );
      if ( $scanval[22] eq $key ) {
        $scanid = $c;
      }
      $c++;
    }
	}
  return $scannen[$scanid];
}



=head2 recordAddOrUpdate($dbh,$db1,$pnr,$ppages,$plockuser,$user)

Adds a new Archivista record and/or does lock it

=cut

sub recordAddOrUpdate {
  my ($dbh,$db1,$pnr,$ppages,$plockuser,$user,$owner) = @_;
  $user = "cupsftp" if $user eq "";
  my $user1 = $dbh->quote($user);
  my ( $sql, $r, @row );
  if ( $$pnr == 0 ) {
    # add a document with the acutal date
    my $date1 = DocumentAddDatForm( TimeStamp() );
    my $foldakt = getParameterRead($dbh,$db1,"ArchivOrdner");
		checkJobAdmin($dbh,$db1,"localhost");
		my $o1="''";
		$o1 = $dbh->quote($owner) if $owner ne "";
    $sql = "insert into $db1.archiv set Seiten=0,Ordner=$foldakt, " .
		       "Eigentuemer=$o1,Erfasst=0," .
           "Datum=$date1,UserNeuName=$user1,ErfasstDatum=$date1,ArchivArt=1";
	  logit($sql);
    $r = $dbh->do($sql);
    if ( $r == 1 ) {
      # Document is ok, we need the number
      $sql = "select Laufnummer from $db1.archiv " .
             "where Laufnummer=LAST_INSERT_ID()";
      @row = $dbh->selectrow_array($sql);
      $$pnr = $row[0];
      $$ppages = 0;
    }
  }
  if ( $$pnr > 0 ) {
    # update the document so it is locked
    my $locked = $dbh->quote($$plockuser);
    $sql = "update $db1.archiv set Akte=$$pnr,UserModName=$user1," .
           "Gesperrt=$locked " .
           "where Laufnummer=$$pnr";
		logit($sql);
    $dbh->do($sql);
    # get the actual pages directly from the document
    $sql = "select Seiten from archiv where Akte=$$pnr";
    @row = $dbh->selectrow_array($sql);
    $$ppages = $row[0] if $row[0] >= 0;
  }
}



=head1 checkJobAdmin($dbh,$db1)

Check if we need to update scripts for a box

=cut

sub checkJobAdmin {
  my ($dbh,$db1,$host) = @_;
  my $jobadmin = getParameterReadWithType($dbh,$db1,"JOBADMIN01","JOBADMIN");
  if ($jobadmin ne "") {
	  my @lines = split("\r\n",$jobadmin);
		my $c0=0;
		foreach (@lines) {
		  my $line = $_;
		  my $update = 0;
			my @vals = split(";",$line);
			if ($vals[4]==1) {
			  my @boxes = split(",",$vals[2]);
				my @found = ();
				my $name = $vals[0];
				my $script = $vals[1];
				my $startpos = index($script,"/home/data/archivista/cust/");
				my $user = $vals[6];
				my $pw = pack("H*",$vals[7]);
				my @updated = split(",",$vals[5]);
				my $pres = $dbh->selectall_arrayref("show processlist");
				my $pres1 = $$pres[0];
				my $ip = $$pres1[2];
				$ip =~ s/^(.*?)(:)(.*)/$1/;
				my $c=0;
				foreach (@boxes) {
				  if ($boxes[$c] eq $ip && $startpos==0) {
			      logit("we need to update a script $script");
					  my $c1=0;
						my $writeit=1;
					  foreach (@updated) {
						  $writeit=0 if $updated[$c1] eq $ip;
							$c1++;
						}
						if ($writeit==1) {
						  my $code = unescape($vals[3]);
							open(FOUT,">$script");
							binmode(FOUT);
							print FOUT $code;
							close(FOUT);
							if (checkFile($script)) {
                chown "root","root",$script;
							  chmod 0750,$script;
							}
							push @updated,$ip;
							$vals[5] = join(",",@updated);
							$lines[$c0]=join(";",@vals);
							checkJobAdminUpdate($ip,$db1,$user,$pw,$name,$vals[5],$host);
						}
					}
				  $c++;
				}
			}
			$c0++;
		}
	}
}



=head1 checkJobAdminUpdate($ip,$db1,$user,$pw,$name,$line)

Send command for updating a given jobadmin definition

=cut

sub checkJobAdminUpdate {
  my ($ip,$db1,$user,$pw,$name,$boxes,$host) = @_;
	my $db2 = "archivista";
	if ($ip eq "localhost") {
	  $user = "root";
		$host = "localhost";
		$pw = $val{pw};
	}
	my $dbh = MySQLOpen($host,$db2,$user,$pw);
	if ($dbh) {
    my $sql1 = "INSERT INTO $db2.";
    my $sql = $sql1."jobs SET job='JOBS',status=110," .
               "host='localhost',db=".$dbh->quote($db1)."," .
               "user=".$dbh->quote($user).",pwd=''";
		$dbh->do($sql);
		$sql = "SELECT LAST_INSERT_ID()";
    my @rows = $dbh->selectrow_array($sql);
		my $id = $rows[0];
		if ($id>0) {
		  my $line = $name.";".$boxes;
      $sql=$sql1."jobs_data set jid=$id,param='LINE',value=".$dbh->quote($line);
			$dbh->do($sql);
      $sql = "UPDATE $db2.jobs SET status=100 where id=$id";
			$dbh->do($sql);
			logit($sql);
    }
	}
}



=head2 $stamp=TimeStamp 

Actual date/time stamp (20040323130556)

=cut

sub TimeStamp {
  my @t = localtime( time() );
  my ( $stamp, $y, $m, $d, $h, $mi, $s );
  $y     = $t[5] + 1900;
  $m     = $t[4] + 1;
  $m     = sprintf( "%02d", $m );
  $d     = sprintf( "%02d", $t[3] );
  $h     = sprintf( "%02d", $t[2] );
  $mi    = sprintf( "%02d", $t[1] );
  $s     = sprintf( "%02d", $t[0] );
  $stamp = $y . $m . $d . $h . $mi . $s;
  return $stamp;
}



=head1 dd.mm.yyyy=DateStamp

Give back a string with the date 

=cut

sub DateStamp {
  my ($date) = @_;
  my @t = localtime( $date );
  my ( $stamp, $y, $m, $d, $h, $mi, $s );
  $y = $t[5] + 1900;
  $m = $t[4] + 1;
  $m = sprintf( "%02d", $m );
  $d = sprintf( "%02d", $t[3] );
  $stamp = $d.'.'.$m.'.'.$y;
  return $stamp;
}



=head2 $stamp=SQLStamp

Actual date as SQL string (2004-03-23)

=cut

sub SQLStamp {
  my @t = localtime( time() );
  my ( $stamp, $y, $m, $d, $h, $mi, $s );
  $y     = $t[5] + 1900;
  $m     = $t[4] + 1;
  $m     = sprintf( "%02d", $m );
  $d     = sprintf( "%02d", $t[3] );
  $stamp = $y . "-" . $m . "-" . $d . " 00:00:00";
  return $stamp;
}



=head2 $sqldat=DocumentAddDatForm(yyyymmdd)

Format a date ('yyyy-mm-dd 00:00:00')

=cut

sub DocumentAddDatForm {
  my ($d) = @_;
  $d = "'".substr($d,0,4)."-".substr($d,4,2)."-".substr($d,6,2)." 00:00:00'";
  return $d;
}



=head2 $@fields=getFields ($dbh, $table)

Gets back in a pointer of an array a hash 
containing all name, type und sizes of an mysql table

=cut

sub getFields {
  my ($dbh,$table,$db1) = @_;
	$table = "archiv" if $table eq "";
	if ($db1 ne "") {
	  $table = "$db1.$table";
	}
  my $sql   = "describe $table";
  my $nr = 0;
  my $st = $dbh->prepare($sql);
  my $r  = $st->execute;
  my @fields;
  while ( my @row = $st->fetchrow_array ) {
    my %f;
    $f{name} = $row[0];
    my $name = "";
    my $lang = 0;
    ( $name, $lang ) = $row[1] =~ /(.*)\((.*)\)/;
    $name = $row[1] if $name eq "";
    $f{type}     = $name;
    $f{size}     = $lang;
    $fields[$nr] = \%f;
    $nr++;
  }
  return \@fields;
}



=head2 savePage($obj)

Save one single page to the database (incl. all processing)

=cut

sub savePage {
	my ($obj) = @_;
	#$obj->logit("start to save ".$obj->file);
	$obj->changedimage(0);
	my $size = -s $obj->file;
	if ($obj->officetyp eq "office" && $obj->pages==0 && $obj->officeimages==1) {
		my $size2 = $size;
		$size2 = -s $obj->source2 if -e $obj->source2;
		my $cmd = "identify -verbose ".$obj->file;
		my $res = `$cmd`;
		$res = "size: $size2\n$res";
		$obj->mediainfo($res);
	}
	if ($size>0) { # there is some content in the file!!!
		if ($obj->callconvert ne "") {
	 		my @parts = split(/\//,$obj->file);
			my $name = pop @parts;
			my $pfad = join('/',@parts);
			@parts = split(/\./,$name);
			my $ext = pop @parts;
			my $base = join('.',@parts);
			my $fin = "$pfad/$base.$ext";
			my $fout = "$pfad/$base-2.$ext";
			if (!-e "$fout") {
				moveFile($fin,$fout);
				my $cmd = "convert $fout ".$obj->callconvert." $fin";
				my $res = callPrg($cmd);
				$size = -s $obj->file;
				if ($size>0) {
					unlinkFile($fout);
				} else {
					moveFile($fout,$fin);
					$size = -s $obj->file;
				}
			}
		}
	}
	if ($size>0) { # there is some content in the file!!!
    my $image = ExactImage::newImage();
	  my $image2 = "";
	  logit("start reading ".$obj->file);
		savePageBW($obj);
    my $res = ExactImage::decodeImageFile($image,$obj->file);
    my $w = ExactImage::imageWidth($image);
    my $h = ExactImage::imageHeight($image);
		$obj->filex($w);
		$obj->filey($h);
		if (($w==0 && $h==0) || $res!=1) {
	    logit("image ".$obj->file."has a problem, try to fix it with convert");
		  my $cmd = "convert ".$obj->file." ".$obj->file;
			callPrg($cmd,1);
      $res = ExactImage::decodeImageFile($image,$obj->file);
	    $obj->changedimage(1);
		}
		my $xres = ExactImage::imageXres($image);
		my $yres = ExactImage::imageYres($image);
    if (($xres!=$obj->dpi || $yres!=$obj->dpi) && $obj->changedimage!=-1) {
      ExactImage::imageSetXres($image,$obj->dpi);
      ExactImage::imageSetYres($image,$obj->dpi);
			if ($obj->nolog==0) {
			  #logit("no resolution in image, so set to ".$obj->dpi);
			}
		  $obj->changedimage(1);
		}
		if ($obj->autocrop==1) {
	    ExactImage::imageFastAutoCrop($image);
	    $obj->changedimage(1);
		}
	  if (checkEmptyPage($obj,$image)==0) {
	    savePageChecks($obj,$image);
		  checkSplitDoc($obj,$image);
      rotate($obj,$image); # check for rotation (according feeder)
		  if ($obj->splitpage) {
        $obj->changedimage(1);
		    $obj->logit("split page");
        my $w = ExactImage::imageWidth($image);
        my $h = ExactImage::imageHeight($image);
		    if ($w>0 && $h>0) {
			    my $w1 = int ($w/2);
			    my $w1a = $w1+1;
			    my $w2 = $w-$w1;
          $image2 = ExactImage::copyImage($image);
          ExactImage::imageCrop($image2,$w1a,0,$w2,$h);
			    ExactImage::imageCrop($image,0,0,$w1,$h);
			  }
		  }
      savePageDo($obj,$image);
		  if ($image2 ne "") {
		    savePageDo($obj,$image2);
        ExactImage::deleteImage($image2);
		  }
	  } else {
		  logit("we got an empty page");
			if ($obj->jobstop ne "" || $obj->maxpages>3) {
		    # check for auto pilot mode (and empty page)
			  logit("jop stopped because of empty page!");
        writeFile($obj->jobstop,\"stop") 
			}
		}
		my $pages2 = $obj->pages2;
    $pages2++;
		$obj->pages2($pages2);
	  ExactImage::deleteImage($image);
		if ($obj->officeimages==1) {
			my $fin = $obj->file;
			$fin = $obj->source2 if checkFile($obj->source2);
			my $fout = $fin.".zip";
			unlinkFile($fout) if checkFile($fout);
			if (checkFile($fin)) {
	    	my $cmd = "zip -q -j '$fout' '$fin' 2>/dev/null 0>/dev/null";
		    my $res = callPrg($cmd);
				if ($res==0) {
					unlinkFile($obj->source2) if checkFile($obj->source2);
					$obj->source2($fout);
				}
			}
		}
  } else {
	  logit("Error: file empty");
    $obj->error(1);
  }
	if ($obj->lastpage>0) {
		# there is a barcode that goes to the last current page of a document
		saveDoc($obj); # close the current doc
		if ($obj->job>0) {
  		$obj->doc(0);
			my $sql = "select value from jobs_data where ".
				"param='SCAN_TO_DOCUMENT' and jid=".$obj->job;
			my @row = $obj->dbh->selectrow_array($sql);
			if ($row[0]>0) {
				my $check=1;
				if ($ENV{'AV_CURRENTPAGE'}>0) {
					$check=0 if $ENV{'AV_CURRENTPAGE'}==$ENV{'AV_MAXPAGES'};
				}
				if ($check==1) {
					checkNewDoc($obj);
					$sql = "update jobs_data set value='".$obj->doc."' ".
						"where jid=".$obj->job;
					$obj->dbh->do($sql);
				}
			}
		}
		$obj->lastpage(0);
	}
  unlinkFile($obj->file) if checkFile($obj->file);
  $obj->last(1) if $obj->error!=0;
}



=head1 savePageBW($obj)

check if we scan color and want to have a tiffg4 file (espceially for iX500)

=cut

sub savePageBW {
  my ($obj) = @_;
  if ($obj->bwoptimize==1 && $ENV{AV_OPTIMIZE_OFF}==0) {
	  $obj->bwdepth(1); # set to black/white optimization
	  if ($obj->depth==24 && $obj->compression>0) {
      # we have a color jpeg image (fast way to transfer it to bw)
		  if ($obj->bwradius==0) {
			  if ($obj->rotate==0 && $obj->splitpage==0 && $obj->autocrop==0) {
			    my $djpeg = "/opt/libjpeg-turbo/bin/djpeg -fast -gray -pnm ";
				  my $pgmtopbm = "/usr/bin/pgmtopbm -threshold";
				  my $pnmtotiff = "/usr/bin/pnmtotiff -g4";
				  my $fin = $obj->file;
					my @parts = split(/\//,$fin);
					my $fout = pop @parts;
					$fout = "/tmp/$fout";
				  my $cmd = "$djpeg $fin | $pgmtopbm | $pnmtotiff >$fout;rm $fin";
				  my $res = callPrg($cmd,1);
					if ($res==0) {
						$obj->file($fout);
						$obj->changedimage(-1);
					} else {
					  logit("$res=>$cmd");
					}
				}
			}
		}
	}
}



=head2 savePageDo($obj,$image)

Process one single page

=cut

sub savePageDo {
  my ($obj,$image) = @_;
	my ($img1,$img2);
  optimizeBW($obj,$image);
  my $bits = ExactImage::imageColorspace($image);
	if ($obj->pages==0) {
		$obj->archtype(1) if $bits eq "gray1" && $obj->archtype==3;
		$obj->archtype(3) if $bits ne "gray1" && $obj->archtype==1;
		$obj->archtype(3) if $bits eq "rgb8" && $obj->archtype==1;
		$obj->archtype(3) if $bits eq "gray8" && $obj->archtype==1;
	}
	if ($obj->archtype!=1) {
		if ($bits eq "gray1") {
	    ExactImage::imageConvertColorspace($image,"gray"); 
      $obj->changedimage(1);
		}
		if ($obj->quality2==1) {
		  logit("compress it all the time");
	    ExactImage::imageInvert($image);
	    ExactImage::imageInvert($image);
      $obj->changedimage(1);
		}
		if ($obj->compression>0 && $obj->changedimage==0) {
			readFile2($obj->file,\$img1);
		} else {
      $img1 = ExactImage::encodeImage($image,"jpeg",$obj->quality,'');
		}
  } else {
	  ExactImage::imageConvertColorspace($image,"gray1") if $bits ne "gray1";
    $img1 = ExactImage::encodeImage($image,"tiff",100,'');
  }  
  if ($obj->factor>0) {
		my $width = ExactImage::imageWidth($image);
		my $height = ExactImage::imageHeight($image);
		if ($width>1024 && $height>786) {
      my $fact1 = $obj->factor/100;
		  if ($obj->archtype!=1) {
        ExactImage::imageScale($image,$fact1);
        $img2 = ExactImage::encodeImage($image,"jpeg",$obj->quality,'');
      } else {
        ExactImage::imageBilinearScale($image,$fact1);
        $img2 = ExactImage::encodeImage($image,"tiff");
      }
	  }
	}
	my $page = $obj->pages;
	$page++;
	$obj->pages($page);
  my $seite = $obj->doc * 1000 + $obj->pages;
	$obj->logit("save docpage:".$seite);
  my $sql = "insert into archivbilder set Seite=$seite,BildInput=";
	$sql .= $obj->dbh2->quote($img1).",Bild=".$obj->dbh2->quote($img2);
	if ($obj->dbh2->do($sql)) {
	  updateDoc($obj,5);
	} else {
	  $obj->error(1);
	}
	MainAccessLog($obj->dbh,$obj->database,$obj->dbh2,$obj->doc,$obj->pages,
	              "add_page",$obj->user,$obj->host) if $obj->error==0;
}



=head createThumbsFirst($dbh,$db,$doc,$seite,$art,$fold,$arch)

Create a small thumb for field Verbindungen in archiv table

=cut

sub createThumbsFirst {
	my ($dbh,$db,$doc,$pages,$art,$arch,$fold,$source2) = @_;
	my $page = 0;
	if ($pages>1) {
  	my $photoMode = getParameterRead($dbh,$db,"PhotoMode");
		if ($photoMode==2) {
			$page = int(rand($pages-1)+1);
			$page = 1 if $page<1;
			$page = $pages if $page>$pages;
		} else {
			$page = 1;
		}
	} elsif ($pages==1) {
		$page=1;
	}
	if ($page>0) {
		logit("add thumb for $doc according $page in $db");
		my $seite = ($doc*1000)+$page;
		my $table = "archivbilder";
		my $archfold = ""; # only needed if it is an archived document
		$table = getBlobTable($dbh,$fold,$arch,$table,$archfold) if $arch==1;
		my $sql = "select BildInput from $db.$table where Seite=$seite";
		my @row = $dbh->selectrow_array($sql);
		if (length($row[0])>0) {
 			my $img0 = ExactImage::newImage();
			my $imgz = "";
			ExactImage::decodeImage($img0,$row[0]);
			my $width = ExactImage::imageWidth($img0);
			my $height = ExactImage::imageHeight($img0);
			if ($width>0 && $height>0) {
				if ($width>150 && $height>150) {
					my $fact0 = 150/$width;
			  	if ($art!=1) {
						my $ok=0;
						if ($ENV{AV_COVER}==1 && -e "$source2") {
							my $fin = "/tmp/$db-$doc-in.png";
							my $fout = "/tmp/$db-$doc-out.png";
							my $cmd = "/usr/bin/ffmpeg -i \"$source2\" $fin 2>/dev/null";
							my $res=callPrg($cmd);
							if ($res==0 && -e "$fin") {
								my $cmd = "convert $fin -colors 4 -scale 150x150 $fout";
								my $res = callPrg($cmd);
								if ($res==0) {
									readFile2($fout,\$imgz,1);
									$ok=1;
								}
								unlinkFile($fin) if checkFile($fin);
							}
						}
						if ($ok==0) {
							logit("normal thumbnailing...");
	    		  	ExactImage::imageThumbnailScale($img0,$fact0);
		 	  			$imgz = ExactImage::encodeImage($img0,"jpeg",25);
						}
	  	  	} else {
	  	  	  ExactImage::imageBilinearScale($img0,$fact0);
		  			$imgz = ExactImage::encodeImage($img0,"tiff");
	  		  }
				} else {
					$imgz = $img0;
				}
				logit("adding thumb with length ".length($imgz)." for $doc in $db");
				$sql = "update $db.archiv set Verbindungen=".$dbh->quote($imgz).
					" where Laufnummer=".$doc;
				$dbh->do($sql);
			}
		} else {
			logit("empty image for $doc at $page in $db");
		}
	}
}



=head1 updateDoc($obj)

Update the document, don't unlock it

=cut

sub updateDoc {
  my ($obj,$step) = @_;
	if (($obj->pages % $step)==0) {
	  my $sql = "update archiv set Erfasst=1,Seiten=".$obj->pages." ".
		          "where Laufnummer=".$obj->doc;
	  $obj->dbh2->do($sql);
  }
}



=head1 saveDoc($obj,$maxpages)

Adjust the current document

=cut

sub saveDoc {
  my ($obj,$maxpages) = @_;
	my $id = "";
	return if $obj->pages==0; # if we have no pages, don't do anything
	if ($obj->pagesstart>0) {
		if ($obj->doc>0) {
	  	for (my $c=$obj->pagesstart;$c<=$obj->pages;$c++) {
			  my $nr = ($obj->doc*1000)+$c;
				my $sql = "select Seite from ".$obj->database.".archivseiten where ".
					"Seite=".$nr;
				my @row = $obj->dbh2->selectrow_array($sql);
				$sql="insert into ";
				$sql ="update " if $row[0]==$nr;
				my $md5 = "";
				if ($obj->md5 ne "" && $c==1) {
					$md5 = ",Schlussel=".$obj->dbh2->quote($obj->md5);
					$obj->md5("");
				}
	  	  $sql .= $obj->database.".archivseiten set Seite=".$nr.",OCR=".$obj->ocr.
					",Erfasst=".$obj->ocrerfasst.",Ausschliessen=".$obj->ocrexclude.$md5;
				$sql .= " where Seite=$nr" if $row[0]==$nr;
  		  $obj->dbh2->do($sql);
			}
		}
	}
	if ($obj->pages>=1) {
		createThumbsFirst($obj->dbh2,$obj->database,$obj->doc,$obj->pages,
			$obj->archtype,0,1,$obj->source2);
	}
	if ($obj->versions ne "") {
	  $id = $obj->versionkey_val;
	  if ($obj->versions_val>0) {
	    my $sql = "select Laufnummer from archiv where ".
				$obj->versions."=".$obj->dbh2->quote($obj->versions_val).
				" and Laufnummer != ".$obj->doc." order by Laufnummer desc";
      my @row = $obj->dbh2->selectrow_array($sql);
	    my $oldnr = $row[0];
	    if ($oldnr>0) {
	      $id = $oldnr;
		    copyValues($obj->dbh2,$obj->database,$obj->fields,$oldnr,$obj->doc);
		  }
		} else {
	    my $sql = "update archiv set ".$obj->versions."=".
			          $obj->dbh2->quote($obj->doc)." where Laufnummer=".$obj->doc;
			$obj->dbh2->do($sql);
		}
	}
	if ($obj->doc>0 && $obj->ftpplus==0) { # check ftpplus once
	  $obj->ftpplus(1);
    my $prg = $val{ftpplus};
    if (checkFile($prg)) {
      # Program is available
      my $cmd = "$prg '".$obj->title."' '".$ENV{AV_SCAN_BASE}."'";
			logit("$cmd");
      my $autofields = `$cmd`;
			logit("ftpplus values:$autofields");
	    AutoFields($obj,$autofields) if $autofields ne "";
		}
	}
	if ($ENV{AV_KFILE} ne "") {
		updateTitelKfile($obj->dbh,$obj->database,$obj->doc,$ENV{AV_KFILE});
	}
	AutoFields($obj,$obj->autofields) if $obj->autofields ne "" && $obj->doc>0;
	my $sqlmedia = "";
	my $flags = "";
  my $mediadb = getParameterRead($obj->dbh2,$obj->database,"MediaFields");
	if ($mediadb==1 && ($obj->pages==1 || $obj->filetyp eq "")) {
		$flags=ExtractMediaInfo($obj,\$sqlmedia);
	}
	if ($obj->nolog==0) { # add if not single page upload from web upload/office
	  FTPaddPagesSource($obj,$maxpages); # pdf file
	  FTPaddPagesSource2($obj); # mail zip file (delete source file)
	}
	my $erfasst="0";
	$erfasst="1" if $obj->pages>0;
  my $sql="update archiv set Erfasst=$erfasst,".
		"Gesperrt='',Seiten=".$obj->pages.",";
	$sql.="Notiz=".$obj->dbh2->quote($flags)."," if $flags ne "";
	$sql.= "ArchivArt=".$obj->archtype." ". # correct type after processing
		"where Laufnummer=".$obj->doc;
	$obj->dbh2->do($sql);
	if ($sqlmedia ne "") {
		my $sql = "update archiv set $sqlmedia where Laufnummer=".$obj->doc;
		logit($sql);
		$obj->dbh2->do($sql);
	}
  my $filename = getParameterRead($obj->dbh2,$obj->database,"FILENAME");
	if ($filename ne "" && $obj->pagesframe_beg==1) {
		my ($type,$name,$ext) = getEDVName($obj->dbh,$obj->database,$obj->doc);
		if ($type eq "office" || $type eq "multi") {
			my $filename0 = $name;
			$filename0 .= "." if $filename0 ne "" && $ext ne "";
			$filename0 .= $ext if $ext ne "";
		  $sql = "update ".$obj->database.".archiv set $filename=".
				$obj->dbh2->quote($filename0)." where Laufnummer=".$obj->doc;
			$obj->dbh2->do($sql);
    }	
  }
	if ($obj->autoprog==1) {
	  foreach (@{$obj->autoprogs}) {
	    AutoPrg($obj,$_);
	  }
	}
	AutoPrg($obj,$obj->database.'.pl'); # call unique program during SaveDoc
	if ($obj->host eq "localhost") {
    my $index = getParameterRead($obj->dbh2,$obj->database,"SEARCH_FULLTEXT");
		if ($index==2) {
	    my $cmd = "/usr/bin/perl /home/cvs/archivista/jobs/sphinxit.pl ".
	              "indexone ".$obj->database." >>/home/data/archivista/av.log";
	    callPrg($cmd,1);
		}
	}
	my $frec = $obj->formrec;
	if ($obj->nolog==0) {
    addLogEntryImport($obj->dbh,$obj->dbh2,$obj->host,$obj->database,
			$obj->user,$obj->password,$obj->doc,$obj->pages,'sne',$frec,1,
			$obj->logdone,$obj->ocrwait);
	}
	if (($obj->pagesframe_beg!=1 && $obj->pagesframe_beg>0) 
	  || $ENV{'AV_EDVNAME_KILL'}==1) { 
	   # remove EDVName in splitted docs or in processweb
     my $sql = "update archiv set EDVName='' where Laufnummer=".$obj->doc;
     $obj->dbh2->do($sql);
	}
	if ($maxpages==0) {
	  # we have a new document, but max_pages was not reached, so init them
	  $obj->pagesframe_beg(0);
		$obj->pagesframe_end(0);
	}
	$sql="select id from jobs where job='LASTDOC' and ".
		"host=".$obj->dbh->quote($obj->host)." and ".
		"db=".$obj->dbh->quote($obj->database)." and ".
		"user=".$obj->dbh->quote($obj->user);
	my @row=$obj->dbh->selectrow_array($sql);
	$sql="update jobs";
	my $jid=$row[0];
	$sql="insert into jobs" if $jid eq "";
   $sql.= " set host=".$obj->dbh->quote($obj->host).",".
		"db=".$obj->dbh->quote($obj->database).",".
		"user=".$obj->dbh->quote($obj->user).",".
		"pwd=".$obj->dbh->quote($obj->doc).",job='LASTDOC'";
	$sql.=" where id=$jid" if $jid ne "";
	#logit($sql);
	$obj->dbh->do($sql);
}



=head1 checkNewDoc($obj)

Check if we need to add a new document

=cut

sub checkNewDoc {
  my ($obj) = @_;
	$obj->error(0);
  if ($obj->doc>0) {
	  if ($obj->locked==0) {
      my $sql = "update archiv set Gesperrt=".
			          $obj->dbh2->quote($obj->lockuser)." ".
		            "where Laufnummer=".$obj->doc." and Gesperrt='' ".
				  			"and Archiviert=0 and ArchivArt=".$obj->archtype;
		  if ($obj->dbh2->do($sql)) {
		    $obj->locked(1);
		  } else {
		    $obj->error(1);
			}
		}
		if ($obj->locked==1 && $obj->pagesloaded==0) {
		  my $sql = "select Seiten from archiv where Laufnummer=".$obj->doc;
			my @row = $obj->dbh2->selectrow_array($sql);
			if ($row[0]<MAX_PAGES) { # we can add more pages
			  $obj->pages($row[0]);
				my $start=$row[0]+1;
				$obj->pagesstart($start);
				$obj->pagesloaded(1);
			} else {
			  $obj->pages(MAX_PAGES);
			}
		}
		if ($obj->pages>=MAX_PAGES) { # got maximum of pages, so add new document
      my $pages = $obj->pages();
		  saveDoc($obj,1);
			$obj->pagesframe_beg($obj->pagesframe_end()+1);
			$obj->pagesframe_end($obj->pagesframe_end()+$pages);
			$obj->doc(0);
			$obj->pages(0);
			$obj->pagesloaded(0);
			$obj->pagesstart(0);
			$obj->locked(0);
		}
	}
	if ($obj->doc==0 && $obj->error==0) {
	  $obj->logit("add new doc for db: ".$obj->database);
    my $date1 = DocumentAddDatForm( TimeStamp() );
		checkOwner($obj);
		checkJobAdmin($obj->dbh2,$obj->database,$obj->host);
		my $ext = 'TIF';
		$ext='JPG' if $obj->depth>1 && $obj->bwoptimize==0;
    my $sql = "insert into archiv set Seiten=0,Ordner=".$obj->folder.",".
		       "Eigentuemer=".$obj->dbh2->quote($obj->owner).",".
					 "UserNeuName=".$obj->dbh2->quote($obj->user).",".
           "Datum=$date1,ErfasstDatum=$date1,ArchivArt=".$obj->archtype.",".
	         "BildInput=1,BildIntern=1,BildInputExt='$ext',Erfasst=0,".
					 "Gesperrt=".$obj->dbh2->quote($obj->lockuser);
		$obj->logit($sql);
		my $res = $obj->dbh2->do($sql);
		if ($res==1) {
		  $obj->logit("added ok");
      # Document is ok, we need the number
      $sql = "select LAST_INSERT_ID()";
      my @row = $obj->dbh2->selectrow_array($sql);
      $obj->doc($row[0]);
			if ($obj->doc>0) {
			  $sql = "update archiv set Akte=".$obj->doc." ".
				       "where Laufnummer=".$obj->doc;
				$obj->logit($sql);
				$obj->dbh2->do($sql);
			} else {
			  $obj->error(1);
			}
			$obj->pages(0);
			$obj->pagesloaded(1);
			$obj->pagesstart(1);
			$obj->locked(1);
		}
  }
}



=head1 $sqlpseudopart=AutoFields($obj,$autofields)

Create an sql fragment from a string Field1=Value1:Field2=Value2

=cut

sub AutoFields {
  my ($obj,$autofields) = @_;
  # check ALWAYS for autofields
  my $sql = ""; # new sql string for barcode information    
  # add the values from autofields
  my @p=split(":",$autofields);
  for (my $idx = 0; $idx <=20; $idx++) {
    my ($f,$v) = split("=",$p[$idx]);
		if ($f eq "" && $v eq "") {
		  last; # no more information go out  
    } else {		
      # there must allways be a fields and value string
      my $sql1 = checkBarcodeAddAutoFields($obj->dbh2,$f,$v,$obj->fields,$obj);
			if ($sql1 ne "") {
			  $sql .= "," if $sql ne "";
			  $sql .= $sql1;  
			}
    }
  }
  if ($sql ne "") {
    # if we could find values for fields, so lets store them
    $sql = "update archiv set $sql where Laufnummer=".$obj->doc;
		logit("AutoFields:$sql");
    $obj->dbh2->do($sql);
  }
}



=head ExtractMediaInfo($obj,$psqlm)

Extract multimedia info and store it to note field in archive table

=cut

sub ExtractMediaInfo {
	my ($obj,$psqlm) = @_;
	my $out = "";
	if ($obj->filetyp eq "office" || $obj->filetyp eq "mail") {
		$$psqlm = "MediaType=".$obj->dbh->quote($obj->filetyp);
		if ($ENV{AV_PATH} ne "") {
			$$psqlm .= ",MediaDevice=".$obj->dbh->quote(fromUTF8($ENV{AV_PATH}));
		}
		my $size = -s $obj->source2;
		if ($size>0) {
			$size = int(($size/(1024*1024)*100)+0.5);
			$size = $size/100;
			$$psqlm .= ",MediaSizeMB=".$obj->dbh->quote($size);
		}
		if ($obj->filetyp eq "office") {
			if ($obj->filename ne "") {
				my $fname = fromUTF8($obj->filename);
				my $fext = fromUTF8($obj->fileext);
				$fname .= "." if $fname ne "" && $fext ne "";
				$fname .= $obj->fileext if $fext ne "";
				$$psqlm .= ",MediaName=".$obj->dbh->quote($fname);
				$$psqlm .= ",MediaCodec=".$obj->dbh->quote(lc($fext));
				if ($obj->officeimages==1) {
					my $width = $obj->filex;
					my $height = $obj->filey;
					$$psqlm.=",MediaWidth=".$obj->dbh->quote($width) if $width>0;
					$$psqlm.=",MediaHeight=".$obj->dbh->quote($height) if $height>0;
				}
			}
		}
	} elsif ($obj->officetyp eq "multi") {
	  my $sound = 0;
	  $sound =1 if $ENV{'AV_MEDIASOUND'}==1;
		my $file = correctFile($obj->source2);
		my $cmd="ffprobe -v error -show_format -show_streams \"$file\"";
		my $info = `$cmd`;
		my @parts = ();
		while ($info =~ /(\[STREAM\])(\n)(.*?)(\[\/STREAM\])/sg) {
			my $part=$3;
			if (index($part,"codec_type=video")>=0) {
				unshift @parts,$part;
			} else {
				push @parts,$part;
			}
		}	
		my %done = ();
		my @values = qw(codec_type codec_name codec_aspect_ratio width height
			duration duration_ts bit_rate bits_per_raw_sample sample_rate
			r_frame_rate);
		foreach my $val (@values) {
			$done{$val}=0;
		}
		foreach my $part (@parts) {
			next if $sound==1 && index($part,"codec_type=audio")==-1;
			foreach my $value (@values) {
				$part =~ /(\n)($value)(=)(.*)/;
				if ($2 eq $value && $3 eq "=" && $4 ne "") {
					my $res = $4;
					next if $res eq "N/A";
					if ($done{$value}==0) {
						$done{$value}=1;
						$out.="$value=$res ";
						if ($value eq "duration" || $value eq "duration_ts" ||
							$value eq "codec_name" || $value eq "codec_type" ||
							$value eq "bits_per_raw_sample" || $value eq "bits_rate" ||
							$value eq "r_frame_rate") {
							ExtractMultimediaField($obj,$psqlm,$value,$res);
						} else {
							ExtractMultimediaField($obj,$psqlm,$value,$res);
						}
					}
				}
			}
		}
		while ($info =~ /(\[FORMAT\])(.*?)(\[\/FORMAT\])/sg) {
			push @parts,$2;
		}	
		foreach my $part (@parts) {
			my @values = qw(filename size);
			foreach my $value (@values) {
				$part =~ /(\n)($value)(=)(.*)/;
				if ($2 eq $value && $3 eq "=" && $4 ne "") {
					my $res = $4;
					if ($value eq "filename") {
						my @parts = split(/\//,$res);
						$res = pop @parts;
					}
					$out.="$value=$res ";
					ExtractMultimediaField($obj,$psqlm,$value,$res);
				}
			}
		}
		if ($ENV{AV_PATH} ne "") {
			$$psqlm .= ",MediaDevice=".$obj->dbh->quote(fromUTF8($ENV{AV_PATH}));
		}
	} else {
		if ($obj->mediainfo ne "") {
			my $res = $obj->mediainfo;
			if ($res ne "") {
				$obj->mediainfo("");
				my @values = qw(Geometry Depth Compression Quality Model filename size); 
				foreach my $value (@values) {
					$res =~ /($value)(\:)(\s)(.*)/;
					if ($1 eq $value && $2 eq ":" && $3 eq " " && $4 ne "") {
						my $res = $4;
						if ($value eq "filename") {
							my @parts = split(/\//,$res);
							$res = pop @parts;
							$out.="$value=$res ";
							ExtractMultimediaField($obj,$psqlm,$value,$res);
						} elsif ($value eq "Geometry") {
							my @parts = split(/\+/,$res);
							$res = shift @parts;
							my ($width,$height) = split('x',$res);
							if ($width>0 && $height>0) {
								$out.="width=$width height=$height ";
								ExtractMultimediaField($obj,$psqlm,"width",$width);
								ExtractMultimediaField($obj,$psqlm,"height",$height);
								ExtractMultimediaField($obj,$psqlm,"codec_type","image");
							}
						} else {
							$out.="$value=$res ";
							ExtractMultimediaField($obj,$psqlm,$value,$res);
						}
					}
				}
			}
		}
	}
	if ($$psqlm eq "") {
		$$psqlm = "MediaType='image'";
		my $dpi = $obj->dpi;
		my $width = $obj->filex;
		my $height = $obj->filey;
		if ($width>0 && $height>0 && $dpi>0 && $obj->pages==1) {
			my $rotation=rotateADF($obj);
			$rotation = (($rotation + ($obj->rotate)) % 360);
			$rotation = (($rotation + $obj->rotate2) % 360);
			if ($rotation>0) {
				my $temp = $width;
				$width = $height;
				$height = $temp;
			}
			$$psqlm.=",MediaWidth=".$obj->dbh->quote($width) if $width>0;
			$$psqlm.=",MediaHeight=".$obj->dbh->quote($height) if $height>0;
			my $depth = $obj->depth;
			my $art = "";
			if ($depth==1) {
				$art = "tif";
			} elsif ($depth>0) {
				$art = "jpg";
			}
			if ($art ne "") {
				$$psqlm.=",MediaBits=".$obj->dbh->quote($depth);
				$$psqlm.=",MediaCodec=".$obj->dbh->quote($art);
			}
			my $nr = ($obj->doc*1000)+1;
			my $sql = "select length(BildInput) from archivbilder ".
				"where Seite=$nr";
	    my @row = $obj->dbh2->selectrow_array($sql);
			my $size = $row[0];
			if ($size>0) {
				$size = int(($size/(1024*1024)*100)+0.5);
				$size = $size/100;
				$$psqlm .= ",MediaSizeMB=".$obj->dbh->quote($size);
			}
		}
	}
	return $out;
}



sub ExtractMultimediaField {
	my ($obj,$psqlm,$field,$res) = @_;
	my $sqlfield = "";
	if ($field eq "codec_type") {
		$sqlfield = "MediaType";
	} elsif ($field eq "codec_name") {
		$sqlfield = "MediaCodec";
	} elsif ($field eq "width") {
		$sqlfield = "MediaWidth";
	} elsif ($field eq "height") {
		$sqlfield = "MediaHeight";
	} elsif ($field eq "duration") {
		$sqlfield = "MediaSeconds";
		$res = sprintf("%.02f",$res);
	} elsif ($field eq "r_frame_rate") {
		my ($first,$second) = split(/\//,$res);
		if ($second>0) {
			$res = (int(($first/$second)*100))/100 if $second>0;
			$sqlfield = "MediaFrames";
		}
	} elsif ($field eq "bits_per_raw_sample" || $field eq "Depth") {
		$sqlfield = "MediaBits";
		$res = int ($res);
	} elsif ($field eq "sample_rate") {
		$sqlfield = "MediaRate";
	} elsif ($field eq "size") {
		$sqlfield = "MediaSizeMB";
		$res = int(($res/(1024*1024)*100)+0.5);
		$res = $res/100;
	} elsif ($field eq "filename") {
		$sqlfield = "MediaName";
	} elsif ($field eq "Model") {
		$sqlfield = "MediaDevice";
	}
	if ($sqlfield ne "") {
		$$psqlm .= "," if $$psqlm ne "";
		$res=fromUTF8($res) if $sqlfield eq "MediaName";
		$$psqlm .= "$sqlfield=".$obj->dbh2->quote($res);
	}
}



=head1 AutoPrg($obj,$prg)

Update the document with programmed fields

=cut

sub AutoPrg {
  my ($obj,$prg) = @_;
  my $prgdo = $val{autoprg} = "/home/data/archivista/cust/autofields/".$prg;
	if (checkFile($prgdo) && $obj->doc>0) {
		my $pw1 = $obj->password;
		$pw1 =~ s/\!/\\\!/g;
		my $prg = "$prgdo ".$obj->host." ".$obj->database." ".
		        $obj->user." ".$pw1." ".$obj->doc;
		my $log = "$prgdo ".$obj->host." ".$obj->database." ".
		        $obj->user." ***** ".$obj->doc;
	  callPrg($prg,0,$log);
	}
}



=head2 savePageChecks($obj,$image)

Check if and to which doc (i.e. barcodes) we can save a page

=cut

sub savePageChecks {
  my ($obj,$image) = @_;
  # do the barcode recognition
  my $barcode = "";
	if ($obj->barcodedef>0 && checkInitialized("24162736f6465637")) {
	  if (check64bit()==64) {
	    $barcode = checkBarcode64($obj,$image);
		} else {
	    $barcode = checkBarcode($obj,$image);
		}
		# check if current == last barcode AND singe page mode is not on
	  $barcode="" if $barcode eq $obj->lastbarcode && $obj->bc_singlepages==0;
	  $obj->lastbarcode($barcode) if $barcode ne ""; # hold last barcode
    if ($barcode ne "") {
		  logit("new barcode:".barcodeOneLiner($barcode));
			$obj->lastpage(1) if $obj->bc_lastpage==1;
      # the barcode is ok, so we need anyway a new record
			if ($obj->pages>0) {
				if ($obj->bc_lastpage==0) {
	      	saveDoc($obj); # close the current doc
			    $obj->doc(0);
				}
			} else {
			  if ($obj->doc>0) {
			    # check if we really have no pages (processweb.pl upload)
		      my $sql = "select Seiten from archiv where Laufnummer=".$obj->doc;
			    my @row = $obj->dbh2->selectrow_array($sql);
			    $obj->pages($row[0]);
				  if ($obj->pages>0) {
						if ($obj->bc_lastpage==0) {
				    	my $oldlog = $obj->nolog(0);
		          saveDoc($obj); # close the current doc
					    $obj->nolog($oldlog);
	  		      $obj->doc(0);

						}
					}
				}
			}
      if ($obj->bc_doc==1) {
        my $sql = "select Laufnummer,Seiten from archiv where " .
                  "Laufnummer=$barcode";
        my @row1=$obj->dbh2->selectrow_array($sql);
        if ($row1[0]>0 && $row1[1]==0) {
          # store to an existing document (if no pages are there)
          $obj->doc($barcode); # barocde is document number
					$sql = "update archiv set UserNeuName=".
					       $obj->dbh2->quote($obj->user)." where ".
								 "Laufnummer=".$obj->doc." and Gesperrt=''";
					$obj->dbh2->do($sql);
				  $obj->pages(0);
				  $obj->pagesloaded(0);
				  $obj->pagesstart(0);
				  $obj->locked(0);
		    }
		  }
		  checkNewDoc($obj);
      # we need a pointer to an array with all fields, where we find a hash
      # with name, type, size -> so we can save the values to sql
      my @bc = split(";",$obj->bc_string);
      my $bcs=21; # start of the first sub setting
      my $sql = ""; # new sql string for barcode information  
    	my $c=0;
	    foreach(@{$obj->fields}) { # initialize bcvalue (no side effects)
		    ${$obj->fields}[$c]->{bcvalue} = "";
	      $c++;
	    }
			my @barcodes = split(/\n/,$barcode);
			my $bcresultnr=0;
			foreach my $barcodex (@barcodes) {
				$bcresultnr++;
        for (my $idx = 0; $idx <=31; $idx++) {
          # get the actual sub setting of the barcode definition
          my $bcset = $bc[$bcs+$idx];
					if ($bcset ne "" && $bcset ne "-1,-1,-1,-1,-1") {
						my $sql1 = checkBarcodeAddField($obj,$barcodex,$bcset,$bcresultnr);
						if ($sql1 ne "") {
						  $sql .= "," if $sql ne "";
							$sql .= $sql1;
						}
					}
				}
      }
      # update the values from the barcode
      if ($sql ne "") {
        # if we could find values for fields, so lets store them
        $sql = "update archiv set $sql where Laufnummer=".$obj->doc;
				logit("Barcodes:$sql");
        $obj->dbh2->do($sql);
      }
	  }
	}
	checkNewDoc($obj) if $obj->initialized==0;
} 
 


=head2 checkOwner($obj)

If activated set lowercase for user and if none gives a default user

=cut

sub checkOwner {
  my ($obj) = @_;
  if ($obj->lcuser==1 || $obj->defuser ne "") {
	  my $tmpuser = $obj->owner;
		my $owner = "";
    # if User not SYSOP or Admin
    unless ($tmpuser eq "SYSOP" || $tmpuser eq "Admin" ) {
		  if ($obj->lcuser == 1) {
        # Check if both are uppercase if not set everything to lowercase
        my $tmp = uc($tmpuser);
        if ($tmpuser ne $tmp) {
          # We have not uppercase so change it to lowercase
          # Set UserName to Lowercase if the option is activated
          $owner = lc($tmpuser);
				}
			}
			if ($obj->defuser ne "" && $tmpuser eq "") {
			  # we have no value in Eigentuemer field and a def user
			  $owner = $obj->defuser; 
			}
		}
		$obj->owner($owner) if $owner ne "";
  }
}



=head2 $barcode=checkBarcode($obj,$image)

Do the barcode recognition and gives back the barcode if it does match

=cut

sub checkBarcode {
  my ($obj,$image) = @_;
  my $barcode1;
	if ($obj->bc_stretch==1) {
    $barcode1 = ExactImage::imageDecodeBarcodesExt($image,$obj->bc_type,
		            $obj->bc_length,$obj->bc_length);
	} else {
	  my $cimage = ExactImage::copyImage($image);
    ExactImage::imageScale($cimage,1,$obj->bc_stretch);
    $barcode1 = ExactImage::imageDecodeBarcodesExt($cimage,$obj->bc_type,
		            $obj->bc_length,$obj->bc_length);
	  ExactImage::deleteImage($cimage);
	}
  my @barcodes;
  for (my $i;$i<scalar(@$barcode1);$i+=2) {
    push @barcodes,@$barcode1[$i];
  }
  my $barcode;
  foreach (@barcodes) {
    # do a check for every barcode, the first one we take
    $barcode = $_;
		$obj->logit("barcode found:$barcode");
    chomp $barcode;
    if ($barcode ne "" && $obj->bc_chars ne "") {
      # if we have a barcode and if we have checkChars, we test the first chars
      my @checks = split(",",$obj->bc_chars);
      my $found = 0;
      foreach (@checks) {
        my $check1 = $_;
        my $fpos = index $barcode,$check1;
        $found=1 if $fpos==0;
      }
      $barcode="" if $found==0;
    }
    last if $barcode ne "";
  }
  return $barcode;
}



=head2 $barcode=checkBarcode64($obj,$image)

Do the barcode recognition and gives back the barcode if it does match

=cut

sub checkBarcode64 {
  my ($obj,$image) = @_;
	my $barcode = "";
	my ($typ1,$typ2) = split(/\|/, $obj->bc_type);
	if ($typ1 eq "esr") {
		$barcode=checkBarcode64esr($obj,$image);
	} else {
		my $alter = 0;
		$barcode=checkBarcode64normal($obj,$image,$typ1,$typ2,$alter);
		if ($barcode eq "" && $obj->bc_engine==0) {
			$alter = 1;	
			$barcode=checkBarcode64normal($obj,$image,$typ1,$typ2,$alter);
		}
	}
	return $barcode;
}

sub checkBarcode64esr {
  my ($obj,$image) = @_;
	my $copy=0;
	my $move=0;
	my $barcode="";
	my $file2=checkBarcode64checkfile($obj,\$copy,\$move,1);
	my $file3 = $file2;
	$file3 =~ s/\/go\//\//;
	if ($obj->depth>1) {
	  $file3 .= ".jpg";
	} else {
	  $file3 .= ".tif";
	}
	my $file4 = $file3.".txt";
  unlinkFile($file4) if checkFile($file4);
	my $cmd = "tesseract -l ocb $file3 $file3 0>/dev/null 2>/dev/null";
	my $res = callPrg($cmd,1);
	if ($res==0) {
	  readFile2($file4,\$barcode,1);
		$barcode=checkBarcode64esrcheck($barcode);
	}
 	unlinkFile($file3) if checkFile($file3);
	if ($barcode eq "") {
		$file2=checkBarcode64checkfile($obj,\$copy,\$move,2);
		my $file3 = $file2;
		$file3 =~ s/\/go\//\//;
		if ($obj->depth>1) {
		  $file3 .= ".jpg";
		} else {
		  $file3 .= ".tif";
		}
		$cmd = "tesseract -l ocb $file3 $file3 2>/dev/null 0>/dev/null";
		$res = callPrg($cmd,1);
		if ($res==0) {
	  	readFile2($file4,\$barcode,1);
			$barcode=checkBarcode64esrcheck($barcode);
		}
	}
 	unlinkFile($file3) if checkFile($file3);
	callPrg("rm -f $file2",1) if $copy==1;
	moveFile($file2,$obj->file) if $move==1;
	return $barcode;
}

sub checkBarcode64esrcheck {
	my ($text) = @_;
	my $part="";
	$text=~s/(O)/0/g;
	$text=~s/(D)/0/g;
	$text=~s/(o)/0/g;
	$text=~s/(z)/2/g;
	$text=~s/(&)/8/g;
	$text=~s/\r/ /g;
	$text=~s/\n/ /g;
	$text=~s/\s{1,10}/ /g;
	$text=~/([0-9]+)(\>)([0-9]+)(\+{1,1})(\s{1,1})([0-9]+)(\>)/;
	if ($1 ne "" && $2 eq ">" && $3 ne "" && $4 eq "+" && 
		$5 eq " " && $6 ne "" && $7 eq ">") {
		$part="$1>$3+ $6>";
		#logit("esr found with $part");
	} else {
		my $text1=$text;
		$text=~s/(\>{1,1})(\s+)/$1/g;
		$text=~s/(\s+)(\>{1,1})/$2/g;
		$text=~s/(\>{1,1})([0-9]+)(\s)([0-9]+)(\+{1,1})/$1$2$4$5/g;
		$text=~/([0-9]+)(\>)([0-9]+$)(\+{1,1}$)/;
		if ($1 ne "" && $2 eq ">" && $3 ne "" && $4 eq "+") {
			$part="$1>$3+ ";
			#logit("esr part 1 found with $part");
			$text=~/([0-9]+)(\>)/;
			if ($1 ne "" && $2 eq ">") {
				$part.="$1>";
				#logit("esr part 2 found with $part");
			} else {
				$part="";
			}
		} else {
			$text=$text1;
			$text=~/([0-9]+)(\+{1,1})(\s{1,1})([0-9]+)(\>)(\s{1,1})([0-9]+)(\>)/;
			if ($1 ne "" && $2 eq "+" && $3 eq " " && $4 ne "" && $5 eq ">" && 
				$6 eq " " && $7 ne "" && $8 eq ">") {
				$part="$1+ $4> $7>";
			} else {
				$part="";
			}
		}
	}
	my $lang=length($part);
	my $first=index($part,">");
	my $second=index($part,"+");
	logit("check esr with $part ($lang,$first,$second)") if $lang>0;
	my $lang2=0;
	if ($first==3) {
		# without amount (short ref=16, long ref=27)
		$part="" if $second!=20 && $second!=31;
	} elsif ($first==13) {
		# with amount (short ref=16, long ref=27)
		$part="" if $second!=30 && $second!=41;
	} else {
		$lang=length($part);
		$first=index($part,"+");
		$second=index($part,">");
		my $third=0;
		if ($second>0) {
			$third=index($part,">",$second+1);
		}
		if ($first==27 && $second==38 && $third==49) {
			# nothing to do	
		} else {
			$part="";
		}
	}
	$lang2=$second+12 if $part ne ""; # check if correct length at all
	$part="" if $lang2!=$lang;
	return $part;
}

sub checkBarcode64checkfile {
	my ($obj,$pcopy,$pmove,$type) = @_;
	my $file2 = "";
	if ($obj->depth>1) {
	  $file2 = $obj->file.".jpg";
	} else {
	  $file2 = $obj->file.".tif";
	}
	callPrg("rm -f $file2",1) if checkFile($file2);
 	if (notFile($file2)) {
		my $cmd2 = "identify -ping ".$obj->file;
		my $res = `$cmd2`;
		#logit("$res=>$cmd2");
		if ($res =~ /(JPEG)/) {
			if ($type>0) {
				callPrg("cp ".$obj->file." $file2",1);
				$$pcopy=1;
			} else {
				moveFile($obj->file,$file2);
				$$pmove=1;
			}
		} else {
	    $$pcopy=1;
			$cmd2 = "econvert -i ".$obj->file;
			$cmd2 .= " --resolution ".$obj->dpi if $obj->changedimage==1;
			$cmd2 .= " -o $file2";
			$res = callPrg($cmd2,1);
		}
	} else {
		$file2 = $obj->file;
	}
	if ($type>0) {
		my $cmd = "";
		my $res = 0;
		my $file3 = $file2;
		$file3 =~ s/\/go\//\//;
		if ($obj->depth>1) {
	  	$file3 .= ".jpg";
		} else {
	  	$file3 .= ".tif";
		}
  	unlinkFile($file3) if checkFile($file3);
		if ($type==1) {
			$cmd = "econvert -i $file2 --crop 750,3220,1750,180 -o $file3";
		} else {
			$cmd = "econvert -i $file2 --crop 750,960,1750,280 -o $file3";
		}
		$res=callPrg($cmd,1);
	}
	return $file2;
}

# get type of barcode if not choosen in definition

sub checkBarcode64normal2 {
	my ($obj,$prg,$alter) = @_;
	my $barcodes2 = 0;
  if (checkFile($prg)) {
 	  my $dbh2 = $obj->dbh2;
    my $db1 = $obj->database;
		my $sql = "select Inhalt from $db1.parameter where Name = 'BARCODES2'";
	  my @res = $dbh2->selectrow_array($sql);
	  $barcodes2 = $res[0];
		if ($alter==1) {
			if ($barcodes2==0) {
				$barcodes2=1;
			} else {
				$barcodes2=0;
			}
		}
	}
	return $barcodes2;
}

sub checkBarcode64normal {
	my ($obj,$image,$typ1,$typ2,$alter) = @_;
  my $barcode; # all bacodes
  my $barcode1; # one barcode (testing and checks)
	my $opt = "";
	my $cmd = "";
	my $lib = "/usr/lib/libtiff.so.3";
	my $prg = "/home/data/archivista/cust/bardecode/bardecode";
	my $copy = 0;
	my $move = 0;
	my $file2 = "";
	my $barcodes2 = 0;
	if ($obj->bc_engine==0) {
	  $barcodes2 = checkBarcode64normal2($obj,$prg,$alter);
	} elsif ($obj->bc_engine==1) {
	  $barcodes2 = 0;
	} else {
	  $barcodes2 = 1 if checkFile($prg);
	}
  logit("alternative barcode recognition activated:".$barcodes2);
	if ($barcodes2==1) {
	  if (notFile($lib)) {
		  my $cmd="cp /home/data/archivista/cust/bardecode/libtiff.so.3 /usr/lib";
		  callPrg($cmd,1);
			callPrg("ldconfig",1);
		}
	  if ($obj->bc_length>0 && $obj->bc_script eq "") {
	    $opt .= " -c ".$obj->bc_length." -C ".$obj->bc_length;
    }
		my $typ = "any";
		if ($typ1 ne "any" || $typ2 ne "any") {
		  $typ="";
		  if ($typ1 ne "") {
			  $typ = $typ1;
			}
			if ($typ2 ne "") {
			  $typ .= "|" if $typ ne "";
        $typ .= $typ2;
			}
		}
		$opt .= " -t \"$typ\"";
		$opt .= " -m" if $obj->allbarcodes==1;
		$file2=checkBarcode64checkfile($obj,\$copy,\$move,0);
		if ($typ eq "qrcode") {
			$prg.="2" if -e "/home/data/archivista/cust/bardecode/bardecode2";
		}
	  $cmd = "$prg $opt $file2";
	} else {
  	if ($typ1 eq "any") {
	    $opt = "-Senable";
  	} elsif ($typ1 ne "" && $typ2 eq "") {
	    $opt = "-Sdisable -S".checkBarcode64Typ($typ1).".enable";
  	} elsif ($typ1 ne "" && $typ2 ne "") {
  	  $opt = "-Sdisable -S".checkBarcode64Typ($typ1).".enable ".
	  	       "-S".checkBarcode64Typ($typ2).".enable";
	  }  
	  if ($obj->bc_length>0 && $obj->bc_script eq "") {
	    $opt .= " -Smin-length=".$obj->bc_length." -Smax-length=".$obj->bc_length;
    }
	  $opt .= " --raw";
	  $cmd = "zbarimg $opt ".$obj->file;
	}
	logit($cmd);
	my $barcode0 = `$cmd 2>/dev/null`;
	logit("result of bc recognition:".barcodeOneLiner($barcode0));
	if ($barcode0 ne "" && $obj->bc_script ne "") {
	  logit("starting script ".$obj->bc_script);
	  my $bcin = "/tmp/in.bc";
		my $bcout = "/tmp/out.bc";
	  writeFile($bcin,\$barcode0,1);
		$barcode0="";
    my $prg = "/home/data/archivista/cust/autofields/".$obj->bc_script;
		if (-e $prg) {
			unlink $bcout if -e $bcout;
		  my $res = callPrg("$prg $bcin $bcout");
			if ($res==0 && -e $bcout) {
			  readFile2($bcout,\$barcode0,1);
				logit("barcode after processing:".barcodeOneLiner($barcode0));
			}
		}
	}
	callPrg("rm -f $file2",1) if $copy==1;
	moveFile($file2,$obj->file) if $move==1;
	if ($typ1 ne "qrcode") {
		my @barcode1 = split(/\n/,$barcode0);
	  my @barcodes = ();
		my $c=0;
		foreach (@barcode1) {
		  if ($barcode1[$c] ne "") {
    	  push @barcodes,$barcode1[$c];
			}
			$c++;
	  }
	  foreach (@barcodes) {
  	  # do a check for every barcode, the first one we take
    	my $barcode1 = $_;
			$obj->logit("barcode found:$barcode1");
  	  chomp $barcode1;
	    if ($barcode1 ne "" && $obj->bc_chars ne "") {
  	    # if we have a barcode and if we have checkChars, test the first chars
	      my @checks = split(",",$obj->bc_chars);
  	    my $found = 0;
    	  foreach (@checks) {
      	  my $check1 = $_;
        	my $fpos = index $barcode1,$check1;
	        $found=1 if $fpos==0;
  	    }
    	  $barcode1="" if $found==0;
	    }
			$barcode.="\n" if $barcode ne "";
			$barcode.=$barcode1 if $barcode1 ne "";
      last if $barcode eq "";
		}
  } else {
		$barcode = $barcode0;
		$barcode =~ s/\r//g;
		$barcode = fromUTF8($barcode);
	}
  return $barcode;
}

sub barcodeOneLiner {
	my ($barcode0) = @_;
	my @codes = split(/\n/,$barcode0);
	my $barcode0a = "";
	my $ccode=1;
	if ($codes[1] ne "") {
		foreach my $code (@codes) {
			$code =~ s/\r//g;
			$barcode0a.="--" if $barcode0a ne "";
			$barcode0a.="$ccode=$code";
			$ccode++;
		}
	} else {
		$barcode0a=$barcode0;
	}
	return $barcode0a;
}



=head1 $type=checkBarcode64Type($typ32)

Convert the code names between old and new barcode solution

=cut

sub checkBarcode64Typ {
  my ($typ) = @_;
	if ($typ eq "code25") {
	  $typ = "i25";
	}
	return $typ;
}



=head2 $sqlpart=checkBarcodeAddField($obj,$barcode,$bcset,$notfirst)

We find out which part of the barcode goes to wich field
  
=cut

sub checkBarcodeAddField {
  my ($obj,$barcode,$bcset,$bcresultnr) = @_;
  my @bc1 = split(",",$bcset);
	my $bcBarcode=$bc1[0]; # desired barcode to match
  my $bcLength=$bc1[1]; # length of the barcode
  my $bcField=$bc1[2]; # field to store the barcode
  my $bcStart=$bc1[3]; # start position in barcode
  my $bcCharacter=$bc1[4]; # end position in barcode
  my $sql;
  my $subBarcode = $barcode;
	if ($bcBarcode==$bcresultnr) {
	  if (length($barcode) == $bcLength || $bcLength==0) {
  	  if ($bcLength>0 && $bcStart>0 && $bcCharacter>0) {
    	  $subBarcode = substr $subBarcode,$bcStart-1,$bcCharacter;
	    }
  	  # remove all leading spaces
    	$subBarcode =~ s/^\s+//;
    	# we don't want Akte/Seite, but the corresponding field nr
    	my $c=0;
	    my $cok=0;
  	  my $cres=-1;
    	foreach(@{$obj->fields}) {
      	my $name=${$obj->fields}[$c]->{name};
	      if ($name ne "Seiten" and $name ne "Akte") {
  	      if ($cok==$bcField) {
    	      $cres=$c;
      	  }
        	$cok++;
	      }
  	    $c++;
    	}
			if ($cres>=0) {
  		  my $name=${$obj->fields}[$cres]->{name};
				if ($name ne "Seiten" && $name ne "Akte") {
		  	  my $type=${$obj->fields}[$cres]->{type};
  		  	my $size=${$obj->fields}[$cres]->{size};
	    		if ($type eq "datetime") {
	  	    	# we have a date
		    	  if (length($subBarcode)==6) {
  		    	   # date has six chars, go to eight chars
	  		       my $y=substr $subBarcode,0,2;
  	  		     if ($y<=30) {
    	  		     $subBarcode="20$subBarcode";
      	  		 } else {
        	  		 $subBarcode="19$subBarcode";
			         }
  			    }
    			  # format the date string
      			$sql="$name=".DocumentAddDatForm("$subBarcode");
		    	} elsif ($type eq "varchar") {
				  	if (${$obj->fields}[$cres]->{bcvalue} ne "") {
				    	$subBarcode = ${$obj->fields}[$cres]->{bcvalue}." ".$subBarcode;
						}
				  	${$obj->fields}[$cres]->{bcvalue} = $subBarcode;
	  		    # we have a text value
  	  		  # if field has less chars then subbarcode, then adjust it
    	  		$subBarcode=substr $subBarcode,0,$size if length($subBarcode)>$size;
		    	  $sql="$name=".$obj->dbh2->quote($subBarcode);
		  	  } elsif ($type eq "int" || $type eq "bigint" || 
						$type eq "tinyint" || $type eq "double") {
    		  	# we have a number value
						if ($size>0 && length($subBarcode)>$size) {
	    		  	$subBarcode=substr $subBarcode,0,$size; # length limit to value
						}
	  	    	$sql="$name=$subBarcode";
		    	}
  				checkLinkFields($obj->dbh2,\$sql,$name,$type,$subBarcode);
		  	}
			}
		}
	}
  return $sql;
}



=head2 checkLinkFields($dbh,$psql,$name,$type,$val,$doc)

Check if we have a code/definition pair for a given value

=cut

sub checkLinkFields {
  my ($dbh,$psql,$name,$type,$val,$doc) = @_;
  return if $type eq "datetime";
  if ($name ne "") {
    if ($type eq "int") {
      $val = int $val;
    }
    # give back the connected field (if available)
    my $sql1="select Definition,FeldDefinition from feldlisten where " .
             "FeldCode=".$dbh->quote($name)." and " .
             "Code=".$dbh->quote($val)." limit 1";
    my @fld = $dbh->selectrow_array($sql1);
    my $def2 = $fld[0];
    my $fld2 = $fld[1];
		if ($def2 ne "" && $fld2 ne "") {
	    $$psql .= ",$fld2=".$dbh->quote($def2);
		} else {
			$sql1 = "select Laufnummer from feldlisten where FeldDefinition=".
			$dbh->quote($name)." and FeldCode='Laufnummer' limit 1";
   		@fld = $dbh->selectrow_array($sql1);
			if ($fld[0]>0) {
				my @best = split(/;/,$val);
				foreach my $best1 (@best) {
					$best1=~s/^\s+|\s+$//g;
					if ($best1 ne "" && $doc>0) {
						$sql1 = "insert into feldlisten set FeldDefinition=".
							$dbh->quote($name).",Definition=".
							$dbh->quote($best1).",Code=".$dbh->quote($doc);
						$dbh->do($sql1);
					}
				}
			}
		}
  }
}



=head2 $sqlpart=checkBarcodeAddAutoFields($dbh,$field,$val,@$fields,$notfirst)

We add desired default values to the database
  
=cut

sub checkBarcodeAddAutoFields {
  my ($dbh,$fld,$val,$pfields,$obj) = @_;
  my @fields = @$pfields;
  my $val2 = unescape($val);
	if ($val2 ne $val) {
	  logit("Unescape:$val=>$val2");
		$val = $val2;
	}
	if ($fld eq "EDVName") {
		my ($typ,$fname,$ext) = splitEDVName($val);
		$obj->filename($fname);
		$obj->fileext($ext);
		$obj->filetyp($typ);
	}
	#my $val3 = fromUTF8($val);
	#if ($val3 ne $val) {
	#  logit("FromUTF8:$val=>$val3");
	#  $val = $val3;
	#}
  my ($cres,$c,$sql);
  $cres=-1;
  $c=0;
  foreach(@fields) {
    my $name = $fields[$c]->{name};
    if ($name ne "Seiten" and $name ne "Akte") {
      # compare to the actual field name
      if ($name eq $fld) {
        $cres=$c;
      }
    }
    $c++;
  }
  if ($cres>=0) {
    my $name=$fields[$cres]->{name};
    my $type=$fields[$cres]->{type};
    my $size=$fields[$cres]->{size};
		if ($name eq "Eigentuemer") {
		  $val = substr($val,0,15);
			$obj->owner($val);
			checkOwner($obj);
			$val = $obj->owner();
		}
    if ($type eq "datetime") {
      # we have a date
      if (length($val)==6) {
        # date has six chars, go to eight chars
        my $y=substr $val,0,2;
        if ($y<=30) {
          $val="20$val";
        } else {
          $val="19$val";
        }
      } elsif (length($val)==10) {
        # Date in format ex: 25.06.2006
        my ($day,$month,$year) = split(/\./,$val);
        $val = "$year$month$day"; 
      }
      # format the date string
      $sql="$name=".DocumentAddDatForm("$val");
    } elsif ($type eq "varchar") {
      # we have a text value
      # if field has less chars then subbarcode, then adjust it
      $sql="$name=".$dbh->quote($val);
    } elsif ($type eq "int" || $type eq "bigint" || 
			$type eq "tinyint" || $type eq "double") {
      # we have a number value
      $sql="$name=$val";
    }
    checkLinkFields($dbh,\$sql,$fld,$type,$val,$obj->doc);
  }
  return $sql;
}



=head2 $supr=checkEmptyPage($obj,$tmpimg)

Checks if the page is empty (1=empty, 0=not activated or not empty)

=cut

sub checkEmptyPage {
  my ($obj,$tmpimg) = @_;
	my $emptypages = $obj->checkemptypage;
  my $rand = 0;
	my $supress = 0;
  ( $emptypages, $rand ) = split /\./, $emptypages;
  $emptypages = $emptypages / 1000;
	# we downsize the image later to 1/8, so reduce margin
	$rand = int($rand/8) if $rand>0 && $obj->depth>1;
  # rand pixel must be a maximum of 8
  $rand = $rand - ( $rand % 8 );
  if ( $emptypages > 0 ) {
    # empty pages are activated
    $supress = 1;
    # Make a working copy for isEmpty so that the original is still unmodified
    my $tmpimg2 = ExactImage::copyImage($tmpimg);
    if ($obj->depth>1) {
      ExactImage::imageBoxScale($tmpimg2,0.125);
      # convert a colour/grayscale image to black/white
      ExactImage::imageOptimize2BW($tmpimg2,0,0,128,1,2.1);
    }
    $rand = 0 if $rand < 0;
    # black/white is analysis
    my $res = ExactImage::imageIsEmpty($tmpimg2,$emptypages,$rand);
		$supress=0 if $res!=1;
    ExactImage::deleteImage($tmpimg2);
  }
  return $supress;
}



=head2 $res=writeFile($file,$pcontent,$killold)

Saves a file (if needed with check) from a pointer variable (1=success)

=cut

sub writeFile {
  my ($file,$pcontent,$killold) = @_;
  my $res=0;
  if ($killold) {
    unlinkFile($file) if checkFile($file);
  }
  if (notFile($file)) {
    open(FOUT,">>$file");
    binmode(FOUT);
    print FOUT $$pcontent;
    close(FOUT);
    $res=1 if checkFile($file);
  }
  return $res;
}



=head2 readFile($file,\$memory)

Reads a file and stores its contents to a pointer (file must exist)

=cut

sub readFile {
  my ($file,$pmemory) = @_;
  open( FIN, $file );
  binmode(FIN);
	$$pmemory = undef; # reset pmemory 
	while(my $line = <FIN>) {
	  $$pmemory .= $line;
	}
  close(FIN);
}



=head1 readFile2($file,$pout,$killit)

Get the file back as a pointer

=cut

sub readFile2 {
  my ($file1,$pout,$killit) = @_;
  my $buf = "";
	$$pout = "";
	eval { open (FH, '< :raw', $file1) or die $!; };
	my $length = 4096*1024;
  while (my $read = sysread( FH, $buf, $length) ) {
    $$pout .= $buf;
	}
	if ($killit==1) {
	  unlinkFile($file1) if checkFile($file1);
	}
}

=head2 optimizeBW($obj,$image)

Check if we need a black/white optimization

=cut

sub optimizeBW {
  my ($obj,$image) = @_;
  if ($obj->bwoptimize==1 && $ENV{AV_OPTIMIZE_OFF}==0) {
	  my $changed = 1;
	  $obj->bwdepth(1); # set to black/white optimization
	  if ($obj->depth>1) {
      # we have a gray or color image
		  if ($obj->bwradius==0) {
			  if ($obj->changedimage==-1) {
				  $changed=0;
				} else {
		      # no optimization, only convert to bw (no scaling)
          ExactImage::imageConvertColorspace($image,"bw");
				}
		  } else {
        # we need to have a look in the scan definitions
	      my $bwoutdpi = $obj->bwoutdpi;
	      $bwoutdpi=0 if ExactImage::imageXres($image)==$obj->bwoutdpi;
        ExactImage::imageOptimize2BW($image,0,0,$obj->bwthreshold,
		                              $obj->bwradius,2.1,$bwoutdpi);
		  }
		}
    $obj->changedimage($changed);
	}
}



=head2 $val=getParameterRead($dbh,$db1,$val)

Read a value form the parameter table and give it back as string

=cut

sub getParameterRead {
  my ($dbh,$db1,$val) = @_;
  return getParameterReadWithType($dbh,$db1,'parameter',$val);
}



=head2 $val=getBoxParameterRead($dbh,$db1,$val,$min,$max,$def)

Read a BOX value form the parameter table and give it back as string

=cut

sub getBoxParameterRead {
  my ($dbh,$db1,$val,$min,$max,$def) = @_;
  my $res = getParameterReadWithType($dbh,$db1,'ArchivistaBox',$val);
  $res=$def if ($res<$min || $res>$max);
  return $res;
}



=head2 $val=getParameterReadWithType($dbh,$db1,$type,$val)

Reads a value from the parameter table and gives it back as a string

=cut

sub getParameterReadWithType {
  my ($dbh,$db1,$type,$val) = @_;
  my $val1 = $dbh->quote($val);
  my ( $sql, @row, );
  # we need to have a look in the scan definitions
  $sql = "select Inhalt from $db1.parameter where Art = " .
         $dbh->quote($type) . " AND Name=$val1";
  @row = $dbh->selectrow_array($sql);
  return $row[0];
}



=head2 FTPaddPages($dbh,$host,$db,$user,$pw,$ftpin,$logid,$fout,$pfinfo) 

Add all pages of a given pdf/ps file with sane-client.pl

=cut

sub FTPaddPages {
  my ($fout,$pfinfo) = @_;
  my $ftpin  = $val{ftpin};
  # check if we have a pdf AND (optional) postscript file (cups)
  my ( $pdffile, $psfile ) = _FTPgetFileNames( $ftpin, $pfinfo );
  # check for the desired database, so we can get options
  my $destdb = $$pfinfo{Destination};
  $destdb =~ s/(.*?)(-)(.*?)$/$3/;
	my $dbh = MySQLOpen();
	if ($dbh) {
    if (HostIsSlave($dbh)==0) {
			my $pdfcheck = "";
	  	my $db = $val{db};
		  $db = $destdb if checkDatabase($dbh,$destdb);
  	  $ENV{'AV_OPTIMIZE_OFF'} = 1; # we dont want B/W optimisation
	  	if ($$pfinfo{'Profile'} eq "" && $$pfinfo{'NoOCRfromCUPS'}==0) {
      	# load the first definition if there is none available
			  # AND if we are NOT in CUPS mode
  	    $$pfinfo{'Profile'}=getScanDefByNumber($dbh,$db,0);
	  	}
	    my $checkdef=getScanDefByName($dbh,$db,$$pfinfo{'Profile'});
  	  my @valdefs=split(";",$checkdef);
		  if ($valdefs[0] eq $$pfinfo{'Profile'} && $$pfinfo{'Profile'} ne "") {
			  # if we have only the name of a def and the name exists,
			  # we switch to this scan def 
		    $$pfinfo{'Profile'} = $checkdef;
  	    $ENV{'AV_OPTIMIZE_OFF'} = 0; # we dont want B/W optimisation
		  }
		  $dbh->disconnect();
	    my $fields = $$pfinfo{'Fields'};
  	  $ENV{'SCAN_FIELDS'} = $fields;
		  my $rotate = 0;
			my $value = `cat /etc/cups-webconfig.conf | grep 'alterps='`;
			my ($v1,$v2) = split(/=/,$value);
			if ($v2==0) {
		  	$rotate = 90 if $$pfinfo{'Paper orientation'} eq 'Landscape';
			}
	    $ENV{'PAGE_ROTATION'} = $rotate;
  	  $ENV{'AV_SCAN_BASE'} = $fout;
	    $ENV{'AV_SCAN_EXT'} = 'jpg'; 
  	  $ENV{'AV_SCAN_EXT'} = 'tif' if $$pfinfo{'Bits per pixel'}==1;
	    $ENV{'AV_SCAN_DEFINITION'} = $$pfinfo{'Profile'};
			if ($psfile ne "" || $$pfinfo{'ImportSource'}==1) {
	  	  $ENV{'AV_SOURCE'}=$pdffile 
			}
			$ENV{'AV_SCAN_HOST'} = $val{host};
  	  $ENV{'AV_SCAN_DB'} = $db;
		  $ENV{'AV_SCAN_USER'} = $val{user};
		  $ENV{'AV_SCAN_PWD'} = $val{pw};
	  	$ENV{'AV_SCAN_PATH'} = $ftpin;
			$ENV{'AV_SOURCE2'} = $$pfinfo{'SourceFile'}; # mail zip file
			$ENV{'ORG_FILE'} = $$pfinfo{'orgfile'};
			if ($$pfinfo{'NoOCRfromCUPS'}) { # we don't want ocr
				$pdfcheck = $pdffile;
 	    	$ENV{'AV_SOURCE3'}=$pdfcheck;
				my @parts = split(/\./,$pdfcheck);
				my $ext = pop @parts;
				if ($ext eq "pdf") {
					$pdfcheck = join('.',@parts).".ps";
					$pdfcheck .= ".pdf" if $v2==1;
					if (-e "$pdfcheck") {
						my $info = $val{pdfinfo}." $pdfcheck | grep 'PDF version'";
						my $info2 = `$info`;
  		    	$ENV{'AV_SOURCE3'}=$pdfcheck if $info2 ne "";
					}
				}
			}
			if (checkFile($val{ftpprocess})) {
			  my $text="";
			  readFile2($val{ftpprocess},\$text);
				my @vals = split("\n",$text);
		    $ENV{'AV_SCAN_HOST'} = $vals[0] if $vals[0] ne "";
    	  $ENV{'AV_SCAN_DB'} = $vals[1] if $vals[1] ne "";
		    $ENV{'AV_SCAN_USER'} = $vals[2] if $vals[2] ne "";
		    $ENV{'AV_SCAN_PWD'} = $vals[3] if $vals[3] ne "";
			}
  	 	callPrg($val{sane});
	    # remove the source files if they exist
  	  unlinkFile($pdffile) if checkFile($pdffile);
    	unlinkFile($psfile) if checkFile($psfile);
  	  unlinkFile($pdfcheck) if checkFile($pdfcheck) && $pdfcheck ne "";
		}
	}
}



=head3 FTPaddPagesSource($obj,$maxpages);

Extract pdf pages and text and store the information to the db

=cut

sub FTPaddPagesSource {
  my ($obj,$maxp) = @_;
  # check if we should add pdf files at all
	if ($obj->officetyp ne "multi" && 
		(checkFile($obj->source) || checkFile($obj->addtext))) {
	  my $file = $obj->source;
		$file = $obj->addtext if notFile($file); 
		$file = $obj->addtext if $obj->addtext =~ /(\.ps)$/;
		$file = $obj->addtext if $obj->addtext =~ /(\.ps\.pdf)$/;
		my $part = 0;
		my $start = 1;
		if ($obj->pagesframe_beg()>0) {
      $start=$obj->pagesframe_beg();
			$part = 1;
		}
		my $end = $obj->pages;
		if ($obj->pagesframe_end()>0) {
      $end=$obj->pagesframe_end();
			$part = 1;
		}
		my $found = 0;
		if ($obj->nosingle==1) { # store everything in one doc
      $found = FTPaddPagesSourceAllPages($obj,$start,$end,$file,$part,$maxp);
		} else { # all pages individuallly
      $found = FTPaddPagesSourceSinglePages($obj,$start,$end,$file,$part);
		}
		if ($found==1) {
		  $obj->logdone(4); # say to ocr we don't want it (source file);
		}
		# say to the document that we have an internal PDF file
    my $sql = "update archiv set QuelleExt='PDF',QuelleIntern=1 ".
              "where Laufnummer=".$obj->doc;
    $obj->dbh2->do($sql);
  }
}



=head1 FTPaddPagesSourceSinglePages($obj,$start,$end,$file,$part)

Store the pdf pages and/or text in single pages

=cut

sub FTPaddPagesSourceSinglePages {
  my ($obj,$start,$end,$file,$part) = @_;
  my $pdftk = $val{pdftk};
	my $c1 = 1;
	my $found = 0;
	$c1 = $start if $part==0;
	for (my $c=$start;$c<=$end;$c++) { 
    my $nr = ( $obj->doc * 1000 ) + $c1;
    # we need two temp. files for the actual page (pdf/txt files)
    my $pdf1 = $file . "$c-s";
    my $ptxt = $file . "$c-t";
    # extract the actual pdf page
    my $cmd = "$pdftk \"".$file."\" cat $c output \"$pdf1\"";
	  ##logit($cmd);
    my $res = callPrg($cmd,1);
    if ( $res == 0 ) {
      if (checkFile($pdf1)) {
				my $text = "";
				if ($obj->pdf==1 || $ENV{'AV_SOURCE'} ne "") {
					FTPaddPagesSourcePDF($obj,$nr,$pdf1);
				}
				FTPaddPagesSourceText($obj,$pdf1,$ptxt,\$text);
				$found=1 if $text ne "";
				FTPaddPagesSourceTextSave($obj,\$text,$nr);
			}
		}
		unlinkFile($pdf1) if checkFile($pdf1);
		unlinkFile($ptxt) if checkFile($ptxt);
	  $c1++;
	}
	return $found;
}



=head FTPaddPagesSourceTextSave($obj,$ptext,$nr)

Save a single page of text in db

=cut

sub FTPaddPagesSourceTextSave {
  my ($obj,$ptext,$nr) = @_;
	if ($$ptext ne "") {
    my $txt = $obj->dbh2->quote($$ptext);
	  my $sql1 = "update archivseiten set Text=$txt where Seite=$nr";
    $obj->dbh2->do($sql1);
	}
}



=head FTPaddPagesSourcePDF($obj,$nr,$pdf1)

Save one/more pdf pages to a document page

=cut

sub FTPaddPagesSourcePDF {
  my ($obj,$nr,$pdf1) = @_;
  my $ps1  = $pdf1;
  $ps1 =~ s/(.*)(\.pdf)/$1.ps/;
  $ps1 =~ s/(.*)(\.PDF)/$1.PS/ if $ps1 eq $pdf1;
	if (-f "$ps1") {
	  my $cmd = "ps2pdf12 $ps1 $pdf1";
		my $res=callPrg($cmd);
	}
  # the actual pdf page is extracted, now read it
  my $pmem;
  # read the content
  readFile2( $pdf1, \$pmem );
  $pmem = $obj->dbh2->quote($pmem);
  my $sql = "update archivbilder set Quelle=$pmem where Seite=$nr";
  # save the page to the database table
  $obj->dbh2->do($sql);
}



=head1 FTPaddPagesSourceAllPages($obj,$start,$end,$file,$part)

Save all pdf pages to one document and store page texts if needed

=cut

sub FTPaddPagesSourceAllPages {
  my ($obj,$start,$end,$file,$part,$maxpages) = @_;
	my $found = 0;
  my $pdftk = $val{pdftk};
	my $pdf1 = $file;
  my $ptxt = $file . "-t";
	my $res = 0;
	if ($part==1 && ($start>1 || $obj->pagesframe_beg>0)) {
    $pdf1 = $file . "-s";
    # extract the actual pdf pages
    my $cmd = "$pdftk '$file' cat $start-$end output '$pdf1'";
    $res = callPrg($cmd,1);
	} 
  if ($res==0) {
		my $text="";
    my $nr = ( $obj->doc * 1000 ) + 1;
		if ($obj->pdf==1 || $ENV{'AV_SOURCE'} ne "") {
			FTPaddPagesSourcePDF($obj,$nr,$pdf1);
		}
	  FTPaddPagesSourceText($obj,$pdf1,$ptxt,\$text);
		if ($text ne "") {
		  my @pages = split(/\14/,$text);
			my $c1 = 1;
	    $c1 = $start if $part==0;
			for (my $c=$start;$c<=$end;$c++) {
			  my $c2 = $c1 - 1;
				my $text1 = $pages[$c2];
				$found=1 if $text1 ne "";
        my $nr1 = ( $obj->doc * 1000 ) + $c1;
				FTPaddPagesSourceTextSave($obj,\$text1,$nr1);
				$c1++;
			}
		}
	} else {
	  logit("error while text extracting");
	}
  if ($ENV{'AV_SOURCE_NOTKILL'}==0) {
	  if ($maxpages==0) {
	    unlinkFile($pdf1) if checkFile($pdf1) && $part==1;
	  }
	  unlinkFile($ptxt) if checkFile($ptxt);
	}
	unlinkFile($pdf1) if checkFile($pdf1);
	unlinkFile($ptxt) if checkFile($ptxt);
	return $found;
}



=head FTPaddPagesSourceText($obj,$pdf1,$ptxt,$presult)

Extract page text from a pdf file and give back the text

=cut

sub FTPaddPagesSourceText {
  my ($obj,$pdf1,$ptxt,$presult) = @_;
  # now extract the page text
	if ($ENV{AV_OCRTXT} eq "") {
  	my $pdf2txt = $val{pdf2txt};
	  my $cmd1 = "$pdf2txt -layout \"$pdf1\" \"$ptxt\"";
  	my $res = callPrg($cmd1,1);
	  if ($res==0 ) {
  	  if (checkFile($ptxt)) {
	  	  if (check64bit()==64) {
				  my $ptxt1 = $ptxt.".ttt";
					unlinkFile($ptxt1) if checkFile($ptxt1);
	        my $cmd = "iconv -c -f utf8 -t iso-8859-1 \"$ptxt\" -o \"$ptxt1\"";
					my $res = callPrg("$cmd",1);
					if (checkFile($ptxt1)) {
					  unlinkFile($ptxt) if checkFile($ptxt);
						moveFile($ptxt1,$ptxt);
					}
				}
	      # there is a txt file, so add the text to the database table
  	    readFile2($ptxt,$presult);
			}
		}
	} else {
		$ptxt=$ENV{AV_OCRTXT};
  	readFile2($ptxt,$presult,1);
	}
}

=head3 FTPaddPagesSource2($obj);

Store mail zip file to BildA

=cut

sub FTPaddPagesSource2 {
  my ($obj) = @_;
	if (checkFile($obj->source2) && ($obj->pagesframe_end==0 || 
	  $obj->pagesframe_beg==1 || $obj->officeimages==1) && $obj->pages>0) {
	  logit("save source file ".$obj->source2);
    my $nr = ($obj->doc*1000)+1;
		my $file = $obj->source2;
		my $size = -s $file;
		my $maxsize = 512*1024*1024; # old max. size of file (512 MByte)
 		my $multi=getParameterRead($obj->dbh2,$obj->database,"SourcesOffline");
		$multi=0 if $obj->officetyp ne "multi";
		if (($size<=$maxsize && $multi==0) || `uname -r`<5) {
			my $pmem;
  	  readFile2($file,\$pmem);
    	$pmem = $obj->dbh2->quote($pmem);
	    my $sql = "update archivbilder set BildA=$pmem where Seite=$nr";
	   	$obj->dbh2->do($sql);
		} else {
			if ($multi==1) {
				my $typ = $obj->filetyp;
				my $name =$obj->filename;
				my $ext = $obj->fileext;
				if ($typ eq "" || $name eq "" || $ext eq "") {
					($typ,$name,$ext)=getEDVName($obj->dbh,$obj->database,$obj->doc);
				}
				my $fname = $obj->database."-".$obj->doc.".".$ext;
				my $path = "/home/data/archivista/archmulti";
				if (!-e "$path") {
					mkdir $path;
					callPrg("chmod -R 770 $path",1);
					callPrg("chown -R www-data.www-data $path",1);
				}
				if (-e "$path") {
					my $res=0;
					my $ok=moveFile($file,"$path/$fname",1);
					$res=-1 if $ok!=1;
					$res=callPrg("chown www-data.www-data $path/$fname",1) if $res==0;
					$res=callPrg("touch /tmp/archmulti",1) if !-e "/tmp/archmulti";
					$multi=0 if $res!=0;
				} else {
					$multi=0;
				}
			} else {
				$multi=0;
			}
			if ($multi==0) {
	  		my $totram = 0;
			  my $mem = `cat /proc/meminfo | grep 'MemTotal'`;
			  $mem =~ s/\s+/ /g;
		  	my @parts = split(/\s/,$mem);
			  $totram = $parts[1]*1024;
				my $anz = int($size/$maxsize)+1;
				my $size2 = $size*2;
				logit("$file with size $size, total memory:$totram");
				my $c=0;
				my $buf = "";
				eval { open (FH, '< :raw', $file) or die $!; };
				while (my $read = sysread( FH, $buf, $maxsize) ) {
					$c++;
					last if $c>640;
					my $nr=($obj->doc*1000)+$c;
					my $file1 = "/tmp/chunk-".$obj->job."-$c.chk";
					open(FOUT,">$file1");
					binmode(FOUT);
					print FOUT $buf;
					close(FOUT);
					my $sql = "update ".$obj->database.".archivbilder set ".
						"BildA=load_file(".$obj->dbh2->quote($file1).") where Seite=$nr";
					$obj->dbh2->do($sql);
					logit("chunk $c of ".$obj->doc." processed");
					unlink $file1 if -e $file1;
					sleep 1;
				}
			}
		}
    if ($ENV{'AV_SOURCE_NOTKILL'}==0) {
		  unlinkFile($obj->source2) if checkFile($obj->source2);
		}
	}
}



=head3 _FTPgetFileNames($path,$pfinfo)

Give back the pdf AND postscript file name if they exist

=cut

sub _FTPgetFileNames {
  my ($path,$pfinfo) = @_;
  # file name without extension
  my $fout = $$pfinfo{'File name'};
  my $fps  = $fout;
  $fps =~ s/(.*)(\.pdf)$/$1.ps/;
  if ( $fps ne $fout ) {
    # check if the postscript file is available
    if (checkDir($path)) {
      $fps = "$path$fps";
      $fps = "" if notFile($fps);
    } else {
      $fps = "";
    }
  } else {
    $fps = "";
  }
  # check if the pdf file is available
  $fout = "$path$fout";
  $fout = "" if notFile($fout);
  # give back both file names
  return ( $fout, $fps );
}



=head3 %newhash=_FTPinitHash($pfinfo)

Give us back a new empty hash for the values from axis file

=cut

sub _FTPinitHash {
  my ($pfinfo) = @_;
  %$pfinfo = (
    'File name'          => '',    # file
    'Date'               => '',    # date
    'Destination'        => '',    # db or owner (at{'destdb'})
    'Paper size'         => '',    # size (A4)
    'Number of pages'    => '',    # pages
    'Width'              => '',    # Pixel x
    'Height'             => '',    # Pixel y
    'X Resolution (DPI)' => '',    # 75-9600
    'Y Resolution (DPI)' => '',    # 75-9600
    'Bits per pixel'     => '',    # 1/8/24
    'Format'             => '',    # Tiff/JPEG
    'Profile'            => '',    # Scan-Defination
    'Paper orientation'  => '',    # Portrait/Landscape
    'Fields'             => '',    # Additional fields
    'ImportSource'       => '',    # 1=import of source file
		'NonAxisFile'        => '',    # normal axis file (no user file name)
		'UserID'             => '',    # userid (unido) -> goes to owner
		'NoOCRfromCUPS'      => '',    # don't call scan def (no ocr after it) 
		'SourceFile'         => '',    # source file (mail archiving)
		'orgfile'            => '',    # original file name (pdf stream)
  );
}



=head2 FTPparseFile ($file,$pfinfo)

Parse the $file from path ftpin and get all informations out of it

=cut

sub FTPparseFile {
  my ($file,$pfinfo) = @_;
  my ($res);
  if (-e "$file") {
    logit("Key file $file found");
    # we read all the files, extract the key information
    _FTPinitHash($pfinfo);    # initialize the hash (no old values)
    # read the axis file
    open FIN, "$file" or die "Can't open $file\n";
		my $meta = 0;
    while (<FIN>) {
		  my $line = $_;
      chomp $line;
			if ( $line =~ /\[Metadata\]/ ) {
			  $meta = 1;
			} elsif ($line =~ /\[^(Metadata)\]/ ) {
			  # if we have [something] then we have left the meta mode
			  $meta = 0;
			}
      # go to each line of the file and split it at "= "
			my ($key,$value) = keyvalue($line,"= ");
      # remove \r and \n chars
      $value =~ s/\n//g;
      $value =~ s/\r//g;
      if ( $value ne "" ) {
        # there is a value, remove spaces after key
        $key =~ s/\s*$//;
				if ($meta == 1) {
				  # We have meta Values.
					# They are saved in Fields
					if ($key eq "UserID") {
					  # Set UserID in the pfinfo hash not in Fields
					  $value = escape($value); # don't forget quoting
					  $$pfinfo{"Fields"} .= ":" if $$pfinfo{"Fields"} ne "";
					  $$pfinfo{"Fields"} .= "Eigentuemer=$value";
					} elsif ($key eq "Title") {
					  $key = "Titel"; # Title to Titel (not possible in db)
			    }
					if ($key ne "UserID") {
					  $value = escape($value); # don't forget quoting
					  $$pfinfo{"Fields"} .= ":" if $$pfinfo{"Fields"} ne "";
					  $$pfinfo{"Fields"} .= "$key=$value";
					}
				} else {
          foreach my $k1 ( keys %$pfinfo ) {
            # go through the hash and if it is there, store the value
            if ( $k1 eq $key ) {
              $$pfinfo{$key} = $value;
            }
          }
				}
      }
    }
    close FIN;
    # kill the axis txt file
    my $f1 = $file;
    unlinkFile($f1) if checkFile($f1);
    $res = 1;
  }
  return $res;
}



=head2 FTPsplitFile ($pfinfo)

Extracts the images from a given pdf file

=cut

sub FTPsplitFile {
  my ($pfinfo) = @_;
  my $gogs   = $val{ftpgs};
  my $path   = $val{ftpin};
  # file name without extension
  my $fout = $$pfinfo{'File name'};
  $fout = checkPathAlreadyHere($fout,$path);
  $$pfinfo{'File name'} = $fout;
  $fout =~ s/(.*)(\.pdf)$/$1/;
	if ($fout eq $$pfinfo{'File name'}) {
    $fout =~ s/(.*)(\.jpg)$/$1/;
		if ($fout ne $$pfinfo{'File name'}) {
		  my $fout1 = $fout."0001.jpg";
			my $in = $path.$$pfinfo{'File name'};
			my $out = "$path$fout1";
		  moveFile($in,$out);
		} else {
		  $fout="";
		}
		return $fout;
  }
	my $ferr = "$path$fout.err";
  # extract resolution information
  my $rx = $$pfinfo{'X Resolution (DPI)'};
  $rx = 300 if ( $rx < 75 || $rx > 9600 );
  my $ry = $$pfinfo{'Y Resolution (DPI)'};
  $ry = 300 if ( $ry < 75 || $ry > 9600 );

  # normally we want tiff g4
  my $dev = "tiffg4";
  my $ext = "tif";

  # check, if we have to process jpeg
  $dev = "jpeggray" if ( $$pfinfo{'Bits per pixel'} == 8 );
  $dev = "jpeg"     if ( $$pfinfo{'Bits per pixel'} == 24 );
  $ext = "jpg"      if ( $dev eq "jpeggray" || $dev eq "jpeg" );

  # check for a postscript file (from cups2axis.pl)
  my $fpdf = $$pfinfo{'File name'};
  my $fps  = $fpdf;
  $fps =~ s/(.*)(\.pdf)/$1.ps/;
  my $ppdf = "$path$fpdf";
  my $pps  = "$path$fps";
  my $fin  = $ppdf;
	my $f1 = "";
  my $pages = $$pfinfo{"Number of pages"};
	my $found=checkmaxi(2);
	my $single=0;
 	# we use gs to extract jpg files
	if (checkFile($pps)) {
		my $value = `cat /etc/cups-webconfig.conf | grep 'alterps='`;
		my ($v1,$v2) = split(/=/,$value);
		if ($v2==1) {
			my $pps1 = $pps.".pdf";
			my $res=callPrg("ps2pdf14 $pps $pps1",1);
			if ($res==0) {
				$fin=$pps1;
				callPrg("rm -f $pps",1);
				logit("use pdf instead of ps source");
			} else {
				$fin=$pps;
			}
		} else {
			$fin=$pps;
		}
  	$f1="$gogs -sDEVICE=$dev -r$rx"."x$ry " . # device and resolution
		  "-sOutputFile=\"$path$fout".'%04d.'."$ext\" \"$fin\""; # out/in file
		$single=1; # ps code does not work with parallel (gs problem)
	} else {
		my $fin1 = $fin;
		$fin1 =~ s/\s/\\ /g;
		my $fout1 = $fout;
		$fout1 =~ s/\s/\\ /g;
	  $f1="seq 1 $pages | parallel ".
			"--tmpdir /home/data/archivista/tmp -P $found ". # call gs parallel
			"$gogs -sDEVICE=$dev -r$rx"."x$ry"." -dFirstPage={} -dLastPage={} ".
    	"-sOutputFile=\"$path$fout1"."_"."{}".'.'."$ext\" \"$fin1\" >>$ferr";
	}
  logit("Extract image files from $fin\n");
	my $jpegonly=_FTPsplitFileImagesSimple($pfinfo,$ext,$path,$fin,$fout);
	if ($jpegonly==0) {
		my $res=0;
		if ($$pfinfo{'COLD2'} eq "") {
		  my $destdb = $$pfinfo{Destination};
		  $destdb =~ s/(.*?)(-)(.*?)$/$3/;
			my $dbh = MySQLOpen();
			if ($dbh) {
				my $cold2=getParameterRead($dbh,$destdb,"COLD2",0,1,0);
			  $dbh->disconnect();
				$$pfinfo{'COLD2'}=1 if $cold2==1;
			}
		}
		if ($$pfinfo{'COLD2'}==0) {
			$res=_FTPsplitBasic($path,$fout,$pages,$ext,$f1,$single);
			my $errmsg = "";
			readFile2($ferr,\$errmsg,1);
			if ($errmsg ne "") {
				logit("there seems to be a problem with $fout");
				$errmsg = uc($errmsg);
				if ($errmsg =~ /ERROR/) {
					logit("there is an ERROR message from $fout");
					$res=-1;
				}
			}
	    # if processing was not sucessfully, then kill fout file
		  if ($res != 0) {
	  	  logit("Use second raster engine");
		  	$res = _FTPsplitExtended($ppdf,"$path$fout",$dev,$ext);
		  }
		  $fout = "" if $res!=0;
		} else {
	  	$res = _FTPsplitExtended($ppdf,"$path$fout",$dev,$ext);
			if ($res!=0) {
	  	  logit("Use first raster engine");
				$res=_FTPsplitBasic($path,$fout,$pages,$ext,$f1,$single);
			}
		}
	  $fout = "" if $res!=0;
	}
  return $fout;
}

sub _FTPsplitBasic {
	my ($path,$fout,$pages,$ext,$f1,$single) = @_;
 	my $res = callPrg($f1);
	if ($single==1) {
		for(my $page=1;$page<=$pages;$page++) {
			my $fileout = "$path$fout".sprintf("%04d",$page).'.'."$ext";
			$res=9999 if !-s "$fileout";
		}
	} else {
		for(my $page=1;$page<=$pages;$page++) {
			my $filein = "$path$fout"."_".$page.'.'."$ext";
			my $fileout = "$path$fout".sprintf("%04d",$page).'.'."$ext";
			if (-s "$filein") {
		  	moveFile($filein,$fileout);
			} else {
				$res=9999;
			}
		}
	}
	return $res;
}

sub _FTPsplitFileImagesSimple {
	my ($pfinfo,$ext,$path,$fin,$fout) = @_;
 	my $jpegonly=0;
	return $jpegonly; # don't use check of jpeg/ppm files at this time
  my $pages = $$pfinfo{"Number of pages"};
	if ($ext eq "jpg" && $pages>0) {
	  my $f2 = "pdfimages -j -f 1 -l $pages \"$fin\" \"$path$fout\"";
	  my $res = callPrg($f2);
	  if ($res==0) {
   	  my $f3 = "ls -1 $path$fout-*.jpg";
		  $res = `$f3`;
			logit("$f3");
		  my @files = split(/\n/,$res);
	    my $pages1 = $#files;
			my $pages2 = $pages1+1;
      $f3 = "ls -1 $path$fout-*.pbm";
		  $res = `$f3`;
			logit("$f3");
		  my @files1 = split(/\n/,$res);
		  if ($pages==$pages2 && $files1[0] eq "") {
			  logit("pdfimages got same number of images as pages:$pages2");
		    my $file1 = $files[0];
			  my $secpage = 2;
			  $secpage = 1 if $pages==1;
		    my $file2 = $files[$secpage];
		    my ($xres1,$yres1) = _FTPsplitFileJPG($file1);
			  my ($xres2,$yres2) = _FTPsplitFileJPG($file2);
				logit("$xres1---$xres2--$yres1---$yres2");
			  if ($xres1==$xres2 && $yres1==$yres2 &&
			      $xres1>0 && $xres2>0 && $yres1>0 && $yres2>0) {
			    $jpegonly=1;
			  }
			}
			if ($jpegonly==0) {
			  foreach my $file (@files) {
					unlink $file if -e $file;
				}
			  foreach my $file (@files1) {
					unlink $file if -e $file;
				}
			} else {
			  foreach my $file (@files) {
				  my @parts = split("-",$file);
					my $last = pop @parts;
					my ($base,$ext) = split(/\./,$last);
					$base++;
		      $last = sprintf("%04d",$base).".$ext";
					my $file1 = join("-",@parts);
		      $file1 .= $last;
					moveFile($file,$file1,1);
				}
			}
	  }
	}
	return $jpegonly;
}



sub _FTPsplitFileJPG {
  my ($file) = @_;
  my $cmd = "identify -ping $file";
  my $res = `$cmd`;
  #logit("$res=>$cmd");
	my @parts = split(/\s/,$res);
	my $format = $parts[1];
	my $size = $parts[2];
	my ($sizex,$sizey) = split("x",$size);
	return ($sizex,$sizey);
}



=head1 $file=checkPathAlreadyHere($file,$path)

Check if the given path is inside the file name

=cut

sub checkPathAlreadyHere {
  my ($file,$path) = @_;
	if (index($file,$path)==0) {
	  my @parts = split(/\//,$file);
		$file = pop @parts;
	}
	return $file;
}



sub _FTPsplitExtended {
  my ($fin,$fout,$dev,$ext) = @_;
	my $maxpage=0;
	my $res=-1;
  my $out = `pdfinfo $fin`;
	$out =~ /(Pages:)(\s+)([0-9]+)/;
	$maxpage = $3;
	my $zeros = getZeros($maxpage);
  my $ext1 = "pbm";
	$ext1 = "pgm" if $dev eq "jpeggray";
	$ext1 = "ppm" if $dev eq "jpeg";
	my $opt = "";
	$opt = "-mono" if $dev eq "tiffg4";
	$opt = "-gray" if $dev eq "jpeggray";
 	if (`cat /proc/cpuinfo | grep 'ARM'` ne "") {
  	$ext1 = "jpg" if $dev eq "jpeggray" || $dev eq "jpeg";
	}
	$opt .= " -jpeg" if $ext1 eq "jpg";
	if ($maxpage>0) {
	  #my $cmd = "pdftoppm $opt -r 300 -f 1 -l $maxpage $fin $fout";
		my $found = checkmaxi(2);
		my $fin0 = $fin;
		$fin0 =~ s/\s/\\ /g;
		my $cmd = "seq 1 $maxpage | parallel ".
			"--tmpdir /home/data/archivista/tmp -P $found ".
			"pdftoppm $opt -r 300 -f {} -l {} '$fin0' '$fout'";
		$res = callPrg("$cmd 2>/dev/null");
		if ($res==0) {
		  logit("all images created");
		  for (my $c=1;$c<=$maxpage;$c++) {
			  my $fout2 = getZeroFile($fout,$c,$ext1);
			  my $fout3 = $fout.sprintf("%04d",$c).".$ext";
			  unlinkFile($fout3) if checkFile($fout3);
			  moveFile($fout2,$fout3);
		  }
		}
	}
	return $res;
}



=head2 CUPSgetPSFile($file,$file2)

Gets the apprpriate postscript file from a cups-pdf file

=cut

sub CUPSgetPSFile {
  my ($file,$file2) = @_;
  my $cupsps = $val{cupsps};
  my $ftpin  = $val{ftpin};
  my $fps    = $file;
  my $res    = 0;
	my $printer = "";
  my ( $db, $user, $defname );

  # now lets have a look at the original ps file
  $fps =~ s/^(job_)([0-9]+)(.*)(pdf)$/$2/;
  $fps =~ s/(.*)(job_)([0-9]+)(.*)(pdf)$/$3/ if $fps==0;
  if ( $fps > 0 ) {
    # store the job number
    my $fnr = $fps;
    # we got the job number
    $fps = "d" . sprintf( "%05d", $fps ) . "-001";
    my $fin  = "$cupsps$fps";
    my $fout = "$ftpin$file2";
    if (checkFile($fin)) {
      unlinkFile($fout) if checkFile($fout);
      $res = moveFile($fin,$fout);
      # now get the desired database
			my $check = $val{cupsinfo};
			$check = $val{cupsinfo64} if check64bit()==64;
      my $jobs = `$check`;
      my @l = split( "\n", $jobs );
      foreach ( reverse @l ) {
        # go through all lines and split with empty chars
        my @l1 = split( " ", $_ );
				my $line = join("",@l1);
				# check for unwanted chars (printer is behind @) -> leuthold
				my @parts0 = split("@",$line);
				my $l2 = pop @parts0;
        # now lets have a look at printer-jobnr
        my @parts = split( "-", $l2 );
				my $pr = shift @parts;
				my $nr = pop @parts;
				my $def = join("-",@parts);
        if ( $nr == $fnr ) {
          # we did find a job number, so store db and user
          $db = $pr;
					$defname = $def;
					$printer = "$db-$defname" if $defname ne "";
          last;
        }
      }
    }
  }
  return ( $res, $db, $defname, $printer );
}



=head2 CUPSparseFile ($file,$psfile)

Create an axis key file with pdfinfo

=cut

sub CUPSparseFile {
  my ($file,$psfile,$db,$defname,$fieldsgl,$sourcefile,$printer) = @_;
  # if we got no desitination, then add standard
  $db = "archivista" if $db eq "";
  my $pdfinfo = $val{pdfinfo};
  my $dir     = $val{ftpin};
  my $color   = 0;
  my $out     = "";
  my $first   = 1;
  my $psdf    = "$dir$psfile";
  if (checkFile($psdf)) {
    # check ps file for orientation
    open( FIN, $psdf );
    while (<FIN>) {
      my $line = $_;
      $line =~ /^(\%\%Orientation:\s)([a-zA-Z]*)(.*)?$/;
      if ( $2 eq "Portrait" or $2 eq "Landscape" ) {
        # check for landscape orientation
        if ( $first == 1 ) {
          $out .= "Paper orientation     = $2\n";
          $first = 0;
        }
      }
      # check for a color printer
			my $value = `cat /etc/cups-webconfig.conf | grep 'defaultcolor='`;
			my ($v1,$v2) = split(/=/,$value);
			if ($v1 eq "defaultcolor" && ($v2==1 || $v2==2)) {
			  $color = 1 if $v2==2;
			} else {
        $line =~ /^(\%\%TargetDevice:\s)(.*)(Color)(.*)?$/;
        $color = 1 if $3 eq "Color";
			}
      # check for end of comments, so we don't read the full file
      $line =~ /^(\%\%)(EndComments)(.*)$/;
      last if $2 eq "EndComments";
    }
    close(FIN);

    my $keys  = `$pdfinfo $dir$file`;
    my $title = $keys;
    $title =~ s/(.*)(Creator\:)(\s+)(.*?)(\n.*)/$4/s;
    $title =~ s/(.*)(Producer\:)(\s+)(.*?)(\n.*)/$4/s if $title eq $keys;
    if ( $title ne $keys ) {
      # document is correct
      # extract title (Fields for COLDplus)
      # check for COLDplus
      my $for = $keys;
			$for =~ s/(.*)(Author\:)(\s+)(.*?)(\n.*)/$4/s;
      if ($for ne $keys) {
        $for =~ s/\s//g; # we did found an author, remove spaces
      } else {
        $for = ""; # not found, not used
      }
      my $fields = $keys;
      $fields =~ s/(.*)(Title\:)(\s+)(.*?)(\n.*)/$4/s;
      # if there was no match, no title, clear the fields
      $fields = "" if $fields eq $keys;
			my $fields2 = $fields;
			CUPSCheckOffice(\$db,\$fields,$file,$psfile,$title,\$sourcefile);
      CUPSCheckCOLDplus(\$db,$fields2,$file,$psfile,$for,\$fields,$printer);
      # extract pages
      my $pages = $keys;
      $pages =~ s/(.*)(\nPages\:)(\s+?)([0-9]+)(.*)/$4/s;
      # extract sizex
      $keys =~ /(Page\ssize\:)(\s+)([0-9]+)?(\sx\s)([0-9]+)?(.*)/;
      my $sizex = int $3 * 4.19498;
      # extract sizey
      $keys =~ /(Page\ssize\:)(\s+)([0-9]+)?(\sx\s)([0-9]+)?(.*)/;
      my $sizey = int $5 * 4.16158;
      # compose axis file
      my @row;
      $row[0] = "Destination           = $db";
      $row[1] = "File name             = $file";
      if ( $color == 0 ) {
        $row[2] = "Bits per pixel        = 1";
        $row[5] = "Format                = TIF";
      } else {
        $row[2] = "Bits per pixel        = 24";
        $row[5] = "Format                = JPEG";
      }
      $row[3] = "X Resolution (DPI)    = 300";
      $row[4] = "Y Resolution (DPI)    = 300";
      $row[6] = "Number of pages       = $pages";
      $row[7] = "Width                 = $sizex";
      $row[8] = "Height                = $sizey";
			$row[9] = "NonAxisFile           = 1";
			$row[10] = "NoOCRfromCUPS        = 1";
			if ($defname ne "") {
				my $dbh = MySQLOpen();
				if ($dbh) {
				  my $scandef = getScanDefByName($dbh,$db,$defname);
					my @valdefs = split(";",$scandef);
					if ($valdefs[0] eq $defname) {
					  my $depth = 1;
		        $depth = 8 if $valdefs[1]==1;
		        $depth = 24 if $valdefs[1]==2;
            $row[2] = "Bits per pixel        = $depth";
            $row[3] = "X Resolution (DPI)    = $valdefs[2]";
            $row[4] = "Y Resolution (DPI)    = $valdefs[2]";
						if ($depth==1) {
              $row[5] = "Format                = TIF";
            } else {
              $row[5] = "Format                = JPEG";
            }
			      $row[11] = "Profile              = $defname";
					}
				}
			}
			if ($sourcefile ne "") {
			  $row[12] = "SourceFile           = $sourcefile";
			}
      $out .= join( "\n", @row ) . "\n";
			if ($fieldsgl ne "") {
			  $fields.=":" if $fields ne "";
				$fields.=$fieldsgl;
			}
      # add field values (form COLDplus)
      $out .= "Fields                = $fields";
      my $fout = $file;
      $fout =~ s/^(.*?)(\.)(pdf)$/$1/;
      $fout .= '.txt';
      $fout = "$dir$fout";
      open( FOUT, ">$fout" );
      print FOUT $out;
      close(FOUT);
    }
  }
}



=head2 $vals=CUPSCheckOffice($pdb,$pfields,$file,$psfile,$title)

Checks if we have a locally office printed document

=cut

sub CUPSCheckOffice {
  my ($pdb,$pfields,$file,$psfile,$title,$psourcefile) = @_;
  $file   = $val{ftpin} . $file;
  $psfile = $val{ftpin} . $psfile;
	my $vers = "3.0";
  $vers = "3.2" if check64bit()==64;
	if ($title eq "(OpenOffice.org $vers)") {
	  my $name = $$pfields;
		$name =~ s/^\(//;
		$name =~ s/\)$//;
		my $pfad = "/home/data/archivista/ftp/officeout/$$pdb/$name";
		my @files = <$pfad*>;
		if (checkFile($files[0])) {
		  my $filein = $files[0];
		  my @parts = split(/\//,$files[0]);
			my $nameold = pop @parts;
			my @parts1 = split("_",$nameold);
			shift @parts1;
			my $namenew = join("_",@parts1);
			push @parts,$namenew;
			my $fileout = join("/",@parts);
			unlinkFile($fileout) if checkFile($fileout);
			my $res=moveFile($filein,$fileout);
			if ($res==1) {
			  my $zipname = $fileout.".zip";
        unlinkFile($zipname) if checkFile($zipname); # make sure we never append
			  my $cmd = "zip -q -j $zipname $fileout 2>/dev/null 0>/dev/null";
				$res = callPrg($cmd,1);
				if ($res==0) {
			    logit("sourcefile added:$zipname");
			    $$psourcefile=$zipname;
			    $namenew = escape($namenew); # don't forget quoting
				  $$pfields = "EDVName=office;$namenew";
				  logit("pfields:$$pfields");
				}
			}
    }		
	}
}



=head2 $vals=CUPSCheckCOLDplus($db,$file)

Checks if we have a coldplus program and gives back the field values

=cut

sub CUPSCheckCOLDplus {
  my ($pdb,$fields,$file,$psfile,$for,$pfields,$printer) = @_;
  $file   = $val{ftpin} . $file;
  $psfile = $val{ftpin} . $psfile;
  my $prg = $val{coldplus};
  if (checkFile($prg)) {
	  logit("coldplus with file $file");
    # Program is available
    my $cmd = "$prg $$pdb \"$$pfields\" \"$file\" \"$psfile\" ".
		  "\"$for\" $printer";
    my $fields = `$cmd`;
    if ( $fields ne "" ) {
      my $db = $fields;
      $db     =~ s/^(.*?)(;)(.*)$/$1/;
      $fields =~ s/(.*?)(;)(.*)$/$3/;
      $$pdb     = $db;
      $$pfields = $fields;
		}
	}
}



=head2 CUPSwaitAndMove($file)

Moves the file to the ftp folder

=cut

sub CUPSwaitAndMove {
  my ($file) = @_;
  my $cupsin = $val{cupsin};
  my $ftpin  = $val{ftpin};
  my $res;
  my $file1 = $file;
  # we only use simple file names (ghostscript does not like strong names)
  $file1 =~ s/(.*?)(\-)(.*)/cups-$1./;
  # give a simple name to the ps file
  my $file2 = "$file1" . 'ps';
  # and now to the pdf file
  $file1 .= 'pdf';
  my $fin  = "$cupsin$file";
  my $fout = "$ftpin$file1";
  unlinkFile($fout) if checkFile($fout);
  # move the file to the ftp dir
  $res = moveFile($fin,$fout);
  return ( $res, $file1, $file2 );
}



=head2 ($doc,$pag,$folder,$typ)=GETnEXTfReeDocument($dbh,$dbn,$dnr,$desc,$sql)

Gives back the next free document we can process

=cut

sub getNextFreeDocument {
  my ($dbh,$db1,$aktnr,$desc,$sqladd) = @_;
  # compose base framgent
  my $sql = "select Laufnummer,Seiten,Ordner,ArchivArt from $db1.archiv " .
            "where (Gesperrt='' || Gesperrt is null) ";
  # add an additional part (if available)
  $sql .= $sqladd if ($sqladd ne "");
  if ($desc) {
    # calcluate last document
    $sql .= "and Laufnummer<=$aktnr order by Laufnummmer desc ";
  } else {
    # calculate first document
    $sql .= "and Laufnummer>=$aktnr order by Laufnummer ";
  }
  # we only want 1 document
  $sql .= "limit 1";
  # do the sql command
  my ( $akte, $seiten, $ordner, $art ) = $dbh->selectrow_array($sql);
  return ($akte,$seiten,$ordner,$art);
}



=head2 $pfadpart=getArchivingFolderPath($folder)

According folder number we calculate path part

=cut

sub getArchivingFolderName {
  my ($ordner) = @_;
  # folders are always in format (ARCH0001,ARCH0002..)
  my $pfad = "ARCH".sprintf("%04d",$ordner).$val{dirsep};
	$pfad = "AR".sprintf("%06d",$ordner).$val{dirsep} if $ordner>=10000;
  return $pfad;
}



=head2 $ext=getArchivingExt($art,$zipped)

Give back the extension from the field value ArchivArt

=cut

sub getArchivingExt {
  my ($art,$zipped) = @_;
  my $ext = "JPG";
  $ext = "PNG" if $art == 2;
  $ext = "TIF" if $art == 1;
  if ($zipped) {
    $ext = "ZIP" if $art == 0;
  } else {
    $ext = "BMP" if $art == 0;
  }
  return $ext;
}



=head2 $file1=getFileToPng($file,$akte,$seite)

Converts the png file into a png file and gives back the fine name

=cut

sub getFileToPng {
  my ($file,$akte,$seite) = @_;
  my ($file1);
  if (checkFile($file)) {
    # file is ok (we actually have a tif file) -> convert it to png
    $file1 = "$val{tmpdir}$akte-$seite.png";
    unlinkFile($file1) if checkFile($file1);
    my $cmd = "$val{tifpng} $file";
    my $res = callPrg($cmd);
    $file1 = "" if $res > 0;    # we got and error
  }
  return $file1;
}



=head2 $file1=getFileUnzip($mode,$akte,$seite,$file)

Does a zip file unzip to a bmp and convert it to a tif file

=cut

sub getFileUnzip {
  my ($mode,$akte,$seite,$file) = @_;
  my $pw1 = $seite + 1000;
  my $pw = "$mode$akte$pw1";
  my $file1 = "$val{tmpdir}$akte-$seite.bmp";

  # unzip the file with the pw and store it to a temp file
  my $cmd = "$val{unzip} $pw $file >$file1";
  if (callPrg($cmd)!=0 ) {
    # we got 0, so it is ok
    my $file2 = "$val{tmpdir}$akte-$seite.tif";
    unlinkFile($file2) if checkFile($file2);
    # now we need to translate the bmp to the tif file
    $cmd = "$val{bmptif} $file1 $file2";
    if (callPrg($cmd)!=0) {
      # we got no error, so the file is ok
      # remove the bmp file, we only want the tif
      unlinkFile($file1) if checkFile($file1);
      $file1 = $file2;
    } else {
      # we got an error, so the file is not ok, kill the bmp file
      unlinkFile($file1) if checkFile($file1);
      $file1 = "";
    }
  }
  return $file1;
}



=head2 $rowsaffected=DocumentLock($dbh,$db1,$akte,$lockuser)

Locks a document to a given user

=cut

sub DocumentLock {
  my ($dbh,$db1,$akte,$lockuser) = @_;
  $lockuser = $dbh->quote($lockuser);
  my $sql = "update $db1.archiv set Gesperrt=$lockuser where Akte=$akte";
  my $res = $dbh->do($sql);
  return $res;
}



=head2 $rowsaffected=DocumentUnlock($dbh,$db1,$akte)

Unlocks a document

=cut

sub DocumentUnlock {
  my ($dbh,$db1,$akte) = @_;
  my $sql  = "update $db1.archiv set Gesperrt='' where Akte=$akte";
  my $res  = $dbh->do($sql);
  return $res;
}



=head1 logit($message)

Writes a message to a log file

=cut

sub logit {
  my ($message) = @_;
  my $stamp = TimeStamp();
  # $log file name comes from outside
  my @parts = split($val{dirsep},$0);
  my $prg = pop @parts;
  open( FOUT, ">>$val{log}" );
  binmode(FOUT);
  my $logtext = $prg . " " . $stamp . " " . $message . "\n";
  print FOUT $logtext;
  close(FOUT);
}



=head2 $dbh=MySQLOpen($host,$db,$user,$pw)

Open a MySQL connection and gives back a db handler

=cut

sub MySQLOpen {
  my ($host,$db,$user,$pw) = @_;
  $host = $val{host} if $host eq "";
  $db = $val{db} if $db eq "";
  $user = $val{user} if $user eq "";
  $pw = $val{pw} if $pw eq "";
  my ($ds,$dbh);
  $ds = "DBI:mysql:host=$host;database=$db";
  $dbh = DBI->connect($ds,$user,$pw,{PrintError=>1,RaiseError=>0});
  return $dbh;
}



=head2 $dbh2=MySQLOpen2($logid,$dbh,$pwd)

Open a second MySQL connection and gives back a db handler 2

=cut

sub MySQLOpen2 {
  my ($logid,$dbh,$pwd) = @_; # attention at last parameter (if no passwd)
  my ( $host, $db, $user, $ds, $dbh2 );
  if ($dbh) {
    my $sql = "select host,db,user from logs where ID=$logid";
    my @row = $dbh->selectrow_array($sql);
    $host = $row[0];
    $db   = $row[1];
    $user = $row[2];
  }
  $ds = "DBI:mysql:host=$host;database=$db";
  $dbh2 = DBI->connect($ds,$user,$pwd,{PrintError=>1,RaiseError=>0});
  return $dbh2;
}



=head1 MySQLClose($dbh) 

Close mysql connection

=cut

sub MySQLClose {
  my ($dbh) = @_;
	logit("Close db connection");
	$dbh->disconnect;
}



=head1 $slave=HostIsSlave($dbh)

gives back a 1 if we are in slave mode

=cut

sub HostIsSlave {
  my ($dbh) = @_;
  return HostIsSlaveSimple($dbh);
}



=head1 $slave=HostIsSlaveSimple($dbh)

gives back a 1 if we are in slave mode (user session)

=cut

sub HostIsSlaveSimple {
  my ($dbh) = @_;
  my $hostIsSlave = 0;
  my @row = $dbh->selectrow_array("SHOW VARIABLES LIKE 'server_id'");
  $hostIsSlave=1 if $row[1]>1;
  return $hostIsSlave;
}



=head2 getFileNameNr($document,$page,$mode)

Gives back a file name from a doc./page/mode
  
=cut

sub getFileNameNr {
  my ($document,$page,$mode) = @_;
  my $t1 = getFileNameCalc( $document, 5 );
  my $t2 = getFileNameCalc( $page, 2 );
  $mode = "A" if ! defined $mode;
  my $fn=$mode . $t1 . $t2;
  return $fn;
}



=head2 $fname=getSourceFileNameNr($document,$mode)

Returns an archive source-filename.

=cut

sub getSourceFileNameNr {
  my ($document,$mode) = @_;
  my $t1 = getFileNameCalc( $document, 5 );
  my $t2 = "0A";
  $mode = "A" if ! defined $mode;
  my $fn=$mode . $t1 . $t2;
  return $fn;
}



=head2 $fnamepart=getFileNameCalc($input,$count)

Returns an archive filename.

=cut

sub getFileNameCalc {
  my ( $input, $count ) = @_;
  my ( $res, $res1, $t, $tcount );
  while ( $input != 0 ) {
    $res = $input % 26;
    $res1 = $res + 65;
    $t  = chr($res1) . $t;
    $input = $input - $res;
    $input = $input / 26;
  }
  $t = "0" x ( $count - length($t) ) . $t;
  return $t;
}



=head2 $kbused=getDiskSpaceUsed($dir)

Returns the used diskspace on the root filesystem in kbytes.

=cut

sub getDiskSpaceUsed {
  my ($folderName) = @_;
  # get actual size of a folder and give back the number of files
  my $cmd = "du --max-depth 0 -k $folderName | awk '{print \$1}'";
  my $folderSize = `$cmd`;
  $folderSize =~ s/\r//;
  $folderSize =~ s/\n//;
  return $folderSize;
}



=head2 $kbfree=getDiskSpaceFree($dir)

Returns the available diskspace on the root filesystem in kbytes.

=cut

sub getDiskSpaceFree {
  my ($dir) = @_;
  return `df -k $dir | sed 1d | awk -F' ' '{ print $4 }'`;
}



=head2 $source=archivingDocExtractSource($dbh,$db1,$doc,$pages,$ext,$tpath)

Extract source file from archivbilder in temp folder and gives back the file

=cut

sub archivingDocSource {
  my ($dbh,$db1,$doc,$pag,$tpath,$table,$fld,$archiviert) = @_;
	$fld = "Quelle" if $fld eq "";
	my $multi = 0;
  my (@row,$sql,$ext,$s1,$s2,$l1,$l2,$file,$file1,$img,$res);
	if ($fld eq "Quelle") { 
    # first, we need the extension of the source file (lower case)
    $sql = "select QuelleExt from $db1.archiv where Laufnummer=$doc";
    @row = $dbh->selectrow_array($sql);
    $ext = lc($row[0]);
	} else {
	  $ext = "ZIP";
		my ($typ,$fname,$ext) = getEDVName($dbh,$db1,$doc);
		$ext=uc($ext);
		if ($typ eq "multi") {
			$multi = 1;
 			my $nr1 = 1;
			my @files = ();
			my $extfile = isSourceExternal($db1,$doc,$typ,$ext);
			logit("check for external file $extfile");
			if (!-e $extfile) {
				logit("no external file, so get it from database");
				my $nr1 = ($doc*1000)+1;
      	while ($nr1>0) {
					my $file = "/home/data/archivista/tmp/av-$db1-$nr1.CHK";
	 	 			$sql="select BildA from $db1.archivbilder where Seite=$nr1";
			    @row = $dbh->selectrow_array($sql);
					if (length($row[0])>0) {
						writeFile($file,\$row[0]);
						push @files,$file;
						$nr1++;
					} else {
						$nr1=0;
					}
				}
			} else {
				logit("external file found, so move it to temp folder");
				my $size = -s $extfile;
				my $maxsize = 4096*1024*1024;
				my $chunksize = 512*1024*1024;
				if ($size > $maxsize) {
					logit("external file over 4GB, so split it");
					my $buf = "";
					open(FIN,$extfile);
					while (my $read = sysread( FIN, $buf, $chunksize) ) {
						last if $nr1>640;
						my $nr = ($doc*1000)+$nr1;
						my $file1 = "/home/data/archivista/tmp/av-$db1-$nr.CHK";
						logit("write chunk $file1 for archiving");
						open(FOUT,">$file1");
						binmode(FOUT);
						print FOUT $buf;
						close(FOUT);
						push @files,$file1;
						$nr1++;
					}
					close(FIN);
				} else {
					my $file1 = "/home/data/archivista/tmp/av-$db1-$nr1.$ext";
					logit("external file under 4GB, so move it to $file1");
					my $res = callPrg("cp $extfile $file1",1);
					push @files,$file1 if $res==0;
				}
			}
			if ($files[0] ne "") {
				if ($files[8] eq "") {
  	      $file = getSourceFileNameNr($doc,"A").".$ext";
	  	    $file1 = "$tpath$file";
					my $cmd = "cat ".join(" ",@files)." >$file1";
					my $res=callPrg($cmd);
					foreach my $file (@files) {
						unlinkFile($file) if checkFile($file);
					}
				} else {
					my $c1=1;
					$file = "";
					foreach my $file0 (@files) {
	    	    my $file1a = getFileNameNr($doc,$c1,"A").".CHK";
 	      		my $file2 = "$tpath$file1a";
						moveFile($file0,$file2,1);
						$c1++;
						$file .= " " if $file ne "";
						$file .= $file1a;
					}
				}
			}
		}
	}
  if ($ext ne "" && $multi==0) {
    # ext is ok, so check if in the first Seite there is a source file
    $s1 = ($doc*1000)+1;
		$table = "archivbilder" if $table ne "archivseiten" && $archiviert==0;
    $sql = "select length($fld) from $db1.$table where Seite=$s1";
    @row = $dbh->selectrow_array($sql);
    $l1 = $row[0];
    if (length($l1>0)) {
      # there is a source file, so now check if it is a single page/pdf
      $s2 = ($doc*1000)+$pag;
      $sql = "select length($fld) from $db1.$table where Seite=$s2";
      @row = $dbh->selectrow_array($sql);
      $l2 = $row[0];
      if (length($l2>0) && $ext eq "pdf" && $pag>1) {
        # we have pdf files in every page
        $file=archivingDocSourceSingle($dbh,$db1,$doc,$pag,$tpath,$ext);
      } else {
        # we only have a single file, so extract it
        $file = getSourceFileNameNr($doc,"A")."\.".$ext;
        $file1 = "$tpath$file";
        $sql = "select $fld from $db1.$table where Seite=$s1";
        @row = $dbh->selectrow_array($sql);
        $img = $row[0];
        $res=writeFile($file1,\$img,1);
        $file="" if $res==0; 
      }
    }  
  }
  logit("Source $file of doc $doc is ok") if $file ne "";
  return $file;
}



=head2 $file=archivingDocSourceSingle($dbh,$db1,$doc,$pag,$tpath,$ext)

Combine single pdf pages to one big pdf file

=cut

sub archivingDocSourceSingle {
  my ($dbh,$db1,$doc,$pag,$tpath,$ext) = @_;
  my ($pf,$anz,$file,$file1,$fin,$fld,$cmd,$res);
  # we have single page pdf files, so combine them together
  $fld = "Quelle";
  $pf = archivingDocExtractImages($dbh,$db1,$doc,$pag,$tpath,$fld,$ext);
  $anz = @$pf;
  if ($anz>0 && $anz==$pag) {
    # all files are ok, now combine them to a single pdf file
    $file = getSourceFileNameNr($doc,"A")."\.".$ext;
    $file1 = "$tpath$file";
    foreach (@$pf) {
      # create a single name string
      $fin .= "$tpath$_ ";
    }
    $cmd = "$val{pdftk} $fin cat output $file1";
    $res = callPrg($cmd,1);
    $file="" if ($res>0); # unmark file if no success
    foreach (@$pf) {
      # delete old files
      $fin = "$tpath$_ ";
      unlinkFile($fin) if checkFile($fin);
     }
  }  
  return $file;  
}



=head2 $pfiles=archivingDocExtractImages($dbh,$db1,$doc,$pages,$ext,$tpath)

Extract files from archivbilder in temp folder and gives back the files

=cut

sub archivingDocExtractImages {
  my ($dbh,$db1,$doc,$pages,$tpath,$field,$ext,$table,$archiviert) = @_;
  my ($c,$c1,$sql,@row,@files);
  for ($c=1;$c<=$pages;$c++) {
    $c1 = ($doc*1000)+$c;
		$table = "archivbilder" if $table ne "archivseiten" && $archiviert==0;
    $sql = "select $field from $db1.$table where Seite=$c1";
    my @row = $dbh->selectrow_array($sql);
    my $img = $row[0];
    my $file = getFileNameNr($doc,$c,"A")."\.".$ext;
    my $file1 = "$tpath$file";
    my $res = writeFile($file1,\$img,1);
    if ($res==1) {
      push @files, $file;
    } else {
      @files = ();
      $c=$pages;
    }
  }
  return \@files;
}



=head2 archivingCheckDirs($db1)

Check if the archiving folder output does exist for a given database

=cut

sub archivingCheckDirs {
  my ($db1) = @_;
  my ($apath,$ds,$tout,$res);
  # get a temp path for the document
  $ds = $val{dirsep};
  $apath = "$val{dirimg}$db1";
  if (notFile($apath) || notDir($apath)) {
    mkdir $apath; 
    archivingDocSetOwner($apath);
  }
  if (checkDir($apath)) {
    $tout = "$ds"."output";
    $apath = "$apath$tout";
    if (notFile($apath) || notDir($apath)) {
      mkdir $apath;
      archivingDocSetOwner($apath);
    }
  }
}



=head2 $res=archivingDocument($dbh,$db1,$doc,$pag,$typ,$f,$s,$pf,$ps,$pf,$ki)

Process a single document and gives back 1 if the archiving process is ok

=cut

sub archivingDocument {
  my ($dbh,$db1,$doc,$pag,$typ,$files,$size,$pfiles,$psize,$pfolder) = @_;
  my ($apath,$apath1,$tpath,$ext,$fld,$pf,$ds,$tout,$psave,$res);
  # get a temp path for the document
  $ds = $val{dirsep};
  $tout = "output$ds";
  $apath = "$val{dirimg}$db1$ds$tout";
  $tpath = "$val{dirimg}$db1-$doc"."s$ds";
  $apath1=archivingDocPathCheck($apath,$$pfolder); # check if folder is ok
  if (checkDir($apath1)) {
    # the archiving folder does exist
    unlinkFile($tpath) if (notDir($tpath) && checkFile($tpath));
    mkdir "$tpath" if notDir($tpath);
    if (checkDir($tpath)) {
      # the temp path does exist, so extract files to it
      $ext = getArchivingExt($typ);
      $fld = "BildInput";
      $pf = archivingDocExtractImages($dbh,$db1,$doc,$pag,$tpath,$fld,$ext);
      my $anz = @$pf;
      if ($anz>0 && $anz==$pag) {
        # extraction was ok, now extract source file (if needed)
				my $tb="archivbilder";
        my $source = archivingDocSource($dbh,$db1,$doc,$pag,$tpath,$tb);
        $source = archivingDocSource($dbh,$db1,$doc,$pag,$tpath,$tb,"BildA");
        $apath1=archivingDocFolder($dbh,$db1,$doc,$tpath,$apath,$apath1,
					$anz,$files,$size,$pfiles,$psize,$pfolder);
        if ($apath1) {
          # now save the files to the folder
          my $err=0;
          foreach (@$pf) {
            $res=archivingDocMoveFile($tpath,$apath1,$_);
            if ($res==0 && $err==0) {
              logit("Error at doc $doc with file $_");
              $err=1;
            }
          }
          if ($source ne "" && $res>0) {
            # move the source file
						my @files = split(/\s/,$source);
						foreach my $source1 (@files) {
							if ($res>0) {
		            $res=archivingDocMoveFile($tpath,$apath1,$source1);
							}
						}
          }
        }
      }
    }
  }
  if ($res>0) {
    my $sql="update $db1.archiv set Archiviert=1, " .
            "Ordner=$$pfolder where Laufnummer=$doc";
    $dbh->do($sql);
    logit("Doc. $doc with $pag page(s) archived");
    logit("Folder $$pfolder: $$psize KByte(s) and $$pfiles files");
  } else {
    logit("ERROR: Doc. $doc NOT archieved");
  }
  # remove the temp path
  my $cmd = "rm -Rf $tpath";
  callPrg($cmd,1);
}



=head2 archivingDocFolder

Gives back the current archiving folder for the actual document

=cut

sub archivingDocFolder {
  my ($dbh,$db1,$doc,$tpath,$apath,$apath1,$anz,$files,
	    $size,$pfiles,$psize,$pfolder) = @_;
	my $empty = 0;
  if ($$psize==0 || $$pfiles==0) {
    # no old size, so it is the first time (does mean: get size/files)
    $$psize = getDiskSpaceUsed($apath1);
    opendir(FOUT,$apath1);
    my @files = readdir(FOUT);
    closedir(FOUT);
    my $anz = @files;
    $$pfiles = $anz-2;
    $$pfiles--;
    $$pfiles--;
		$$pfiles=0 if $$pfiles<0;
		$empty=1 if $$pfiles==0;
  }
          
  # now check if we need a new folder
  my $size1 = getDiskSpaceUsed($tpath);
  $$psize += $size1;
  $$pfiles += $anz;
  
	if ((($$pfiles>$files || $$psize>$size) && $empty==0)) {
    # we need a new folder
    $$pfolder++;
    archivingFolderUpdate($dbh,$db1,$$pfolder);
    $$pfiles = $anz;
    $$psize = $size1;
    logit("New folder $$pfolder at $doc");
    $apath1=archivingDocPathCheck($apath,$$pfolder); # get new folder
  }
  # check if the maximum number of folder is achieved
  $apath1="" if $$pfolder>$val{avfoldermax};
  return $apath1;
}



=head2 $path = archivingDocPathCheck($pathin,$folder)

Check if the desired folder does exist and gives it back

=cut

sub archivingDocPathCheck {
  my ($apath,$folder) = @_;
  my $apath1;
  if (checkDir($apath)) {
    # base folder does exist, so get actual folder (and check it)
    $apath1 = $apath.getArchivingFolderName($folder);
    mkdir $apath1 if notFile($apath1);
    archivingDocSetOwner($apath1);  
  }
  return $apath1;
} 



=head2 $res=archivingDocumentMoveFile($pathin,$pathout,$file)

Move an file from temp to archiving folder

=cut

sub archivingDocMoveFile {
  my ($tpath,$apath,$file,$always) = @_;
	my ($res,$fin,$fout,$error);
 	$fin = "$tpath$file";
  $fout = "$apath$file";
 	if (checkFile($fin) && (notFile($fout) || $always==1)) {
   	# move the file to the new folder location
    $res=moveFile($fin,$fout);
 	  archivingDocSetOwner($fout);    
  }
  return $res;
}



=head2 archivingDocSetOwner($fout) 

Set to a file/folder the standard user

=cut

sub archivingDocSetOwner {
  my ($fout) = @_;
  # change ownership to archivista user
  chown $val{avuser},$val{avowner},$fout if checkFile($fout);
}



=head2 archivingFolderUpdate($dbh,$db1,$folder)

Store the folder in the parameter table

=cut

sub archivingFolderUpdate {
  my ($dbh,$db1,$folder) = @_;
  # update the folder number
  my $sql="update $db1.parameter set Inhalt=$folder where " .
          "Name='ArchivOrdner' and Art='parameter' and " .
          "Tabelle='parameter'";
  $dbh->do($sql);
}



=head2 $res=archivingDatabase($dbh,$dbname)

Establishes the connection to the database and also starts the
archiving process.

=cut

sub archivingDatabase {
  my ($dbh,$db1) = @_;
  my $lockuser = "avarch";
  logit("Start archiving in $db1");
  if (jobStart(20,"archiving")==1) {  
    # first we need the smallest and biggest document
    my $sql="select min(Laufnummer), max(Laufnummer) " .
            "from $db1.archiv " .
            "where Archiviert=0 and Erfasst=1 and Seiten>0 ";
    my ($aakt,$amax) = $dbh->selectrow_array($sql);
    # current folder, files, size, path
    my $foldakt = getParameterRead($dbh,$db1,"ArchivOrdner");
		my $foldstart = $foldakt;
    logit("Current folder is: $foldakt");
    my $files = getParameterRead($dbh,$db1,"ArchivDateien");
    my $size = getParameterRead($dbh,$db1,"ArchivMByte")*1024;
    my $killinput = getParameterRead($dbh,$db1,"ArchivingDeleteInput");
    my ($fakt,$sakt); # hold the number of files and the size
    # now extract the additional conditions
    my $sqladd = getParameterRead($dbh,$db1,"ArchivSQL");
    $sqladd =~ s/(^\s*)(.*?)(\s*$)/$2/;
    # we only want the not archived and those who have pages
    $sqladd = "and ($sqladd) " if ($sqladd ne "");
    $sqladd .= "and Archiviert=0 and Erfasst=1 and Seiten>0 ";
    archivingCheckDirs($db1); # check for existing folders
    while ($aakt<=$amax && $aakt>0 && $amax>0) {
      # archive document after document
      my ($doc,$pag,$fold,$art)=getNextFreeDocument($dbh,$db1,$aakt,0,$sqladd);
      if ( $doc > 0 and DocumentLock($dbh,$db1,$doc,$lockuser ) ) {
        # we only process documents if > 0 and if current doc is unlocked
        my $error=archivingDocument($dbh,$db1,$doc,$pag,$art,$files,$size,
                                  \$fakt,\$sakt,\$foldakt,$killinput);
        DocumentUnlock($dbh,$db1,$doc); # Unlock document after processing it
        $aakt = $doc; # adjust the doc number to the counter
      } else {
        $aakt = $amax if $doc == 0; # no doc was found, stop process
      }  
      $aakt++; # go to the next document
      if (jobCheckStop(30,"archiving")==1) {
        # if someone want to cancel the job, so do it
        $aakt=$amax+1;
      }
    }
    # update the folder pointer to the current position
    $sql = "update $db1.parameter set Inhalt=$foldakt where " .
           "Name='ArchivOrdner' and Art='parameter' and Tabelle='parameter'";
		$dbh->do($sql);
    my $splittedfolders = getParameterRead($dbh,$db1,"ArchivExtended");
		if ($splittedfolders > 0) {
		  my $cmd = "/usr/bin/perl /home/cvs/archivista/jobs/splitTableImages.pl";
		  logit("$cmd $db1 $splittedfolders $foldstart");
		  my $res = callPrg("$cmd $db1 $splittedfolders $foldstart",1);
		}
    dumpKeyInfos($dbh,$db1,0);
		my $isoMByte = create_iso($dbh,$db1);
    jobStop(40,"archiving");
  } else {
    logit("Another job is running, please wait for the end or stop it!");
  }
  logit("Ending archiving process in $db1");
}



sub create_iso {
  my ($dbh,$db1) = @_;
	my $res = 0;
  my $isoMByte = getParameterRead($dbh,$db1,"isoMByte");
	my @folders = ();
  if ($isoMByte>0) {
	  logit("iso size is:$isoMByte");
	  my $folder0 = "/home/data/archivista/images/$db1/output";
    opendir(DIR,$folder0);
    my @files = readdir(DIR); 
    closedir(DIR);
    foreach my $file (@files) {
      push @folders,$file if $file ne "." && $file ne "..";
	  }
	  @folders = sort @folders;
		my $lastone = pop @folders;
		my @iso = ();
		my $size = 0;
		my $sizemax = $isoMByte*1024*1024;
		my $first = "";
		my $flist = "";
	  foreach my $folder (@folders) {
			if ($first eq "") {
				$first = $folder;
			  $flist = "$folder=/home/data/archivista/images/$db1/output/$folder";
			}
		  logit("read folder $folder for database $db1");
			my $folder1 = "$folder0/$folder";
      my $sizecd  = _sizeOfDir($folder1);
			my $sizenew = $size+$sizecd;
			logit("sizenew:$sizenew and sizemax:$sizemax");
			if ($sizenew >= $sizemax) {
        if ($flist eq "") {
				  logit("first folder $folder is to big, but create iso");
				  $flist = "$folder=/home/data/archivista/images/$db1/output/$folder";
				}
			  my $opt="-rJ --graft-points";
				my $pfadiso = "/var/lib/vz/template/iso";
				my $ftpgo=0;
 				if (`cat /proc/cpuinfo | grep 'ARM'` ne "") {
					$pfadiso = "/home/data/archivista/ftp/temp";
					$ftpgo=1;
				}
			  my $oiso = "$pfadiso/$db1"."_".$first.".iso";
				$size = $sizenew if $size==0;
			  logit("now, create iso with size $size");
				my $mkisofs=`which mkisofs`;
				$mkisofs=`which genisoimage` if $mkisofs eq "";
				chomp $mkisofs;
        my $cmd = "$mkisofs -o $oiso $opt $flist 2>/dev/null";
				$res = callPrg($cmd);
				if ($res==0) {
				  logit("iso file $oiso created");
					callPrg("chown ftp.users $oiso",1) if $ftpgo==1;
					my @fdel = split(/\s/,$flist);
					foreach my $fdel (@fdel) {
					  my ($short,$long) = split("=",$fdel);
            $res = callPrg("rm -Rf $long",1);
						if ($res==0) {
						  logit("folder $short sucessfully deleted");
						} else {
						  logit("ERROR: folder $fdel NOT sucessfully deleted");
							last;
						}
					}
				  $size = $sizecd;
					$first = $folder;
				  $flist = "$folder=/home/data/archivista/images/$db1/output/$folder";
				} else {
				  logit("ERROR: $oiso NOT created");
          last;
				}
			} else {
				$size  += $sizecd;
				if ($folder ne $first) {
				  $flist .= " " if $flist ne "";
					$flist .= "$folder=/home/data/archivista/images/$db1/output/$folder";
				}
			}
		}
	}
	return $isoMByte;
}


=head2 $res=archivingDatabase1($dbh,$dbname,$killinput)

Establishes the connection to the database and also starts the
archiving process.

=cut

sub archivingDatabase1 {
  my ($dbh,$db1,$killit) = @_;
  my $lockuser = "avarch";
  logit("Start archiving in $db1");
  if (jobStart(20,"archiving")==1) {  
   # current folder, files, size, path
    my $foldakt = getParameterRead($dbh,$db1,"ArchivOrdner");
		my $foldstart = $foldakt;
    logit("Current folder is: $foldakt");
    my $files = getParameterRead($dbh,$db1,"ArchivDateien");
    my $size = getParameterRead($dbh,$db1,"ArchivMByte")*1024;
    my $killinput = getParameterRead($dbh,$db1,"ArchivingDeleteInput");
    my ($fakt,$sakt); # hold the number of files and the size
    # now extract the additional conditions
    my $sqladd = getParameterRead($dbh,$db1,"ArchivSQL");
    $sqladd =~ s/(^\s*)(.*?)(\s*$)/$2/;
    # we only want the not archived and those who have pages
    my $sql="select Laufnummer,Seiten,Ordner,ArchivArt " .
            "from $db1.archiv " .
            "where Archiviert=0 and Erfasst=1 and Seiten>0 ";
    $sql .= "and ($sqladd) " if $sqladd ne "";
    my $res = $dbh->selectall_arrayref($sql);
    archivingCheckDirs($db1); # check for existing folders
		foreach my $res1 (@$res) {
		  my $doc = $$res1[0];
			my $pag = $$res1[1];
			my $fold = $$res1[2];
			my $art = $$res1[3];
      if ( $doc > 0 ) {
        # we only process documents if > 0 and if current doc is unlocked
        my $error=archivingDocument($dbh,$db1,$doc,$pag,$art,$files,$size,
                                  \$fakt,\$sakt,\$foldakt,$killinput);
			}
    }
    # update the folder pointer to the current position
    $sql = "update $db1.parameter set Inhalt=$foldakt where " .
           "Name='ArchivOrdner' and Art='parameter' and Tabelle='parameter'";
    $dbh->do($sql);
	  my $splittedfolders = getParameterRead($dbh,$db1,"ArchivExtended");
		if ($splittedfolders > 0) {
		  my $cmd = "/usr/bin/perl /home/cvs/archivista/jobs/splitTableImages.pl";
		  my $res = callPrg("$cmd $db1 $splittedfolders $foldstart");
		}
    dumpKeyInfos($dbh,$db1,0);
		my $isoMByte = create_iso($dbh,$db1);
    jobStop(10,"archiving");
  } else {
    logit("Another job is running, please wait for the end or stop it!");
  }
  logit("Ending archiving process in $db1");
}



=head2 dumpKeyInfos

Dumps the Infos of a Folder into the ARCH00XX Folder.

=cut

sub dumpKeyInfos {
  my ($dbh,$db,$always) = @_;
  my $mysqldump = "/usr/opt/mysql/bin/mysqldump";
	# mysqldump is another place in 64bit release
	$mysqldump = "/usr/bin/mysqldump" if check64bit()==64;
  my $table = "archiv";
  my $user = $val{user};
  my $pw   = $val{pw};
  my $folder = "/home/data/archivista/images/$db/output/";
  my $last_folder = getParameterRead($dbh,$db,"ArchivOrdner");
  # Remove one from the last folder because the last isn't finish yet
  $last_folder -= 1;
  for (my $c=$last_folder;$c>0;$c--) {
    my $folder_nr = getArchivingFolderName($c);
		$folder_nr =~ s/\///g;
    my $folder_name = $folder.$folder_nr."/";
		if ($always==0) {
	    last if notFile($folder_name);
		} else {
			next if notFile($folder_name);
		}
    my $outfile = $folder_name.lc($folder_nr).".sql";
    if (notFile($outfile)) {
      my $cmd="$mysqldump --hex-blob -u$user -p$pw -w \"ordner=$c\" ".
				"$db $table >$outfile";
      my $cmd1="$mysqldump --hex-blob -u$user -w \"ordner=$c\" $db $table";
      my $res=callPrg($cmd,0,$cmd1);
      archivingDocSetOwner($outfile);
    }
  }
}



# calculate the size of a directory

sub _sizeOfDir {
  my ($folder) = @_;
  opendir(DIR,$folder);
  my @files = readdir(DIR); 
  closedir(DIR);
  my $size = 0;
  my @stats = stat($folder);
  # Add Folder Size
  $size += $stats[7];
  foreach my $file (@files) {
    next if $file =~ /^\./;
    my $work_file = $folder."/".$file;
    if(checkDir($work_file)) {
      $size += _sizeOfDir($work_file);
    } else {
      my @stats = stat($work_file);
      $size += $stats[7];
    }
  }
  return $size;
}



=head2 $cancel=jobCheckStop($wait,$mess)

Checks if a running job should be stopped

=cut

sub jobCheckStop {
  my ($wait,$mess) = @_;
  my $pf = $val{jobpf};
  my $wr = $val{jobwr};
  my $st = $val{jobst};
  my $en = $val{joben};
  my $cancel = 0;
  if (checkFile("$pf$wr") && checkFile("$pf$st")) {
	  $wait = 10 if $wait>60;
		$wait = 10 if $wait<1;
	  sleep $wait;
    open(FOUT,">$pf$en");
    my $ms = TimeStamp() . " stop: $mess\n";
    print FOUT $ms;
    close(FOUT);
    $cancel=1;
  }
  return $cancel;  
}



=head jobCheckInit

Check if the ocr job is running, give a time to wait, but after it close it

=cut

sub jobCheckInit {
  my ($mess) = @_;
  my $pf = $val{jobpf};
  my $wr = $val{jobwr};
  my $st = $val{jobst};
	my $file1 = "$pf$wr";
	my $file2 = "$pf$st";
	if (checkFile($file1)) {
	  $mess = "init" if $mess eq "";
	  writeFile($file2,\$mess);
    jobCheckStop(60,"Initialize $0 now, ocr seems to hang");
		jobStop(10,"now stop it");
	}
}



=head2 $ok=jobStop($wait,$jmess)

Tries to stop a job

=cut

sub jobStop {
  my ($wait,$mess) = @_;
	$wait = 10 if $wait>60;
	$wait = 10 if $wait<1;
	sleep $wait;
	$mess = "job stopped" if $mess == "";
	logit("$mess");
  my $pf = $val{jobpf};
  my $wr = $val{jobwr};
  my $st = $val{jobst};
  my $en = $val{joben};
  # stop a job
  unlinkFile("$pf$wr") if checkFile("$pf$wr");
  unlinkFile("$pf$en") if checkFile("$pf$en");
  unlinkFile("$pf$st") if checkFile("$pf$st");
  return 1;
}



=head2 $ok=jobStart($wait,$mess)

Checks if a job is running (OCR or archiving)

=cut

sub jobStart {
  my ($wait,$mess) = @_;
  my $pf = $val{jobpf};
  my $wr = $val{jobwr};
  my $ok = 0;
  # start job action 
  for(my $c=0;$c<=$wait;$c++) {
    if (notFile("$pf$wr")) {
      open(FOUT,">$pf$wr");
      my $ms = TimeStamp() . " start: $mess\n";
      print FOUT $ms;
      close(FOUT);
      $c = $wait;
      $ok = 1;
    }
    sleep 1;
  }
  return $ok;
}



=head2 $ver=checkDatabase($dbh,$dbname,$log)

Does a check if it is an archivista database (and logs it if wished)
(gives back the version)

=cut

sub checkDatabase {
  my ($dbh,$db1,$log) = @_;
  my ($message,$sql,@res);
	$sql = "show tables from $db1";
	my $ptab = $dbh->selectall_arrayref($sql);
	foreach (@$ptab) {
	  my @row = @{$_};
		if ($row[0] eq "parameter") {
      $sql = "select Inhalt from $db1.parameter where Name like 'AVVersion%'";
      @res = $dbh->selectrow_array($sql);
      if ($res[0]>= $val{avversion}) {
        $message = "Archivista database found";
			}
		}
  }
  $message = "Sorry, $db1 is not an archivista database" if $message eq "";
  return $res[0];
}



=head2 $pav=getValidDatabases($dbh)

Gives back a list of all archivista databases

=cut

sub getValidDatabases {
  my ($dbh) = @_;
  my $sql = "show databases";
  my ($pdbs,@av);
  $pdbs = $dbh->selectall_arrayref($sql);
  foreach (@$pdbs) {
    my $db = $$_[0];
    push @av,$db if (checkDatabase($dbh,$db)>=$val{avversion});
  }
  return \@av;
}



=head2 $psel=chooseXValues($pvals,$text)

Uses Xdialog to get a selection of a list

=cut

sub chooseXValues {
  my ($pvals,$text) = @_;
  my $args = "";
  for my $db (@$pvals) {
    # compose all values
    my $tag   = "'" . $db . "'";
    my $item  = "''";
    my $value = "'" . "0" . "'";
    $args .= " $tag $item $value";
  }
  # special handling of height so it shoes correct
	my $height = 20;
	my $dialog = "dialog";
	my $lang = getLang();
	my $cancel=findit("CANCEL",$lang);
  my $cmd = qq($dialog --cancel-label "$cancel" --stdout --checklist "$text" ) .
            qq("$height" "50" "$height" $args);
  # get back all select entries
  my $RC = `$cmd`;
  my @lines = split "\n", $RC;
  my @selection = split "/", $lines[0];
  # return the values as a pointer of an array
  return \@selection;
}



=head2 restartOCRbatch($db,$range) 

Restart the OCR batch for some documents again

=cut

sub restartOCRbatch {
  my ($db1,$range,$host,$user,$pw) = @_;
  my $dbh=MySQLOpen();
  if ($dbh) {
    if (HostIsSlave($dbh)==0) {
      # first, we need all Archivista databases
      my $host1 = $dbh->quote($host);
      my $user1 = $dbh->quote($user);
      my $db2 = $dbh->quote($db1);
      my $pw1 = $dbh->quote($pw);
      my $dbh2 = MySQLOpen($host,$db1,$user,$pw);
      if ($dbh2) {
        if (HostIsSlaveSimple($dbh2)==0) {
          my $pdb = getValidDatabases($dbh2); 
          foreach (@$pdb) {
            if ($_ eq $db1) {
				      restartOCRbatch1($dbh,$db1,$range,$dbh2,
								$host1,$db2,$user1,$pw1,$host);
				    }
			    }
        } else {
          logit("No ocr in slave mode at $host and $db1");
        }
        $dbh2->disconnect();
			} else {
        logit("Restart of ocr batch failed -- no connection at $host in $db1");
			}
		} else {
      logit("No ocr in slave mode at current box");
		}
    logit("End activate ocr batch again in $db1");
	  $dbh->disconnect();
  } else {
    logit("Restart of ocr batch failed -- no connection at current box");
	}
}



=head1 restartOCRbatch1($dbh,$db1,$range,$dbh2,$host1,$db2,$user1,$pw1)

Restart all ocr jobs in one database

=cut

sub restartOCRbatch1 {
  my ($dbh,$db1,$range,$dbh2,$host1,$db2,$user1,$pw1,$host) = @_;
  my ($sql,@row);
  logit("Start activate ocr batch again in $db1");
  my $archivfolders = ""; # check if we have extended tables
	$archivfolders = getBlockTableCheck($dbh,$archivfolders,$db1);
	my $maxfiles = 10000;
	my $curfiles = 0;
	my $serverfounds=_ocr_cluster($host,$dbh2,$db1,$val{pw},$range);
	if ($serverfounds==1) {
		logit("ocr cluster started with $db1 and $range");
	} else {
	  # get back the range of document (or nothing if all docs.
  	my ($sqladd,$aakt,$amax) = getSQLrange($range);
	  # we want to reprocess documents again
  	$sql="select min(Laufnummer), max(Laufnummer) " .
    	   "from $db1.archiv where Erfasst=1 and Seiten>0 $sqladd";
	  ($aakt,$amax) = $dbh2->selectrow_array($sql);
  	while ($aakt<=$amax) {
    	# archive document after document
	    my ($doc,$pag,$fold,$art)=getNextFreeDocument($dbh2,$db1,$aakt,0,$sqladd);
  	  if ($pag>0) {
			  $curfiles++;
				if ($curfiles<=$maxfiles) {
			  	restartOCRbatchCheckTable($dbh,$db1,$dbh2,$doc,$archivfolders);
				}
  	    # only process documents that have pages
    	  $sql = "select ID,DONE,ERROR from logs where Laufnummer=$doc " .
      	       "and db=$db2 and host=$host1 and user=$user1"; # check db
	      @row = $dbh->selectrow_array($sql);
  	    if ($row[0]==0) {
    	    # no log entry found
	        $sql = "insert into logs set " .
  	             "host=$host1,db=$db2,user=$user1,pwd=$pw1," .
    	           "type='sne',Laufnummer=$doc,DONE=".LOG_DONE.
      	         ",pages=$pag";
        	$dbh->do($sql);
	      } else {
  	      # there was a log entry found
    	    my $done=$row[1];
      	  my $err=$row[2];
	        if (($done==LOG_DONE || $done==LOG_AXISDONE ||
  	           $done==LOG_OCRDONE) && $err==0) {
    	      $sql="update logs set DONE=".LOG_DONE." where ID=$row[0]";
      	    $dbh->do($sql);
	        }
  	    }
    	  $aakt = $doc;
	    } else {
  	    $aakt = $amax if $doc == 0; # no doc was found, stop process
    	}
	    $aakt++; # go to the next document
  	}
	}
}



sub restartOCRbatchCheckTable {
  my ($dbh,$db1,$dbh2,$doc,$archivfolders) = @_;
  my $fr =  "/home/archivista/.wine/drive_c/Programs/Av5e/avformrc.exe";
	my $size = -s $fr;
	if ($archivfolders>0 && $size != 115200) {
	  my $sql = "select Akte,Archiviert,Ordner,Seiten from $db1.archiv where ".
	            "Laufnummer = $doc";
	  my ($lnr,$archiviert,$folder,$seiten) = $dbh2->selectrow_array($sql);
		my $table = "archivbilder";
		my $tablego = getBlobTable($dbh2,$folder,$archiviert,$table,$archivfolders);
		if ($tablego ne $table && $doc == $lnr) {
		  for (my $c=1;$c<=$seiten;$c++) {
			   my $nr = ($doc*1000)+$c;
			   $sql = "select Seite from $db1.archivbilder where Seite=$nr";
				 my ($res) = $dbh2->selectrow_array($sql);
				 if ($res==0) {
				   $sql = "insert into $db1.archivbilder ".
					        "(Seite,Bild,BildA,BildInput,Quelle) ".
					        "select Seite,Bild,BildA,BildInput,Quelle ".
									"from $tablego where Seite=$nr";
					 logit("insert temp image $nr to $db1.archivbilder");
					 $dbh2->do($sql);
				 }
			}
		}
	} else {
	  logit("no need to resave docs in archivbilder");
	}
}



=head2 ($sql,$start,$ende)=getSQLrange($range)

Gives back an sql fragment either 'and Laufnummer=x' or
'and Laufnummer between x and y' and start/end document

=cut

sub getSQLrange {
  my ($val) = @_;
  my ($sql,$x,$y,$start,$ende);
  # check if we only have one document or several documents
  $val=~/^(\d+)(-*)(\d*)/;
  $x=$1;
  $y=$3;
  if ($y != ''){
	  if ($y>0 && $y>$x) {
      $sql = "and Laufnummer between $x and $y ";
		} else {
		  $sql = "and Laufnummer=$x ";
		}
  } else {
    $sql = "and Laufnummer=$x " if ($x>0);
  }
  $start=$x if $x>0;
  $ende=$x if $x>0;
  $ende=$y if $y>0;
  return ($sql,$start,$ende); 
}



=head2 getFileExtensions

Return the extension of a file or in other words the filetype.

=cut

sub getFileExtension {
  my ($fn) = @_;
  my @sections=split(/\./,$fn);
  return $sections[$#sections];
}



=head2 get_rotation($obj)

This method calculates the rotation of the image with respect to the scanmode.

=cut

sub rotateADF {
  my ($obj) = @_;
  my $factor=0;
  $factor=3 if ($obj->adf==-2) && (($obj->pages2 & 1));
  $factor=1 if ($obj->adf==-2) && (!($obj->pages2 & 1));
  my $angle=$factor*90;
  return $angle%360;
}



=head2 rotate

Rotates the incomming picture with respect of its type.
This method returns the orientation of the document:

0=portrait
1=landscape

=cut

sub rotate {
  my ($obj,$image) = @_;
  my $rotation=rotateADF($obj);
	$rotation = (($rotation + ($obj->rotate)) % 360);
	$rotation = (($rotation + $obj->rotate2) % 360);
  return 0 if (!$rotation);
  # We want to rotate our picture because the scanner gives it reversed
  # if we have a 90 Rotation we need to make it 270 and vice versa
  my $rotation1 = $rotation;
  if ($rotation==90) {
    $rotation1=270;
  } elsif ($rotation==270) {
    $rotation1=90;
  } 
  ExactImage::imageRotate($image,$rotation1);
  $obj->changedimage(1);
}



=head1 checkSplitDoc($obj)

Check if we need to open a new document

=cut

sub checkSplitDoc {
  my ($obj,$image) = @_;
  if ($obj->newdocpages>0 && $obj->pages>=$obj->newdocpages) {
	  saveDoc($obj); # close the current doc
		my $oldnr = $obj->doc;
		$obj->doc(0);
		checkNewDoc($obj);
    copyValues($obj->dbh2,$obj->database,$obj->fields,$oldnr,$obj->doc);
	}
}



=head1 copyValues($dbh,$pfields,$lold,$lnew)

Copy the fields from one document to another

=cut

sub copyValues {
  my ($dbh,$db1,$pfields,$lold,$lnew) = @_;
  return if $lold==0;
  my $sql=""; # now get all field values from the old document
  my (@flds,@typ);
  foreach(@$pfields) {
    my $f1 = $_->{name};
    last if $f1 eq "Laufnummer"; # stop for field Notes
    my $t1 = $_->{type};
    # don't update Document, Datum and Pages
    if ($f1 ne "Akte" && $f1 ne "Datum" &&
		    $f1 ne "Seiten" && $f1 ne "Gesperrt") {
      $sql .= "," if $sql ne "";
      $sql .= $f1;
      push @flds,$f1;
      push @typ,$t1;
    }  
  }
  $sql = "select $sql from $db1.archiv where Laufnummer=$lold";
  my @row = $dbh->selectrow_array($sql);
  $sql=""; # now update the old values to the new document
  my $c=0;
  foreach(@row) {
    my $v = $_;
    my $dont = 0;
    if ($typ[$c] eq "varchar" or $typ[$c] eq "datetime") {
      $v = $dbh->quote($v);
    } else {
      $dont=1 if $v eq "";
    }
    if ($dont==0) {
      $sql .= "," if $sql ne "";
      $sql .= $flds[$c]."=".$v;
    }
    $c++;
  }
  if ($sql ne "") {
    $sql = "update $db1.archiv set $sql where Laufnummer=$lnew";
		logit($sql);
    $dbh->do($sql);
  }
}



=head1 MainAccessLog()

Check if we need to log the the actions

=cut

sub MainAccessLog {
  my ($dbh,$db1,$dbh2,$doc,$page,$action,$user,$host) = @_;
  return if $dbh2==0 or $dbh==0;
  my $dbhgo = $dbh2;
  $dbhgo = $dbh if $host eq "localhost";
  if (getParameterRead($dbh2,$db1,"ACCESS_LOG")) {
    my $string = "page=$page;";
    if ($action eq "add_page") {
      my $pagedoc = ($doc*1000)+$page;
      my $sql = "select BildInput from $db1.archivbilder where Seite=$pagedoc";
      my @row = $dbh2->selectrow_array($sql);
      my $imagehash = md5_hex $row[0];
      $string .= "imagehash=$imagehash;";
      $sql = "select UserModDatum,UserModName from $db1.archiv ".
			       "where Laufnummer=$doc";
      @row = $dbh2->selectrow_array($sql);
      $string .= "moddate=$row[0];moduser=$row[1];";
    }
    my $sql = "host=".$dbh->quote($val{host});
    $sql .= ",db=".$dbh->quote($db1);
    $sql .= ",user=".$dbh->quote($user);
    $sql .= ",document=".$doc;
    $sql .= ",action=".$dbh->quote($action);
    $sql .= ",additional=".$dbh->quote($string);
    $sql = "insert into access set $sql";
    $dbhgo->do($sql);
    $sql = "select LAST_INSERT_ID()";
    my @row = $dbhgo->selectrow_array($sql);
    my $id = $row[0];
    my $id2 = $id-1;
    my $hash = "";
    if ($id2>0) {
      my $sql2 = "select hash from access where id=$id2";
      @row = $dbhgo->selectrow_array($sql);
      $hash = $row[0];
    }
    $string = "$sql$hash";
    $hash = md5_hex $string;
    $sql = "update access set hash=".$dbh->quote($hash)." ".
           "where id=$id";
    $dbhgo->do($sql);
  }
}



=head1 emptypath($path)

remove $path and create it again

=cut

sub emptypath {
  my ($path) = @_;
  if (checkDir($path)) {
    callPrg("rm -Rf $path",1);
    print "$path killed\n";
  }
  callPrg("mkdir $path",1);
}



=head1 sqlrange($range)

gives back an sql fragment either 'and Laufnummer=x' or
'and Laufnummer between x and y'

=cut

sub sqlrange {
  my ($val,$field) = @_;
	$field = "Laufnummer" if $field ne "Ordner";
  my ($sql,$x,$y);
  # check if we only have one document or several documents
  $val=~/^(\d+)(-*)(\d*)/;
  $x=$1;
  $y=$3;
  if ($y != ''){
	  if ($y>0 && $y>$x) {
      $sql = "and $field between $x and $y";
		} else {
		  $sql = "and $field=$x" if $x eq $y;
		}
  } else {
    $sql = "and $field=$x" if $x>0 && $x eq $val;
  }
	if ($sql eq "") {
		my @multiple = split(/\,/,$val);
		my $or = "";
		foreach my $nr (@multiple) {
			$or .= " or " if $or ne "";
			$or .= "$field=$nr";
		}
		$sql = "and ($or)" if $or ne "";
	}
  return $sql; 
}



=head1 $d=DocumentAddDatGerman($d)

Format a date ('dd.mm.yyyy'')

=cut

sub DocumentAddDatGerman {
  my ($d) = @_;
  $d=substr($d,8,2).".".substr($d,5,2).".".substr($d,0,4);
  return $d;
}



=head1 $d=DocumentAddDatSQL($d)

German date to SQL ('yyyy-mm-dd 00:00:00')

=cut

sub DocumentAddDatSQL {
  my ($d) = @_;
  $d= "'".substr($d,6,4)."-".substr($d,3,2)."-".substr($d,0,2)." 00:00:00'";
  return $d;
}



=head1 exportformatvalue($val,$typ)

does format a mysql field value to the archivista export format

=cut

sub exportformatvalue {
  my ($val,$typ) = @_;
  my $t="";
  if ($val ne "") {
    if ($typ eq "varchar" or $typ eq "text") {
      $t=$val;
    } elsif ($typ eq "int" || $typ eq "bigint") {
      $t=$val;
    } elsif ($typ eq "tinyint") {
      if ($val==0) {
        $t="Nein";
      } else {
        $t="Ja";
      }
    } elsif ($typ eq "double") {
      $t=$val;
    } elsif ($typ eq "datetime") {
      $t=DocumentAddDatGerman($val);
    }
  }
  $t.="\t";
  return $t;
}



=head1 importformatvalue ($dbh,$bisher,$val,$feld,$typ,$size)

does format a mysql field value to the archivista export format

=cut

sub importformatvalue {
  my ($dbh,$bisher,$val,$feld,$typ,$size) = @_;
  my $t="";
  if ($val ne "") {
    if ($typ eq "varchar") {
      if (length($val)>$size) {
        $val = substr $val, 0, $size-1;
      }   
      $t=$dbh->quote($val);
    } elsif ($typ eq "text") {
      $t=$dbh->quote($val);
    } elsif ($typ eq "int" || $typ eq "bigint") {
      $t=int $val;
    } elsif ($typ eq "tinyint") {
      if ($val eq "Ja") {
        $t=1
      } else {
        $t=0
      }
    } elsif ($typ eq "double") {
      $t=$val;
    } elsif ($typ eq "datetime") {
      $t=DocumentAddDatSQL($val);
    }
    if ($t ne "") {
      $t="$feld=$t";
      if ($bisher ne "") {
        $t=", ".$t;
      }
    }
  }
  return $t;
}



=head1 addLogEntryImport($dbh,$host,$db,$user,$pw,$akte,$seiten)

Add a log entry from import

=cut

sub addLogEntryImport {
  my ($dbh,$dbh2,$host,$db,$user,$pw,$akte,$seite,$mod,
		$formrec,$old,$done,$wait) = @_;
	$wait=0 if $wait eq '';
	$formrec=0 if $formrec eq '';
	$done=LOG_DONE if $done eq '';
	$done=LOG_DONE if $done != LOG_OCRDONE;
	$done=LOG_FORMREC if $formrec>0;
	my $pw1 = $pw;
	my $serverfounds=0;
	$serverfounds=_ocr_cluster($host,$dbh2,$db,$val{pw},$akte) if $done==LOG_DONE;
	if ($serverfounds==1) {
		logit("ocr cluster started at document $akte in $db");
	} else {
		$pw1 = "" if $host eq "localhost";
		my $plus=$wait*60;
		my @row = $dbh->selectrow_array("select unix_timestamp()+$plus");
		$wait=$row[0];
		my $sql="select id from archivista.logs where ".
		  "host=".$dbh->quote($host)." and db=".$dbh->quote($db)." and ".
		  "user=".$dbh->quote($user)." and pwd=".$dbh->quote($pw1)." and ".
		  "Laufnummer=$akte and idstart=$akte and formrec=$formrec and ".
			"DONE=$done and ERROR=0 and TYPE='$mod'";
		@row=$dbh->selectrow_array($sql);
  	$sql="insert into";
		$sql="update" if $row[0]>0 && $old==1;
		$sql.=" archivista.logs set host=".$dbh->quote($host).",".
		  "db=".$dbh->quote($db).",user=".$dbh->quote($user).",".
		  "pwd=".$dbh->quote($pw1).",Laufnummer=$akte,idstart=$akte,".
			"formrec=$formrec,pages=$seite,wait=$wait,DONE=$done,ERROR=0,TYPE='$mod'";
		$sql.=" where id=$row[0]" if $row[0]>0 && $old==1;
		#logit($sql);
		$dbh->do($sql);
	}
}	



sub _ocr_cluster {
	my ($host,$dbh,$db,$pw,$range) = @_;
	my $serverfounds=0;
	my $ocr_cluster = getParameterRead($dbh,$db,"OCR_CLUSTER");
	if ($ocr_cluster ne "" && $host eq "localhost") {
		if ($range ne "") {
			my ($min,$max)=split(/\-/,$range);
			$max = $min if $max eq "";
			$min = ($min*1000)+1;
			$max = ($max*1000)+999;
			my $sql = "update $db.archivseiten set Zipped=-1 ".
				"where Seite between $min and $max";
			$dbh->do($sql);
			my $count=`ps ax | grep 'allocrpdf' | grep '$db'`;
			my @counts = split(/\n/,$count);
			my @outs = ();
			foreach my $file (@counts) {
			  push @outs,$file if index($file,'grep')==-1;
			}
			my $maxi=0;
			foreach my $out (@outs) {
	  		$maxi++ if $out ne "";
			}
			if ($maxi==0) {
				my $prg = "/usr/bin/perl /home/cvs/archivista/jobs/allocrpdf.pl ".
					"localhost $db root";
				my $res = callPrg("nohup $prg $pw &",1);
				logit("$res=>$prg *** calling from $range");
			}
		}
		my $server = "";
		if (`uname -r`<5) {
			$server = `cd /etc/perl;perl install.pl getip`;
		} else {
			$server = `cd /;perl install.pl getip`;
		}
		chomp $server;
		if ($server ne "") {
			my @hosts = ();
			my @hosts0 = split(/\,/,$ocr_cluster);
			my $base = "";
			my $ext = "";
			foreach my $host (@hosts0) {
				my @parts = split(/\./,$host);
				if ($parts[3] ne "") {
					$base = $parts[0].'.'.$parts[1].'.'.$parts[2];
					$ext = $parts[3];
				} else {
					$ext = $parts[0];
				}
				push @hosts,$base.'.'.$ext if $ext ne "" && $base ne "";
			}
			my $goips = join(',',@hosts);
			if ($goips ne "") {
				my $prg0 = "/usr/bin/perl /home/cvs/archivista/jobs/allocrz.pl ".
					"$server $goips 8 $db";
				my $prg = "$prg0 $pw";
				my $res = callPrg($prg,1);
				logit("$res=>$prg0");
				$serverfounds=1 if $res==0;
			}
		}
	}
	return $serverfounds;
}



=head1 $string=findit($id)

Get back a string (given an id), lang comes from global var

=cut

sub findit {
  my ($id,$lang) = @_;
	return findit1($id,$lang);
}



=head1 $string=findit1($id)

Get back a string (given an id), lang comes from global var

=cut

sub findit1 {
  my ($id,$lang) = @_;
	my $file="/usr/lib/perl5/site_perl/languages.txt";
	my $file1="/usr/lib/perl5/site_perl/languages1.txt";
  $file1="$file" if !-e "$file1";
	my $res = "";
  if ($lang eq "de") {
		$res = `cat $file1 | grep -w \"$id\" | awk -F\"\\t\" '{print \$3}'`;
	} elsif ($lang eq "fr") {
		$res = `cat $file1 | grep -w \"$id\" | awk -F\"\\t\" '{print \$5}'`;
	} elsif ($lang eq "it") {
		$res = `cat $file1 | grep -w \"$id\" | awk -F\"\\t\" '{print \$6}'`;
	} else {
		$res = `cat $file1 | grep -w \"$id\" | awk -F\"\\t\" '{print \$4}'`;
	}
	chomp $res;
	return $res;
}



=head1 $lang=lang()

Give back the language id (en/de)

=cut

sub getLang {
	my $kbd = "";
	my $lang = "de";
	if (-e "/etc/archivista/getlang.pl") {
		$lang = `cd /etc/archivista;/usr/bin/perl getlang.pl`;
	} else {
		readFile2("/etc/lang.conf",\$kbd);
		my @lines = split(/\n/,$kbd);
		$lang = $lines[0];
		if ($lang eq "") {
			readFile2("/home/archivista/.xkb-layout",$kbd);
			@lines = split(/\n/,$kbd);
			my $kbd1 = $lines[0];
			my $kbd2 = "";
			($lang,$kbd2) = split(/\_/,$kbd1);
		}
		$lang = "en" if $lang ne "fr" && $lang ne "it" && $lang ne "de";
	}
	return $lang;
}



=head1 $val=escape($val)

Quote special chars in value parts (axis/xerox/autofields)

=cut

sub escape {
  my ($str) = @_;
  $str =~ s/([^a-zA-Z0-9_.-])/sprintf("%%%02x", ord($1))/ge;
	return $str;
}



=head1 $val=unescape($val)

Unquote special chars in value parts (axis/xerox/autofields)

=cut

sub unescape {
  my ($str) = @_;
  $str =~ tr/+/ /;
	$str =~ s/\\$//;
  $str =~ s/%([0-9a-fA-Z]{2})/pack("c",hex($1))/eg;
  $str =~ s/\r?\n/\n/g;
	return $str;
}



=head1 ($key,$value)=keyvalue($line,$seperator)

Separates at the first seperator to key and value.

=cut

sub keyvalue {
  my ($line,$seperator) = @_;
  my @vals = split( $seperator, $line );
	my $key = shift @vals;
	my $value = join($seperator,@vals);
	return ($key,$value);
}



=head1 $pblob=getBlobFile($dbh,$fld,$page,$ordner,$arch,$tbl,$archfolders)

Give back a file from the blob table archivbilder or archimg...

=cut

sub getBlobFile {
  my ($dbh,$field,$page,$ordner,$archiviert,$table,$archivfolders) = @_;
	$table = getBlobTable($dbh,$ordner,$archiviert,$table,$archivfolders);
  my $sql = "select $field from $table where Seite=$page";
  my @row = $dbh->selectrow_array($sql);
  return \$row[0];
}



=head1 $pblob=getBlobTable($dbh,$ordner,$arch,$tbl,$archfolders)

Give back the correct table name for blob table archivbilder

=cut

sub getBlobTable {
  my ($dbh,$ordner,$archiviert,$table,$archivfolders) = @_;
	$table = "archivbilder" if $table ne "archivseiten";
	$archivfolders = getBlockTableCheck($dbh,$archivfolders);
	if ($archiviert==1 && $archivfolders>0 && $table eq "archivbilder") {
	  my $nr = int(($ordner-1)/$archivfolders);
		$nr = $nr * $archivfolders;
		$table = "archimg".sprintf("%05d",$nr);
	}
	return $table;
}



=head archivfolders = getBlockTableCheck($dbh,$archivfolders,$db1)

Check if we have extended tables or not

=cut

sub getBlockTableCheck {
  my ($dbh,$archivfolders,$db1) = @_;
	if ($archivfolders eq "") {
	  $db1 .= "." if $db1 ne "";
    my $sql = "select Inhalt from ".$db1."parameter where Art = " .
           "'parameter' AND Name='ArchivExtended'";
    my @row = $dbh->selectrow_array($sql);
		$archivfolders = $row[0];
	}
	return $archivfolders;
}



=head2 createThumbsAndSave($dbh,$db1,$dbh2,$img,$depth,$nr,$page,$sd,$od)

Create thumbnails from the original images

=cut

sub createThumbsAndSave {
  my ($dbh,$db1,$image,$depth,$nr,$page,$scanid,$depthold) = @_;
  my $seite = $nr * 1000 + $page;
  if ($depth != $depthold) {
    imageReadAndSave($dbh,$db1,$image,"BildInput",$depth,$seite);
  }
  my $fact=getBoxParameterRead($dbh,$db1,"PrevScaling",0,100,20);
  $fact=10 if $fact<10 && $fact !=0;
  if ($fact>0) {
    my $fact1 = $fact/100;
		my $width = ExactImage::imageWidth($image);
		my $height = ExactImage::imageHeight($image);
		if ($width>1024 && $height>786) {
      if ($depth==1) {
        ExactImage::imageBilinearScale($image,$fact1);
      } else {
        ExactImage::imageScale($image,$fact1);
      }
      imageReadAndSave($dbh,$db1,$image,"Bild",$depth,$seite);
		} else {
		  logit("no screen copy (under 1024x768)");
		}
  }
  updateArchivTable($dbh,$db1,$nr,$seite,$depth,$scanid);
}



=head1 updateArchivTable($dbh2,$db1,$nr,$seite,$depth,$scanid)

Update the archiv table to the current values

=cut

sub updateArchivTable {
  my ($dbh2,$db1,$nr,$seite,$depth,$scanid) = @_;
  my $ext1="TIF";
  my $ext2="PNG";
  my $archivart=1;
  if ($depth>1) {
    $ext1="JPG";
    $ext2="JPG";
    $archivart=3;
  }
  my $sql = "update $db1.archiv set BildInputExt='$ext1',BildAExt='$ext2', " .
         "Erfasst=1,ArchivArt=$archivart where Laufnummer=$nr";
  $dbh2->do($sql);
  if ($scanid ne "") {
    # add the page to the archivseiten table
    my @scan = split(";",$scanid);
    my $ocr = $scan[10];
    my $erf = 0;
    my $excl = 0;
    $ocr=0 if ($ocr <0);
    if ($ocr==26) {
      $erf = 1;
      $excl = 0;
    } elsif ($ocr==27) {
      $erf = 0;
      $excl = 1;
    }
    $ocr=0 if ($ocr >20);
    $sql = "insert into $db1.archivseiten set Seite=$seite, " .
           "OCR=$ocr,Erfasst=$erf,Ausschliessen=$excl";
    $dbh2->do($sql);
  }
}



=head2 imageReadAndSave($dbh,$db1,$image,$feld,$depth)

Save an image to the archivbilder table

=cut

sub imageReadAndSave {
  my ($dbh,$db1,$image,$feld,$depth,$seite) = @_; # ExactImage image object
  my $image1 = "";
  my $sql1 = "update $db1.archivbilder set ";
  my $sql2 = " where Seite=$seite";
  my $w = ExactImage::imageWidth($image);
  my $h = ExactImage::imageHeight($image);
  if ($depth == 1) {
    $image1 = ExactImage::encodeImage($image,"tiff");
  } else {
    my $fact=getBoxParameterRead($dbh,$db1,"JpegQuality",10,100,33);
    $image1 = ExactImage::encodeImage($image,"jpeg",$fact);
  }
  $image1 = $dbh->quote($image1);
  my $sql = "$sql1 $feld=$image1";
  my $name = "Bild";
  $name .= "A" if $feld eq "BildInput";
  my $namex = $name."X";
  my $namey = $name."Y";
  $sql .= ",$namex=$w,$namey=$h" if $w>0 && $h>0;
  $sql .= $sql2;
  eval( $dbh->do($sql));
}



=head1 ($host,$db,$user,$pw)=getConnectionValues()

Give back the connection information

=cut

sub getConnectionValues {
  return ($val{host},$val{db},$val{user},$val{pw});
}



=head1 $fpdf=HTMLtoPDF($fin1,$tpfad)

Create a pdf file from html file (calling Firefox)

=cut

sub HTMLtoPDF {
  my ($fin1,$tpfad) = @_;
	my $fpng = "$tpfad/screenshot.png";
	my $fpdf = "";
  unlinkFile($fpng) if checkFile($fpng);
	if (!checkFile($fpng)) {
	  my ($fh,$fhtml) = tempfile(DIR=>$tpfad,SUFFIX=>'.html');
  	if ($fh) {
			close($fh);
			my $cont = "";
			readFile2($fin1,\$cont);
			$cont =~ s/(\sloading=\"lazy\"\s)/ /g;
			$cont =~ s/(\sloading=lazy\s)/ /g;
			writeFile($fhtml,\$cont,1);
			if (checkFile($fhtml)) {
				my $res=callPrg("chown archivista.archivista \"$fhtml\"");
			}
		}
		$fpdf = PNGtoPDF($fpng,$tpfad) if HTMLScreenshot($fhtml,$tpfad,$fpng);
		unlinkFile($fhtml) if checkFile($fhtml);
  	unlinkFile($fpng) if checkFile($fpng);
	}
	return $fpdf;
}



=head1 $ftext=HTMLtoText($fin1,$tpfad)

Create a text file form html file (calling html2text)

=cut

sub HTMLtoText {
	my ($fin,$tpfad) = @_;
	my $ftext = "";
  my ($fh,$ftmp) = tempfile(DIR=>$tpfad,SUFFIX=>'.tmp');
 	if ($fh) {
		close($fh);
	  my ($fh,$fout) = tempfile(DIR=>$tpfad,SUFFIX=>'.txt');
 		if ($fh) {
			close($fh);
			my $cont = "";
			my $url = "";
			readFile2($fin,\$cont);
			$cont =~ /(\surl:\s)(.*?)(\n)/s;
			$url=$2 if $1 eq " url: " && $2 ne "";
			$cont = "";
			my $cmd = "html2text -utf8 \"$fin\" >$ftmp";
			my $res=callPrg($cmd);
			readFile2($ftmp,\$cont,1);
			$cont =~ s/(\[data:)(.*?)(\])//smg;
			$cont =~ s/\_/ /g;
			$cont =~ s/\[/ /g;
			$cont =~ s/\]/ /g;
			$cont =~ s/(\r\n\r\n)+/\r\n/g;
			$cont =~ s/(\n\n+)/\n/sg;
			$cont = fromUTF8($cont);
			$cont = "URL: $url\n".$cont if $url ne "" && length($url)>=8;
			writeFile($fout,\$cont,1);
			$ftext=$fout if checkFile($fout);
		}
	}
	return $ftext;
}



=head1 $ok=HTMLScreenshot($fhtml,$tpfad,$fpng)

Create a png file from html file (calling Firefox)

=cut

sub HTMLScreenshot {
	my ($fhtml,$tpfad,$fpng) = @_;
	my $res=-1;
	my $ok=0;
	if (checkFile($fhtml)) {
		my $prof = '. /etc/profile'; # import default settings
		my $nr = ":0";
		$nr = `cat /tmp/display` if -e "/tmp/display";
	 	my $disp = "export DISPLAY=$nr"; # export display to default
		for (my $c=0;$c<2;$c++) {
			my $psfirefox = `ps ax | grep firefox`;
			my @files = split(/\n/,$psfirefox);
			if ($files[1] ne "") {
				callPrg("killall firefox-esr");
				callPrg("killall firefox-bin");
				callPrg("killall firefox");
			} else {
				last;
			}
			sleep 1;
		}
		my $prg = "/bin/su - archivista -c \"$prof; $disp; cd /usr/share/firefox;".
			"./firefox --headless --width ".HTML_WIDTH." --height ".HTML_HEIGHT.
			" --window-size ".HTML_WIDTH.
			" --screenshot $fpng file://$fhtml\"";
		logit("$prg");
		$res=callPrg($prg);
		if (!checkFile($fpng)) {
			$prg = "/bin/su - archivista -c \"$prof; $disp; cd /usr/share/firefox;".
				"./firefox --headless --width ".HTML_WIDTH." --height ".HTML_HEIGHT.
				" --window-size ".HTML_WIDTH.",32700".
				" --screenshot $fpng file://$fhtml\"";
			logit("$prg");
			$res=callPrg($prg);
		}
	}
	$ok=1 if checkFile($fpng);
	return $ok;
}



=head1 $fpdf=HTMLScreenshot($fpng,$tpfad)

Create a pdf file from large png file

=cut

sub PNGtoPDF {
	my ($fpng,$tpfad) = @_;
	my @jpgs = ();
	my @pdfs = ();
	my $fpdf = "";
	my $wimg=0; # width of the image
	my $himg=0; # height of the image
	my $scale=HTML_SCALE;
	my $scale2=HTML_SCALE2;
	my $scroll=HTML_SCROLL*$scale;
	my $width=HTML_WIDTH*$scale;
	my $height=HTML_HEIGHT*$scale;
	if (checkFile($fpng)) {
		my $fpng2 = "/tmp/screenshot2.png";
		unlinkFile($fpng2) if checkFile($fpng2);
		my $cmd = "econvert -i $fpng --colorspace RGB -o $fpng2";
		my $res = callPrg($cmd);
		if (checkFile($fpng2)) {
			unlinkFile($fpng);
			$cmd = "mv $fpng2 $fpng";
			$res = callPrg($cmd);
		}
	}
	if (checkFile($fpng)) {
		my $info = `file $fpng`;
		$info =~ /([0-9]+)(\sx\s)([0-9]+)(,)/;
		if ($1>0 && $3>0) {
			$wimg = $1;
			$himg = $3;
		}
		if ($wimg>0 && $himg>0) {
			my $pages = int($himg/$scroll);
			my $top = 0;
			for(my $c=0;$c<=$pages;$c++) {
			  my ($fh,$fjpg1) = tempfile(DIR=>$tpfad,SUFFIX=>'.png');
		  	if ($fh) {
					close($fh);
					my $cmd = "econvert -i $fpng --crop 0,$top,$wimg,$height -o $fjpg1";
					my $res=callPrg($cmd);
					push @jpgs,$fjpg1 if checkFile($fjpg1);
					last if $res!=0;
					$top+=$scroll;
				}
			}
			foreach my $fjpg2 (@jpgs) {
			  my ($fh,$fpdf2) = tempfile(DIR=>$tpfad,SUFFIX=>'.pdf');
		  	if ($fh) {
					close($fh);
					my $cmd = "econvert -i $fjpg2 --scale $scale2 ".
						"--resolution 300x300 -o $fpdf2";
					my $res=callPrg($cmd);
					push @pdfs,$fpdf2 if checkFile($fpdf2);
					last if $res!=0;
				}
			}
		  my ($fh,$fpdfall) = tempfile(DIR=>$tpfad,SUFFIX=>'.pdf');
	  	if ($fh) {
				close($fh);
				my $cmd = "pdftk ".join(" ",@pdfs)." output $fpdfall";
				my $res=callPrg($cmd);
				$fpdf=$fpdfall if checkFile($fpdfall);
			}
			foreach my $file (@jpgs) {
				unlink($file) if checkFile($file);
			}
			foreach my $file (@pdfs) {
				unlink($file) if checkFile($file);
			}
		}
	}
	return $fpdf;
}



=head1 $res=callPrgf($cmd2,$nolog)

Do a job and log it to default log file (no log if nolog!=0

=cut

sub callPrg {
	my ($cmd2,$nolog,$log) = @_;
	my $res=system($cmd2);
	$log=$cmd2 if $log eq "";
	logit("$res=>$log") if $nolog==0;
	return $res;
}



=head1 $res=OpenOfficeConvert($fin1,$path,$base1)

Create a pdf file from openoffice file and give back res (0=not,1=ok)

=cut

sub OpenOfficeConvert {
  my ($fin1,$path,$base1,$stage) = @_;
  my $fin1q = qq{'$fin1'};
  my $fin2 = "$path$base1\.pdf";
  my $prof = '. /etc/profile'; # import default settings
	my $nr = ":0";
	$nr = `cat /tmp/display` if -e "/tmp/display";
  my $disp = "export DISPLAY=$nr"; # export display to default
	my $pfad = "/opt/openoffice.org3/program";
	my $opt = "-nofirststartwizard -invisible";
	my $loffice = `which loffice`;
	logit("check Libre/OpenOffice: ".$loffice); 
	if (check64bit()==64 || $loffice ne "") {
	  $pfad = "/usr/bin";
		$opt = "-nologo";
	}
  my $prg = qq(/bin/su - archivista -c "$prof; $disp;)." ".
	          "$pfad/soffice $opt ".
			  	  qq{macro:///Standard.MyConversions.SaveAsPDF\\\(};
  my $prg1 = $prg.$fin1q.'\) "';
	if ($loffice ne "") {
	  $opt = "--invisible --nologo --nodefault --convert-to pdf ";
		my $go = "cd /home/data/archivista/tmp";
		$prg = qq(/bin/su - archivista -c "$prof; $disp; $go;)." ".
			     "$pfad/soffice $opt ";
	  $prg1 = $prg.$fin1q."\"";
	}
	my $res = 0;
	$res = callPrg($prg1);
	if (notFile($fin2) && $stage==0) {
    my $fin1a = "$path$base1\.txt";
    my $fin1q = qq{'$fin1a'};
    unlinkFile($fin1a) if checkFile($fin1a);
    $res=moveFile($fin1,$fin1a) if notFile($fin1a);
    if ($res==1) {
      $fin1 = $fin1a;
      $prg1 = $prg.$fin1q.'\) "';
	    $res = callPrg($prg1,1);
    } else {
		  $res=-1;
	  }
	}
	return ($res,$fin1,$fin2);
}



=head1 ($fin1,$path,$base1)=CheckFileNamePathBase($fnew)

Give back a new file name, the path and the base

=cut

sub CheckFileNamePathBase {
  my ($fnew) = @_;
  my $path = $val{tmpdir};
	my @parts = split(/\./,$fnew);
	my $ext = pop @parts;
	my $base = join("\.",@parts);
	$base = "document" if $base eq "";
	my $base1 = $base;
	$ext = "txt" if $base eq "";
	$ext = "txt" if $ext eq "";
	my $fin1 = "$path$base\.$ext";
	for (my $c=0;$c<10;$c++) {
	  if (checkFile($fin1)) {
	    $base1 = "$base-$c";
		  $fin1 = "$path$base1\.$ext";
			sleep 1;
		} else {
		  last;
		}
	}
	return ($fin1,$path,$base1);
}



=head1 $val=selectValue($dbh,$jobid,$attr)

Give back a volue from job_data table

=cut

sub selectValue {
  my ($dbh,$jobid,$attr) = @_;
	my $attr1 = $dbh->quote($attr);
  my $sql = "select value from jobs_data " .
            "where jid=$jobid and param=$attr1 limit 1";
  my @f = $dbh->selectrow_array($sql);
  # store the actual scan definition
	return $f[0];
}



=head1 $langs = OCRDoOpenSourceLang($dbh,$db1,$dbh2,$ocr,$nr)

Get the current language(s) string

=cut

sub OCRDoOpenSourceLang {
  my ($dbh,$db1,$dbh2,$ocr,$onr) = @_;
  my $sql = "select Inhalt from $db1.parameter where Name = 'OCRSets'";
	my @row = $dbh2->selectrow_array($sql);
	my $ocr1 = $row[0];
	my $langs = "";
	my $detect = 0;
	my $clean = 0;
	my $tables = 0;
	if ($ocr1 ne "") {
		my @defs = split(/\r\n/,$ocr1);
		my $found = 0;
		my $c = 0;
		foreach (@defs) {
		  my $ocrdef = $_;
		  my @entries = split(";",$ocrdef);
			if ($entries[0] eq $ocr && $ocr ne "") {
			  $found=$c;
				last;
			}
			if ($ocr eq "" && $onr==$c) {
			  $found=$c;
				last;
			}
			$c++;
		}
		my @entries1 = split(";",$defs[$found]);
		for (my $c1=1;$c1<=5;$c1++) {
		  my $idx = $entries1[$c1];
			if ($idx>=0) {
				if ($ocr{$idx} ne "") {
				  $langs .= "," if $langs ne "";
					$langs .= $ocr{$idx};
				}
			}
		}
		$detect = $entries1[7];
		$clean = $entries1[8];
		$tables = $entries1[11];
	}
	return ($langs,$detect,$clean,$tables);
}



=head2 $xrandr=getXRandr()

Give back if we have to fix intel graphic card

=cut

sub getXRandr {
  my ($profile_cmd,$display_cmd) = @_;
	my $xrandr = "";
  if (notFile("/tmp/xrandr.done") && check64bit()!=64) {
    my $card = `lspci`;
    $card =~ /(Display controller:)(\s)(.+?)(\s)/;
	  if ($1 eq "Display controller:" && $3 eq "Intel") {
		  my $randr = "/usr/X11/bin/xrandr";
		  logit("Intel card found");
		  my $cmd1 = "$profile_cmd; $display_cmd; $randr";
			my $res = `$cmd1`;
		  $res =~ /(VGA\sconnected)(\s)([0-9]+)(x)([0-9]+)(\+)(.*)/;
		  logit("Current resolution:$3--$5");
		  if ($1 eq "VGA connected" && $3>0 && $5>0 && $5>=768) {
	      my $x1=$3;
	      my $y1=$5;
				my $minimal = "1024x768";
				$minimal = "800x600" if $5==768;
        $xrandr = $cmd1." --size $minimal; $randr --size ".$x1."x".$y1.";";
        logit("$xrandr");
				callPrg($xrandr,1);
	      callPrg("touch /tmp/xrandr.done",1);
	    } else {
			  sleep 30;
			}
	  } else {
	    callPrg("touch /tmp/xrandr.done",1);
	  }	
  }
	return $xrandr;
}



=head2 OCRDoOpenSource($fin,$fout,$lang,$doocr)

Do an ocr job with the open sourc ocr ocrad

=cut

sub OCRDoOpenSource {
  my ($doc,$pages,$lang,$doocr,$dopdf,$pdfwhole,$pfad,
		$ext,$detect,$clean,$tables,$page,$db) = @_;
	if ($pfad eq "") {
		$pfad = $val{frpath};
	} else {
		$pfad .= "/";
	}
	my $start = 1;
	if ($page>0) {
		$start=$page;
		$pages=$page;
	}
	for(my $c=$start;$c<=$pages;$c++) {
	  my $file = $db."_".$doc."_".$c.".".$ext;
		if ($doocr==3 || $lang eq "Digits") {
    	my $image = ExactImage::newImage();
	    if (ExactImage::decodeImageFile($image,"$pfad$file")) {
				unlink "$pfad$file" if checkFile("$pfad$file");
			  my $bits = ExactImage::imageColorspace($image);
				$dopdf = 0;
        ExactImage::imageConvertColorspace($image,"gray1") if $bits ne "gray1";
	    	ExactImage::imageCrop($image,750,960,1750,180) if $lang eq "Digits";
				ExactImage::encodeImageFile($image,"$pfad$file");
				ExactImage::deleteImage($image);
			}
		}
	}
	my $found=checkmaxi(2);
	my $marker='{}';
	my $ord="seq $start $pages | parallel ".
		"--tmpdir /home/data/archivista/tmp -P $found ";
	if ($page>0) {
		$ord="";
		$marker=$page;
	}
	my $base = $pfad.$db."_".$doc."_".$marker;
	if ($doocr==3) {
		$ord .= "/usr/bin/ocrad -l $base.$ext > $base.txt";
	} else {
	  $ord .= "/usr/bin/tesseract ";
		my $lng1 = "";
		my @lang1 = split(",",$lang);
		foreach my $lang (@lang1) {
			$lng1.="+" if $lng1 ne "";
			if ($lang eq "German") {
	  		$lng1 .= "deu";
			} elsif ($lang eq "GermanNewSpelling") {
		  	$lng1 .= "deu-frak";
			} elsif ($lang eq "French") {
		 	 	$lng1 .= "fra";
			} elsif ($lang eq "Italian") {
			  $lng1 .= "ita";
			} elsif ($lang eq "Spanish") {
   	    $lng1 .= "spa";
			} elsif ($lang eq "Dutch") {
       	$lng1 .= "nld";
			} elsif ($lang eq "Digits") {
				$lng1 .= "ocb";
			} else {
			  $lng1 .= "eng";
			}
		}
		$ord .= $base.'.'.$ext.' '.$base;
		$ord .= " -c textord_tabfind_vertical_text=0" if $detect==0;
	  $ord .= " -c chop_enable=0" if $clean==0;
		$ord .= " -c textord_tabfind_find_tables=0" if $tables==0;
		$ord .= " -l $lng1" if $lng1 ne "";
		$ord .= " pdf" if $dopdf !=0 && $lang ne "Digits";
	}
  callPrg("$ord 2>/dev/null");
	for(my $c=$start;$c<=$pages;$c++) {
		if ($dopdf == 1) {
			$ord="cd $pfad;".$val{pdf2txt}." -raw ".$pfad.$db."_".$doc."_".$c.".pdf";
			callPrg($ord,1);
		}
		my $txt = $pfad.$db."_".$doc."_".$c.".txt";
		my $con = $pfad.$db."_".$doc."_".$c.".con";
		my $pgm = $pfad.$db."_".$doc."_".$c.".pgm";
    $ord = "iconv -c -f utf8 -t iso-8859-1 $txt -o $con";
		my $res=callPrg("$ord",1);
		if (checkFile("$con")) {
			unlinkFile("$txt"); 
			moveFile("$con","$txt");
		}
		unlinkFile("$pgm") if checkFile("$pgm");
	}
}



=head1 ($dopdf,$doocr,$pdfwholedoc,$osocr)=getOCRVals($dbh2,$db1)

Check for the user defined ocr/pdf parameters

=cut

sub getOCRVals {
  my ($dbh2,$db1) = @_;
  my $sql = "select Inhalt from $db1.parameter where Name = 'PDFFiles'";
  my @felder=$dbh2->selectrow_array($sql);
  my $dopdf=$felder[0];
	$dopdf=1 if $dopdf eq "";
  $sql = "select Inhalt from $db1.parameter where " .
         "Name = 'JobsOCRRecognition'";
  @felder=$dbh2->selectrow_array($sql);
  my $doocr=$felder[0];
	$doocr=1 if $doocr eq "";
	$doocr=2 if `uname -r` > 5;
  $sql = "select Inhalt from $db1.parameter where Name = 'PDFWHOLEDOC'";
  @felder=$dbh2->selectrow_array($sql);
  my $pdfwholedoc=$felder[0];
	$pdfwholedoc=1 if $pdfwholedoc eq "";
	if ($doocr!=1 && $doocr !=2) {
	  $dopdf = 0;
		$pdfwholedoc = 0;
	}
	# Activate Tesseract if ArchivistaOCR option is not available (always)
	if (notFile('/home/archivista/.wine/drive_c/Programs/Av5e/avocr.exe')) {
	  $doocr=2;
	}
  $sql = "select Inhalt from $db1.parameter where Name = 'OCRLIMIT01'";
	my ($vals) = $dbh2->selectrow_array($sql);
	my ($limit,$start,$end,$pdftemp,$pdfhost,
		$pdfdomain,$pdffold,$pdfuser,$pdfpw) = split(";",$vals);
	return ($dopdf,$doocr,$pdfwholedoc,$limit,$start,$end,
		$pdftemp,$pdfhost,$pdfdomain,$pdffold,$pdfuser,$pdfpw);
}



=head1 vers=check64bit 

Give back if we are under 32 or 64 bit

=cut

sub check64bit {
  my $vers = 32;
	my $linux = `cat /proc/version`;
	$vers=64 if $linux =~ /Debian/;
	return $vers;
}



=head1 MainUpdateText($fld,$val,%text) 

Store all fields we have right to save

=cut

sub MainUpdateText {
  my ($fld,$val,$ptext) = @_;
	my @parts = split("_",$fld);
  my $fld1 = pop @parts;
	$val = "NULL" if $val eq "";
	$$ptext{$fld1} = $val;
}



=head1 MainUpdateTextUpdate($dbh,$db1,$doc,$pages,$ptext)

Save the new values to page text table

=cut

sub MainUpdateTextUpdate {
  my ($dbh,$db1,$doc,$pages,$ptext,$kill) = @_;
	if ($pages>0) {
    my $seitennr = ($doc*1000)+1;
		my $sql = "select Text,Seite from $db1.archivseiten where Seite=$seitennr";
		my @row = $dbh->selectrow_array($sql);
		my $string = $row[0];
		my $seite = $row[1];
		my $insert = 0;
		$insert=1 if $seite==0;
		my $sep = "-" x 40;
		$sep = "\r\n".$sep."\r\n";
		my @parts = split($sep,$string);
		my $text = "";
		$text = pop @parts if $parts[1] ne "";
		if ($kill != 1) {
		  my @fields = split("\r\n",$text);
		  my %saved = ();
		  foreach my $line (@fields) {
        my @parts1 = split(": ",$line);
			  my $fld1 = shift @parts1;
			  my $val1 = join(": ",@parts1);
			  if (!exists $$ptext{$fld1}) {
			    $$ptext{$fld1} = $val1;
			  }
		  }
		  my $out = "";
		  foreach my $fld2 (keys %$ptext) {
		    $out .= "\r\n" if $out ne "";
			  $out .= "$fld2: ".$$ptext{$fld2};
		  }
		  push @parts,$out;
		}
		my $outtext = join($sep,@parts);
		$outtext = $sep . $outtext if $insert==1;
    $sql = "update $db1.archivseiten set ";
		$sql = "insert into $db1.archivseiten set Seite=$seitennr," if $insert==1;
		$sql .= "Text = ".$dbh->quote($outtext);
		$sql .= " where Seite=$seitennr" if $insert==0;
		$dbh->do($sql);
	}
}



=head1 $formated=getZeros($pages)

Give back the format string we need for pdftoppm (different 32/64 bit)

=cut

sub getZeros {
  my ($pages) = @_;
  my $zeros = 6;
	# version 3.02 of xpdf does not have the zero problem
	my $raspi = 0;
  my $res = `cat /proc/cpuinfo | grep 'ARM'`;
	$raspi = 1 if $res ne "";
	if (check64bit()==64 || $raspi==1) {
	  if ($pages<10) {
		  $zeros = 1;
		} elsif ($pages<100) {
		  $zeros = 2;
		} elsif ($pages<1000) {
		  $zeros = 3;
		} elsif ($pages<10000) {
      $zeros = 4;
		} elsif ($pages<100000) {
		  $zeros = 5;
		}
	}
  #$zeros = 0 if `cat /proc/version | grep 'Debian 4.4.'` ne "";
	$zeros = "%0".$zeros."d";
  return $zeros;
}



=head1 $file3=getZeroFile($file1,$seite,$ext)

Give back the appropriate name after pdftoppm call

=cut

sub getZeroFile {
  my ($file1,$seite,$ext) = @_;
	my @parts = split(/\//,$file1);
	my $basename = pop @parts;
	my $basepath = join('/',@parts);
	my $file3 = `ls $file1\*$seite\.$ext 2>/dev/null`;
	chomp $file3;
	@parts = split(/\n/,$file3);
	$file3 = $parts[0];
	chomp $file3;
	return $file3;
}



=head2 ($fname,$db,$scandef)=getFileDB($file)

Gives back the filename and the database

=cut

sub getFileDB {
  my $file = shift;
  my @parts = split($val{dirsep},$file);
  my $fname = pop @parts;
  my $db = pop @parts;
	my $db1 = pop @parts;
	my $typ = pop @parts;
	my $ftp = pop @parts;
	my $scandef = "";
	if (($typ eq "pdf" || $typ eq "xerox") && $ftp eq "ftp") {
		$scandef = $db;
	  $db = $db1;
	} else {
    if (($db eq "pdf" || $db eq "xerox") && 
			$db1 eq	"ftp" && $typ eq "archivista") {
			$db = "archivista";
		}
	}
  @parts = split(/\./,$fname);
	my $ext = pop @parts;
	if ($ext eq "PDF") {
	  $ext = "pdf";
	  push @parts,$ext;
		$fname = join('.',@parts);
	}
	return ($fname,$db,$scandef);
}



sub getInfos {
  my ($dir,$file,$db,$scandefname,$bits,$savescandef) = @_;
  my %info;
  my $line = getIdentify("$dir$file");
  my @lines = split("\n",$line);
	foreach (@lines) {
    my ($name,$val) = split(/\s{2,10}/,$_);
		if ($name eq "Pages:") {
      $info{'Number of pages'}=$val;
		} elsif ($name eq "Page size:") {
      my @temp = split(" ",$val);
      $info{'Width'}=$temp[0];
			$info{'Height'}=$temp[2];
		}
	}
  $info{'Format'}=1;	
  $info{'Bits per pixel'}=1;
	if ($bits>0) {
    $info{'Bits per pixel'}=$bits;
	} else {
	  my $dbh = MySQLOpen();
	  my $scandef=getScanDefByName($dbh,$db,$scandefname);
	  $dbh->disconnect();
    my @parts=split(/;/,$scandef);
	  $info{'Bits per pixel'}=8 if $parts[1]==1;
	  $info{'Bits per pixel'}=24 if $parts[1]==2;
	  $info{'Profile'}=$scandef if $savescandef==1;
	}
  my $dbh = MySQLOpen();
	my $cold2=getParameterRead($dbh,$db,"COLD2",0,1,0);
  $dbh->disconnect();
	$info{'COLD2'}=1 if $cold2==1;
  $info{'Format'}=3 if $info{'Bits per pixel'}!=1;
  $file =~ s/\.[a-zA-Z]{3,3}$/\.pdf/g;
  $info{'File name'}="$dir$file";
  $info{'Date'}=$file;
  $info{'Paper size'}="A4";
  $info{'Destination'}=$db;
	# correction from points to pixels
	$info{Width} = int $info{Width} * 4.166;
	$info{Height} = int $info{Height} * 4.166;
  $info{'X Resolution (DPI)'}=300;
  $info{'Y Resolution (DPI)'}=300;
  if($info{'Height'} > $info{'Width'}){
    $info{'Paper Orientation'}='Portrait';
  } else {
    $info{'Paper Orientation'}='Landscape';
  }
  return \%info;
}



=head2 String getIdentify( $dir,$file )

Returns the Information of identify $file.

=cut

sub getIdentify {
  my ($file) = @_;
  my $result;
  my $system = "pdfinfo \"$file\"";
  $result = `$system`;
  return $result;
}



=head2 void createAXIS( $phash,$dir )

Writes an Index-File like AXIS for our PDF-File.

=cut

sub createAXIS {
  my $phash = shift;
	my $noocr = shift;
  my $outfile = $phash->{'File name'};
  $outfile =~ s/\.pdf/\.txt/g;
  open(FOUT,">$outfile");
  binmode(FOUT);
  foreach my $key (keys %$phash){
    print FOUT "$key"," "x(21-length($key)),"= ",$phash->{"$key"},"\n";
  }
  print FOUT "NonAxisFile  "," "x9,"= 1\n";
  print FOUT "NoOCRfromCUPS"," "x9,"= 1\n" if $noocr==1;
  close(FOUT);
}




sub closeSessionKey {
  my ($message) = @_;
	my $output=unpack("h*",$message);
  my @length=$output=~/.{0,256}/g;
	return $output;
}



sub openSessionKey {
  my ($message) = @_;
  chomp($message);
  my $output.=pack"h*",$message;
	return $output;
}


sub checkIt {
  my ($liz) = @_;
	my $mode="";
  use constant CHECKPRG => '/home/cvs/archivista/webclient/perl/checkit';
	if (checkFile(CHECKPRG)) {
	  my $prg = CHECKPRG;
	  $mode = `$prg`;
	} else {
	  die;
	}
  return $mode;
}



sub checkInitialized {
  my ($key) = @_;
  my $res = checkIt();
	my $str1 = openSessionKey($key);
	my $str2 = openSessionKey("955435");
  $res =~ /($str1)(:)(\s)($str2)/;
  return 1 if $4 eq $str2;
}



=head1 fromUTF8

Convert a string fromUTF8 and give it back as iso-8859-1

=cut

sub fromUTF8 {
  my ($string) = @_;
  $string = Encode::decode_utf8($string);
  $string = encode("iso-8859-1", $string);
  return $string;
}



=head1 toUTF8

Convert a string toUTF8 (from iso-8859-1)

=cut

sub toUTF8 {
  my ($string) = @_;
  Encode::from_to($string,"iso-8859-1","utf8");
	return $string;
}



sub correctFile {
	my ($file) = @_;
	chomp $file;
	$file =~ s/\r//g;
	$file =~ s/\n//g;
	$file =~ s/\"/\\\"/g;
	return $file;
}



=head1 $res=moveFile($in,$out)

Use system call to move a file (1=success,0=no success

=cut 

sub moveFile {
  my ($in,$out,$logit) = @_;
	my $nolog=0;
	$nolog=1 if $logit==0;
	$in=correctFile($in);
	$out=correctFile($out);
	my $ok=1;
	my $cmd = "mv \"$in\" \"$out\"";
	my $res=callPrg($cmd,$nolog);
	$ok=0 if $res != 0;
	return $ok;
}



=head1 $res=checkFile($file)

Check if a file does exist (gives back 1 if it is so)

=cut

sub checkFile {
  my ($file) = @_;
	my $res = 0;
	$res=1 if -e "$file";
	return $res;
}



=head1 $res=checkDir($file)

Check if a direcotry does exist (gives back 1 if it is so)

=cut

sub checkDir {
  my ($file) = @_;
	my $res = 0;
	$res=1 if -d "$file";
	return $res;
}



=head1 $res=notFile($file)

Check if a file does NOT exist (gives back 1 if it is so)

=cut

sub notFile {
  my ($file) = @_;
	my $res = 0;
	$res=1 if !-e "$file";
	return $res;
}



=head1 $res=notDir($file)

Check if a file is NOT a directory (gives back 1 if it is so)

=cut

sub notDir {
  my ($file) = @_;
	my $res = 0;
	$res=1 if !-d "$file";
	return $res;
}



=head1 $res=unlinkFile($file)

Remove a file (give back 1 if it is successfully)

=cut

sub unlinkFile {
  my ($file) = @_;
	my $res = 0;
	$res=1 if unlink "$file";
	#logit("$res=>unlink $file");
	return $res;
}



=head1 MainNoteHash

Returns a hash of all notes for a page, keyed on the note index
Also updates the params argument with image size info if any of the
notes contain it, or gets it from image if all of them need it.

=cut
sub MainNoteHash {
  my ($dbh,$db1,$params,$imgo) = @_;
  # $params = contains: doc, page, cWidth, cHeight, and more
  # $imgo = only send BildInput!
  my $pn = ($params->{doc}*1000)+$params->{page};
  
  ##############################################################
  # pull existing notes from db and cache them in @dbnotes
  # also determine the maximum note index
  my @dbnotes = ();
  my $maxIdx = 0;

  my $sql = "select Notes from $db1.archivseiten where Seite=$pn";
	my @res = $dbh->selectrow_array($sql);
  foreach my $line (split(/\r\n/,$res[0])) {
    my $note = obj::Note->new();
    $note->fromString($line);

    # if the caller gave us incomplete data, use _ALL_ data from note
    if ( !$params->{'iWidth'} || !$params->{'iHeight'}
      || !$params->{'iXRes'} || !$params->{'iYRes'}
    ){
      $params->{'iWidth'} = $note->get('iWidth');
      $params->{'iHeight'} = $note->get('iHeight');
      $params->{'iXRes'} = $note->get('iXRes');
      $params->{'iYRes'} = $note->get('iYRes');
    }

    if($maxIdx < $note->get('index')){
      $maxIdx = $note->get('index');
    }

    push (@dbnotes, $note);
  }

  # if the caller AND all the notes gave us incomplete data,
  # get the correct data from image
  if ( (!$params->{'iWidth'} || !$params->{'iHeight'}
    || !$params->{'iXRes'} || !$params->{'iYRes'})
    && (scalar @dbnotes)
  ){
    my $deleteImgo = 0;
    ##############################################################
    # extract size and resolution
    $params->{'iWidth'} = ExactImage::imageWidth($imgo);
    $params->{'iHeight'} = ExactImage::imageHeight($imgo);
    $params->{'iXRes'} = ExactImage::imageXres($imgo);
    $params->{'iYRes'} = ExactImage::imageYres($imgo);
  
    # some images dont have resolution encoded. we make a guess
    if(!$params->{'iXRes'}){
      $params->{'iXRes'} = 300;
    }
    if(!$params->{'iYRes'}){
      $params->{'iYRes'} = 300;
    }
  }

  ##############################################################
  # add missing params and index to notes made by RichClient
  # rotate note params if page view is rotated (not for pdfs)
  # convert array @dbnotes to hash %dbnotes
  my %dbnotes;
  foreach my $note (@dbnotes){
    $note->set(%{$params});
    $note->updateBgParams();
    if($note->get('index') < 1){
      $maxIdx++;
      $note->set('index'=>$maxIdx);
    }
    $dbnotes{$note->get('index')} = $note;
  }
  return \%dbnotes;
}



# check for the ammount of ram (small is <5gb)

sub smallram {
  my $smallram = 0;
  my $mem = `cat /proc/meminfo | grep 'MemTotal'`;
  $mem =~ s/\s+/ /g;
  my @parts = split(/\s/,$mem);
  $smallram=1 if $parts[1]<5000000;
	return $smallram;
}


sub md5get {
  my ($pdata) = @_;
	my $md5 = md5_hex $$pdata;
	return $md5;
}


sub md5check {
  my ($dbh,$db1,$md5) = @_;
	my $sql = "select count(Seite) from archivseiten where ".
	          "Schlussel=".$dbh->quote($md5);
  my @row = $dbh->selectrow_array($sql);
	return $row[0];
}


sub splitEDVName {
	my ($edvname) = @_;
	my $type = "";
	my $name = "";
	my $ext = "";
	my $mailrestore = "";
	if ($edvname ne "") {
		my @parts0 = split(";",$edvname);
		$type = shift @parts0;
		if ($type eq "mail") {
			$name = $parts0[0];
			$mailrestore = $parts0[1];
		} else {
			$name = join(';',@parts0);
			my @parts = split(/\./,$name);
			$ext = pop @parts;
			$name = join('.',@parts);
			if ($ext ne "" && $name eq "") {
				$name = $ext;
				$ext = "";
			}
		}
	}
	return ($type,$name,$ext,$mailrestore);
}


sub isSourceExternalLC {
	my ($archpfad,$db1,$lnr,$ext) = @_;
	my $file = "$archpfad/$db1-$lnr";
	$file = "$archpfad/$db1-$lnr.$ext" if $ext ne "";
	my $file1 = "$archpfad/$db1-$lnr.".lc($ext);
	my $file2 = "$archpfad/$db1-$lnr.".uc($ext);
	my $fileout = "";
	if (-e "$file") {
		$fileout=$file;
	} else {
		if (-e "$file1") {
			$fileout=$file1;
		} else {
			if (-e "$file2") {
				$fileout=$file2;
			}
		}
	}
	return $fileout;
}


# check if a file is in archmulti storage
sub isSourceExternal {
	my ($db1,$lnr,$type,$ext,$pathin) = @_;
	my $fileout = "";
	my $archpfad = "/home/data/archivista/archmulti";
	$archpfad = $pathin if $pathin ne "";
	if ($type eq "multi") {
		my $file = isSourceExternalLC($archpfad,$db1,$lnr,$ext);
		if ($file eq "") {
			for (my $c=0;$c<=9;$c++) {
				$file = isSourceExternalLC("$archpfad$c",$db1,$lnr,$ext);
				if (-e "$file") {
					$fileout = $file;
					last;
				}
			}
		} else {
			$fileout = $file;
		}
	}
	return $fileout;
}



# Save pdata (scalar or pointer) to temp outfile
sub writeTempFile {
  my $pdata = shift;
  my ($fh,$tempfile) = tempfile(DIR=>"/home/data/archivista/tmp");
  if ($fh) {
    binmode($fh);
    if (ref($pdata) eq "SCALAR") {
      print $fh $$pdata;
    } else {
      print $fh $pdata;
    }
    close($fh);
  } else {
    $tempfile = "";
  }
  return $tempfile;
}



# Get a temp file with an extension and kill it if wished
sub getTempFile {
  my ($file1,$ext,$killit) = @_;
  unlink $file1 if -e $file1 && $killit==1;
  $file1 .= $ext;
  unlink $file1 if -e $file1 && $killit==1;
  return $file1;
}



# export source file from blob talbe to disk
sub exportDbToDisk {
	my ($dbh,$db1,$type,$lnr,$seiten,$ordner,$archiviert,$archivfolders) = @_;
	my $file = ""; # give back exported file name
	my @files = (); # storage for all file chunks (1-x parts)A
	if ($type eq "mail" || $type eq "office" || $type eq "multi") {
		for (my $seite=1;$seite<=$seiten;$seite++) {
  		my $nr = ($lnr*1000)+$seite;
   		my $prow = getBlobFile($dbh,"BildA",$nr,$ordner,$archiviert,
				"archivbilder",$archivfolders);
			if (length($$prow)>0) {
	    	my $file1 = writeTempFile(\$$prow);
				if (-e "$file1") {
					push @files,$file1;
				} else {
					last;
				}
			}
		}
		if ($files[1] eq "") {
			$file = $files[0]; # there is only one file
		} else {
 			$file = writeTempFile(""); # multimedia files over 512 MByte
			my $cmd = "cat ".join(" ",@files)." >$file";
			my $res = callPrg($cmd);
			foreach my $file1 (@files) {
				unlink $file1;
			}
		}
	}
	return $file;
}



sub md5File {
	my ($file) = @_;
	$file=correctFile($file);
	my $cmd = "md5sum \"$file\"";
	my $md5all = `$cmd`;
	my ($md5,$fname) = split(/\s/,$md5all);
	return $md5;
}



# unzip first file from zipped source file
sub unzipFirstFile {
	my ($filezip,$ext) = @_;
	my $file2 = writeTempFile("");
	$file2 = getTempFile($file2,'.'.$ext,1);
	my $zip = Archive::Zip->new();
	my $res = $zip->read($filezip);
	my @files = $zip->memberNames();
	if ($files[0] ne "") {
		$res=$zip->extractMember($files[0],$file2);
		unlink "$file2" if $res>0;
	} else {
		$file2="";
	}
	return $file2;
}



# add the md5 hash to the database (from file or check over all pages)
sub md5add2 {
	my ($dbh,$db1,$lnr,$seite,$ordner,$archiviert,$archivfolders,$file) = @_;
	my $md5 = "";
	my $fname = "";
	if (-e "$file") {
		$md5 = md5File($file);
	} else {
  	my $pblob = getBlobFile($dbh,"BildA",$seite,$ordner,
			$archiviert,"archivbilder",$archivfolders);
		if ($$pblob ne "") {
			$md5 = md5_hex($$pblob);
		}
	}
	if ($md5 ne "") {
		my $page = ($lnr*1000)+$seite;
		my $sql = "select Seite from $db1.archivseiten where Seite=$page";
		my @res = $dbh->selectrow_array($sql);
		my $page1 = $res[0];
		$sql = "insert into $db1.archivseiten ";
		$sql = "update $db1.archivseiten " if $page1==$page;
		$sql .= "set Schlussel=".$dbh->quote($md5).",Seite=$page";
		$sql .= " where Seite=$page" if $page1==$page;
    $dbh->do($sql);
	}
}



# add md5 hashes to archivseiten table
sub md5add {
  my ($dbh,$db1) = @_;
  return 0 if HostIsSlave($dbh)==1;
	my $aktnr = 1;
  my $archivfolders = ""; # check if we have extended tables
	$archivfolders = getBlockTableCheck($dbh,$archivfolders,$db1);
	while ($aktnr) {
 		my $sql = "SELECT Laufnummer,Seiten,Ordner,Archiviert,EDVName,Gesperrt ".
			"FROM $db1.archiv WHERE Laufnummer>=$aktnr order by Laufnummer limit 1";
		my @row = $dbh->selectrow_array($sql);
		my $lnr = $row[0];
		if ($lnr>0) {
			my $seiten = $row[1];
			my $ordner = $row[2];
			my $archiviert = $row[3];
			my $edvname = $row[4];
			my $gesperrt = $row[5];
			my $page = ($lnr*1000)+1;
			my $sql = "select Schlussel from $db1.archivseiten where Seite=$page";
			my ($key) = $dbh->selectrow_array($sql);
			if ($seiten>0 && $gesperrt eq "" && $key eq "") {
		    logit("add md5 to $lnr with $seiten pages in $db1");
		    $sql = "update $db1.archiv set Gesperrt='md5imp' where ".
			         "Laufnummer=$lnr and (Gesperrt is null or Gesperrt='')";
				$dbh->do($sql);
				$sql = "select Gesperrt from $db1.archiv where Laufnummer=$lnr";
				my ($locked) = $dbh->selectrow_array($sql);
				if ($locked eq "md5imp") {
					my $file = "";
					my $killit = 0;
					if ($edvname ne "") {
						my ($type,$name,$ext) = splitEDVName($edvname);
						$file = isSourceExternal($db1,$lnr,$type,$ext) if $type eq "multi";
						if ($file eq "") {
							$killit = 1;
							$file = exportDbToDisk($dbh,$db1,$type,$lnr,$seiten,$ordner,
								$archiviert,$archivfolders);
						}
						if (-e "$file") {
							my $file1=unzipFirstFile($file,$ext) if $type ne "multi";
							if ($file1 ne $file && $killit==1) {
								unlink "$file";
								$file = $file1;
							}
						}
						md5add2($dbh,$db1,$lnr,1,$ordner,$archiviert,$archivfolders,$file);
						unlink "$file" if $killit==1;
					} else {
						for (my $seite=1;$seite<$seiten;$seite++) {
							md5add2($dbh,$db1,$lnr,$seite,$ordner,$archiviert,$archivfolders,"");
						}
					}
    	  	DocumentUnlock($dbh,$db1,$lnr);
				} else {
					logit("Document $lnr in $db1 is locked for $locked, so no md5 adding");
				}
			}
			$aktnr = $lnr+1;
		} else {
			$aktnr=0;
		}
	}
	logit("finishing adding md5 hashes");
	return 1;
}



# check if adding to last document
sub checklastdocadd {
	my ($obj) = @_;
	if ($obj->doc==0 && $obj->addlastdoc>0 && $ENV{SCAN_FILE} eq "") {
		my $sql="select pwd from jobs where job='LASTDOC' and ".
			"host=".$obj->dbh->quote($obj->host)." and ".
			"db=".$obj->dbh->quote($obj->database)." and ".
			"user=".$obj->dbh->quote($obj->user);
		logit($sql);
		my @row=$obj->dbh->selectrow_array($sql);
		if ($row[0]>0) {
 			$ENV{'AV_SCAN_TO_DOCUMENT'}=$row[0];
			$obj->doc($row[0]);
		}
	}
}

# Check if we have a maxi=x flag

sub checkmaxi {
	my ($part) = @_; # ie 2 => found/2 (1/2 of cpus)
	$part=1 if $part<1;
	my $cmdline=`cat /proc/cmdline`;
	my $found=checkmaxi1($cmdline);
	if ($found==0 && -s "/iso/syslinux.cfg") {
		$cmdline=`grep 'maxi' /iso/syslinux.cfg`;
		$found=checkmaxi1($cmdline);
	}
	$found = int($found/$part);
	$found = 1 if $found<1;
	return $found;
}

sub checkmaxi1 {
	my ($cmdline) = @_;
	my $found=0;
	my @values=split(/\s/,$cmdline);
	foreach my $val (@values) {
		my ($name,$val) = split('=',$val);
		if ($name eq "maxi") {
			$found=$val;
			callPrg("echo $val >/tmp/maxi",1);
			last;
		}
	}
	return $found;
}

sub OCRScriptPDFCheck {
	my ($dbh,$host1,$db1,$user1,$pw1,$lnr,$pdftemp,$pdfhost,
		$pdfdomain,$pdffold,$pdfuser,$pdfpw) = @_;
 	my $job2="/home/data/archivista/cust/autofields/$db1"."ocr.pl";
	if (-e $job2) {
 		my $cmd="/usr/bin/perl $job2 $host1 $db1 $user1 $pw1 $lnr";
 	 	my $cmd1="/usr/bin/perl $job2 $host1 $db1 $user1 *** $lnr";
		my $res=callPrg($cmd,1);
		logit("$res=>$cmd1");
	}
	if ($pdftemp==1) {
		my $doc=$lnr;
		my $sql = "select Laufnummer,Seiten,Titel,Ordner,Archiviert,ArchivArt ".
			"from archiv where Laufnummer=$doc";
		my ($akte,$pages,$titel,$ord,$arch,$art) = $dbh->selectrow_array($sql);
		if ($akte==$doc && $pages>0) {
			$titel=$db1."_".$doc if $titel eq "";
			my @parts = ();
			my $pfad = "/home/data/archivista/ftp/temp";
			my $fend="$titel.pdf";
			my $single = 0;
			for(my $page=1;$page<=$pages;$page++) {
				$single=1 if $pages==1;
				if ($page==1 && $single==0) {
					my $lnr = ($doc*1000)+$pages;
					my $pblob=getBlobFile($dbh,"Quelle",$lnr,$ord,$arch,"archivbilder");
					$single=1 if length($$pblob)>0;
				}
				my $lnr = ($doc*1000)+$page;
				my $pblob=getBlobFile($dbh,"Quelle",$lnr,$ord,$arch,"archivbilder");
				my $fname="$titel"."_".$page.".pdf";
				if (length($$pblob)>0) {
					my $res=writeFile("$pfad/$fname",$pblob,1);
				} else {
					$pblob=getBlobFile($dbh,"BildInput",$lnr,$ord,$arch,"archivbilder");
					my $fimg="$pfad/$titel"."_".$page;
					my $ext = "jpg";
					$ext = "tif" if $art==1;
					$fimg .= ".$ext";
					my $res=writeFile($fimg,$pblob,1);
					my $cmd = "econvert -i $fimg -o $pfad/$fname";
					$res=callPrg($cmd,1);
					unlinkFile($fimg) if checkFile($fimg);
				}
				push @parts,$fname if -e "$pfad/$fname";
				last if $single==0;
			}
			my $in = join("\" \"",@parts);
			$in = "\"$in\"";
			my $cmd = "cd $pfad;/usr/bin/pdftk $in output \"$fend\"";
			my $res=callPrg($cmd,1);
			foreach my $fdel (@parts) {
				unlink("$pfad/$fdel") if checkFile("$pfad/$fdel");
			}
			callPrg("chown ftp.users $pfad/$fend",1);
			if ($pdfhost ne "" && $pdffold ne "" && $pdfuser ne "" && $pdfpw ne "") {
				my $pfad1 = "/mnt/pdf";
				mkdir $pfad1 if !-e $pfad1;
				my $mnt = "//$pdfhost/$pdffold $pfad1 -o user=$pdfuser,passwd=$pdfpw";
				$mnt .= ",domain=$pdfdomain" if $pdfdomain ne "";
				$cmd = "mount -t cifs $mnt";
				$res=callPrg($cmd,1);
				if ($res==0) {
					$cmd="cp -f \"$pfad/$fend\" $pfad1";
					$res=callPrg($cmd);
					callPrg("umount $pfad1",1);
					unlinkFile("$pfad/$fend") if checkFile("$pfad/$fend");
				}
			}
		}
	}
}



=head1 exportfiles ($dbh, $db, $doc, $pages,, $type (JPG/TIF/PNG), $source)
 
does export all images of one document

=cut

sub exportfiles {
  my ($dbh,$db,$pfad,$akte,$seiten,$typ,$quelle) = @_;
	$pfad = $pfad.'/';
	$pfad =~ s/(.*?)(\/)(\/)/$1$2/;
  my $pfadname;
  my $fname;
  my $out="\t";
	my $sql1 = "select Ordner,Archiviert,EDVName ".
		"from $db.archiv where Laufnummer=$akte";
	my ($ordner,$arch,$edvname) = $dbh->selectrow_array($sql1);
	my $archfolders = "";
	$archfolders = getBlockTableCheck($dbh,$archfolders,$db);
  my @sources = (); # pdf parts
  for(my $c=1;$c<=$seiten;$c++) {
    my $seite=$akte*1000+$c;
		if ($typ ne "") { # normal export
	    my $pblob=getBlobFile($dbh,"BildInput",$seite,$ordner,$arch,$archfolders);
  	  if ($$pblob eq "") {
	      $pblob=getBlobFile($dbh,"Bild",$seite,$ordner,$arch,$archfolders);
			}
    	if ($$pblob ne "") {
      	$fname="$akte"."_"."$c\.$typ";
				my $fname1="Z:\\transfer\\".$fname;
  	    $out.="$fname1;";
    	  $pfadname="$pfad$fname";
      	open(FOUT1,">$pfadname");
	      binmode(FOUT1);
  	    print FOUT1 $$pblob;
    	  close(FOUT1);
			}
			$pblob=getBlobFile($dbh,"Quelle",$seite,$ordner,$arch,$archfolders);
			if ($$pblob ne "") {
				$fname="$akte"."_"."$c\.$quelle";
				$pfadname="$pfad$fname";
				open(FOUT1,">$pfadname");
				binmode(FOUT1);
				print FOUT1 $$pblob;
				close(FOUT1);
				push @sources,$pfadname;
			}
		}
		if ($edvname ne "" && $c==1) {
			my $copyit = 1;
			my $file = "";
			my ($type,$name,$ext) = splitEDVName($edvname);
			$file = isSourceExternal($db,$akte,$type,$ext) if $type eq "multi";
			if ($file eq "") {
				$copyit=0;;
				$file = exportDbToDisk($dbh,$db,$type,$akte,$seiten,
					$ordner,$arch,$archfolders);
			}
			if (-e "$file") {
				$ext = "zip" if $type ne "multi";
				my $fname="$akte"."_"."$c\.$ext";
				my $pfadname="$pfad$fname";
				if ($copyit==1) {
					my $cmd = "cp -f \"$file\" $pfadname";
					my $res = callPrg($cmd);
				} else {
					moveFile($file,$pfadname,1);
				}
			}
		}
		if ($typ ne "") {
	    my $sql = "select Text from archivseiten where Seite = $seite";
      my @r=$dbh->selectrow_array($sql);
 	    if ($r[0] ne "") {
   	    $fname="$akte"."_"."$c\.TXT";
     	  $pfadname="$pfad$fname";
       	open(FOUT1,">$pfadname");
        binmode(FOUT1);
 	      print FOUT1 $r[0];
   	    close(FOUT1);   
	  	} else {
				$out.=";";
			}
		}
	}
	if ($typ ne "") {
 		if ($sources[0] ne "") {
	    my $pdf1="/tmp/all.pdf";
   		my $dopdf="pdftk ".join(" ",@sources)." output $pdf1";
	    calPrg("$dopdf",1);
			foreach (@sources) {
			  my $file = $_;
			  unlink $file if -e $file;
			}
			move($pdf1,$sources[0]);
		}
	}
  return $out;
}

sub getEDVName {
	my ($dbh,$db,$doc) = @_;
	my $sql = "select EDVName from $db.archiv where Laufnummer=$doc";
	my @row = $dbh->selectrow_array($sql);
	my ($typ,$file,$ext) = splitEDVName($row[0]);
	return ($typ,$file,$ext);
}

sub updateTitelKfile {
	my ($dbh,$db,$lnr,$bas) = @_;
	my $sql = "select Laufnummer,Titel from $db.archiv where Laufnummer=$lnr";
	my @row1 = $dbh->selectrow_array($sql);
	my $lnr1 = $row1[0];
	if ($lnr1 == $lnr) {
		my $titel = $row1[1];
		if ($titel ne "") {
			my $pos = index($titel,$bas);
			$titel .= " - $bas" if $pos==-1;
		} else {
			$titel = $bas;
		}
		$sql = "update $db.archiv set Titel=".$dbh->quote($titel).
			" where Laufnummer=$lnr";
		$dbh->do($sql);
	}
}

sub speechRecognition {
	my ($db,$lnr) = @_;
	my $maxi = 4;
	$maxi=`cat /tmp/maxi` if -e "/tmp/maxi";
	my $maxcpus = `cat /proc/cpuinfo | grep 'bogomips'`;
	my @cpus = split(/\n/,$maxcpus);
	my $totcpus = @cpus;
	$totcpus = $totcpus-1;
	$totcpus=1 if $totcpus<1;
	$maxi=$totcpus if $maxi>$totcpus;
	my $pfad = "/home/cvs/archivista/jobs";
	my $vosk = "vosk2db.pl";
	if (-e "/home/data/vosk/vosk2txt.py") {
		while (1) {
			my $jobs0 = `ps ax | grep $vosk`;
			my @psjobs = split(/\n/,$jobs0);
			last if $psjobs[$maxi] eq "";
			sleep 3;
		}
		my $cmd = "cd $pfad;/usr/bin/perl $pfad/vosk2db.pl $db $lnr";
		my $res=callPrg($cmd);
	}
}

# must be 1
1;

